@@ -14,11 +14,15 @@
 
 package com.obrador;
 
-import java.awt.*;
-import java.awt.image.*;
-import java.io.*;
-import java.util.ArrayList;
-import java.util.List;
+import java.awt.AWTException;
+import java.awt.Frame;
+import java.awt.Image;
+import java.awt.MediaTracker;
+import java.awt.image.PixelGrabber;
+import java.io.BufferedOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.util.Vector;
 
 /*
  * JpegEncoder - The JPEG main program which performs a jpeg compression of
@@ -274,16 +278,16 @@ public class JpegEncoder extends Frame
     DHT4[1] = (byte) 0xC4;
     for (i = 0; i < 4; i++ ) {
       bytes = 0;
-      DHT1[index++ - oldindex] = (byte) ((int[]) Huf.bits.get(i))[0];
+      DHT1[index++ - oldindex] = (byte) Huf.bits.elementAt(i)[0];
       for (j = 1; j < 17; j++) {
-        temp = ((int[]) Huf.bits.get(i))[j];
+        temp = Huf.bits.elementAt(i)[j];
         DHT1[index++ - oldindex] =(byte) temp;
         bytes += temp;
       }
       intermediateindex = index;
       DHT2 = new byte[bytes];
       for (j = 0; j < bytes; j++) {
-        DHT2[index++ - intermediateindex] = (byte) ((int[]) Huf.val.get(i))[j];
+        DHT2[index++ - intermediateindex] = (byte) Huf.val.elementAt(i)[j];
       }
       DHT3 = new byte[index];
       java.lang.System.arraycopy(DHT4, 0, DHT3, 0, oldindex);
@@ -847,8 +851,8 @@ class Huffman
       0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 
       0xea, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
       0xf9, 0xfa };
-  public List bits;
-  public List val;
+  public Vector<int[]> bits;
+  public Vector<int[]> val;
   
   /*
    * jpegNaturalOrder[i] is the natural-order position of the i'th element
@@ -870,16 +874,16 @@ class Huffman
   public Huffman(int Width,int Height)
   {
     
-    bits = new ArrayList();
-    bits.add(bitsDCluminance);
-    bits.add(bitsACluminance);
-    bits.add(bitsDCchrominance);
-    bits.add(bitsACchrominance);
-    val = new ArrayList();
-    val.add(valDCluminance);
-    val.add(valACluminance);
-    val.add(valDCchrominance);
-    val.add(valACchrominance);
+    bits = new Vector<int[]>();
+    bits.addElement(bitsDCluminance);
+    bits.addElement(bitsACluminance);
+    bits.addElement(bitsDCchrominance);
+    bits.addElement(bitsACchrominance);
+    val = new Vector<int[]>();
+    val.addElement(valDCluminance);
+    val.addElement(valACluminance);
+    val.addElement(valDCchrominance);
+    val.addElement(valACchrominance);
     initHuf();
     ImageWidth=Width;
     ImageHeight=Height;
