@@ -26,8 +26,15 @@ package org.jmol.adapter.readers.cifpdb;
 import org.jmol.adapter.smarter.*;
 import org.jmol.api.JmolAdapter;
 import org.jmol.api.JmolLineReader;
+import org.jmol.constant.EnumStructure;
 
+import java.util.ArrayList;
+import java.util.BitSet;
 import java.util.Hashtable;
+import java.util.List;
+import java.util.Map;
+
+import javax.vecmath.Point3f;
 
 import org.jmol.util.CifDataReader;
 import org.jmol.util.Logger;
@@ -66,9 +73,31 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
   private String thisFormula = "";
   private boolean iHaveDesiredModel;
   private boolean isPDB = false;
-  private Hashtable htHetero;
+  private Map<String, String> htHetero;
+  private boolean isMolecular;
+  private String molecularType = "GEOM_BOND default";
+  private char lastAltLoc;
+  private int configurationPtr = Integer.MIN_VALUE;
+  private int conformationIndex;
 
+  
+  @Override
   public void initializeReader() throws Exception {
+    if (checkFilter("CONF ")) {
+      configurationPtr = parseInt(filter, filter.indexOf("CONF ") + 5);
+    }
+
+    isMolecular = (filter != null && filter.indexOf("MOLECUL") >= 0);
+    if (isMolecular) {
+      if (!doApplySymmetry) {
+        doApplySymmetry = true;
+        latticeCells[0] = 1;
+        latticeCells[1] = 1;
+        latticeCells[2] = 1;
+      }
+      molecularType = "filter \"MOLECULAR\"";
+    }
+
     int nAtoms = 0;
     /*
      * Modified for 10.9.64 9/23/06 by Bob Hanson to remove as much as possible
@@ -76,7 +105,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
      * 
      * blah blah blah loop_ _a _b _c 0 1 2 0 3 4 0 5 6 loop_......
      * 
-     * we don't actually check that any skpped loop has the proper number of
+     * we don't actually check that any skipped loop has the proper number of
      * data points --- some multiple of the number of data keys -- but other
      * than that, we are checking here for proper CIF syntax, and Jmol will
      * report if it finds data where a key is supposed to be.
@@ -133,11 +162,11 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
       }
       if (!skipping) {
         key = key.replace('.', '_');
-        if (key.startsWith("_chemical_name")) {
-          processChemicalInfo("name");
+        if (key.startsWith("_chemical_name") || key.equals("_chem_comp_name")) {
+          atomSetCollection.setAtomSetCollectionAuxiliaryInfo("modelLoadNote", processChemicalInfo("name"));
         } else if (key.startsWith("_chemical_formula_structural")) {
           processChemicalInfo("structuralFormula");
-        } else if (key.startsWith("_chemical_formula_sum")) {
+        } else if (key.startsWith("_chemical_formula_sum") || key.equals("_chem_comp_formula")) {
           processChemicalInfo("formula");
         } else if (key.startsWith("_cell_")) {
           processCellParameter();
@@ -163,6 +192,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
     continuing = false;
   }
 
+  @Override
   protected void finalizeReader() throws Exception {
     super.finalizeReader();
     String header = tokenizer.getFileHeader();
@@ -171,13 +201,19 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
           header);
   }
 
+  private int nMolecular = 0;
+
+  @Override
   public void applySymmetryAndSetTrajectory() throws Exception {
     // This speeds up calculation, because no crosschecking
     // No special-position atoms in mmCIF files, because there will
     // be no center of symmetry, no rotation-inversions, 
     // no atom-centered rotation axes, and no mirror or glide planes. 
     atomSetCollection.setCheckSpecial(!isPDB);
+    boolean doCheck = doCheckUnitCell && !isPDB;
     super.applySymmetryAndSetTrajectory();
+    if (doCheck && (bondTypes.size() > 0 || isMolecular))
+      setBondingAndMolecules();
   }
 
   ////////////////////////////////////////////////////////////////
@@ -189,6 +225,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
    *  
    */
   private void processDataParameter() {
+    bondTypes.clear();
     tokenizer.getTokenPeeked();
     thisDataSetName = (key.length() < 6 ? "" : key.substring(5));
     if (thisDataSetName.length() > 0) {
@@ -208,9 +245,10 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
    * into specific atomSetAuxiliaryInfo elements
    * 
    * @param type    "name" "formula" etc.
+   * @return data
    * @throws Exception
    */
-  private void processChemicalInfo(String type) throws Exception {
+  private String processChemicalInfo(String type) throws Exception {
     if (type.equals("name"))
       chemicalName = data = tokenizer.fullTrim(data);
     else if (type.equals("structuralFormula"))
@@ -220,6 +258,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
     if (Logger.debugging) {
       Logger.debug(type + " = " + data);
     }
+    return data;
   }
 
   /**
@@ -228,7 +267,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
    * @throws Exception
    */
   private void processSymmetrySpaceGroupName() throws Exception {
-    setSpaceGroupName(data);
+    setSpaceGroupName((key.equals("_symmetry_space_group_name_H-M") ? "HM:" : "Hall:") + data);
   }
 
   final public static String[] cellParamNames = { 
@@ -335,8 +374,10 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
     String str = tokenizer.peekToken();
     if (str == null)
       return;
-    if (str.startsWith("_atom_site_") || str.startsWith("_atom_site.")) {
-      if (!processAtomSiteLoopBlock())
+    boolean isLigand = false;
+    if (str.startsWith("_atom_site_") || str.startsWith("_atom_site.") 
+        || (isLigand = str.equals("_chem_comp_atom.comp_id"))) {
+      if (!processAtomSiteLoopBlock(isLigand))
         return;
       atomSetCollection.setAtomSetName(thisDataSetName);
       atomSetCollection.setAtomSetAuxiliaryInfo("chemicalName", chemicalName);
@@ -349,11 +390,23 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
       processAtomTypeLoopBlock();
       return;
     }
+    if (str.startsWith("_chem_comp_bond")) {
+      processLigandBondLoopBlock();
+      return;
+    }
+    
     if (str.startsWith("_geom_bond")) {
-      if (doApplySymmetry && !applySymmetryToBonds)
-        skipLoop();
-      else
+      if (!doApplySymmetry) {
+        isMolecular = true;
+        doApplySymmetry = true;
+        latticeCells[0] = 1;
+        latticeCells[1] = 1;
+        latticeCells[2] = 1;
+      }
+      if (isMolecular)
         processGeomBondLoopBlock();
+      else 
+        skipLoop();
       return;
     }
     if (str.startsWith("_pdbx_entity_nonpoly")) {
@@ -394,7 +447,11 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
   ////////////////////////////////////////////////////////////////
 
 
-  private Hashtable atomTypes;
+  private Map<String, Float> atomTypes;
+  private List<Object[]> bondTypes = new ArrayList<Object[]>();
+
+  private String disorderAssembly = ".";
+  private String lastDisorderAssembly;
   
   final private static byte ATOM_TYPE_SYMBOL = 0;
   final private static byte ATOM_TYPE_OXIDATION_NUMBER = 1;
@@ -437,7 +494,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
       if (atomTypeSymbol == null || Float.isNaN(oxidationNumber))
         continue;
       if (atomTypes == null)
-        atomTypes = new Hashtable();
+        atomTypes = new Hashtable<String, Float>();
       atomTypes.put(atomTypeSymbol, new Float(oxidationNumber));
     }
   }
@@ -466,6 +523,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
   final private static byte GROUP_PDB = 16;
   final private static byte MODEL_NO = 17;
   final private static byte DUMMY_ATOM = 18;
+ 
   final private static byte DISORDER_GROUP = 19;
   final private static byte ANISO_LABEL = 20;
   final private static byte ANISO_MMCIF_ID = 21;
@@ -495,6 +553,18 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
   final private static byte ANISO_Beta_13 = 45;
   final private static byte ANISO_Beta_23 = 46;
   final private static byte ADP_TYPE = 47;
+  final private static byte CHEM_COMP_AC_ID = 48;
+  final private static byte CHEM_COMP_AC_NAME = 49;
+  final private static byte CHEM_COMP_AC_SYM = 50;
+  final private static byte CHEM_COMP_AC_CHARGE = 51;
+  final private static byte CHEM_COMP_AC_X = 52;
+  final private static byte CHEM_COMP_AC_Y = 53;
+  final private static byte CHEM_COMP_AC_Z = 54;
+  final private static byte CHEM_COMP_AC_X_IDEAL = 55;
+  final private static byte CHEM_COMP_AC_Y_IDEAL = 56;
+  final private static byte CHEM_COMP_AC_Z_IDEAL = 57;
+  final private static byte DISORDER_ASSEMBLY = 58;
+
 
   final private static String[] atomFields = { 
       "_atom_site_type_symbol",
@@ -545,6 +615,17 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
       "_atom_site_aniso_Beta_13",
       "_atom_site_aniso_Beta_23",
       "_atom_site_adp_type",
+      "_chem_comp_atom_comp_id",
+      "_chem_comp_atom_atom_id", 
+      "_chem_comp_atom_type_symbol", 
+      "_chem_comp_atom_charge",
+      "_chem_comp_atom_model_Cartn_x", 
+      "_chem_comp_atom_model_Cartn_y", 
+      "_chem_comp_atom_model_Cartn_z", 
+      "_chem_comp_atom_pdbx_model_Cartn_x_ideal", 
+      "_chem_comp_atom_pdbx_model_Cartn_y_ideal", 
+      "_chem_comp_atom_pdbx_model_Cartn_z_ideal", 
+      "_atom_site_disorder_assembly"
   };
 
   /* to: hansonr@stolaf.edu
@@ -558,15 +639,19 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
 
   /**
    * reads atom data in any order
+   * @param isLigand 
    * 
    * @return TRUE if successful; FALS if EOF encountered
    * @throws Exception
    */
-  boolean processAtomSiteLoopBlock() throws Exception {
+  boolean processAtomSiteLoopBlock(boolean isLigand) throws Exception {
     int currentModelNO = -1;
     boolean isAnisoData = false;
     parseLoopParameters(atomFields);
-    if (fieldOf[CARTN_X] != NONE) {
+    if (fieldOf[CHEM_COMP_AC_X_IDEAL] != NONE) {
+      isPDB = false;
+      setFractionalCoordinates(false);
+    } else if (fieldOf[CARTN_X] != NONE || fieldOf[CHEM_COMP_AC_X] != NONE) {
       setFractionalCoordinates(false);
       disableField(FRACT_X);
       disableField(FRACT_Y);
@@ -595,6 +680,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         switch (fieldProperty(i)) {
         case NONE:
           break;
+        case CHEM_COMP_AC_SYM:
         case TYPE_SYMBOL:
           String elementSymbol;
           if (field.length() < 2) {
@@ -608,7 +694,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
           }
           atom.elementSymbol = elementSymbol;
           if (atomTypes != null && atomTypes.containsKey(field)) {
-            float charge = ((Float) atomTypes.get(field)).floatValue();
+            float charge = atomTypes.get(field).floatValue();
             atom.formalCharge = (int) (charge + (charge < 0 ? -0.5 : 0.5));
             //because otherwise -1.6 is rounded UP to -1, and  1.6 is rounded DOWN to 1
             if (Math.abs(atom.formalCharge - charge) > 0.1)
@@ -618,22 +704,44 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
               }
           }
           break;
+        case CHEM_COMP_AC_NAME:
         case LABEL:
         case AUTH_ATOM:
           atom.atomName = field;
           break;
+        case CHEM_COMP_AC_X_IDEAL:
+          float x = parseFloat(field);
+          if (!Float.isNaN(x))
+            atom.x = x;
+          break;
+        case CHEM_COMP_AC_Y_IDEAL:
+          float y = parseFloat(field);
+          if (!Float.isNaN(y))
+            atom.y = y;
+          break;
+        case CHEM_COMP_AC_Z_IDEAL:
+          float z = parseFloat(field);
+          if (!Float.isNaN(z))
+            atom.z = z;
+          break;
+        case CHEM_COMP_AC_X:
         case CARTN_X:
         case FRACT_X:
           atom.x = parseFloat(field);
           break;
+        case CHEM_COMP_AC_Y:
         case CARTN_Y:
         case FRACT_Y:
           atom.y = parseFloat(field);
           break;
+        case CHEM_COMP_AC_Z:
         case CARTN_Z:
         case FRACT_Z:
           atom.z = parseFloat(field);
           break;
+        case CHEM_COMP_AC_CHARGE:
+          atom.formalCharge = parseInt(field);
+          break;
         case OCCUPANCY:
           float floatOccupancy = parseFloat(field);
           if (!Float.isNaN(floatOccupancy))
@@ -642,6 +750,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         case B_ISO:
           atom.bfactor = parseFloat(field) * (isPDB ? 1 : 100f);
           break;
+        case CHEM_COMP_AC_ID:
         case COMP_ID:
           atom.group3 = field;
           break;
@@ -659,7 +768,10 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
           break;
         case ALT_ID:
           atom.alternateLocationID = firstChar;
-        break;
+          break;
+        case DISORDER_ASSEMBLY:
+          disorderAssembly = field;
+          break;
         case DISORDER_GROUP:          
           if (firstChar == '-' && field.length() > 1) {
             atom.alternateLocationID = field.charAt(1);
@@ -703,7 +815,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         case ANISO_LABEL:
           iAtom = atomSetCollection.getAtomNameIndex(field);
           if (iAtom < 0)
-            return false;
+            continue;
           atom = atomSetCollection.getAtom(iAtom);
           break;
         case ANISO_MMCIF_ID:
@@ -776,25 +888,119 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
       }
     }
     if (isPDB) {
-      atomSetCollection.setIsPDB();
-      atomSetCollection.setAtomSetAuxiliaryInfo("isPDB", Boolean.TRUE);
+      setIsPDB();
     }
     atomSetCollection.setAtomSetAuxiliaryInfo("isCIF", Boolean.TRUE);
     return true;
   }
      
+  
+  @Override
+  protected boolean filterAtom(Atom atom, int iAtom) {
+    if (!super.filterAtom(atom, iAtom))
+      return false;
+    if (configurationPtr > 0) {
+      if (!disorderAssembly.equals(lastDisorderAssembly)) {
+        lastDisorderAssembly = disorderAssembly;
+        lastAltLoc = '\0';
+        conformationIndex = configurationPtr;
+      }
+      // ignore atoms that have no designation
+      if (atom.alternateLocationID != '\0') {
+        // count down until we get the desired index into the list
+        if (conformationIndex >= 0 && atom.alternateLocationID != lastAltLoc) {
+          lastAltLoc = atom.alternateLocationID;
+          conformationIndex--;
+        }
+        if (conformationIndex != 0) {
+          Logger.info("ignoring " + atom.atomName);
+          return false;
+        }
+      }
+    }
+    return true;
+  }
+
+
   ////////////////////////////////////////////////////////////////
   // bond data
   ////////////////////////////////////////////////////////////////
 
+  final private static byte CHEM_COMP_BOND_ATOM_ID_1 = 0;
+  final private static byte CHEM_COMP_BOND_ATOM_ID_2 = 1;
+  final private static byte CHEM_COMP_BOND_VALUE_ORDER = 2;
+  final private static byte CHEM_COMP_BOND_AROMATIC_FLAG = 3;
+  final private static String[] chemCompBondFields = {
+    "_chem_comp_bond_atom_id_1",
+    "_chem_comp_bond_atom_id_2",
+    "_chem_comp_bond_value_order",
+    "_chem_comp_bond_pdbx_aromatic_flag", 
+  };
+  
+  private void processLigandBondLoopBlock() throws Exception {
+    parseLoopParameters(chemCompBondFields);
+    for (int i = propertyCount; --i >= 0;)
+      if (fieldOf[i] == NONE) {
+        Logger.warn("?que? missing _chem_comp_bond property:" + i);
+        skipLoop();
+        return;
+      }
+    int order = 0;
+    boolean isAromatic = false;
+    while (tokenizer.getData()) {
+      int atomIndex1 = -1;
+      int atomIndex2 = -1;
+      order = 0;
+      isAromatic = false;
+      for (int i = 0; i < tokenizer.fieldCount; ++i) {
+        switch (fieldProperty(i)) {
+        case CHEM_COMP_BOND_ATOM_ID_1:
+          atomIndex1 = atomSetCollection.getAtomNameIndex(field);
+          break;
+        case CHEM_COMP_BOND_ATOM_ID_2:
+          atomIndex2 = atomSetCollection.getAtomNameIndex(field);
+          break;
+        case CHEM_COMP_BOND_AROMATIC_FLAG:
+          isAromatic = (field.charAt(0) == 'Y');
+          break;
+        case CHEM_COMP_BOND_VALUE_ORDER:
+          order = JmolAdapter.ORDER_COVALENT_SINGLE;
+          if (field.equals("SING"))
+            order = JmolAdapter.ORDER_COVALENT_SINGLE;
+          else if (field.equals("DOUB"))
+            order = JmolAdapter.ORDER_COVALENT_DOUBLE;
+          else if (field.equals("TRIP"))
+            order = JmolAdapter.ORDER_COVALENT_TRIPLE;
+          else
+            Logger.warn("unknown CIF bond order: " + field);
+          break;
+        }
+      }
+      if (atomIndex1 < 0 || atomIndex2 < 0)
+        continue;
+      if (isAromatic)
+        switch (order) {
+        case JmolAdapter.ORDER_COVALENT_SINGLE:
+          order = JmolAdapter.ORDER_AROMATIC_SINGLE;
+          break;
+        case JmolAdapter.ORDER_COVALENT_DOUBLE:
+          order = JmolAdapter.ORDER_AROMATIC_DOUBLE;
+          break;
+        }
+      atomSetCollection.addNewBond(atomIndex1, atomIndex2, order);
+    }
+  }
+
   final private static byte GEOM_BOND_ATOM_SITE_LABEL_1 = 0;
   final private static byte GEOM_BOND_ATOM_SITE_LABEL_2 = 1;
-  final private static byte GEOM_BOND_SITE_SYMMETRY_2 = 2;
+  final private static byte GEOM_BOND_DISTANCE = 2;
+  //final private static byte GEOM_BOND_SITE_SYMMETRY_2 = 3;
 
   final private static String[] geomBondFields = { 
       "_geom_bond_atom_site_label_1",
-      "_geom_bond_atom_site_label_2", 
-//      "_geom_bond_site_symmetry_2",
+      "_geom_bond_atom_site_label_2",
+      "_geom_bond_distance",
+    //  "_geom_bond_site_symmetry_2",
   };
 
   /**
@@ -815,30 +1021,55 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         return;
       }
 
+    String name1 = null;
+    String name2 = null;
     while (tokenizer.getData()) {
       int atomIndex1 = -1;
       int atomIndex2 = -1;
+      float distance = 0;
+      float dx = 0;
+      //String siteSym2 = null;
       for (int i = 0; i < tokenizer.fieldCount; ++i) {
         switch (fieldProperty(i)) {
         case NONE:
           break;
         case GEOM_BOND_ATOM_SITE_LABEL_1:
-          atomIndex1 = atomSetCollection.getAtomNameIndex(field);
+          atomIndex1 = atomSetCollection.getAtomNameIndex(name1 = field);
           break;
         case GEOM_BOND_ATOM_SITE_LABEL_2:
-          atomIndex2 = atomSetCollection.getAtomNameIndex(field);
+          atomIndex2 = atomSetCollection.getAtomNameIndex(name2 = field);
           break;
-        case GEOM_BOND_SITE_SYMMETRY_2:
-          //symmetry = field;
+        case GEOM_BOND_DISTANCE:
+          distance = parseFloat(field);
+          int pt = field.indexOf('('); 
+          if (pt >= 0) {
+            char[] data = field.toCharArray();
+            String sdx = field.substring(pt + 1, field.length() - 1);
+            int n = sdx.length();
+            for (int j = pt; --j >= 0;) {
+              if (data[j] == '.')
+                --j;
+               data[j] = (--n < 0 ? '0' : sdx.charAt(n));
+            }
+            dx = parseFloat(String.valueOf(data));
+            if (Float.isNaN(dx)) {
+              Logger.info("error reading uncertainty for " + line);
+              dx = 0.015f;
+            }
+            // TODO -- this is the full +/- (dx) in x.xxx(dx) -- is that too large?
+          } else {
+            dx = 0.015f;
+          }
           break;
+        //case GEOM_BOND_SITE_SYMMETRY_2:
+          //siteSym2 = field;
+          //break;
         }
       }
-      if (/*symmetry != null || */ atomIndex1 < 0 || atomIndex2 < 0)
+      if (atomIndex1 < 0 || atomIndex2 < 0)
         continue;
-      Bond bond = new Bond();
-      bond.atomIndex1 = atomIndex1;
-      bond.atomIndex2 = atomIndex2;
-      atomSetCollection.addBond(bond);
+      if (distance > 0) 
+        bondTypes.add(new Object[] { name1, name2, new Float(distance), new Float(dx) });
     }
   }
   
@@ -876,13 +1107,12 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
     hetatmData = null;
   }
 
-
   final private static byte CHEM_COMP_ID = 0;
   final private static byte CHEM_COMP_NAME = 1;
 
   final private static String[] chemCompFields = { 
       "_chem_comp_id",
-      "_chem_comp_name",  
+      "_chem_comp_name",
   };
   
 
@@ -909,9 +1139,8 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
           break;
         }
       }
-      if (groupName == null || hetName == null)
-        return;
-      addHetero(groupName, hetName);
+      if (groupName != null && hetName != null)
+        addHetero(groupName, hetName);
     }
   }
 
@@ -949,7 +1178,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
     if (!JmolAdapter.isHetero(groupName))
       return;
     if (htHetero == null)
-      htHetero = new Hashtable();
+      htHetero = new Hashtable<String, String>();
     htHetero.put(groupName, hetName);
     if (Logger.debugging) {
       Logger.debug("hetero: " + groupName + " = " + hetName);
@@ -971,6 +1200,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
   final private static byte SERIAL_NO = 8;
   final private static byte HELIX_CLASS = 9;
 
+
   final private static String[] structConfFields = { 
       "_struct_conf_conf_type_id",
       "_struct_conf_beg_auth_asym_id", 
@@ -998,16 +1228,16 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         return;
       }
     while (tokenizer.getData()) {
-      Structure structure = new Structure(Structure.PROTEIN_STRUCTURE_HELIX);
+      Structure structure = new Structure(EnumStructure.HELIX);
       for (int i = 0; i < tokenizer.fieldCount; ++i) {
         switch (fieldProperty(i)) {
         case NONE:
           break;
         case CONF_TYPE_ID:
           if (field.startsWith("TURN"))
-            structure.structureType = structure.substructureType = Structure.PROTEIN_STRUCTURE_TURN;
+            structure.structureType = structure.substructureType = EnumStructure.TURN;
           else if (!field.startsWith("HELX"))
-            structure.structureType = structure.substructureType = Structure.PROTEIN_STRUCTURE_NONE;
+            structure.structureType = structure.substructureType = EnumStructure.NONE;
           break;
         case BEG_ASYM_ID:
           structure.startChainID = firstChar;
@@ -1074,7 +1304,7 @@ public class CifReader extends AtomSetCollectionReader implements JmolLineReader
         return;
       }
     while (tokenizer.getData()) {
-      Structure structure = new Structure(Structure.PROTEIN_STRUCTURE_SHEET);
+      Structure structure = new Structure(EnumStructure.SHEET);
       for (int i = 0; i < tokenizer.fieldCount; ++i) {
         switch (fieldProperty(i)) {
         case BEG_ASYM_ID:
@@ -1149,7 +1379,7 @@ _struct_site_gen.details
 */
   
   //private int siteNum;
-  private Hashtable htSites;
+  private Map<String, Map<String, Object>> htSites;
   
   /**
    * 
@@ -1171,8 +1401,8 @@ _struct_site_gen.details
     String chainID = "";
     String resID = "";
     String group = "";
-    Hashtable htSite = null;
-    htSites = new Hashtable();
+    Map<String, Object> htSite = null;
+    htSites = new Hashtable<String, Map<String, Object>>();
     while (tokenizer.getData()) {
       for (int i = 0; i < tokenizer.fieldCount; ++i) {
         switch (fieldProperty(i)) {
@@ -1184,9 +1414,9 @@ _struct_site_gen.details
             htSite.put("groups", groups);
           }
           siteID = field;
-          htSite = (Hashtable)htSites.get(siteID);
+          htSite = htSites.get(siteID);
           if (htSite == null) {
-            htSite = new Hashtable();
+            htSite = new Hashtable<String, Object>();
             //htSite.put("seqNum", "site_" + (++siteNum));
             htSite.put("groups", "");
             htSites.put(siteID, htSite);
@@ -1311,6 +1541,7 @@ _struct_site_gen.details
       tokenizer.loopData = new String[tokenizer.fieldCount];
   }
 
+  @Override
   public String readLine() throws Exception {
     super.readLine();
     if (line.indexOf("#jmolscript:") >= 0)
@@ -1369,5 +1600,277 @@ _struct_site_gen.details
         return false;
     }
     return true;
-  }  
+  }
+  
+  /////////////////////////////////////
+  //  bonding and molecular 
+  /////////////////////////////////////
+  
+  private float[] atomRadius;
+  private BitSet[] bsConnected;
+  private BitSet[] bsSets;
+  final private Point3f ptOffset = new Point3f();
+  private BitSet bsMolecule;
+  private BitSet bsExclude;
+  private int firstAtom;
+  private int atomCount;
+  private Atom[] atoms;
+  
+  /**
+   * (1) If GEOM_BOND records are present, we
+   *     (a) use them to generate bonds
+   *     (b) add H atoms to bonds if necessary
+   *     (c) turn off autoBonding ("hasBonds")
+   * (2) If MOLECULAR, then we
+   *     (a) use {1 1 1} if lattice is not defined
+   *     (b) use atomSetCollection.bonds[] to construct 
+   *         a preliminary molecule and connect as we go
+   *     (c) check symmetry for connections to molecule in any
+   *         one of the 27 3x3 adjacent cells
+   *     (d) move those atoms and their connected branch set
+   *     (e) iterate as necessary to get all atoms desired
+   *     (f) delete unselected atoms
+   *     (g) set all coordinates as Cartesians
+   *     (h) remove all unit cell information
+   */
+  private void setBondingAndMolecules() {
+    Logger.info("CIF creating molecule "
+        + (bondTypes.size() > 0 ? " using GEOM_BOND records" : ""));
+    atoms = atomSetCollection.getAtoms();
+    firstAtom = atomSetCollection.getLastAtomSetAtomIndex();
+    int nAtoms = atomSetCollection.getLastAtomSetAtomCount();
+    atomCount = firstAtom + nAtoms;
+
+    // get list of sites based on atom names
+
+    bsSets = new BitSet[nAtoms];
+    symmetry = atomSetCollection.getSymmetry();
+    for (int i = firstAtom; i < atomCount; i++) {
+      int ipt = atomSetCollection.getAtomNameIndex(atoms[i].atomName)
+          - firstAtom;
+      if (bsSets[ipt] == null)
+        bsSets[ipt] = new BitSet();
+      bsSets[ipt].set(i - firstAtom);
+    }
+
+    // if molecular, we need atom connection lists and radii
+
+    if (isMolecular) {
+      atomRadius = new float[atomCount];
+      for (int i = firstAtom; i < atomCount; i++) {
+        short elemnoWithIsotope = atoms[i].elementNumber = JmolAdapter
+            .getElementNumber(atoms[i].getElementSymbol());
+        int charge = (atoms[i].formalCharge == Integer.MIN_VALUE ? 0
+            : atoms[i].formalCharge);
+        if (elemnoWithIsotope > 0)
+          atomRadius[i] = JmolAdapter.getBondingRadiusFloat(elemnoWithIsotope, charge);
+      }
+      bsConnected = new BitSet[atomCount];
+      for (int i = firstAtom; i < atomCount; i++)
+        bsConnected[i] = new BitSet();
+
+      // Set up a working set of atoms in the "molecule".
+
+      bsMolecule = new BitSet();
+
+      // Set up a working set of atoms that should be excluded 
+      // because they would map onto an equivalent atom's position.
+
+      bsExclude = new BitSet();
+    }
+
+    boolean isFirst = true;
+    while (createBonds(isFirst)) {
+      isFirst = false;
+      // main loop continues until no new atoms are found
+    }
+
+    if (isMolecular) {
+
+      // Set bsAtoms to control which atoms and 
+      // bonds are delivered by the iterators.
+
+      if (atomSetCollection.bsAtoms == null)
+        atomSetCollection.bsAtoms = new BitSet();
+      atomSetCollection.bsAtoms.clear(firstAtom, atomCount);
+      atomSetCollection.bsAtoms.or(bsMolecule);
+      atomSetCollection.bsAtoms.andNot(bsExclude);
+
+      // Set atom positions to be Cartesians and clear out unit cell
+      // so that the model displays without it.
+
+      for (int i = firstAtom; i < atomCount; i++) {
+        if (atomSetCollection.bsAtoms.get(i))
+          symmetry.toCartesian(atoms[i], true);
+        else if (Logger.debugging)
+          Logger.info(molecularType + " removing " + i + " "
+              + atoms[i].atomName + " " + atoms[i]);
+      }
+      atomSetCollection.setAtomSetAuxiliaryInfo("notionalUnitcell", null);
+      if (nMolecular++ == atomSetCollection.getCurrentAtomSetIndex()) {
+        atomSetCollection
+            .clearGlobalBoolean(AtomSetCollection.GLOBAL_FRACTCOORD);
+        atomSetCollection.clearGlobalBoolean(AtomSetCollection.GLOBAL_SYMMETRY);
+        atomSetCollection
+            .clearGlobalBoolean(AtomSetCollection.GLOBAL_UNITCELLS);
+      }
+      
+    }
+    
+    // Set return info to enable desired defaults.
+
+    if (bondTypes.size() > 0)
+      atomSetCollection.setAtomSetAuxiliaryInfo("hasBonds", Boolean.TRUE);
+    atomSetCollection.setAtomSetAuxiliaryInfo("fileHasUnitCell", Boolean.TRUE);
+
+    // Clear temporary fields.
+
+    bondTypes.clear();
+    atomRadius = null;
+    bsSets = null;
+    bsConnected = null;
+    bsMolecule = null;
+    bsExclude = null;
+  }
+
+  /**
+   * Use the site bitset to check for atoms that are within 
+   * +/-dx Angstroms of the specified distances in GEOM_BOND
+   * where dx is determined by the uncertainty (dx) in the record.
+   * Note that this also "connects" the atoms that might have 
+   * been moved in a previous iteration.
+   * 
+   * Also connect H atoms based on a distance <= 1.1 Angstrom
+   * from a nearby atom. 
+   * 
+   * Then create molecules.
+   * 
+   * @param doInit 
+   * @return TRUE if need to continue
+   */
+  private boolean createBonds(boolean doInit) {
+    
+    // process GEOM_BOND records
+    for (int i = bondTypes.size(); --i >= 0;) {
+      Object[] o = bondTypes.get(i);
+      float distance = ((Float) o[2]).floatValue();
+      float dx = ((Float) o[3]).floatValue();
+      int iatom1 = atomSetCollection.getAtomNameIndex((String) o[0]);
+      int iatom2 = atomSetCollection.getAtomNameIndex((String) o[1]);
+      BitSet bs1 = bsSets[iatom1 - firstAtom];
+      BitSet bs2 = bsSets[iatom2 - firstAtom];
+      if (bs1 == null || bs2 == null)
+        continue;
+      for (int j = bs1.nextSetBit(0); j >= 0; j = bs1.nextSetBit(j + 1))
+        for (int k = bs2.nextSetBit(0); k >= 0; k = bs2.nextSetBit(k + 1)) {
+          if ((!isMolecular || !bsConnected[j + firstAtom].get(k))
+              && symmetry.checkDistance(atoms[j + firstAtom], atoms[k
+                  + firstAtom], distance, dx, 0, 0, 0, ptOffset))
+            addNewBond(j + firstAtom, k + firstAtom);
+        }
+    }
+    
+    // do a quick check for H-X bonds if we have GEOM_BOND
+    
+    if (bondTypes.size() > 0)
+      for (int i = firstAtom; i < atomCount; i++)
+        if (atoms[i].elementNumber == 1) {
+          boolean checkAltLoc = (atoms[i].alternateLocationID != '\0');
+          for (int k = firstAtom; k < atomCount; k++)
+            if (k != i && atoms[k].elementNumber != 1 && 
+                (!checkAltLoc 
+                    || atoms[k].alternateLocationID == '\0' 
+                    || atoms[k].alternateLocationID == atoms[i].alternateLocationID)) {
+              if (!bsConnected[i].get(k)
+                  && symmetry.checkDistance(atoms[i], atoms[k], 1.1f, 0, 0, 0,
+                      0, ptOffset))
+                addNewBond(i, k);
+            }
+        }
+    if (!isMolecular)
+      return false;
+    
+    // generate the base atom set
+
+    if (doInit)
+      for (int i = firstAtom; i < atomCount; i++)
+        if (atoms[i].atomSite + firstAtom == i && !bsMolecule.get(i))
+          setBs(atoms, i, bsConnected, bsMolecule);
+    
+    // Now look through unchecked atoms for ones that
+    // are within bonding distance of the "molecular" set
+    // in any one of the 27 adjacent cells in 444 - 666.
+    // If an atom is found, move it along with its "branch"
+    // to the new location. BUT also check that we are
+    // not overlaying another atom -- if that happens
+    // go ahead and move it, but mark it as excluded.
+    
+    float bondTolerance = viewer.getBondTolerance();
+    BitSet bsBranch = new BitSet();
+    Point3f cart1 = new Point3f();
+    Point3f cart2 = new Point3f();
+    int nFactor = 2; // 1 was not enough. (see data/cif/triclinic_issue.cif)
+    for (int i = firstAtom; i < atomCount; i++)
+      if (!bsMolecule.get(i) && !bsExclude.get(i))
+        for (int j = bsMolecule.nextSetBit(0); j >= 0; j = bsMolecule
+            .nextSetBit(j + 1))
+          if (symmetry.checkDistance(atoms[j], atoms[i], atomRadius[i]
+              + atomRadius[j] + bondTolerance, 0, nFactor, nFactor, nFactor, ptOffset)) {
+            setBs(atoms, i, bsConnected, bsBranch);
+            for (int k = bsBranch.nextSetBit(0); k >= 0; k = bsBranch
+                .nextSetBit(k + 1)) {
+              atoms[k].add(ptOffset);
+              cart1.set(atoms[k]);
+              symmetry.toCartesian(cart1, true);
+              BitSet bs = bsSets[atomSetCollection
+                  .getAtomNameIndex(atoms[k].atomName)
+                  - firstAtom];
+              if (bs != null)
+                for (int ii = bs.nextSetBit(0); ii >= 0; ii = bs.nextSetBit(ii + 1)) {
+                  if (ii + firstAtom == k)
+                    continue;
+                  cart2.set(atoms[ii + firstAtom]);
+                  symmetry.toCartesian(cart2, true);
+                  if (cart2.distance(cart1) < 0.1f) {
+                    bsExclude.set(k);
+                    break;
+                  }
+                }
+              bsMolecule.set(k);
+            }
+            return true;
+          }
+    return false;
+  }
+
+  /**
+   * add the bond and mark it for molecular processing
+   * 
+   * @param i
+   * @param j
+   */
+  private void addNewBond(int i, int j) {
+    atomSetCollection.addNewBond(i, j);
+    if (!isMolecular)
+      return;
+    bsConnected[i].set(j);
+    bsConnected[j].set(i);
+  }
+
+  /**
+   * iteratively run through connected atoms, adding them to the set
+   * 
+   * @param atoms
+   * @param iatom
+   * @param bsBonds
+   * @param bs
+   */
+  private void setBs(Atom[] atoms, int iatom, BitSet[] bsBonds, BitSet bs) {
+    BitSet bsBond = bsBonds[iatom];
+    bs.set(iatom);
+    for (int i = bsBond.nextSetBit(0); i >= 0; i = bsBond.nextSetBit(i + 1)) {
+      if (!bs.get(i))
+        setBs(atoms, i, bsBonds, bs);
+    }
+  }
 }
