@@ -106,6 +106,7 @@ public class PdbReader extends AtomSetCollectionReader {
  private StringBuffer pdbHeader;
  private int configurationPtr = Integer.MIN_VALUE;
  private boolean applySymmetry;
+private boolean isConnectStateBug;
 
  @Override
  protected void initializeReader() throws Exception {
@@ -119,6 +120,10 @@ public class PdbReader extends AtomSetCollectionReader {
      sbIgnored = new StringBuffer();
      sbSelected = new StringBuffer();
    }
+   
+   isConnectStateBug = (stateScriptVersionInt >= 120151 && stateScriptVersionInt <= 120220
+       || stateScriptVersionInt >= 120300 && stateScriptVersionInt <= 120320);
+
  }
 
   @Override
@@ -145,7 +150,7 @@ public class PdbReader extends AtomSetCollectionReader {
       modelNumber = (bsModels == null ? modelNo : modelNumber + 1);
       if (!doGetModel(modelNumber))
         return checkLastModel();
-      atomSetCollection.connectAll(maxSerial);
+      atomSetCollection.connectAll(maxSerial, isConnectStateBug);
       if (atomCount > 0)
         applySymmetryAndSetTrajectory();
       // supposedly MODEL is only for NMR
@@ -238,7 +243,7 @@ public class PdbReader extends AtomSetCollectionReader {
   @Override
   protected void finalizeReader() throws Exception {
     checkNotPDB();
-    atomSetCollection.connectAll(maxSerial);
+    atomSetCollection.connectAll(maxSerial, isConnectStateBug);
     if (biomolecules != null && biomolecules.size() > 0
         && atomSetCollection.getAtomCount() > 0) {
       atomSetCollection.setAtomSetAuxiliaryInfo("biomolecules", biomolecules);
