@@ -23,12 +23,14 @@
  */
 package org.jmol.adapter.readers.quantum; 
 
-import org.jmol.adapter.smarter.*;
+import org.jmol.adapter.smarter.Atom;
+import org.jmol.adapter.smarter.Bond;
 import org.jmol.api.JmolAdapter;
 
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import org.jmol.util.Logger;
 import org.jmol.util.Parser;
@@ -55,8 +57,9 @@ public class CsfReader extends MopacSlaterReader {
   private int nGaussians = 0;
   private int nSlaters = 0;
   
-  private Hashtable htBonds;
+  private Map<String, Bond> htBonds;
   
+  @Override
   protected boolean checkLine() throws Exception {
     if (line.equals("local_transform")) {
       processLocalTransform();
@@ -112,11 +115,11 @@ public class CsfReader extends MopacSlaterReader {
         );
   }
 
-  private Hashtable propertyItemCounts = new Hashtable();
+  private Map<String, Integer> propertyItemCounts = new Hashtable<String, Integer>();
   private final int[] fieldTypes = new int[100]; // should be enough
   
   private int getPropertyCount(String what) {
-    Integer count = (Integer)(propertyItemCounts.get(what));
+    Integer count = propertyItemCounts.get(what);
     return (what.equals("ID") ? 1 : count == null ? 0 : count.intValue());
   }
   
@@ -136,7 +139,7 @@ public class CsfReader extends MopacSlaterReader {
     while (line != null) {
       tokens = getTokens();
       if (line.indexOf("property ") == 0)
-        propertyItemCounts.put(tokens[1], new Integer((tokens[6].equals("STRING") ? 1 : parseInt(tokens[5]))));
+        propertyItemCounts.put(tokens[1], Integer.valueOf((tokens[6].equals("STRING") ? 1 : parseInt(tokens[5]))));
       else if (line.indexOf("ID") == 0)
         break;
       readLine();
@@ -192,10 +195,10 @@ public class CsfReader extends MopacSlaterReader {
     objCls1, objID1, objCls2, objID2
   };
   
-  private Hashtable connectors;
+  private Map<String, int[]> connectors;
   
   private void processConnectorObject() throws Exception {
-    connectors = new Hashtable();
+    connectors = new Hashtable<String, int[]>();
     readLine();
     parseLineParameters(connectorFields, connectorFieldMap);
     out: for (; readLine() != null;) {
@@ -237,10 +240,10 @@ public class CsfReader extends MopacSlaterReader {
       }
       if (thisAtomID != Integer.MIN_VALUE && thisBondID != null) {
         if (connectors.containsKey(thisBondID)) {
-          int[] connect = (int[])connectors.get(thisBondID);
+          int[] connect = connectors.get(thisBondID);
           connect[1] = thisAtomID;
           if (htBonds != null) {
-            Bond bond = (Bond) htBonds.get(thisBondID);
+            Bond bond = htBonds.get(thisBondID);
             setBond(bond, connect);
           }
         } else {
@@ -368,10 +371,10 @@ public class CsfReader extends MopacSlaterReader {
           bond.order = order;
           if (connectors == null) {
             if (htBonds == null)
-              htBonds = new Hashtable();
+              htBonds = new Hashtable<String, Bond>();
             htBonds.put(thisBondID, bond);
           } else {
-            int[] connect = (int[]) connectors.get(thisBondID);
+            int[] connect = connectors.get(thisBondID);
             setBond(bond, connect);
           }
           break;
@@ -562,7 +565,7 @@ public class CsfReader extends MopacSlaterReader {
       for (int i = 0; i < nOrbitals; i++)
         if (Math.abs(list[iMo][i]) < MIN_COEF)
           list[iMo][i] = 0;
-      Hashtable mo = new Hashtable();
+      Map<String, Object> mo = new Hashtable<String, Object>();
       mo.put("energy", new Float(energy[iMo]));
       mo.put("occupancy", new Float(occupancy[iMo]));
       mo.put("coefficients", list[iMo]);
@@ -634,8 +637,8 @@ public class CsfReader extends MopacSlaterReader {
       }
     }
     if (isGaussian) {
-      List sdata = new ArrayList();
-      List gdata = new ArrayList();
+      List<int[]> sdata = new ArrayList<int[]>();
+      List<float[]> gdata = new ArrayList<float[]>();
       int iShell = 0;
       int gaussianCount = 0;
       for (int ipt = 0; ipt < nGaussians; ipt++) {
@@ -643,8 +646,7 @@ public class CsfReader extends MopacSlaterReader {
           iShell = shells[ipt];
           int[] slater = new int[4];
           int iAtom = atomSetCollection
-              .getAtomSerialNumberIndex(((int[]) (connectors.get(sto_gto
-                  + "_basis_fxn" + (ipt + 1))))[0]);
+              .getAtomSerialNumberIndex((connectors.get(sto_gto + "_basis_fxn" + (ipt + 1)))[0]);
           slater[0] = iAtom;
           slater[1] = JmolAdapter.getQuantumShellTagID(types[ipt]
               .substring(0, 1));
@@ -656,19 +658,18 @@ public class CsfReader extends MopacSlaterReader {
           sdata.add(slater);
           gaussianCount += nZ;
           for (int i = 0; i < nZ; i++)
-            gdata.add(new float[] { zetas[ipt][i],
-                contractionCoefs[ipt][i] });
+            gdata.add(new float[] { zetas[ipt][i], contractionCoefs[ipt][i] });
         }
       }
       float[][] garray = new float[gaussianCount][];
       for (int i = 0; i < gaussianCount; i++)
-        garray[i] = (float[]) gdata.get(i);
+        garray[i] = gdata.get(i);
       moData.put("shells", sdata);
       moData.put("gaussians", garray);
     } else {
       for (int ipt = 0; ipt < nSlaters; ipt++) {
-        int iAtom = atomSetCollection.getAtomSerialNumberIndex(((int[]) (connectors
-            .get(sto_gto + "_basis_fxn" + (ipt + 1))))[0]);
+        int iAtom = atomSetCollection.getAtomSerialNumberIndex((connectors
+            .get(sto_gto + "_basis_fxn" + (ipt + 1)))[0]);
         for (int i = 0; i < nZetas; i++) {
           if (zetas[ipt][i] == 0)
             break;
