@@ -26,12 +26,14 @@ package org.jmol.adapter.readers.quantum;
 
 import org.jmol.adapter.smarter.*;
 import org.jmol.api.JmolAdapter;
+import org.jmol.util.ArrayUtil;
 import org.jmol.util.Logger;
 import org.jmol.util.TextFormat;
 
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 /**
  * General methods for reading molecular orbital data,
@@ -87,7 +89,7 @@ abstract public class MOReader extends BasisFunctionReader {
 
   protected String energyUnits = "";
   
-  protected List moTypes;
+  protected List<String> moTypes;
   private boolean getNBOs;
   private boolean getNBOCharges;
   protected boolean haveNboCharges;
@@ -99,6 +101,7 @@ abstract public class MOReader extends BasisFunctionReader {
   final protected int HEADER_GAMESS_ORIGINAL = 1;
   final protected int HEADER_NONE = 0;
   
+  @Override
   protected void initializeReader() throws Exception {
     line = "\nNBOs in the AO basis:";
     getNBOs = filterMO();
@@ -129,6 +132,10 @@ abstract public class MOReader extends BasisFunctionReader {
         readMolecularOrbitals(HEADER_NONE);
         return false;
       }
+      if (line.indexOf(" SECOND ORDER PERTURBATION THEORY ANALYSIS") >= 0) {
+        readSecondOrderData();
+        return false;
+      }
     }
     if (getNBOCharges && line.indexOf("Summary of Natural Population Analysis:") >= 0) {
       getNboCharges();
@@ -195,7 +202,8 @@ abstract public class MOReader extends BasisFunctionReader {
 
    */
   protected void getNboTypes() throws Exception {
-    moTypes = new ArrayList();
+    moTypes = new ArrayList<String>();
+    iMo0 = (orbitals == null ? 0 : orbitals.size()) + 1;
     readLine();
     readLine();
     int n = 0;
@@ -288,13 +296,13 @@ abstract public class MOReader extends BasisFunctionReader {
     // and these will replace previous results. 
     // we still need atom positions and bases functions.
     if (haveNboOrbitals) {
-      orbitals = new ArrayList();
+      orbitals = new ArrayList<Map<String,Object>>();
       alphaBeta = "";
     }
     haveNboOrbitals = true;
     orbitalsRead = true;
-    Hashtable[] mos = null;
-    List[] data = null;
+    Map<String, Object>[] mos = null;
+    List<String>[] data = null;
     String dCoeffLabels = "";
     String fCoeffLabels = "";
     String pCoeffLabels = "";
@@ -376,7 +384,7 @@ abstract public class MOReader extends BasisFunctionReader {
           float[] coefs = new float[data[iMo].size()];
           int iCoeff = 0;
           while (iCoeff < coefs.length) {
-            coefs[iCoeff] = parseFloat((String) data[iMo].get(iCoeff));
+            coefs[iCoeff] = parseFloat(data[iMo].get(iCoeff));
             iCoeff++;
           }
           haveMOs = true;
@@ -402,19 +410,19 @@ abstract public class MOReader extends BasisFunctionReader {
           // NBOs
         }
         if (mos == null || nThisLine > mos.length) {
-          mos = new Hashtable[nThisLine];
-          data = new ArrayList[nThisLine];
+          mos = ArrayUtil.createArrayOfHashtable(nThisLine);
+          data = ArrayUtil.createArrayOfArrayList(nThisLine);
         }
         for (int i = 0; i < nThisLine; i++) {
-          mos[i] = new Hashtable();
-          data[i] = new ArrayList();
+          mos[i] = new Hashtable<String, Object>();
+          data[i] = new ArrayList<String>();
         }
         getMOHeader(headerType, tokens, mos, nThisLine);
         continue;
       }
-      char ch;
       int nSkip = tokens.length - nThisLine;
       String type = tokens[nSkip - 1];
+      char ch;
       if (type.charAt(0) == '(') {
         ch = type.charAt(1);
         if (!haveCoeffMap) {
@@ -464,9 +472,11 @@ abstract public class MOReader extends BasisFunctionReader {
     dfCoefMaps = null;
   }
   
-  protected int setMOType(Hashtable mo, int i) {
+  private int iMo0 = 1;
+  
+  protected int setMOType(Map<String, Object> mo, int i) {
     if (moTypes != null) {
-      String s = (String) moTypes.get(i % moTypes.size());
+      String s = moTypes.get(i % moTypes.size());
       i++;
       mo.put("type", s);
       mo.put("occupancy", new Float(s.indexOf("*") >= 0 ? 0 : 2));
@@ -476,21 +486,17 @@ abstract public class MOReader extends BasisFunctionReader {
     return i;
   }
   
-  protected void getMOHeader(int headerType, String[] tokens, Hashtable[] mos, int nThisLine)
+  protected void getMOHeader(int headerType, String[] tokens, Map<String, Object>[] mos, int nThisLine)
       throws Exception {
     readLine();
     switch (headerType) {
     default:
     case HEADER_NONE:
-      // this means there are no energies, occupancies or symmetries
-      for (int i = 0; i < nThisLine; i++) {
-        mos[i].put("energy", "");
-      }
       return;
     case HEADER_GAMESS_UK_MO:
       for (int i = 0; i < nThisLine; i++)
         mos[i].put("energy", new Float(tokens[i]));
-      discardLines(5);
+      readLines(5);
       return;
     case HEADER_GAMESS_ORIGINAL:
       // this is the original functionality
@@ -522,30 +528,83 @@ abstract public class MOReader extends BasisFunctionReader {
     }
   }
 
-  protected void addMOData(int nColumns, List[] data, Hashtable[] mos) {
+  protected void addMOData(int nColumns, List<String>[] data, Map<String, Object>[] mos) {
     for (int i = 0; i < nColumns; i++) {
       float[] coefs = new float[data[i].size()];
       for (int j = coefs.length; --j >= 0;)
-        coefs[j] = parseFloat((String) data[i].get(j));
+        coefs[j] = parseFloat(data[i].get(j));
       mos[i].put("coefficients", coefs);
       setMO(mos[i]);
     }
   }
 
+  private Map<String, Object> lastMoData;
+  
   protected void setMOData(boolean clearOrbitals) {
-    if (shells != null && gaussians != null) {
+    if (shells != null && gaussians != null && orbitals.size() != 0) {
       moData.put("calculationType", calculationType);
       moData.put("energyUnits", energyUnits);
       moData.put("shells", shells);
       moData.put("gaussians", gaussians);
       moData.put("mos", orbitals);
-      setMOData(moData);
+      setMOData(lastMoData = moData);
     }
     if (clearOrbitals) {
-      orbitals = new ArrayList();
-      moData = new Hashtable();
+      orbitals = new ArrayList<Map<String, Object>>();
+      moData = new Hashtable<String, Object>();
       alphaBeta = "";
     }
   }
- 
+  
+  /*
+  SECOND ORDER PERTURBATION THEORY ANALYSIS OF FOCK MATRIX IN NBO BASIS
+
+    Threshold for printing:   0.50 kcal/mol
+                                                          E(2)  E(j)-E(i) F(i,j)
+     Donor NBO (i)              Acceptor NBO (j)       kcal/mol   a.u.    a.u. 
+  ===============================================================================
+
+ within unit  1
+   1. BD ( 1) C 1- C 2       47. RY*( 1) O 4              1.28    1.86    0.044
+   1. BD ( 1) C 1- C 2       66. BD*( 1) C 2- O 4         0.67    1.27    0.026
+xxxxxxxxxxxxxxxxxxxxxxxxxx yyyyyyyyyyyyyyyyyyyyyyyyyyy zzzzzzz ....... ffffffff 
+0         1         2         3         4         5         6         7         8        
+01234567890123456789012345678901234567890123456789012345678901234567890123456789012
+   */
+  private void readSecondOrderData() throws Exception {
+
+    if (lastMoData == null || moTypes == null)
+      return;
+    Hashtable<String, Integer> ht = new Hashtable<String, Integer>();
+    for (int i = moTypes.size(); --i >= 0;)
+      ht.put(TextFormat.simpleReplace(moTypes.get(i).substring(10), " ", ""),
+          new Integer(i + iMo0));
+
+    List<String[]> strSecondOrderData = new ArrayList<String[]>();
+    readLines(5);
+    while (readLine() != null && line.indexOf("NBO") < 0) {
+      if (line.length() < 5 || line.charAt(4) != '.')
+        continue;
+      strSecondOrderData.add(new String[] {
+          TextFormat.simpleReplace(line.substring(5, 27).trim(), " ", ""),
+          TextFormat.simpleReplace(line.substring(32, 54).trim(), " ", ""),
+          line.substring(55, 62).trim(), line.substring(71).trim() });
+    }
+    float[][] secondOrderData = new float[strSecondOrderData.size()][4];
+    lastMoData.put("secondOrderData", secondOrderData);
+    lastMoData = null;
+    Integer IMO;
+    for (int i = strSecondOrderData.size(); --i >= 0;) {
+      String[] a = strSecondOrderData.get(i);
+      IMO = ht.get(a[0]);
+      if (IMO != null)
+        secondOrderData[i][0] = IMO.intValue();
+      IMO = ht.get(a[1]);
+      if (IMO != null)
+        secondOrderData[i][1] = IMO.intValue();
+      secondOrderData[i][2] = parseFloat(a[2]);
+      secondOrderData[i][3] = parseFloat(a[3]);
+    }
+  }
+  
 }
