@@ -24,18 +24,24 @@
 
 package org.jmol.adapter.readers.quantum;
 
+
 import org.jmol.adapter.smarter.*;
 import org.jmol.api.JmolAdapter;
 
-import java.util.List;
 import java.util.ArrayList;
+import java.util.List;
 import java.util.Hashtable;
+import java.util.Map;
+
 import javax.vecmath.Vector3f;
 
 import org.jmol.util.Logger;
-import org.jmol.util.Parser;
 
 class SpartanArchive {
+  
+  // OK, a bit of a hack.
+  // It's not a reader, but it needs the capabilities of BasisFunctionReader
+  
 
   private int atomCount = 0;
   private String bondData; // not in archive; may or may not have
@@ -44,33 +50,22 @@ class SpartanArchive {
   private int shellCount = 0;
   private int gaussianCount = 0;
   private String endCheck;
-  private String calculationType = "";
-  private String line;
   
-  private AtomSetCollection atomSetCollection;
-  private AtomSetCollectionReader r;
-  private Hashtable moData;
-  private List orbitals = new ArrayList();
+  private BasisFunctionReader r;
 
-  SpartanArchive(AtomSetCollectionReader r,
-      AtomSetCollection atomSetCollection, Hashtable moData) {
-    initialize(r, atomSetCollection, moData, "");
+  SpartanArchive(BasisFunctionReader r) {
+    initialize(r, "");
   }
 
-  SpartanArchive(AtomSetCollectionReader r,
-      AtomSetCollection atomSetCollection, Hashtable moData, String bondData, String endCheck) {
-    initialize(r, atomSetCollection, moData, bondData);
+  SpartanArchive(BasisFunctionReader r, String bondData, String endCheck) {
+    initialize(r, bondData);
     this.endCheck = endCheck;
   }
 
-  private void initialize(AtomSetCollectionReader r,
-                          AtomSetCollection atomSetCollection,
-                          Hashtable moData, String bondData) {
+  private void initialize(BasisFunctionReader r, String bondData) {
     this.r = r;
-    this.atomSetCollection = atomSetCollection;
-    this.moData = moData;
-    moData.put("isNormalized", Boolean.TRUE);
-    moData.put("energyUnits","");
+    r.moData.put("isNormalized", Boolean.TRUE);
+    r.moData.put("energyUnits","");
     this.bondData = bondData;
   }
 
@@ -91,6 +86,7 @@ class SpartanArchive {
           readMolecularOrbital();
           haveMOData = true;
         }
+        
       } else if (line.indexOf("ENERGY") == 0) {
         readEnergy();
       } else if (line.equals("ENDARCHIVE")
@@ -100,31 +96,19 @@ class SpartanArchive {
       readLine();
     }
     if (haveMOData)
-      r.setMOData(moData);
+      r.setMOData(r.moData);
     return atomCount;
   }
 
   private void readEnergy() throws Exception {
     String[] tokens = getTokens(readLine());
     float value = parseFloat(tokens[0]);
-    atomSetCollection.setAtomSetAuxiliaryInfo("energy", new Float(value));
+    r.atomSetCollection.setAtomSetAuxiliaryInfo("energy", new Float(value));
     if (r instanceof SpartanSmolReader) {
       String prefix = ((SpartanSmolReader)r).constraints;
-      atomSetCollection.setAtomSetName(prefix + (prefix.length() == 0 ? "" : " ") + "Energy=" + value + " KJ");
+      r.atomSetCollection.setAtomSetName(prefix + (prefix.length() == 0 ? "" : " ") + "Energy=" + value + " KJ");
     }
-    atomSetCollection.setAtomSetEnergy(tokens[0], value);
-  }
-
-  private static String[] getTokens(String info) {
-    return Parser.getTokens(info);
-  }
-
-  private int parseInt(String info) {
-    return r.parseInt(info);
-  }
-
-  private float parseFloat(String info) {
-    return r.parseFloat(info);
+    r.atomSetCollection.setAtomSetEnergy(tokens[0], value);
   }
 
   private int modelAtomCount;
@@ -143,25 +127,26 @@ class SpartanArchive {
     gaussianCount = parseInt(tokens[3]);
     //overallCharge = parseInt(tokens[4]);
     moCount = parseInt(tokens[6]);
-    calculationType = tokens[9];
-    String s = (String) moData.get("calculationType");
+    r.calculationType = tokens[9];
+    String s = (String) r.moData.get("calculationType");
     if (s == null)
-      s = calculationType;
-    else if (s.indexOf(calculationType) < 0)
-      s = calculationType + s;
-    moData.put("calculationType", r.calculationType = s);
+      s = r.calculationType;
+    else if (s.indexOf(r.calculationType) < 0)
+      s = r.calculationType + s;
+    r.moData.put("calculationType", r.calculationType = s);
     return modelAtomCount;
   }
 
-  void readAtoms(int atomCount0, boolean doAddAtoms) throws Exception {
+  private void readAtoms(int atomCount0, boolean doAddAtoms) throws Exception {
     for (int i = 0; i < modelAtomCount; i++) {
       String tokens[] = getTokens(readLine());
-      Atom atom = (doAddAtoms ? atomSetCollection.addNewAtom()
-          : atomSetCollection.getAtom(atomCount0 - modelAtomCount + i));
+      Atom atom = (doAddAtoms ? r.atomSetCollection.addNewAtom()
+          : r.atomSetCollection.getAtom(atomCount0 - modelAtomCount + i));
       atom.elementSymbol = AtomSetCollectionReader
           .getElementSymbol(parseInt(tokens[0]));
-      atom.set(parseFloat(tokens[1]), parseFloat(tokens[2]), parseFloat(tokens[3]));
-      atom.scale(AtomSetCollectionReader.ANGSTROMS_PER_BOHR);
+      r.setAtomCoord(atom, parseFloat(tokens[1]) * AtomSetCollectionReader.ANGSTROMS_PER_BOHR, 
+          parseFloat(tokens[2]) * AtomSetCollectionReader.ANGSTROMS_PER_BOHR, 
+          parseFloat(tokens[3]) * AtomSetCollectionReader.ANGSTROMS_PER_BOHR);
     }
     if (doAddAtoms && Logger.debugging) {
       Logger.debug(atomCount + " atoms read");
@@ -187,16 +172,34 @@ class SpartanArchive {
       int targetIndex = parseInt(tokens[i++]) - 1 + atomCount0;
       int bondOrder = parseInt(tokens[i++]);
       if (bondOrder > 0) {
-        atomSetCollection.addBond(new Bond(sourceIndex, targetIndex,
+        r.atomSetCollection.addBond(new Bond(sourceIndex, targetIndex,
             bondOrder < 4 ? bondOrder : bondOrder == 5 ? JmolAdapter.ORDER_AROMATIC : 1));
       }
     }
-    int bondCount = atomSetCollection.getBondCount();
+    int bondCount = r.atomSetCollection.getBondCount();
     if (Logger.debugging) {
       Logger.debug(bondCount + " bonds read");
     }
   }
 
+  /* 
+   * Jmol:   XX, YY, ZZ, XY, XZ, YZ 
+   * qchem: dxx, dxy, dyy, dxz, dyz, dzz : VERIFIED
+   * Jmol:   d0, d1+, d1-, d2+, d2-
+   * qchem: d 1=d2-, d 2=d1-, d 3=d0, d 4=d1+, d 5=d2+
+   * Jmol:   XXX, YYY, ZZZ, XYY, XXY, XXZ, XZZ, YZZ, YYZ, XYZ
+   * qchem: fxxx, fxxy, fxyy, fyyy, fxxz, fxyz, fyyz, fxzz, fyzz, fzzz
+   * Jmol:   f0, f1+, f1-, f2+, f2-, f3+, f3-
+   * qchem: f 1=f3-, f 2=f2-, f 3=f1-, f 4=f0, f 5=f1+, f 6=f2+, f 7=f3+
+   * 
+   */
+
+  //private static String DS_LIST = "d2-   d1-   d0    d1+   d2+";
+  //private static String FS_LIST = "f3-   f2-   f1-   f0    f1+   f2+   f3+";
+  //private static String DC_LIST = "DXX   DXY   DYY   DXZ   DYZ   DZZ";
+  //private static String FC_LIST = "XXX   XXY   XYY   YYY   XXZ   XYZ   YYZ   XZZ   YZZ   ZZZ";
+
+  @SuppressWarnings("incomplete-switch")
   void readBasis() throws Exception {
     /*
      * standard Gaussian format:
@@ -216,9 +219,15 @@ class SpartanArchive {
 
      */
 
-    List sdata = new ArrayList();
-    float[][] garray = new float[gaussianCount][];
+    ArrayList<int[]> shells = new ArrayList<int[]>();
+    float[][] gaussians = new float[gaussianCount][];
     int[] typeArray = new int[gaussianCount];
+    //if (false) { // checking these still
+    // r.getDFMap(DC_LIST, JmolAdapter.SHELL_D_CARTESIAN, BasisFunctionReader.CANONICAL_DC_LIST, 3);
+    // r.getDFMap(FC_LIST, JmolAdapter.SHELL_F_CARTESIAN, BasisFunctionReader.CANONICAL_FC_LIST, 3);
+    // r.getDFMap(DS_LIST, JmolAdapter.SHELL_D_SPHERICAL, BasisFunctionReader.CANONICAL_DS_LIST, 3);
+    // r.getDFMap(FS_LIST, JmolAdapter.SHELL_F_SPHERICAL, BasisFunctionReader.CANONICAL_FS_LIST, 3);
+    // }
     for (int i = 0; i < shellCount; i++) {
       String[] tokens = getTokens(readLine());
       boolean flag4 = (tokens[4].charAt(0) == '1');
@@ -244,7 +253,7 @@ class SpartanArchive {
       int nGaussians = slater[3] = parseInt(tokens[1]);
       for (int j = 0; j < nGaussians; j++)
         typeArray[gaussianPtr + j] = iBasis;
-      sdata.add(slater);
+      shells.add(slater);
     }
     for (int i = 0; i < gaussianCount; i++) {
       float alpha = parseFloat(readLine());
@@ -254,41 +263,53 @@ class SpartanArchive {
       data[0] = alpha;
       //we put D and F into coef 1. This may change if I find that Gaussian output
       //lists D and F in columns 3 and 4 as well.
-      switch (typeArray[i]) {
-      case JmolAdapter.SHELL_S:
+      switch (JmolAdapter.getShellEnumeration(typeArray[i])) {
+      case S:
         data[1] = parseFloat(tokens[0]);
         break;
-      case JmolAdapter.SHELL_SP:
+      case SP:
         data[1] = parseFloat(tokens[0]);
         data[2] = parseFloat(tokens[1]);
-        typeArray[i] = (data[1] == 0 ? JmolAdapter.SHELL_P : JmolAdapter.SHELL_SP);
+        if (data[1] == 0) {
+          data[1] = data[2];
+          typeArray[i] = JmolAdapter.SHELL_P;
+        }
         break;
-      case JmolAdapter.SHELL_D_CARTESIAN:
+      case D_CARTESIAN:
+      case D_SPHERICAL:
         data[1] = parseFloat(tokens[2]);
         break;
-      case JmolAdapter.SHELL_F_CARTESIAN:
+      case F_CARTESIAN:
+      case F_SPHERICAL:
         data[1] = parseFloat(tokens[3]);
         break;
       }
-      garray[i] = data;
+      gaussians[i] = data;
     }
     int nCoeff = 0;
     for (int i = 0; i < shellCount; i++) {
-      int[] slater = (int[]) sdata.get(i);
-      switch(typeArray[slater[2]]) {
-      case JmolAdapter.SHELL_S:
+      int[] slater = shells.get(i);
+      switch(JmolAdapter.getShellEnumeration(typeArray[slater[2]])) {
+      case S:
         nCoeff++;
         break;
-      case JmolAdapter.SHELL_P:
+      case P:
+        slater[1] = JmolAdapter.SHELL_P;
         nCoeff += 3;
         break;
-      case JmolAdapter.SHELL_SP:
+      case SP:
         nCoeff += 4;
         break;
-      case JmolAdapter.SHELL_D_CARTESIAN:
+      case D_SPHERICAL:
+        nCoeff += 5;
+        break;
+      case D_CARTESIAN:
         nCoeff += 6;
         break;
-      case JmolAdapter.SHELL_F_CARTESIAN:
+      case F_SPHERICAL:
+        nCoeff += 7;
+        break;
+      case F_CARTESIAN:
         nCoeff += 10;
         break;
       }
@@ -296,25 +317,27 @@ class SpartanArchive {
     boolean isD5F7 = (nCoeff < coefCount);
     if (isD5F7)
     for (int i = 0; i < shellCount; i++) {
-      switch (typeArray[i]) {
-      case JmolAdapter.SHELL_D_CARTESIAN:
-        garray[i][0] = JmolAdapter.SHELL_D_SPHERICAL;
+      int[] slater = shells.get(i);
+      switch (JmolAdapter.getShellEnumeration(typeArray[i])) {
+      case D_CARTESIAN:
+        slater[1] = JmolAdapter.SHELL_D_SPHERICAL;
         break;
-      case JmolAdapter.SHELL_F_CARTESIAN:
-        garray[i][0] = JmolAdapter.SHELL_F_SPHERICAL;
+      case F_CARTESIAN:
+        slater[1] = JmolAdapter.SHELL_F_SPHERICAL;
         break;
       }
     }
-    moData.put("shells", sdata);
-    moData.put("gaussians", garray);
+    r.moData.put("shells", shells);
+    r.moData.put("gaussians", gaussians);
     if (Logger.debugging) {
-      Logger.debug(sdata.size() + " slater shells read");
-      Logger.debug(garray.length + " gaussian primitives read");
+      Logger.debug(shells.size() + " slater shells read");
+      Logger.debug(gaussians.length + " gaussian primitives read");
     }
   }
 
   void readMolecularOrbital() throws Exception {
     int tokenPt = 0;
+    r.orbitals = new ArrayList<Map<String, Object>>();
     String[] tokens = getTokens("");
     float[] energies = new float[moCount];
     float[][] coefficients = new float[moCount][coefCount];
@@ -335,23 +358,22 @@ class SpartanArchive {
       }
     }
     for (int i = 0; i < moCount; i++) {
-      Hashtable mo = new Hashtable();
-      mo.put("energy", new Float(energies[i]));
+      Map<String, Object> mo = new Hashtable<String, Object>();
+      mo.put("energy", Float.valueOf(energies[i]));
       //mo.put("occupancy", new Float(-1));
       mo.put("coefficients", coefficients[i]);
-      orbitals.add(mo);
+      r.setMO(mo);
     }
     if (Logger.debugging) {
-      Logger.debug(orbitals.size() + " molecular orbitals read");
+      Logger.debug(r.orbitals.size() + " molecular orbitals read");
     }
-    moData.put("mos", orbitals);
+    r.moData.put("mos", r.orbitals);
   }
 
   void readProperties() throws Exception {
     Logger.debug("Reading PROPARC properties records...");
     while (readLine() != null
         && !line.startsWith("ENDPROPARC") && !line.startsWith("END Directory Entry ")) {
-      System.out.println(line);
       if (line.startsWith("PROP"))
         readProperty();
       else if (line.startsWith("DIPOLE"))
@@ -372,7 +394,7 @@ class SpartanArchive {
       return;
     Vector3f dipole = new Vector3f(parseFloat(tokens[0]),
         parseFloat(tokens[1]), parseFloat(tokens[2]));
-    atomSetCollection.setAtomSetAuxiliaryInfo("dipole", dipole);
+    r.atomSetCollection.setAtomSetAuxiliaryInfo("dipole", dipole);
   }
 
   private void readProperty() throws Exception {
@@ -384,7 +406,7 @@ class SpartanArchive {
     String keyName = tokens[2];
     boolean isDipole = (keyName.equals("DIPOLE_VEC"));
     Object value = new Object();
-    List vector = new ArrayList();
+    List<Object> vector = new ArrayList<Object>();
     if (tokens[3].equals("=")) {
       if (isString) {
         value = getQuotedString(tokens[4].substring(0, 1));
@@ -396,7 +418,7 @@ class SpartanArchive {
       if (nValues == 0)
         nValues = 1;
       boolean isArray = (tokens.length == 6);
-      List atomInfo = new ArrayList();
+      List<Float> atomInfo = new ArrayList<Float>();
       int ipt = 0;
       while (readLine() != null
           && !line.substring(0, 3).equals("END")) {
@@ -409,13 +431,13 @@ class SpartanArchive {
             setDipole(tokens2);
           for (int i = 0; i < tokens2.length; i++, ipt++) {
             if (isArray) {
-              atomInfo.add(new Float(parseFloat(tokens2[i])));
+              atomInfo.add(Float.valueOf(parseFloat(tokens2[i])));
               if ((ipt + 1) % nValues == 0) {
                 vector.add(atomInfo);
-                atomInfo = new ArrayList();
+                atomInfo = new ArrayList<Float>();
               }
             } else {
-              value = new Float(parseFloat(tokens2[i]));
+              value = Float.valueOf(parseFloat(tokens2[i]));
               vector.add(value);
             }
           }
@@ -429,9 +451,9 @@ class SpartanArchive {
     }
     //Logger.debug(keyName + " = " + value + " ; " + vector);
     if (value != null)
-      atomSetCollection.setAtomSetCollectionAuxiliaryInfo(keyName, value);
+      r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo(keyName, value);
     if (vector.size() != 0)
-      atomSetCollection.setAtomSetCollectionAuxiliaryInfo(keyName, vector);
+      r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo(keyName, vector);
   }
 
   // Logger.debug("reading property line:" + line);
@@ -440,35 +462,36 @@ class SpartanArchive {
     readLine();
     String label = "";
     int frequencyCount = parseInt(line);
-    List vibrations = new ArrayList();
-    List freqs = new ArrayList();
+    List<List<List<Float>>> vibrations = new ArrayList<List<List<Float>>>();
+    List<Map<String, Object>> freqs = new ArrayList<Map<String,Object>>();
     if (Logger.debugging) {
       Logger.debug("reading VIBFREQ vibration records: frequencyCount = "
           + frequencyCount);
     }
     boolean[] ignore = new boolean[frequencyCount];
     for (int i = 0; i < frequencyCount; ++i) {
-      int atomCount0 = atomSetCollection.getAtomCount();
+      int atomCount0 = r.atomSetCollection.getAtomCount();
       ignore[i] = !r.doGetVibration(i + 1);
       if (!ignore[i] && r.desiredVibrationNumber <= 0) {
-        atomSetCollection.cloneLastAtomSet();
+        r.atomSetCollection.cloneLastAtomSet();
         addBonds(bondData, atomCount0);
       }
       readLine();
-      Hashtable info = new Hashtable();
+      Map<String, Object> info = new Hashtable<String, Object>();
       float freq = parseFloat(line);
       info.put("freq", new Float(freq));
       if (line.length() > 15
           && !(label = line.substring(15, line.length())).equals("???"))
         info.put("label", label);
       freqs.add(info);
-      if (!ignore[i])
-        atomSetCollection.setAtomSetFrequency(null, label, "" + freq, null);
+      if (!ignore[i]) {
+        r.atomSetCollection.setAtomSetFrequency(null, label, "" + freq, null);
+      }
     }
-    atomSetCollection.setAtomSetCollectionAuxiliaryInfo("VibFreqs", freqs);
-    int atomCount = atomSetCollection.getFirstAtomSetAtomCount();
-    List vib = new ArrayList();
-    List vibatom = new ArrayList();
+    r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo("VibFreqs", freqs);
+    int atomCount = r.atomSetCollection.getFirstAtomSetAtomCount();
+    List<List<Float>> vib = new ArrayList<List<Float>>();
+    List<Float> vibatom = new ArrayList<Float>();
     int ifreq = 0;
     int iatom = atomCount;
     int nValues = 3;
@@ -478,82 +501,84 @@ class SpartanArchive {
       for (int i = 0; i < tokens2.length; i++) {
         float f = parseFloat(tokens2[i]);
         atomInfo[i % nValues] = f;
-        vibatom.add(new Float(f));
+        vibatom.add(Float.valueOf(f));
         if ((i + 1) % nValues == 0) {
           if (!ignore[ifreq]) {
-            atomSetCollection.addVibrationVector(iatom, atomInfo[0], atomInfo[1],
-                atomInfo[2]);
+            r.atomSetCollection.addVibrationVector(iatom, atomInfo[0], atomInfo[1], atomInfo[2]);
             vib.add(vibatom);
-            vibatom = new ArrayList();
+            vibatom = new ArrayList<Float>();
           }
           ++iatom;
         }
       }
       if (iatom % atomCount == 0) {
-        if (!ignore[ifreq])
+        if (!ignore[ifreq]) {
           vibrations.add(vib);
-        vib = new ArrayList();
-        if (++ifreq == frequencyCount)
+        }
+        vib = new ArrayList<List<Float>>();
+        if (++ifreq == frequencyCount) {
           break; // /loop exit
+        }
       }
     }
-    atomSetCollection
-        .setAtomSetCollectionAuxiliaryInfo("vibration", vibrations);
+    r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo("vibration", vibrations);
   }
 
+  @SuppressWarnings("unchecked")
   private void setVibrationsFromProperties() throws Exception {
-    List freq_modes = (List) atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_MODES");
-    if (freq_modes == null)
+    List<List<Float>> freq_modes = (List<List<Float>>) r.atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_MODES");
+    if (freq_modes == null) {
       return;
-    List freq_lab = (List) atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_LAB");
-    List freq_val = (List) atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_VAL");
+    }
+    List<String> freq_lab = (List<String>) r.atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_LAB");
+    List<Float> freq_val = (List<Float>) r.atomSetCollection.getAtomSetCollectionAuxiliaryInfo("FREQ_VAL");
     int frequencyCount = freq_val.size();
-    List vibrations = new ArrayList();
-    List freqs = new ArrayList();
+    List<List<List<Float>>> vibrations = new ArrayList<List<List<Float>>>();
+    List<Map<String, Object>> freqs = new ArrayList<Map<String,Object>>();
     if (Logger.debugging) {
       Logger.debug(
           "reading PROP VALUE:VIB FREQ_MODE vibration records: frequencyCount = " + frequencyCount);
     }
-    Object v;
+    Float v;
     for (int i = 0; i < frequencyCount; ++i) {
-      int atomCount0 = atomSetCollection.getAtomCount();
-      atomSetCollection.cloneLastAtomSet();
+      int atomCount0 = r.atomSetCollection.getAtomCount();
+      r.atomSetCollection.cloneLastAtomSet();
       addBonds(bondData, atomCount0);
-      Hashtable info = new Hashtable();
+      Map<String, Object> info = new Hashtable<String, Object>();
       info.put("freq", (v = freq_val.get(i)));
-      float freq = ((Float) v).floatValue();
-      String label = (String) freq_lab.get(i);
-      if (!label.equals("???"))
+      float freq = v.floatValue();
+      String label = freq_lab.get(i);
+      if (!label.equals("???")) {
         info.put("label", label);
+      }
       freqs.add(info);
-      atomSetCollection.setAtomSetName(label + " " + freq + " cm^-1");
-      atomSetCollection.setAtomSetProperty("Frequency", freq + " cm^-1");
-      atomSetCollection.setAtomSetProperty(SmarterJmolAdapter.PATH_KEY,
-          "Frequencies");
+      r.atomSetCollection.setAtomSetName(label + " " + freq + " cm^-1");
+      r.atomSetCollection.setAtomSetModelProperty("Frequency", freq + " cm^-1");
+      r.atomSetCollection.setAtomSetModelProperty(SmarterJmolAdapter.PATH_KEY, "Frequencies");
     }
-    atomSetCollection.setAtomSetCollectionAuxiliaryInfo("VibFreqs", freqs);
-    int atomCount = atomSetCollection.getFirstAtomSetAtomCount();
+    r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo("VibFreqs", freqs);
+    int atomCount = r.atomSetCollection.getFirstAtomSetAtomCount();
     int iatom = atomCount; // add vibrations starting at second atomset
     for (int i = 0; i < frequencyCount; i++) {
       if (!r.doGetVibration(i + 1))
         continue;
       int ipt = 0;
-      List vib = new ArrayList();
-      List mode = (List) freq_modes.get(i);
+      List<List<Float>> vib = new ArrayList<List<Float>>();
+      List<Float> mode = freq_modes.get(i);
       for (int ia = 0; ia < atomCount; ia++, iatom++) {
-        List vibatom = new ArrayList();
-        float vx = ((Float)(v = mode.get(ipt++))).floatValue();
+        List<Float> vibatom = new ArrayList<Float>();
+        float vx = (v = mode.get(ipt++)).floatValue();
         vibatom.add(v);
-        float vy = ((Float)(v = mode.get(ipt++))).floatValue();
+        float vy = (v = mode.get(ipt++)).floatValue();
         vibatom.add(v);
-        float vz = ((Float)(v = mode.get(ipt++))).floatValue();
+        float vz = (v = mode.get(ipt++)).floatValue();
         vibatom.add(v);
-        atomSetCollection.addVibrationVector(iatom, vx, vy, vz);
+        r.atomSetCollection.addVibrationVector(iatom, vx, vy, vz);
         vib.add(vibatom);
       }
       vibrations.add(vib);
     }
-    atomSetCollection.setAtomSetCollectionAuxiliaryInfo("vibration", vibrations);
+    r.atomSetCollection.setAtomSetCollectionAuxiliaryInfo("vibration", vibrations);
   }
 
   private String getQuotedString(String strQuote) {
@@ -562,9 +587,21 @@ class SpartanArchive {
     return (j == i ? "" : line.substring(i + 1, j));
   }
   
-  //because this is NOT an extension of AtomSetCollectionReader
+  private int parseInt(String info) {
+    return r.parseInt(info);
+  }
+
+  private float parseFloat(String info) {
+    return r.parseFloat(info);
+  }
+
+  private String[] getTokens(String s) {
+    return AtomSetCollectionReader.getTokens(s);
+  }
+  
+  private String line;
+ 
   private String readLine() throws Exception {
-    line = r.readLine();
-    return line;
+    return (line = r.readLine());
   }
 }
