@@ -25,9 +25,8 @@ package org.jmol.adapter.readers.xml;
 
 import org.jmol.adapter.smarter.*;
 
-
 import java.io.BufferedReader;
-import java.util.HashMap;
+import java.util.Map;
 import java.util.NoSuchElementException;
 import java.util.Properties;
 import java.util.StringTokenizer;
@@ -83,7 +82,7 @@ public class XmlCmlReader extends XmlReader {
    */
 
   private String[] cmlImplementedAttributes = { "id", //general
-      "title", //molecule, atom
+      "title", //molecule, atom, scalar
       "label", "name", //old atom
       "x3", "y3", "z3", "x2", "y2", "isotope", //atom 
       "elementType", "formalCharge", //atom
@@ -160,6 +159,7 @@ public class XmlCmlReader extends XmlReader {
   XmlCmlReader() {
   }
 
+  @Override
   protected void processXml(XmlReader parent,
                            AtomSetCollection atomSetCollection,
                            BufferedReader reader, XMLReader xmlReader) {
@@ -170,13 +170,14 @@ public class XmlCmlReader extends XmlReader {
     parseReaderXML(xmlReader);
   }
 
+  @Override
   protected void processXml(XmlReader parent,
                             AtomSetCollection atomSetCollection,
                             BufferedReader reader, JSObject DOMNode) {
     this.parent = parent;
     this.atomSetCollection = atomSetCollection;
     implementedAttributes = cmlImplementedAttributes;
-    ((CmlHandler) (new CmlHandler())).walkDOMTree(DOMNode);
+    (new CmlHandler()).walkDOMTree(DOMNode);
   }
 
   private String scalarDictRef;
@@ -195,8 +196,9 @@ public class XmlCmlReader extends XmlReader {
   private boolean embeddedCrystal = false;
   private Properties atomIdNames;
 
+  @Override
   public void processStartElement(String uri, String name, String qName,
-                                  HashMap atts) {
+                                  Map<String, String> atts) {
     // if (!uri.equals(NAMESPACE_URI))
     // return;
 
@@ -220,10 +222,10 @@ public class XmlCmlReader extends XmlReader {
       } else if (name.equals("symmetry")) {
         state = SYMMETRY;
         if (atts.containsKey("spaceGroup")) {
-          localSpaceGroupName = (String) atts.get("spaceGroup");
+          localSpaceGroupName = atts.get("spaceGroup");
         } else {
           localSpaceGroupName = "P1";
-          parent.clearLatticeParameters();
+          parent.clearUnitCell();
         }
       } else if (name.equals("module")) {
         moduleNestingLevel++;
@@ -235,15 +237,18 @@ public class XmlCmlReader extends XmlReader {
 
       break;
     case CRYSTAL:
+      // we force this to be NOT serialized by number, because we might have a1 and a1_....
+      checkedSerial = true;
+      isSerial = false;
       if (name.equals("scalar")) {
         state = CRYSTAL_SCALAR;
         setKeepChars(true);
-        scalarTitle = (String) atts.get("title");
+        scalarTitle = atts.get("title");
         getDictRefValue(atts);
       } else if (name.equals("symmetry")) {
         state = CRYSTAL_SYMMETRY;
         if (atts.containsKey("spaceGroup")) {
-          localSpaceGroupName = (String) atts.get("spaceGroup");
+          localSpaceGroupName = atts.get("spaceGroup");
           for (int i = 0; i < localSpaceGroupName.length(); i++)
             if (localSpaceGroupName.charAt(i) == '_')
               localSpaceGroupName = localSpaceGroupName.substring(0, i)
@@ -251,7 +256,7 @@ public class XmlCmlReader extends XmlReader {
         }
       } else if (name.equals("cellParameter")) {
         if (atts.containsKey("parameterType")) {
-          cellParameterType = (String) atts.get("parameterType");
+          cellParameterType = atts.get("parameterType");
           setKeepChars(true);
         }
       }
@@ -290,18 +295,18 @@ public class XmlCmlReader extends XmlReader {
         state = MOLECULE_BOND_ARRAY;
         bondCount = 0;
         if (atts.containsKey("order")) {
-          breakOutBondTokens((String) atts.get("order"));
+          breakOutBondTokens(atts.get("order"));
           for (int i = tokenCount; --i >= 0;)
             bondArray[i].order = parseBondToken(tokens[i]);
         }
         if (atts.containsKey("atomRef1")) {
-          breakOutBondTokens((String) atts.get("atomRef1"));
+          breakOutBondTokens(atts.get("atomRef1"));
           for (int i = tokenCount; --i >= 0;)
             bondArray[i].atomIndex1 = atomSetCollection
                 .getAtomNameIndex(tokens[i]);
         }
         if (atts.containsKey("atomRef2")) {
-          breakOutBondTokens((String) atts.get("atomRef2"));
+          breakOutBondTokens(atts.get("atomRef2"));
           for (int i = tokenCount; --i >= 0;)
             bondArray[i].atomIndex2 = atomSetCollection
                 .getAtomNameIndex(tokens[i]);
@@ -312,38 +317,38 @@ public class XmlCmlReader extends XmlReader {
         atomCount = 0;
         boolean coords3D = false;
         if (atts.containsKey("atomID")) {
-          breakOutAtomTokens((String) atts.get("atomID"));
+          breakOutAtomTokens(atts.get("atomID"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].atomName = tokens[i];
         }
         if (atts.containsKey("x3")) {
           coords3D = true;
-          breakOutAtomTokens((String) atts.get("x3"));
+          breakOutAtomTokens(atts.get("x3"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].x = parseFloat(tokens[i]);
         }
         if (atts.containsKey("y3")) {
-          breakOutAtomTokens((String) atts.get("y3"));
+          breakOutAtomTokens(atts.get("y3"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].y = parseFloat(tokens[i]);
         }
         if (atts.containsKey("z3")) {
-          breakOutAtomTokens((String) atts.get("z3"));
+          breakOutAtomTokens(atts.get("z3"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].z = parseFloat(tokens[i]);
         }
         if (atts.containsKey("x2")) {
-          breakOutAtomTokens((String) atts.get("x2"));
+          breakOutAtomTokens(atts.get("x2"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].x = parseFloat(tokens[i]);
         }
         if (atts.containsKey("y2")) {
-          breakOutAtomTokens((String) atts.get("y2"));
+          breakOutAtomTokens(atts.get("y2"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].y = parseFloat(tokens[i]);
         }
         if (atts.containsKey("elementType")) {
-          breakOutAtomTokens((String) atts.get("elementType"));
+          breakOutAtomTokens(atts.get("elementType"));
           for (int i = tokenCount; --i >= 0;)
             atomArray[i].elementSymbol = tokens[i];
         }
@@ -351,7 +356,6 @@ public class XmlCmlReader extends XmlReader {
           Atom atom = atomArray[i];
           if (!coords3D)
             atom.z = 0;
-          parent.setAtomCoord(atom);
           addAtom(atom);
         }
       }
@@ -365,9 +369,9 @@ public class XmlCmlReader extends XmlReader {
         int order = -1;
         tokenCount = 0;
         if (atts.containsKey("atomRefs2"))
-          breakOutTokens((String) atts.get("atomRefs2"));
+          breakOutTokens(atts.get("atomRefs2"));
         if (atts.containsKey("order"))
-          order = parseBondToken((String) atts.get("order"));
+          order = parseBondToken(atts.get("order"));
         if (tokenCount == 2 && order > 0) {
           addNewBond(tokens[0], tokens[1], order);
         }
@@ -378,18 +382,19 @@ public class XmlCmlReader extends XmlReader {
         state = MOLECULE_ATOM;
         atom = new Atom();
         parent.setFractionalCoordinates(false);
-        String id = (String) atts.get("id");
+        String id = atts.get("id");
         if (atts.containsKey("name"))
-          atom.atomName = (String) atts.get("name");
+          atom.atomName = atts.get("name");
         else if (atts.containsKey("title"))
-          atom.atomName = (String) atts.get("title");
+          atom.atomName = atts.get("title");
         else if (atts.containsKey("label"))
-          atom.atomName = (String) atts.get("label");
+          atom.atomName = atts.get("label");
         else
           atom.atomName = id;
         if (!checkedSerial) {
           // this is important because the atomName may not be unique
           // (as in PDB files)
+          // but it causes problems in cif-derived files that involve a1 and a1_1, for instance
           isSerial = (id != null && id.length() > 1 && id.startsWith("a")
               && Parser.parseInt(id.substring(1)) != Integer.MIN_VALUE);
           checkedSerial = true;
@@ -399,26 +404,23 @@ public class XmlCmlReader extends XmlReader {
         if (atts.containsKey("xFract")
             && (parent.iHaveUnitCell || !atts.containsKey("x3"))) {
           parent.setFractionalCoordinates(true);
-          parent.setAtomCoord(atom, parseFloat((String) atts.get("xFract")),
-              parseFloat((String) atts.get("yFract")), parseFloat((String) atts
-                  .get("zFract")));
+          atom.set(parseFloat(atts.get("xFract")),
+              parseFloat(atts.get("yFract")), parseFloat(atts.get("zFract")));
         } else if (atts.containsKey("x3")) {
-          parent.setAtomCoord(atom, parseFloat((String) atts.get("x3")),
-              parseFloat((String) atts.get("y3")), parseFloat((String) atts
-                  .get("z3")));
+          atom.set(parseFloat(atts.get("x3")),
+              parseFloat(atts.get("y3")), parseFloat(atts.get("z3")));
         } else if (atts.containsKey("x2")) {
-          parent.setAtomCoord(atom, parseFloat((String) atts.get("x2")),
-              parseFloat((String) atts.get("y2")), 0);
+          atom.set(parseFloat(atts.get("x2")),
+              parseFloat(atts.get("y2")), 0);
         }
         if (atts.containsKey("elementType")) {
-          String sym = (String) atts.get("elementType");
+          String sym = atts.get("elementType");
           if (atts.containsKey("isotope"))
-            atom.elementNumber = (short) ((parseInt((String) atts
-                .get("isotope")) << 7) + JmolAdapter.getElementNumber(sym));
+            atom.elementNumber = (short) ((parseInt(atts.get("isotope")) << 7) + JmolAdapter.getElementNumber(sym));
           atom.elementSymbol = sym;
         }
         if (atts.containsKey("formalCharge"))
-          atom.formalCharge = parseInt((String) atts.get("formalCharge"));
+          atom.formalCharge = parseInt(atts.get("formalCharge"));
       }
 
       break;
@@ -426,19 +428,19 @@ public class XmlCmlReader extends XmlReader {
       if (atts.containsKey("builtin")) {
         setKeepChars(true);
         state = MOLECULE_BOND_BUILTIN;
-        scalarDictValue = (String) atts.get("builtin");
+        scalarDictValue = atts.get("builtin");
       }
       break;
     case MOLECULE_ATOM:
       if (name.equals("scalar")) {
         state = MOLECULE_ATOM_SCALAR;
         setKeepChars(true);
-        scalarTitle = (String) atts.get("title");
+        scalarTitle = atts.get("title");
         getDictRefValue(atts);
       } else if (atts.containsKey("builtin")) {
         setKeepChars(true);
         state = MOLECULE_ATOM_BUILTIN;
-        scalarDictValue = (String) atts.get("builtin");
+        scalarDictValue = atts.get("builtin");
       }
       break;
     case MOLECULE_ATOM_SCALAR:
@@ -454,6 +456,7 @@ public class XmlCmlReader extends XmlReader {
 
   private void addNewBond(String a1, String a2, int order) {
     parent.applySymmetryToBonds = true;
+    //System.out.println("atomsetcollection addnewbond " + a1 + " " + a2);
     if (isSerial)
       atomSetCollection.addNewBondWithMappedSerialNumbers(Parser.parseInt(a1.substring(1)),
           Parser.parseInt(a2.substring(1)), order);
@@ -461,14 +464,15 @@ public class XmlCmlReader extends XmlReader {
         atomSetCollection.addNewBond(a1, a2, order);
   }
 
-  private void getDictRefValue(HashMap atts) {
-    scalarDictRef = (String) atts.get("dictRef");
+  private void getDictRefValue(Map<String, String> atts) {
+    scalarDictRef = atts.get("dictRef");
     if (scalarDictRef != null) {
       int iColon = scalarDictRef.indexOf(":");
       scalarDictValue = scalarDictRef.substring(iColon + 1);
     }
   }
 
+  @Override
   public void processEndElement(String uri, String name, String qName) {
     // if (!uri.equals(NAMESPACE_URI))
     // return;
@@ -535,9 +539,8 @@ public class XmlCmlReader extends XmlReader {
       }
       break;
     case LATTICE_VECTOR:
-      float[] values = new float[3];
-      getTokensFloat(chars, values, 3);
-      parent.addPrimitiveLatticeVector(latticeVectorPtr, values);
+      float[] values = getTokensFloat(chars, null, 3);
+      parent.addPrimitiveLatticeVector(latticeVectorPtr, values, 0);
       latticeVectorPtr = (latticeVectorPtr + 1) % 3;
       setKeepChars(false);
       state = START;
@@ -649,11 +652,12 @@ public class XmlCmlReader extends XmlReader {
   private void addAtom(Atom atom) {
     if ((atom.elementSymbol == null && atom.elementNumber < 0)
         || Float.isNaN(atom.z))
-        return;
-      if (isSerial)
-        atomSetCollection.addAtomWithMappedSerialNumber(atom);
-      else
-        atomSetCollection.addAtomWithMappedName(atom);
+      return;
+    parent.setAtomCoord(atom);
+    if (isSerial)
+      atomSetCollection.addAtomWithMappedSerialNumber(atom);
+    else
+      atomSetCollection.addAtomWithMappedName(atom);
   }
 
   int parseBondToken(String str) {
@@ -734,18 +738,19 @@ public class XmlCmlReader extends XmlReader {
     }
   }
 
-  private void createNewAtomSet(HashMap atts) {
+  private void createNewAtomSet(Map<String, String> atts) {
     atomSetCollection.newAtomSet();
     String collectionName = null;
     if (atts.containsKey("title"))
-      collectionName = (String) atts.get("title");
+      collectionName = atts.get("title");
     else if (atts.containsKey("id"))
-      collectionName = (String) atts.get("id");
+      collectionName = atts.get("id");
     if (collectionName != null) {
       atomSetCollection.setAtomSetName(collectionName);
     }
   }
   
+  @Override
   public void applySymmetryAndSetTrajectory() {
     if (moduleNestingLevel > 0 || !haveMolecule)
       return;
