@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2012-01-11 12:59:35 +0100 (mer., 11 janv. 2012) $
- * $Revision: 16660 $
+ * $Date: 2012-04-20 18:07:52 +0200 (ven., 20 avr. 2012) $
+ * $Revision: 17022 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -549,7 +549,11 @@ public class AtomSetCollection {
     vConnect.add(connectLast = is);
   }
 
-  public void connectAll(int maxSerial) {
+  private void connectAllBad(int maxSerial) {
+    // between 12.1.51-12.2.20 and 12.3.0-12.3.20 we have 
+    // a problem in that this method was used for connect
+    // this means that scripts created during this time could have incorrect 
+    // BOND indexes in the state script. It was when we added reading of H atoms
     int firstAtom = connectNextAtomIndex;
     for (int i = connectNextAtomSet; i < atomSetCount; i++) {
       setAtomSetAuxiliaryInfo("PDB_CONECT_firstAtom_count_max", new int[] {
@@ -565,6 +569,29 @@ public class AtomSetCollection {
     connectNextAtomIndex = firstAtom;
   }
 
+
+  public void connectAll(int maxSerial, boolean isConnectStateBug) {
+    if (currentAtomSetIndex < 0)
+      return;
+    if (isConnectStateBug) {
+      connectAllBad(maxSerial);
+      return;
+    }
+    setAtomSetAuxiliaryInfo("PDB_CONECT_firstAtom_count_max", new int[] {
+        getLastAtomSetAtomIndex(), getLastAtomSetAtomCount(), maxSerial });
+    if (vConnect == null)
+      return;
+    int firstAtom = connectNextAtomIndex;
+    for (int i = connectNextAtomSet; i < atomSetCount; i++) {
+      setAtomSetAuxiliaryInfo("PDB_CONECT_bonds", vConnect, i);
+      setGlobalBoolean(GLOBAL_CONECT);
+      firstAtom += atomSetAtomCounts[i];
+    }
+    vConnect = null;
+    connectNextAtomSet = currentAtomSetIndex + 1;
+    connectNextAtomIndex = firstAtom;
+  }
+
   public void addBond(Bond bond) {
     if (trajectoryStepCount > 0)
       return;
@@ -864,8 +891,14 @@ public class AtomSetCollection {
   private int dtype = 3;
   private Vector3f[] unitCellTranslations;
   
+  private int baseSymmetryAtomCount;
+  
+  public void setBaseSymmetryAtomCount(int n) {
+    baseSymmetryAtomCount = n;
+  }
+  
   private void applyAllSymmetry() throws Exception {
-    int noSymmetryCount = getLastAtomSetAtomCount();
+    int noSymmetryCount = (baseSymmetryAtomCount == 0 ? getLastAtomSetAtomCount() : baseSymmetryAtomCount);
     int iAtomFirst = getLastAtomSetAtomIndex();
     if (needEllipsoids)
       for (int i = iAtomFirst; i < atomCount; i++)
