@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-09-14 13:32:25 +0200 (mer., 14 sept. 2011) $
- * $Revision: 16068 $
+ * $Date: 2012-06-12 07:48:40 +0200 (mar., 12 juin 2012) $
+ * $Revision: 17284 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -319,6 +319,8 @@ public class SmarterJmolAdapter extends JmolAdapter {
     boolean exceptFiles = (manifest.indexOf("EXCEPT_FILES") >= 0);
     if (selectAll || subFileName != null)
       haveManifest = false;
+    if (haveManifest && manifest.indexOf("$SCRIPT_PATH$") >= 0)
+      return "NOTE: file recognized as a script file: " + fileName + "|JmolManifest.txt\n";
     List<Object> vCollections = new ArrayList<Object>();
     Map<String, Object> htCollections = (haveManifest ? new Hashtable<String, Object>() : null);
     int nFiles = 0;
