@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-12-18 20:32:45 +0100 (sam., 18 déc. 2010) $
- * $Revision: 14833 $
+ * $Date: 2011-09-03 16:55:30 +0200 (sam., 03 sept. 2011) $
+ * $Revision: 16030 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -28,9 +28,15 @@ import java.io.BufferedReader;
 import java.io.InputStream;
 import java.util.Hashtable;
 import java.util.BitSet;
+import java.util.Map;
 
+import javax.vecmath.Point3f;
+
+import org.jmol.constant.EnumStructure;
+import org.jmol.constant.EnumQuantumShell;
 import org.jmol.modelset.Group;
 import org.jmol.util.Elements;
+import org.jmol.util.JmolEdge;
 import org.jmol.viewer.JmolConstants;
 
 /****************************************************************
@@ -53,6 +59,7 @@ import org.jmol.viewer.JmolConstants;
  * @see org.jmol.adapter.smarter.SmarterJmolAdapter
  ****************************************************************/
 public abstract class JmolAdapter {
+
   
   public final static short ORDER_COVALENT_SINGLE = JmolEdge.BOND_COVALENT_SINGLE;
   public final static short ORDER_COVALENT_DOUBLE = JmolEdge.BOND_COVALENT_DOUBLE;
@@ -69,15 +76,18 @@ public abstract class JmolAdapter {
   public final static short ORDER_PARTIAL32       = JmolEdge.BOND_PARTIAL32;
   public final static short ORDER_UNSPECIFIED     = JmolEdge.BOND_ORDER_UNSPECIFIED;
   
-  public final static int        SHELL_S           = JmolConstants.SHELL_S;
-  public final static int        SHELL_P           = JmolConstants.SHELL_P;
-  public final static int        SHELL_SP          = JmolConstants.SHELL_SP;
-  public final static int        SHELL_L           = JmolConstants.SHELL_L;
-  public final static int        SHELL_D_SPHERICAL = JmolConstants.SHELL_D_SPHERICAL;
-  public final static int        SHELL_D_CARTESIAN = JmolConstants.SHELL_D_CARTESIAN;
-  public final static int        SHELL_F_SPHERICAL = JmolConstants.SHELL_F_SPHERICAL;
-  public final static int        SHELL_F_CARTESIAN = JmolConstants.SHELL_F_CARTESIAN;
-  public static final String SUPPORTED_BASIS_FUNCTIONS = JmolConstants.SUPPORTED_BASIS_FUNCTIONS;
+  public final static EnumQuantumShell getShellEnumeration(int i) { return EnumQuantumShell.getItem(i); }
+  public final static int[][] getNewDfCoefMap() { return EnumQuantumShell.getNewDfCoefMap(); }
+  // We need the shell ids only because we are storing these in the (int) shell[1] spot
+  public final static int        SHELL_S           = EnumQuantumShell.S.id;
+  public final static int        SHELL_P           = EnumQuantumShell.P.id;
+  public final static int        SHELL_SP          = EnumQuantumShell.SP.id;
+  public final static int        SHELL_L           = EnumQuantumShell.SP.id;
+  public final static int        SHELL_D_SPHERICAL = EnumQuantumShell.D_SPHERICAL.id;
+  public final static int        SHELL_D_CARTESIAN = EnumQuantumShell.D_CARTESIAN.id;
+  public final static int        SHELL_F_SPHERICAL = EnumQuantumShell.F_SPHERICAL.id;
+  public final static int        SHELL_F_CARTESIAN = EnumQuantumShell.F_CARTESIAN.id;
+  public static final String SUPPORTED_BASIS_FUNCTIONS = EnumQuantumShell.SUPPORTED_BASIS_FUNCTIONS;
   
   public static String getElementSymbol(int elementNumber) {
     return Elements.elementSymbolFromNumber(elementNumber);
@@ -96,17 +106,20 @@ public abstract class JmolAdapter {
   }
   
   public static int getQuantumShellTagID(String tag) {
-    return JmolConstants.getQuantumShellTagID(tag);
+    return EnumQuantumShell.getQuantumShellTagID(tag);
   }
                                            
   public static int getQuantumShellTagIDSpherical(String tag) {
-    return JmolConstants.getQuantumShellTagIDSpherical(tag);
+    return EnumQuantumShell.getQuantumShellTagIDSpherical(tag);
   }
   
   final public static short lookupGroupID(String group3) {
     return Group.lookupGroupID(group3);
   }
 
+  public static float getBondingRadiusFloat(short atomicNumberWithIsotope, int charge) {
+    return Elements.getBondingRadiusFloat(atomicNumberWithIsotope, charge);
+  }
 
   //////////////////////////////////////////////////////////////////
   // file related
@@ -146,7 +159,7 @@ public abstract class JmolAdapter {
  * @return The atomSetCollection or String with an error message
  */
 abstract public Object getAtomSetCollectionReader(String name, String type,
-                                 BufferedReader bufferedReader, Hashtable htParams);
+                                 BufferedReader bufferedReader, Map<String, Object> htParams);
 
 abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
   /**
@@ -174,13 +187,13 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
    * @return The atomSetCollection or String with an error message
    */
 
-  abstract public Object getAtomSetCollectionReaders(JmolFileReaderInterface fileReader, String[] names, String[] types,
-                                    Hashtable htParams, boolean getReadersOnly);
+  abstract public Object getAtomSetCollectionReaders(JmolFilesReaderInterface fileReader, String[] names, String[] types,
+                                    Map<String, Object> htParams, boolean getReadersOnly);
 
-  abstract public Object getAtomSetCollectionFromSet(Object readers, Object atomSets, Hashtable htParams);
+  abstract public Object getAtomSetCollectionFromSet(Object readers, Object atomSets, Map<String, Object> htParams);
 
   abstract public Object getAtomSetCollectionOrBufferedReaderFromZip(InputStream is, String fileName, String[] zipDirectory,
-                             Hashtable htParams, boolean asBufferedReader);
+                             Map<String, Object> htParams, boolean asBufferedReader, boolean asBufferendInputStream);
 
   /**
    * all in one -- for TestSmarterJmolAdapter
@@ -193,9 +206,9 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
    */
   public Object getAtomSetCollectionFromReader(String name, String type,
                                                BufferedReader bufferedReader,
-                                               Hashtable htParams) {
+                                               Map<String, Object> htParams) {
     if (htParams == null)
-      htParams = new Hashtable();
+      htParams = new Hashtable<String, Object>();
     // viewer is now needed for CIF and GenNBO file reading
     if (!htParams.containsKey("viewer"))
       htParams.put("viewer", JmolViewer.allocateViewer(null, this));
@@ -213,7 +226,7 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
   }
 
   public Object openBufferedReader(String name, BufferedReader bufferedReader,
-                                   Hashtable htParams) {
+                                   Map<String, Object> htParams) {
     return getAtomSetCollectionFromReader(name, null, bufferedReader, htParams);
   }
 
@@ -222,8 +235,11 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
     return getAtomSetCollectionFromReader(name, type, bufferedReader, null);
   }
 
-  abstract public Object getAtomSetCollectionFromDOM(Object DOMNode, Hashtable htParams);
+  abstract public Object getAtomSetCollectionFromDOM(Object DOMNode, Map<String, Object> htParams);
 
+  /**
+   * @param atomSetCollection  
+   */
   public void finish(Object atomSetCollection) {}
 
   /**
@@ -253,7 +269,7 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
    * @return The auxiliaryInfo Hashtable that may be available for particular
    * filetypes for this atomSetCollection or <code>null</code>
    */
-  abstract public Hashtable getAtomSetCollectionAuxiliaryInfo(Object atomSetCollection);
+  abstract public Map<String, Object> getAtomSetCollectionAuxiliaryInfo(Object atomSetCollection);
   
   /**
    * Get number of atomSets in the file.
@@ -298,7 +314,7 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
    * @return The auxiliaryInfo Hashtable that may be available for particular
    * filetypes for this atomSet or <code>null</code>
    */
-  abstract public Hashtable getAtomSetAuxiliaryInfo(Object atomSetCollection, int atomSetIndex);
+  abstract public Map<String, Object>  getAtomSetAuxiliaryInfo(Object atomSetCollection, int atomSetIndex);
 
   /**
    * Get the estimated number of atoms contained in the file.
@@ -309,7 +325,22 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
    */
   abstract public int getAtomCount(Object atomSetCollection);
 
-  
+  /**
+   * Get the hydrogen atom count -- for ligand files
+   * 
+   * @param atomSetCollection
+   * @return  number of hydrogen atoms in this model
+   */
+  abstract public int getHydrogenAtomCount(Object atomSetCollection);
+
+ 
+  /**
+   * 
+   * @param atomSetCollection 
+   * @return   [ [atomName1 atomName2 order] ...]
+   */
+  abstract public String[][] getBondList(Object atomSetCollection);
+
   /**
    * Get the boolean whether coordinates are fractional.
    * @param atomSetCollection The client file
@@ -390,7 +421,6 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
     public int getAtomSite() { return Integer.MIN_VALUE; }
     abstract public Object getUniqueID();
     public short getElementNumber() { return -1; } // may be atomicNumber + isotopeNumber*128
-    public String getElementSymbol() { return null; }
     public String getAtomName() { return null; }
     public int getFormalCharge() { return 0; }
     public float getPartialCharge() { return Float.NaN; }
@@ -399,6 +429,7 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
     abstract public float getX();
     abstract public float getY();
     abstract public float getZ();
+    abstract public Point3f getXYZ();
     public float getVectorX() { return Float.NaN; }
     public float getVectorY() { return Float.NaN; }
     public float getVectorZ() { return Float.NaN; }
@@ -432,8 +463,8 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
   public abstract class StructureIterator {
     public abstract boolean hasNext();
     public abstract int getModelIndex();
-    public abstract int getStructureType();
-    public abstract int getSubstructureType();
+    public abstract EnumStructure getStructureType();
+    public abstract EnumStructure getSubstructureType();
     public abstract String getStructureID();
     public abstract int getSerialID();
     public abstract int getStrandCount();
@@ -470,6 +501,11 @@ abstract public Object getAtomSetCollection(Object atomSetCollectionReader);
     return canonizeAlphaDigit(altLoc);
   }
 
+  /**
+   * @param name 
+   * @param type  
+   * @return Special load array
+   */
   public String[] specialLoad(String name, String type) {
     return null;
   }
