@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-03-12 01:34:14 +0100 (sam., 12 mars 2011) $
- * $Revision: 15275 $
+ * $Date: 2011-10-03 01:29:23 +0200 (lun., 03 oct. 2011) $
+ * $Revision: 16204 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -24,24 +24,20 @@
 
 package org.jmol.api;
 
-import java.awt.Component;
-import java.awt.Dimension;
-import java.awt.Event;
-import java.awt.Graphics;
-import java.awt.Image;
-import java.awt.Rectangle;
 import java.io.OutputStream;
 import java.io.Reader;
 import java.net.URL;
 import java.util.BitSet;
+import java.util.Map;
 import java.util.Properties;
-import java.util.Hashtable;
 import java.util.List;
-//import java.io.Reader;
+
 import javax.vecmath.Matrix4f;
 import javax.vecmath.Point3f;
+import javax.vecmath.Vector3f;
 
-
+import org.jmol.script.Token;
+import org.jmol.util.BoxInfo;
 import org.jmol.viewer.Viewer;
 
 /**
@@ -74,7 +70,7 @@ abstract public class JmolViewer extends JmolSimpleViewer {
    *    all at once.
    *    
    * 
-   * @param awtComponent
+   * @param container
    * @param jmolAdapter
    * @param htmlName 
    * @param documentBase 
@@ -83,17 +79,45 @@ abstract public class JmolViewer extends JmolSimpleViewer {
    * @param statusListener 
    * @return              a JmolViewer object
    */
-  static public JmolViewer allocateViewer(Component awtComponent,
+  static public JmolViewer allocateViewer(Object container,
                                           JmolAdapter jmolAdapter,
                                           String htmlName, URL documentBase, 
                                           URL codeBase,
                                           String commandOptions, 
                                           JmolStatusListener statusListener) {
     
-    return Viewer.allocateViewer(awtComponent, jmolAdapter,
+    return Viewer.allocateViewer(container, jmolAdapter,
         htmlName, documentBase, codeBase, commandOptions, statusListener);
   }
 
+  /**
+   * default null htmlName, URL bases, comandOptions, and statusListener.
+   * 
+   * @param container
+   * @param jmolAdapter
+   * @return             a viewer
+   */
+  public static JmolViewer allocateViewer(Object container, JmolAdapter jmolAdapter) {
+    return Viewer.allocateViewer(container, jmolAdapter, null, null, null, null, null);
+  }
+  
+  /**
+   * sets a custom console -- should be called IMMEDIATELY following allocateViewer
+   * 
+   * create your console with, perhaps:
+   * 
+   * new org.openscience.jmol.app.jmolPanel.AppConsole(viewer, displayFrame, 
+   *                               externalJPanel, buttonsEnabled);
+   * 
+   * (see examples/basic/org/jmol/Integration.java
+   * 
+   * @param console        the console to use  
+   * 
+   */
+  public void setConsole(JmolAppConsoleInterface console) {
+    getProperty("DATA_API", "getAppConsole", console); 
+  }
+
   abstract public BitSet getSmartsMatch(String smarts, BitSet bsSelected);
   
   /**
@@ -101,23 +125,11 @@ abstract public class JmolViewer extends JmolSimpleViewer {
    * 
    * @param gLeft
    * @param gRight
-   * @param size
-   * @param clip
+   * @param width 
+   * @param height 
    */
-  abstract public void renderScreenImage(Graphics gLeft, Graphics gRight, Dimension size,
-                                         Rectangle clip);
+  abstract public void renderScreenImage(Object gLeft, Object gRight, int width, int height);
 
-  /**
-   * OK, but safer to assign htmlName, URL bases, comandOptions, and statusListener now.
-   * 
-   * @param awtComponent
-   * @param jmolAdapter
-   * @return             a viewer
-   */
-  public static JmolViewer allocateViewer(Component awtComponent, JmolAdapter jmolAdapter) {
-    return Viewer.allocateViewer(awtComponent, jmolAdapter, null, null, null, null, null);
-  }
-  
   static public String getJmolVersion() {
     return Viewer.getJmolVersion();
   }
@@ -156,15 +168,15 @@ abstract public class JmolViewer extends JmolSimpleViewer {
 
 
   // change this to width, height
-  abstract public void setScreenDimension(Dimension dim);
+  abstract public void setScreenDimension(int width, int height);
   abstract public int getScreenWidth();
   abstract public int getScreenHeight();
 
-  public Image getScreenImage() {
-    return getScreenImage(null);
-  }
+//  public Image getScreenImage() {
+//    return getScreenImage(null);
+//  }
 
-  abstract public Image getScreenImage(Graphics g);
+  abstract public Object getScreenImage(Object g);
   abstract public void releaseScreenImage();
   
   abstract public void writeTextFile(String string, String data);
@@ -200,8 +212,6 @@ abstract public class JmolViewer extends JmolSimpleViewer {
    */
   abstract public Object getImageAs(String type, int quality, int width, int height, String fileName, OutputStream os);
 
-  abstract public boolean handleOldJvm10Event(Event e);
-
   abstract public int getMotionEventNumber();
 
   /**
@@ -236,12 +246,14 @@ abstract public class JmolViewer extends JmolSimpleViewer {
 
   abstract public void showUrl(String urlString);
 
-
+  abstract public void calcAtomsMinMax(BitSet bs, BoxInfo boxInfo);
+  abstract public Point3f getBoundBoxCenter();
+  abstract public Vector3f getBoundBoxCornerVector();
   abstract public int getMeasurementCount();
   abstract public String getMeasurementStringValue(int i);
   abstract public int[] getMeasurementCountPlusIndices(int i);
 
-  abstract public Component getDisplay();
+  abstract public Object getDisplay();
 
   abstract public BitSet getElementsPresentBitSet(int modelIndex);
 
@@ -268,12 +280,12 @@ abstract public class JmolViewer extends JmolSimpleViewer {
    * 
    * NOTE: THIS METHOD DOES NOT PRESERVE THE STATE
    * 
-   * @param arrayData a List of models, where each model is either a String
+   * @param arrayData a Vector of models, where each model is either a String
    *                  or a String[] or a Vector<String>
    * @param isAppend TRUE to append models (no ZAP)
    * @return null or error message
    */
-  abstract public String loadInline(List arrayData, boolean isAppend);
+  abstract public String loadInline(List<Object> arrayData, boolean isAppend);
 
   abstract public String evalStringQuiet(String script);
   abstract public boolean isScriptExecuting();
@@ -284,13 +296,13 @@ abstract public class JmolViewer extends JmolSimpleViewer {
   abstract public String getFileAsString(String filename);
   abstract public boolean getFileAsString(String[] data, int nBytesMax, boolean doSpecialLoad);
   abstract public Properties getModelSetProperties();
-  abstract public Hashtable getModelSetAuxiliaryInfo();
+  abstract public Map<String, Object> getModelSetAuxiliaryInfo();
   abstract public int getModelNumber(int modelIndex);
   abstract public String getModelName(int modelIndex);
   abstract public String getModelNumberDotted(int modelIndex);
   abstract public Properties getModelProperties(int modelIndex);
   abstract public String getModelProperty(int modelIndex, String propertyName);
-  abstract public Hashtable getModelAuxiliaryInfo(int modelIndex);
+  abstract public Map<String, Object> getModelAuxiliaryInfo(int modelIndex);
   abstract public Object getModelAuxiliaryInfo(int modelIndex, String keyName);
   abstract public boolean modelHasVibrationVectors(int modelIndex);
 
@@ -315,7 +327,7 @@ abstract public class JmolViewer extends JmolSimpleViewer {
 
   abstract public void homePosition();
 
-  abstract public Hashtable getHeteroList(int modelIndex);
+  abstract public Map<String, String> getHeteroList(int modelIndex);
 
 
   abstract public boolean getPerspectiveDepth();
@@ -371,6 +383,11 @@ abstract public class JmolViewer extends JmolSimpleViewer {
   abstract public void notifyViewerRepaintDone();
 
   abstract public boolean getBooleanProperty(String propertyName);
+  /**
+   * @param key 
+   * @param doICare IGNORED  
+   * @return T/F
+   */
   public boolean getBooleanProperty(String key, boolean doICare) {
     return getBooleanProperty(key); // don't ask for what doesn't exist; you should care!
   }
@@ -386,7 +403,7 @@ abstract public class JmolViewer extends JmolSimpleViewer {
 
   abstract public String getStateInfo();
   
-  abstract public void syncScript(String script, String applet);  
+  abstract public void syncScript(String script, String applet, int port);  
 
   //but NOTE that if you use the following, you are
   //bypassing the script history. If you want a script history, use
@@ -466,9 +483,40 @@ abstract public class JmolViewer extends JmolSimpleViewer {
 
   abstract public String getModelFileName(int modelIndex);
 
+  /**
+   * @param privateKey  
+   * @return T/F
+   */
   abstract public boolean checkPrivateKey(double privateKey);
-
-  public String menuStructure;
   
+  public String menuStructure;
+
+  /**
+   * retrieves list of draw/isosurface objects
+   * 
+   * @param map 
+   * @param b 
+   * 
+   */
+  abstract public void getObjectMap(Map<String, Token> map, boolean b);
+  
+  abstract public void processEvent(int groupID, int eventType, int touchID, int iData,
+                           Point3f pt, long time);
+
+  abstract public boolean handleOldJvm10Event(int id, int x, int y, int modifiers,
+                                     long when);
+
+  /**
+   * old -- not used in 12.2
+   * 
+   * @param g 
+   * @param currentSize 
+   * @param rectClip  // unused
+   */
+  public void renderScreenImage(Object g, Object currentSize,
+                                Object rectClip) {
+    apiPlatform.renderScreenImage(this, g, currentSize);
+  }
+
 }
 
