@@ -35,7 +35,7 @@ import javax.swing.JDialog;
 import javax.swing.JPanel;
 import javax.swing.JFrame;
 import javax.swing.JSplitPane;
-import javax.swing.JTextPane; //import javax.swing.SwingUtilities;
+import javax.swing.JTextPane; 
 import javax.swing.event.UndoableEditEvent;
 import javax.swing.event.UndoableEditListener;
 import javax.swing.text.DefaultEditorKit;
@@ -123,7 +123,7 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
   private JPanel buttonPanel = new JPanel();
 
   void layoutWindow(Container container) {
-    editor = new EditorTextPane(this);
+    editor = new EditorTextPane();
     editor.setDragEnabled(true);
     JScrollPane editorPane = new JScrollPane(editor);
 
@@ -189,6 +189,7 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
     editor.setCaretPosition(editor.editorDoc.getLength());
   }
 
+  @Override
   public void setVisible(boolean b) {
     super.setVisible(b);
     viewer.getProperty("DATA_API", "scriptEditorState", b ? Boolean.TRUE : Boolean.FALSE);
@@ -196,10 +197,6 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
       editor.grabFocus();
   }
   
-  public Object getMyMenuBar() {
-    return null;
-  }
-
   public String getText() {
     return editor.getText();
   }
@@ -209,6 +206,7 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
     editor.clearContent(message);
   }
 
+  @Override
   public void dispose() {
     super.dispose();
   }
@@ -221,12 +219,12 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
    * 
       if (isScriptCompletion && viewer.getMessageStyleChime()
           && viewer.getDebugScript()) {
-        jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_SCRIPT,
+        jmolCallbackListener.notifyCallback(EnumCallback.SCRIPT,
             new Object[] { null, "script <exiting>", statusMessage,
                 new Integer(-1), strErrorMessageUntranslated });
         strStatus = "Jmol script completed.";
       }
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_SCRIPT,
+      jmolCallbackListener.notifyCallback(EnumCallback.SCRIPT,
           new Object[] { sJmol, strStatus, statusMessage,
               new Integer(isScriptCompletion ? -1 : msWalltime),
               strErrorMessageUntranslated });
@@ -418,7 +416,7 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
 
     boolean checking = false;
 
-    EditorTextPane(ScriptEditor scriptEditor) {
+    EditorTextPane() {
       super(new EditorDocument());
       editorDoc = (EditorDocument) getDocument();
       editorDoc.setEditorTextPane(this);
@@ -435,6 +433,7 @@ public final class ScriptEditor extends JDialog implements JmolScriptEditorInter
       parseScript(text);
     }
     
+    @Override
     protected void processKeyEvent(KeyEvent ke) {
       int kcode = ke.getKeyCode();
       int kid = ke.getID();
