@@ -49,6 +49,7 @@ import org.jmol.viewer.Viewer;
 final public class Export3D implements JmolRendererInterface {
 
   private ___Exporter exporter;
+  private double privateKey;
 
   private Graphics3D g3d;
   private short colix;
@@ -72,13 +73,13 @@ final public class Export3D implements JmolRendererInterface {
     return exportName;
   }
 
-  public boolean initializeExporter(String type, Viewer viewer, Graphics3D g3d,
+  public boolean initializeExporter(String type, Viewer viewer, double privateKey, Graphics3D g3d,
                                     Object output) {
     exportName = type;
     try {
       String name = "org.jmol.export._"
           + type + "Exporter";
-      Class exporterClass = Class.forName(name);
+      Class<?> exporterClass = Class.forName(name);
       // Class exporterClass =
       // Class.forName("org.jmol.export.NewPovrayExporter");
       exporter = (___Exporter) exporterClass.newInstance();
@@ -91,7 +92,8 @@ final public class Export3D implements JmolRendererInterface {
     slab = g3d.getSlab();
     width = g3d.getRenderWidth();
     height = g3d.getRenderHeight();
-    return exporter.initializeOutput(viewer, g3d, output);
+    this.privateKey = privateKey;
+    return exporter.initializeOutput(viewer, privateKey, g3d, output);
   }
 
   public String finalizeOutput() {
@@ -358,14 +360,14 @@ final public class Export3D implements JmolRendererInterface {
     exporter.plotText(xBaseline, yBaseline, z, colix, str, font3d);
   }
 
-  public void drawImage(Image image, int x, int y, int z, int zSlab,
+  public void drawImage(Object objImage, int x, int y, int z, int zSlab,
                         short bgcolix, int width, int height) {
-    if (image == null || width == 0 || height == 0)
+    if (objImage == null || width == 0 || height == 0)
       return;
     if (isClippedZ(zSlab))
       return;
     z = Math.max(slab, z);
-    exporter.plotImage(x, y, z, image, bgcolix, width, height);
+    exporter.plotImage(x, y, z, (Image) objImage, bgcolix, width, height);
   }
 
   // mostly public drawing methods -- add "public" if you need to
@@ -512,17 +514,17 @@ final public class Export3D implements JmolRendererInterface {
   }
 
   public void fillConeScreen(byte endcap, int screenDiameter, Point3i pointBase,
-                       Point3i screenTip) {
+                       Point3i screenTip, boolean isBarb) {
     // dipole, vector, draw arrow/vector
     ptA.set(pointBase.x, pointBase.y, pointBase.z);
     ptB.set(screenTip.x, screenTip.y, screenTip.z);
-    fillConeSceen(endcap, screenDiameter, ptA, ptB);
+    exporter.fillConeScreen(colix, endcap, screenDiameter, ptA, ptB, isBarb);
   }
 
   public void fillConeSceen(byte endcap, int screenDiameter, Point3f pointBase,
                        Point3f screenTip) {
     // cartoons, rockets
-    exporter.fillConeScreen(colix, endcap, screenDiameter, pointBase, screenTip);
+    exporter.fillConeScreen(colix, endcap, screenDiameter, pointBase, screenTip, false);
   }
 
   public void drawHermite(int tension, Point3i s0, Point3i s1, Point3i s2,
@@ -666,14 +668,8 @@ final public class Export3D implements JmolRendererInterface {
         colixD, normixD);
   }
 
-  public void drawSurface(MeshSurface meshSurface, Point3f[] vertices, Point3f offset) {
-    exporter.drawSurface(meshSurface.vertexCount, meshSurface.polygonCount,
-        meshSurface.haveQuads ? 4 : 3, vertices == null ? meshSurface.vertices
-            : vertices, meshSurface.vertexNormals,
-        meshSurface.isColorSolid ? null : meshSurface.vertexColixes,
-        meshSurface.polygonIndexes,
-        meshSurface.isColorSolid ? meshSurface.polygonColixes : null,
-        meshSurface.bsFaces, meshSurface.colix, offset);
+  public void drawSurface(MeshSurface meshSurface, short colix) {
+    exporter.drawSurface(meshSurface, colix);
   }
 
   public short[] getBgColixes(short[] bgcolixes) {
@@ -718,6 +714,10 @@ final public class Export3D implements JmolRendererInterface {
     return true;
   }
 
+  public void setColor(int color) {
+    g3d.setColor(color);
+  }
+
   /**
    * gets g3d width
    * 
@@ -736,6 +736,10 @@ final public class Export3D implements JmolRendererInterface {
     return g3d.getRenderHeight();
   }
 
+  public boolean isPass2() {
+    return g3d.isPass2();
+  }
+
   /**
    * gets g3d slab
    * 
@@ -839,4 +843,17 @@ final public class Export3D implements JmolRendererInterface {
     // ignore
   }
 
+  public double getPrivateKey() {
+    return privateKey;
+  }
+
+  public void volumeRender(boolean TF) {
+    // not implemented
+  }
+
+  public void volumeRender(int diam, int x, int y, int z) {
+    fillSphere(diam, x, y, z);
+    
+  }
+
 }
