@@ -29,14 +29,13 @@
  
 package org.jmol.export;
 
+import java.util.ArrayList;
 import java.util.BitSet;
-import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import javax.vecmath.AxisAngle4f;
-import javax.vecmath.Matrix3f;
 import javax.vecmath.Matrix4f;
 import javax.vecmath.Point3f;
 import javax.vecmath.Tuple3f;
@@ -217,6 +216,7 @@ public class _IdtfExporter extends __CartesianExporter {
   private boolean haveCone;
   private boolean haveCircle;
   
+  @Override
   protected void output(Tuple3f pt) {
     output(pt, sbTemp, true);
   }
@@ -246,7 +246,7 @@ public class _IdtfExporter extends __CartesianExporter {
   }
   
   private int iObj;
-  private Hashtable htDefs = new Hashtable();
+  private Map<String, Boolean> htDefs = new Hashtable<String, Boolean>();
   
   final private Matrix4f m = new Matrix4f();
 
@@ -254,6 +254,7 @@ public class _IdtfExporter extends __CartesianExporter {
   final private StringBuffer resources = new StringBuffer();
   final private StringBuffer modifiers = new StringBuffer();
 
+  @Override
   protected void outputHeader() {
     // next is an approximation only 
     output("FILE_FORMAT \"IDTF\"\nFORMAT_VERSION 100\n");
@@ -292,6 +293,7 @@ public class _IdtfExporter extends __CartesianExporter {
     
   }
 
+  @Override
   String finalizeOutput() {
     super.finalizeOutput();
     return getAuxiliaryFileData();
@@ -340,7 +342,7 @@ public class _IdtfExporter extends __CartesianExporter {
         + "\n\\end{center}" 
         + "\n\\end{document}"
         +"\n\\begin{comment}"
-        + viewer.getWrappedState(true)
+        + viewer.getWrappedState(true, 0, 0)
         +"\n\\end{comment}";
 }
 
@@ -397,6 +399,7 @@ public class _IdtfExporter extends __CartesianExporter {
     modifiers.append("}}}}}\n");
   }
 
+  @Override
   protected void outputFooter() {
     htDefs = null;
     outputNodes();
@@ -475,32 +478,34 @@ public class _IdtfExporter extends __CartesianExporter {
   //    return q.q0 + " " + q.q1  + " " + q.q2 + " " + q.q3;
  // }
 
-  private Hashtable htNodes = new Hashtable();
+  private Map<String, List<String>> htNodes = new Hashtable<String, List<String>>();
   
   private void outputNodes() {
-    Enumeration e = htNodes.keys();
-    while (e.hasMoreElements()) {
-      String key = (String) e.nextElement();
-      List v = (List) htNodes.get(key);
+    for (Map.Entry<String, List<String>> entry : htNodes.entrySet()) {
+      String key = entry.getKey();
+      List<String> v = entry.getValue();
       output("NODE \"MODEL\" {\n");
       output("NODE_NAME \"" + key + "\"\n");
       int n = v.size();
       output("PARENT_LIST {\nPARENT_COUNT " + n + "\n"); 
       for (int i = 0; i < n; i++) {
         output("PARENT " + i + " {\n");
-        output((String)v.get(i));
+        output(v.get(i));
         output("}\n");
       }
       output("}\n");
       int i = key.indexOf("_");
-      if (i > 0)
+      if (i > 0) {
         key = key.substring(0,i);
-      if (key.equals("Ellipse"))
+      }
+      if (key.equals("Ellipse")) {
         key = "Circle";
+      }
       output("RESOURCE_NAME \"" + key + "_Mesh\"\n}\n");
     }
   }
 
+  @Override
   protected void outputEllipsoid(Point3f center, Point3f[] points, short colix) {
     //Hey, hey -- quaternions to the rescue!
     // Just send three points to Quaternion to define a plane and return
@@ -510,13 +515,13 @@ public class _IdtfExporter extends __CartesianExporter {
     float sx = points[1].distance(center);
     float sy = points[3].distance(center);
     float sz = points[5].distance(center);
-    outputEllipsoid(center, sx, sy, sz, a, colix);
+    setSphereMatrix(center, sx, sy, sz, a, sphereMatrix);
+    outputEllipsoid(center, sphereMatrix, colix);
   }
 
-  private Matrix4f sphereMatrix = new Matrix4f();
   private Matrix4f cylinderMatrix = new Matrix4f();
 
-  private void outputEllipsoid(Point3f center, float rx, float ry, float rz, AxisAngle4f a, short colix) {
+  private void outputEllipsoid(Point3f center, Matrix4f sphereMatrix, short colix) {
     if (!haveSphere) {
       models.append(getSphereResource());
       haveSphere = true;
@@ -524,31 +529,12 @@ public class _IdtfExporter extends __CartesianExporter {
     checkPoint(center);
     addColix(colix, false);
     String key = "Sphere_" + colix;
-    List v = (List) htNodes.get(key);
+    List<String> v = htNodes.get(key);
     if (v == null) {
-      v = new ArrayList();
+      v = new ArrayList<String>();
       htNodes.put(key, v);
       addShader(key, colix);
     }
-    if (a != null) {
-      Matrix3f mq = new Matrix3f();
-      Matrix3f m = new Matrix3f();
-      m.m00 = rx;
-      m.m11 = ry;
-      m.m22 = rz;
-      mq.set(a);
-      mq.mul(m);
-      sphereMatrix.set(mq);
-    } else {
-      sphereMatrix.setIdentity();
-      sphereMatrix.m00 = rx;
-      sphereMatrix.m11 = ry;
-      sphereMatrix.m22 = rz;
-    }
-    sphereMatrix.m03 = center.x;
-    sphereMatrix.m13 = center.y;
-    sphereMatrix.m23 = center.z;
-    sphereMatrix.m33 = 1;
     v.add(getParentItem("Jmol", sphereMatrix));
   }
 
@@ -628,9 +614,10 @@ public class _IdtfExporter extends __CartesianExporter {
            .append("SHADER_ID 0\n}}\n");
   }
 
+  @Override
   protected boolean outputCylinder(Point3f ptCenter, Point3f pt1, Point3f pt2,
                                    short colix, byte endcaps, float radius,
-                                   Point3f ptX, Point3f ptY) {
+                                   Point3f ptX, Point3f ptY, boolean checkRadius) {
     if (ptX != null) {
       if (endcaps == Graphics3D.ENDCAPS_FLAT) {
         outputEllipse(ptCenter, pt1, ptX, ptY, colix);
@@ -641,8 +628,8 @@ public class _IdtfExporter extends __CartesianExporter {
       }
 
     } else if (endcaps == Graphics3D.ENDCAPS_SPHERICAL) {
-      outputSphere(pt1, radius * 1.01f, colix);
-      outputSphere(pt2, radius * 1.01f, colix);
+      outputSphere(pt1, radius * 1.01f, colix, true);
+      outputSphere(pt2, radius * 1.01f, colix, true);
     } else if (endcaps == Graphics3D.ENDCAPS_FLAT) {
       outputCircle(pt1, pt2, colix, radius);
       outputCircle(pt2, pt1, colix, radius);
@@ -661,9 +648,9 @@ public class _IdtfExporter extends __CartesianExporter {
     int n = (ptX != null && endcaps == Graphics3D.ENDCAPS_NONE ? 2 : 1);
     for (int i = 0; i < n; i++) {
       String key = "Cylinder" + (i == 0 ? "_" : "In_") + colix;
-      List v = (List) htNodes.get(key);
+      List<String> v = htNodes.get(key);
       if (v == null) {
-        v = new ArrayList();
+        v = new ArrayList<String>();
         htNodes.put(key, v);
         addShader(key, colix);
       }
@@ -682,6 +669,7 @@ public class _IdtfExporter extends __CartesianExporter {
     return true;
   }
 
+  @Override
   protected void outputCircle(Point3f pt1, Point3f pt2, float radius,
                               short colix, boolean doFill) {
     if (doFill) {
@@ -713,9 +701,9 @@ public class _IdtfExporter extends __CartesianExporter {
     }
     addColix(colix, false);
     String key = "Ellipse_" + colix;
-    List v = (List) htNodes.get(key);
+    List<String> v = htNodes.get(key);
     if (v == null) {
-      v = new ArrayList();
+      v = new ArrayList<String>();
       htNodes.put(key, v);
       addShader(key, colix);
     }
@@ -737,9 +725,9 @@ public class _IdtfExporter extends __CartesianExporter {
     }
     addColix(colix, false);
     String key = "Circle_" + colix;
-    List v = (List) htNodes.get(key);
+    List<String> v = htNodes.get(key);
     if (v == null) {
-      v = new ArrayList();
+      v = new ArrayList<String>();
       htNodes.put(key, v);
       addShader(key, colix);
     }
@@ -796,6 +784,7 @@ public class _IdtfExporter extends __CartesianExporter {
 
   private StringBuffer sbTemp;
   
+  @Override
   protected void outputFace(int[] face, int[] map, int faceVertexMax) {
     sbTemp.append(" " + map[face[0]] + " " + map[face[1]] + " "
         + map[face[2]]);
@@ -805,12 +794,13 @@ public class _IdtfExporter extends __CartesianExporter {
     }
   }
 
+  @Override
   protected void outputSurface(Point3f[] vertices, Vector3f[] normals,
                                short[] colixes, int[][] indices,
                                short[] polygonColixes, int nVertices,
-                               int nPolygons, int nFaces, BitSet bsFaces,
+                               int nPolygons, int nFaces, BitSet bsPolygons,
                                int faceVertexMax, short colix,
-                               List colorList, Hashtable htColixes,
+                               List<Short> colorList, Map<Short, Integer> htColixes,
                                Point3f offset) {
     addColix(colix, polygonColixes != null || colixes != null);
     if (polygonColixes != null) {
@@ -822,17 +812,17 @@ public class _IdtfExporter extends __CartesianExporter {
 
     StringBuffer sbFaceCoordIndices = sbTemp = new StringBuffer();
     int[] map = new int[nVertices];
-    int nCoord = getCoordinateMap(vertices, map);
-    outputIndices(indices, map, nPolygons, bsFaces, faceVertexMax);
+    int nCoord = getCoordinateMap(vertices, map, null);
+    outputIndices(indices, map, nPolygons, bsPolygons, faceVertexMax);
 
     // normals, part 1
 
     StringBuffer sbFaceNormalIndices = sbTemp = new StringBuffer();
-    List vNormals = null;
+    List<String> vNormals = null;
     if (normals != null) {
-      vNormals = new ArrayList();
-      map = getNormalMap(normals, nVertices, vNormals);
-      outputIndices(indices, map, nPolygons, bsFaces, faceVertexMax);
+      vNormals = new ArrayList<String>();
+      map = getNormalMap(normals, nVertices, null, vNormals);
+      outputIndices(indices, map, nPolygons, bsPolygons, faceVertexMax);
     }
 
     map = null;
@@ -841,9 +831,9 @@ public class _IdtfExporter extends __CartesianExporter {
 
     StringBuffer sbColorIndexes = new StringBuffer();
     if (colorList != null) {
-      boolean isAll = (bsFaces == null);
-      int i0 = (isAll ? nPolygons - 1 : bsFaces.nextSetBit(0));
-      for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsFaces.nextSetBit(i + 1))) {
+      boolean isAll = (bsPolygons == null);
+      int i0 = (isAll ? nPolygons - 1 : bsPolygons.nextSetBit(0));
+      for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsPolygons.nextSetBit(i + 1))) {
         if (polygonColixes == null) {
           sbColorIndexes.append(" "
               + htColixes.get("" + colixes[indices[i][0]]) + " "
@@ -884,7 +874,7 @@ public class _IdtfExporter extends __CartesianExporter {
     if (colorList != null) {
       nColors = colorList.size();
       for (int i = 0; i < nColors; i++) {
-        short c = ((Short) colorList.get(i)).shortValue();
+        short c = colorList.get(i).shortValue();
         sbColors.append(rgbFractionalFromColix(c, ' ')).append(" ").append(
             translucencyFractionalFromColix(c)).append(" ");
       }
@@ -892,7 +882,7 @@ public class _IdtfExporter extends __CartesianExporter {
     String key = "mesh" + (++iObj);
     addMeshData(key, nFaces, nCoord, nNormals, nColors, sbFaceCoordIndices,
         sbFaceNormalIndices, sbColorIndexes, sbCoords, sbNormals, sbColors);
-    List v = new ArrayList();
+    List<String> v = new ArrayList<String>();
     htNodes.put(key, v);
     addShader(key, colix);
     cylinderMatrix.setIdentity();
@@ -929,6 +919,7 @@ public class _IdtfExporter extends __CartesianExporter {
     models.append("}}}\n");
   }
 
+  @Override
   protected void outputCone(Point3f ptBase, Point3f ptTip, float radius,
                             short colix) {
     if (!haveCone) {
@@ -939,9 +930,9 @@ public class _IdtfExporter extends __CartesianExporter {
     checkPoint(ptTip);
     addColix(colix, false);
     String key = "Cone_" + colix;
-    List v = (List) htNodes.get(key);
+    List<String> v = htNodes.get(key);
     if (v == null) {
-      v = new ArrayList();
+      v = new ArrayList<String>();
       htNodes.put(key, v);
       addShader(key, colix);
     }
@@ -978,20 +969,24 @@ public class _IdtfExporter extends __CartesianExporter {
     return getMeshData("Circle", faces, vertexes, normals);
   }
   
-  protected void outputSphere(Point3f center, float radius, short colix) {
-    outputEllipsoid(center, radius, radius, radius, null, colix);
+  @Override
+  protected void outputSphere(Point3f center, float radius, short colix, boolean checkRadius) {
+    setSphereMatrix(center, radius, radius, radius, null, sphereMatrix);
+    outputEllipsoid(center, sphereMatrix, colix);
   }
 
+  @Override
   protected void outputTextPixel(Point3f pt, int argb) {    
     short colix = Graphics3D.getColix(argb); 
-    outputSphere(pt, 0.02f, colix);
+    outputSphere(pt, 0.02f, colix, true);
   }
 
+  @Override
   protected void outputTriangle(Point3f pt1, Point3f pt2, Point3f pt3, short colix) {
     addColix(colix, false);
     String key = "T" + (++iObj);
     models.append(getTriangleResource(key, pt1, pt2, pt3));
-    List v = new ArrayList();
+    List<String> v = new ArrayList<String>();
     htNodes.put(key, v);
     addShader(key, colix);
     if (cylinderMatrix == null)
