@@ -25,15 +25,18 @@
 
 package org.jmol.export;
 
+
 import java.util.BitSet;
-import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.Point3f;
+import javax.vecmath.Point4f;
 import javax.vecmath.Tuple3f;
 import javax.vecmath.Vector3f;
 
 import org.jmol.g3d.Graphics3D;
+import org.jmol.util.Measure;
 import org.jmol.viewer.Viewer;
 
 /*
@@ -49,11 +52,13 @@ public class _PovrayExporter extends __RayTracerExporter {
     commentChar = "// ";
   }
 
+  @Override
   String finalizeOutput() {
     super.finalizeOutput();
     return getAuxiliaryFileData();
   }
 
+  @Override
   protected void outputHeader() {
     super.outputHeader();
     output("// ******************************************************\n");
@@ -62,7 +67,7 @@ public class _PovrayExporter extends __RayTracerExporter {
     output("// This script was generated on " + getExportDate() + "\n");
     output("// ******************************************************\n");
     try {
-    output(viewer.getWrappedState(true));
+    output(viewer.getWrappedState(true, 0, 0));
     } catch (Exception e) {
       // tough luck
     }
@@ -117,9 +122,9 @@ public class _PovrayExporter extends __RayTracerExporter {
 
   private void writeMacros() {
     output("#default { finish {\n" + "  ambient "
-        + (float) Graphics3D.getAmbientPercent() / 100f + "\n" + "  diffuse "
-        + (float) Graphics3D.getDiffusePercent() / 100f + "\n" + "  specular "
-        + (float) Graphics3D.getSpecularPercent() / 100f + "\n"
+        + Graphics3D.getAmbientPercent() / 100f + "\n" + "  diffuse "
+        + Graphics3D.getDiffusePercent() / 100f + "\n" + "  specular "
+        + Graphics3D.getSpecularPercent() / 100f + "\n"
         + "  roughness .00001\n  metallic\n  phong 0.9\n  phong_size 120\n}}"
         + "\n\n");
 
@@ -173,8 +178,6 @@ public class _PovrayExporter extends __RayTracerExporter {
     writeMacrosFinish();
     writeMacrosAtom();
     writeMacrosBond();
-    writeMacrosTriangle();
-    writeMacrosTextPixel();
     //    writeMacrosRing();
   }
 
@@ -188,9 +191,9 @@ public class _PovrayExporter extends __RayTracerExporter {
         + "  #declare shineFactor = 0;\n"
         + " #end\n"
         + " finish {\n" + "  ambient "
-        + (float) Graphics3D.getAmbientPercent() / 100f + "\n" + "  diffuse "
-        + (float) Graphics3D.getDiffusePercent() / 100f + "\n" + "  specular "
-        + (float) Graphics3D.getSpecularPercent() / 100f + "\n"
+        + Graphics3D.getAmbientPercent() / 100f + "\n" + "  diffuse "
+        + Graphics3D.getDiffusePercent() / 100f + "\n" + "  specular "
+        + Graphics3D.getSpecularPercent() / 100f + "\n"
         + "  roughness .00001\n"  
         + "  metallic shineFactor\n"  
         + "  phong 0.9*shineFactor\n"  
@@ -254,7 +257,10 @@ public class _PovrayExporter extends __RayTracerExporter {
 
   }
 
-  private void writeMacrosTriangle() {
+  private boolean haveMacros;
+  
+  private void writeMacros2() {
+    // triangle
     output("#macro r(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,R,G,B,T)\n"
         + " triangle{<X1,Y1,Z1>,<X2,Y2,Z2>,<X3,Y3,Z3>\n"
         + "  pigment{rgbt<R,G,B,T>}\n"
@@ -262,27 +268,31 @@ public class _PovrayExporter extends __RayTracerExporter {
         + "  clip()\n"
         + "  check_shadow()}\n" 
         + "#end\n\n");
-  }
-
-  private void writeMacrosTextPixel() {
+    // text pixel
     output("#macro p(X,Y,Z,R,G,B)\n" 
         + " box{<X,Y,Z>,<X+1,Y+1,Z+1>\n"
         + "  pigment{rgb<R,G,B>}\n"
         + "  clip()\n"
         + "  check_shadow()}\n" 
         + "#end\n\n");
+    // draw arrow BARB
+    output("#macro barb(X1,Y1,Z1,RADIUS1,X2,Y2,Z2,RADIUS2,R,G,B,T,X3,Y3,Z3,W3)\n"
+        + " cone{<X1,Y1,Z1>,RADIUS1,<X2,Y2,Z2>,RADIUS2\n"
+        + "  pigment{rgbt<R,G,B,T>}\n"
+        + "  translucentFinish(T)\n"
+        + "  clip()\n"
+        + "  clipped_by{plane{<X3,Y3,Z3>,W3}}\n"
+        + "  check_shadow()}\n" + "#end\n\n");
+    /*
+    // This type of ring does not take into account perspective effects!
+    output("#macro o(X,Y,Z,RADIUS,R,G,B,T)\n"
+    + " torus{RADIUS,wireRadius pigment{rgbt<R,G,B,T>}\n"
+    + " translate<X,Z,-Y> rotate<90,0,0>\n" + "  check_shadow()}\n"
+    + "#end\n\n");
+    */
+    haveMacros = true;
   }
-
-  /*
-  private void writeMacrosRing() {
-  // This type of ring does not take into account perspective effects!
-  output("#macro o(X,Y,Z,RADIUS,R,G,B,T)\n"
-  + " torus{RADIUS,wireRadius pigment{rgbt<R,G,B,T>}\n"
-  + " translate<X,Z,-Y> rotate<90,0,0>\n" + "  check_shadow()}\n"
-  + "#end\n\n");
-  }
-  */
-
+  
   private String triad(Tuple3f pt) {
     if (Float.isNaN(pt.x))
       return "0,0,0";
@@ -321,10 +331,12 @@ public class _PovrayExporter extends __RayTracerExporter {
 
   }
 
+  @Override
   protected void output(Tuple3f pt) {
     output(", <" + triad(pt) + ">");    
   }
   
+  @Override
   protected void outputCircle(int x, int y, int z, float radius, short colix,
                               boolean doFill) {
     output((doFill ? "b(" : "c(") + x + "," + y + "," + z + "," + radius + ","
@@ -332,12 +344,26 @@ public class _PovrayExporter extends __RayTracerExporter {
         + color4(colix) + ")\n");
   }
 
-  protected void outputCone(Point3f screenBase, Point3f screenTip, float radius,
-                            short colix) {
-    output("b(" + triad(screenBase) + "," + radius + ","
-        + triad(screenTip) + ",0" + "," + color4(colix) + ")\n");
+  @Override
+  protected void outputCone(Point3f screenBase, Point3f screenTip,
+                            float radius, short colix, boolean isBarb) {
+    if (isBarb) {
+      if (!haveMacros)
+        writeMacros2();
+      Point4f plane = new Point4f();
+      tempP1.set(screenBase.x, screenTip.y, 12345.6789f);
+      Measure.getPlaneThroughPoints(screenBase, screenTip, tempP1, tempV1,
+          tempV2, tempV3, plane);
+      output("barb(" + triad(screenBase) + "," + radius + ","
+          + triad(screenTip) + ",0" + "," + color4(colix) + "," + plane.x + ","
+          + plane.y + "," + plane.z + "," + -plane.w + ")\n");
+    } else {
+      output("b(" + triad(screenBase) + "," + radius + "," + triad(screenTip)
+          + ",0" + "," + color4(colix) + ")\n");
+    }
   }
 
+  @Override
   protected void outputCylinder(Point3f screenA, Point3f screenB, float radius,
                               short colix, boolean withCaps) {
     String color = color4(colix);
@@ -346,12 +372,14 @@ public class _PovrayExporter extends __RayTracerExporter {
         + radius + "," + color + ")\n");
   }
   
+  @Override
   protected void outputCylinderConical(Point3f screenA, Point3f screenB,
                                        float radius1, float radius2, short colix) {
     output("b(" + triad(screenA) + "," + radius1 + "," + triad(screenB) + ","
         + radius2 + "," + color4(colix) + ")\n");
   }
 
+  @Override
   protected void outputEllipsoid(Point3f center, float radius, double[] coef, short colix) {
     // no quadrant cut-out here
     String s = coef[0] + "," + coef[1] + "," + coef[2] + "," + coef[3] + ","
@@ -360,15 +388,17 @@ public class _PovrayExporter extends __RayTracerExporter {
     output("q(" + s + ")\n");
   }
 
+  @Override
   protected void outputSurface(Point3f[] vertices, Vector3f[] normals,
                                   short[] colixes, int[][] indices, 
                                   short[] polygonColixes,
-                                  int nVertices, int nPolygons, int nFaces, BitSet bsFaces,
-                                  int faceVertexMax, short colix, List colorList, Hashtable htColixes, Point3f offset) {
+                                  int nVertices, int nPolygons, int nFaces, BitSet bsPolygons,
+                                  int faceVertexMax, short colix,
+                                  List<Short> colorList, Map<Short, Integer> htColixes, Point3f offset) {
     if (polygonColixes != null) {
-      boolean isAll = (bsFaces == null);
-      int i0 = (isAll ? nPolygons - 1 : bsFaces.nextSetBit(0));
-      for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsFaces.nextSetBit(i + 1))) {
+      boolean isAll = (bsPolygons == null);
+      int i0 = (isAll ? nPolygons - 1 : bsPolygons.nextSetBit(0));
+      for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsPolygons.nextSetBit(i + 1))) {
         //if ((p++) % 10 == 0)
         //  output("\n");
         output("polygon { 4\n"); 
@@ -409,25 +439,25 @@ public class _PovrayExporter extends __RayTracerExporter {
       String finish = ">}" + " translucentFinish("
         + translucencyFractionalFromColix(colixes[0]) + ")}";
       for (int i = 0; i < nColix; i++)
-        output("\n, texture{pigment{rgbt<" + color4(((Short)colorList.get(i)).shortValue()) + finish);
+        output("\n, texture{pigment{rgbt<" + color4(colorList.get(i).shortValue()) + finish);
       output("\n}\n");
     }
     output("face_indices { " + nFaces);
-    boolean isAll = (bsFaces == null);
-    int i0 = (isAll ? nPolygons - 1 : bsFaces.nextSetBit(0));
-    for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsFaces.nextSetBit(i + 1))) {
+    boolean isAll = (bsPolygons == null);
+    int i0 = (isAll ? nPolygons - 1 : bsPolygons.nextSetBit(0));
+    for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsPolygons.nextSetBit(i + 1))) {
       output(", <" + triad(indices[i]) + ">");
       if (colixes != null) {
-        output("," + htColixes.get("" + colixes[indices[i][0]]));
-        output("," + htColixes.get("" + colixes[indices[i][1]]));
-        output("," + htColixes.get("" + colixes[indices[i][2]]));
+        output("," + htColixes.get(Short.valueOf(colixes[indices[i][0]])));
+        output("," + htColixes.get(Short.valueOf(colixes[indices[i][1]])));
+        output("," + htColixes.get(Short.valueOf(colixes[indices[i][2]])));
       }
       if (faceVertexMax == 4 && indices[i].length == 4) {
         output(", <" + indices[i][0] + "," + indices[i][2] + "," + indices[i][3] + ">");
         if (colixes != null) {
-          output("," + htColixes.get("" + colixes[indices[i][0]]));
-          output("," + htColixes.get("" + colixes[indices[i][2]]));
-          output("," + htColixes.get("" + colixes[indices[i][3]]));
+          output("," + htColixes.get(Short.valueOf(colixes[indices[i][0]])));
+          output("," + htColixes.get(Short.valueOf(colixes[indices[i][2]])));
+          output("," + htColixes.get(Short.valueOf(colixes[indices[i][3]])));
         }
       }
       output("\n");
@@ -475,19 +505,26 @@ public class _PovrayExporter extends __RayTracerExporter {
 
   }
 
+  @Override
   protected void outputSphere(float x, float y, float z, float radius,
                                   short colix) {
    output("a(" + x + "," + y + "," + z + "," + radius + ","
         + color4(colix) + ")\n");
   }
   
+  @Override
   protected void outputTextPixel(int x, int y, int z, int argb) {
+    if (!haveMacros)
+      writeMacros2();
     //text only
     output("p(" + x + "," + y + "," + z + "," + 
         rgbFractionalFromArgb(argb, ',') + ")\n");
   }
   
+  @Override
   protected void outputTriangle(Point3f ptA, Point3f ptB, Point3f ptC, short colix) {
+    if (!haveMacros)
+      writeMacros2();
     //cartoons, mesh, isosurface
     output("r(" + triad(ptA) + "," + triad(ptB) + "," + triad(ptC) + ","
         + color4(colix) + ")\n");
