@@ -25,6 +25,7 @@
 
 package org.jmol.export;
 
+
 import java.awt.Image;
 import java.io.BufferedWriter;
 import java.io.File;
@@ -33,11 +34,12 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.text.SimpleDateFormat;
+import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Date;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import javax.vecmath.AxisAngle4f;
 import javax.vecmath.Matrix3f;
@@ -140,6 +142,7 @@ public abstract class ___Exporter {
   // generally useful functionality:
 
   protected Viewer viewer;
+  protected double privateKey;
   protected JmolRendererInterface jmolRenderer;
   protected StringBuffer output;
   protected BufferedWriter bw;
@@ -197,9 +200,10 @@ public abstract class ___Exporter {
     this.jmolRenderer = jmolRenderer;
   }
   
-  boolean initializeOutput(Viewer viewer, Graphics3D g3d, Object output) {
+  boolean initializeOutput(Viewer viewer, double privateKey, Graphics3D g3d, Object output) {
     this.viewer = viewer;
     this.g3d = g3d;
+    this.privateKey = privateKey;
     backgroundColix = viewer.getObjectColix(StateManager.OBJ_BACKGROUND);
     center.set(viewer.getRotationCenter());
     if ((screenWidth <= 0) || (screenHeight <= 0)) {
@@ -344,11 +348,11 @@ public abstract class ___Exporter {
   }
 
   protected static String translucencyFractionalFromColix(short colix) {
-    return round(Graphics3D.translucencyFractionalFromColix(colix));
+    return round(Graphics3D.getColixTranslucencyFractional(colix));
   }
 
   protected static String opacityFractionalFromColix(short colix) {
-    return round(1 - Graphics3D.translucencyFractionalFromColix(colix));
+    return round(1 - Graphics3D.getColixTranslucencyFractional(colix));
   }
 
   protected static String opacityFractionalFromArgb(int argb) {
@@ -369,27 +373,26 @@ public abstract class ___Exporter {
   }
   
   /**
-   * input an array of colixes; returns a List for the color list and a
+   * input an array of colixes; returns a Vector for the color list and a
    * HashTable for correlating the colix with a specific color index
    * @param i00 
    * @param colixes
    * @param nVertices
    * @param bsSelected
    * @param htColixes
-   * @return List and HashTable
+   * @return Vector and HashTable
    */
-  protected List getColorList(int i00, short[] colixes, int nVertices,
-                                BitSet bsSelected, Hashtable htColixes) {
-    String color;
+  protected List<Short> getColorList(int i00, short[] colixes, int nVertices,
+                                BitSet bsSelected, Map<Short, Integer> htColixes) {
     int nColix = 0;
-    List list = new ArrayList();
+    List<Short> list = new ArrayList<Short>();
     boolean isAll = (bsSelected == null);
     int i0 = (isAll ? nVertices - 1 : bsSelected.nextSetBit(0));
     for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsSelected.nextSetBit(i + 1))) {
-      color = "" + colixes[i];
+      Short color = Short.valueOf(colixes[i]);
       if (!htColixes.containsKey(color)) {
-        list.add(new Short(colixes[i]));
-        htColixes.put(color, "" + (i00 + nColix++));
+        list.add(color);
+        htColixes.put(color, Integer.valueOf(i00 + nColix++));
       }
     }
     return list;
@@ -412,12 +415,12 @@ public abstract class ___Exporter {
     }
     ms.vertices[n] = new Point3f(0, 0, 1);
     if (matRotateScale != null) {
-      ms.vertexNormals = new Vector3f[ms.vertexCount];
+      ms.normals = new Vector3f[ms.vertexCount];
       for (int i = 0; i < ms.vertexCount; i++) {
         matRotateScale.transform(ms.vertices[i]);
-        ms.vertexNormals[i] = new Vector3f();
-        ms.vertexNormals[i].set(ms.vertices[i]);
-        ms.vertexNormals[i].normalize();
+        ms.normals[i] = new Vector3f();
+        ms.normals[i].set(ms.vertices[i]);
+        ((Vector3f) ms.normals[i]).normalize();
         ms.vertices[i].add(centerBase);
       }
     }
@@ -432,9 +435,7 @@ public abstract class ___Exporter {
       m1.setIdentity();
       if (pt1.z > pt2.z) // 180-degree rotation about X
         m1.m11 = m1.m22 = -1;
-/*      if (pt1.z > pt2.z)
-        m1.mul(-1);
-*/    } else {
+    } else {
       tempV1.set(pt2);
       tempV1.sub(pt1);
       tempV2.set(0, 0, 1);
@@ -472,35 +473,69 @@ public abstract class ___Exporter {
   abstract boolean drawEllipse(Point3f ptAtom, Point3f ptX, Point3f ptY,
                              short colix, boolean doFill);
 
-  void drawSurface(int nVertices, int nPolygons, int faceVertexMax,
-                      Point3f[] vertices, Vector3f[] normals, short[] colixes,
-                      int[][] indices, short[] polygonColixes, BitSet bsFaces,
-                      short colix, Point3f offset) {
+  void drawSurface(MeshSurface meshSurface, short colix) {
+    int nVertices = meshSurface.vertexCount;
     if (nVertices == 0)
       return;
     int nFaces = 0;
-    boolean isAll = (bsFaces == null);
-    int i0 = (isAll ? nPolygons - 1 : bsFaces.nextSetBit(0));
-    for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsFaces.nextSetBit(i + 1))) 
+    int nPolygons = meshSurface.polygonCount;
+    BitSet bsPolygons = meshSurface.bsPolygons;
+    int faceVertexMax = (meshSurface.haveQuads ? 4 : 3);
+    int[][] indices = meshSurface.polygonIndexes;
+    boolean isAll = (bsPolygons == null);
+    int i0 = (isAll ? nPolygons - 1 : bsPolygons.nextSetBit(0));
+    for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsPolygons.nextSetBit(i + 1)))
       nFaces += (faceVertexMax == 4 && indices[i].length == 4 ? 2 : 1);
     if (nFaces == 0)
       return;
-    Hashtable htColixes = new Hashtable();
-    List colorList = null;
+
+    Point3f[] vertices = (Point3f[]) meshSurface.getVertices();
+    Vector3f[] normals = (Vector3f[]) meshSurface.normals;
+
+    boolean colorSolid = (colix != 0);
+    short[] colixes = (colorSolid ? null : meshSurface.vertexColixes);
+    short[] polygonColixes = (colorSolid ? meshSurface.polygonColixes : null);
+    Map<Short, Integer> htColixes = new Hashtable<Short, Integer>();
+    List<Short> colorList = null;
     if (polygonColixes != null)
-      colorList = getColorList(0, polygonColixes, nPolygons, bsFaces, htColixes);
+      colorList = getColorList(0, polygonColixes, nPolygons, bsPolygons,
+          htColixes);
     else if (colixes != null)
       colorList = getColorList(0, colixes, nVertices, null, htColixes);
+    
     outputSurface(vertices, normals, colixes, indices, polygonColixes,
-        nVertices, nPolygons, nFaces, bsFaces, faceVertexMax, colix, colorList,
-        htColixes, offset);
+        nVertices, nPolygons, nFaces, bsPolygons, faceVertexMax, colix,
+        colorList, htColixes, meshSurface.offset);
   }
 
-  abstract protected void outputSurface(Point3f[] vertices, Vector3f[] normals,
+  /**
+   * @param vertices      generally unique vertices [0:nVertices)
+   * @param normals       one per vertex
+   * @param colixes       one per vertex, or null
+   * @param indices       one per triangular or quad polygon;
+   *                      may have additional elements beyond vertex indices if faceVertexMax = 3
+   *                      triangular if faceVertexMax == 3; 3 or 4 if face VertexMax = 4
+   * @param polygonColixes face-based colixes
+   * @param nVertices      vertices[nVertices-1] is last vertex
+   * @param nPolygons     indices[nPolygons - 1] is last polygon
+   * @param nFaces        number of triangular faces required
+   * @param bsPolygons    number of polygons (triangles or quads)   
+   * @param faceVertexMax (3) triangles only, indices[][i] may have more elements
+   *                      (4) triangles and quads; indices[][i].length determines 
+   * @param colix         overall (solid) color index
+   * @param colorList     list of unique color IDs
+   * @param htColixes     map of color IDs to colorList
+   * @param offset 
+   * 
+   */
+  protected void outputSurface(Point3f[] vertices, Vector3f[] normals,
                                 short[] colixes, int[][] indices,
                                 short[] polygonColixes,
-                                int nVertices, int nPolygons, int nFaces, BitSet bsFaces,
-                                int faceVertexMax, short colix, List colorList, Hashtable htColixes, Point3f offset);
+                                int nVertices, int nPolygons, int nFaces, BitSet bsPolygons,
+                                int faceVertexMax, short colix, List<Short> colorList,
+                                Map<Short, Integer> htColixes, Point3f offset) {
+    // not implemented in _ObjExporter
+  }
 
   abstract void drawPixel(short colix, int x, int y, int z, int scale); //measures
   
@@ -508,7 +543,7 @@ public abstract class ___Exporter {
 
   //rockets and dipoles
   abstract void fillConeScreen(short colix, byte endcap, int screenDiameter, 
-                         Point3f screenBase, Point3f screenTip);
+                         Point3f screenBase, Point3f screenTip, boolean isBarb);
   
   abstract void drawCylinder(Point3f atom1, Point3f atom2, short colix1, short colix2,
                              byte endcaps, int madBond, int bondOrder);
@@ -564,7 +599,7 @@ public abstract class ___Exporter {
 
 }
 
-class UseTable extends Hashtable {
+class UseTable extends Hashtable<String, String> {
   private int iObj;
   private String keyword;
   private char term;
