@@ -23,6 +23,8 @@
  */
 package org.jmol.export.dialog;
 
+import java.awt.Dimension;
+import java.awt.Point;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import javax.swing.JFileChooser;
@@ -32,6 +34,7 @@ import javax.swing.JPanel;
 import javax.swing.JComboBox;
 import javax.swing.LookAndFeel;
 import javax.swing.SwingConstants;
+import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
 import javax.swing.UnsupportedLookAndFeelException;
 import javax.swing.border.TitledBorder;
@@ -41,9 +44,7 @@ import javax.swing.filechooser.FileFilter;
 
 import java.awt.BorderLayout;
 import java.awt.Component;
-import java.awt.Dimension;
 import java.awt.FlowLayout;
-import java.awt.Point;
 import java.io.File;
 
 import org.jmol.api.JmolDialogInterface;
@@ -51,7 +52,9 @@ import org.jmol.api.JmolViewer;
 import org.jmol.export.history.HistoryFile;
 import org.jmol.i18n.GT;
 import org.jmol.util.Escape;
+import org.jmol.util.Logger;
 import org.jmol.viewer.FileManager;
+import org.jmol.viewer.Viewer;
 
 public class Dialog extends JPanel implements JmolDialogInterface {
 
@@ -181,7 +184,7 @@ public class Dialog extends JPanel implements JmolDialogInterface {
     if (type != null)
       saveChooser.addChoosableFileFilter(new TypeFilter(type));
     saveChooser.setSelectedFile(file);
-    if ((file = showSaveDialog(this, this, saveChooser, file)) == null)
+    if ((file = showSaveDialog(this, saveChooser, file)) == null)
       return null;
     FileManager.setLocalPath(viewer, file.getParent(), true);
     return file.getAbsolutePath();
@@ -235,7 +238,7 @@ public class Dialog extends JPanel implements JmolDialogInterface {
     }
     createExportPanel(imageChoices, imageExtensions, type);
     imageChooser.setSelectedFile(initialFile = file);
-    if ((file = showSaveDialog(this, this, imageChooser, file)) == null)
+    if ((file = showSaveDialog(this, imageChooser, file)) == null)
       return null;
     qualityJPG = qSliderJPEG.getValue();
     qualityPNG = qSliderPNG.getValue();
@@ -376,7 +379,7 @@ public class Dialog extends JPanel implements JmolDialogInterface {
     return (opt == 0);
   }
 
-  private File showSaveDialog(Dialog sd, Component c,
+  private File showSaveDialog(Component c,
                                      JFileChooser chooser, File file) {
     while (true) {
       if (chooser.showSaveDialog(c) != JFileChooser.APPROVE_OPTION)
@@ -397,6 +400,7 @@ public class Dialog extends JPanel implements JmolDialogInterface {
       thisType = type.toLowerCase();
     }
 
+    @Override
     public boolean accept(File f) {
       if (f.isDirectory() || thisType == null) {
         return true;
@@ -406,6 +410,7 @@ public class Dialog extends JPanel implements JmolDialogInterface {
       return (pt >= 0 && ext.substring(pt + 1).toLowerCase().equals(thisType));
     }
 
+    @Override
     public String getDescription() {
       return thisType.toUpperCase() + " (*." + thisType + ")";
     }
@@ -506,4 +511,54 @@ public class Dialog extends JPanel implements JmolDialogInterface {
       fc.updateUI();
     }
   }
+
+  protected String[] imageChoices = { "JPEG", "PNG", "GIF", "PPM" };
+  protected String[] imageExtensions = { "jpg", "png", "gif", "ppm" };
+  protected String outputFileName;
+  protected String dialogType;
+  protected String inputFileName;
+  protected Viewer viewer;
+  protected int qualityJ = -1;
+  protected int qualityP = -1;
+  protected String imageType;
+  
+  public void setImageInfo(int qualityJPG, int qualityPNG, String imageType) {
+    qualityJ = qualityJPG;
+    qualityP = qualityPNG;
+    this.imageType = imageType;
+  }
+  
+  public String getFileNameFromDialog(Viewer v, String dType, String iFileName) {
+    this.viewer = v;
+    this.dialogType = dType;
+    this.inputFileName = iFileName;
+    outputFileName = null;
+    try {
+      SwingUtilities.invokeAndWait(new Runnable() {
+        public void run() {
+          if (dialogType.equals("load")) {
+            outputFileName = getOpenFileNameFromDialog(
+                viewer.getAppletContext(), viewer, inputFileName, null, null, false);
+            return;
+          }
+          if (dialogType.equals("save")) {
+            outputFileName = getSaveFileNameFromDialog(viewer,
+                inputFileName, null);
+            return;
+          }
+          if (dialogType.startsWith("saveImage")) {
+            outputFileName = getImageFileNameFromDialog(viewer,
+                inputFileName, imageType, imageChoices, imageExtensions,
+                qualityJ, qualityP);
+            return;
+          }
+          outputFileName = null;
+        }
+      });
+    } catch (Exception e) {
+      Logger.error(e.getMessage());
+    }
+    return outputFileName;
+  }
+
 }
