@@ -43,14 +43,17 @@
  */
 package org.jmol.export.image;
 
-import java.io.*;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.Enumeration;
 import java.util.Hashtable;
-import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.Map;
 import java.awt.Image;
-import java.awt.image.*;
+import java.awt.image.ImageProducer;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
 
 import org.jmol.util.Logger;
 
@@ -130,16 +133,14 @@ public class GifEncoder extends ImageEncoder {
     }
   }
 
-  static class ColorVector extends ArrayList {
+  static class ColorVector extends ArrayList<ColorItem> {
     void sort() {
       CountComparator comparator = new CountComparator();
       Collections.sort(this, comparator);
     }
     
-    static class CountComparator implements Comparator {
-      public int compare(Object arg0, Object arg1) {
-        ColorItem a = (ColorItem)arg0;
-        ColorItem b = (ColorItem)arg1;
+    static class CountComparator implements Comparator<ColorItem> {
+      public int compare(ColorItem a, ColorItem b) {
         return (a == null ? 1 : b == null ? -1 : a.count < b.count ? -1 : a.count > b.count ? 1 : 0);
       }    
     }
@@ -173,12 +174,14 @@ public class GifEncoder extends ImageEncoder {
   int width, height;
   int[][] rgbPixels;
 
+  @Override
   void encodeStart(int width, int height) throws IOException {
     this.width = width;
     this.height = height;
     rgbPixels = new int[height][width];
   }
 
+  @Override
   void encodePixels(int x, int y, int w, int h, int[] rgbPixels, int off,
                     int scansize) throws IOException {
     // Save the pixels.
@@ -187,14 +190,16 @@ public class GifEncoder extends ImageEncoder {
           this.rgbPixels[y + row], x, w);
   }
 
-  Hashtable colorHash;
-  ColorVector colorVector;
+  Map<String, AdaptiveColorCollection>colorHash;
 
+  @Override
   void encodeDone() throws IOException {
+    Map<String, ColorItem> ciHash;
+    ColorVector colorVector;
     int transparentIndex = -1;
     int transparentRgb = -1;
     // Put all the pixels into a hash table.
-    colorHash = new Hashtable();
+    ciHash = new Hashtable<String, ColorItem>();
     colorVector = new ColorVector();
     int index = 0;
     String srgb;
@@ -214,8 +219,7 @@ public class GifEncoder extends ImageEncoder {
             rgbPixels[row][col] = rgb = transparentRgb;
           }
         }
-        ColorItem item = (ColorItem) colorHash
-            .get(srgb = getKey(rgb));
+        ColorItem item = ciHash.get(srgb = getKey(rgb));
         if (item == null) {
           if (index < 0)
             throw new IOException("too many colors for a GIF");
@@ -223,19 +227,17 @@ public class GifEncoder extends ImageEncoder {
           //index = 255;
           item = new ColorItem(rgb, 1, index, isTransparent);
           ++index;
-          colorHash.put(srgb, item);
+          ciHash.put(srgb, item);
           colorVector.add(item);
         } else
           ++item.count;
       }
     }
-    
+    ciHash = null;    
     colorVector.sort();
     
     int mask = 0x010101;
-    colorHash = null;
-    int nTotal = index;
-  
+    int nTotal = index;  
     int nMax = Math.max(index - 1, 0); // leave top 1 untouched
 
 //    for (int i = 0; i < index; i++) {
@@ -246,14 +248,14 @@ public class GifEncoder extends ImageEncoder {
     Logger.debug("# colors = " + nTotal);
     while (true) {
       nTotal = index;
-      colorHash = new Hashtable();
+      colorHash = new Hashtable<String, AdaptiveColorCollection>();
       AdaptiveColorCollection acc;
       for (int i = 0; i < nMax; i++) {
-        ColorItem item = (ColorItem) colorVector.get(i);
+        ColorItem item = colorVector.get(i);
         int rgb = (nTotal <= 256 ? item.rgb : item.rgb & ~mask);
         item.rgb2 = rgb;
         srgb = getKey(rgb);
-        if ((acc = (AdaptiveColorCollection) colorHash.get(srgb)) == null) {
+        if ((acc = colorHash.get(srgb)) == null) {
           colorHash.put(srgb, acc = new AdaptiveColorCollection(rgb));
         } else {
           nTotal--;
@@ -280,9 +282,9 @@ public class GifEncoder extends ImageEncoder {
     byte[] reds = new byte[mapSize];
     byte[] grns = new byte[mapSize];
     byte[] blus = new byte[mapSize];
-    Hashtable ht = new Hashtable();
+    Map<String, AdaptiveColorCollection> ht = new Hashtable<String, AdaptiveColorCollection>();
     for (int i = 0; i < index; i++) {
-      ColorItem item = (ColorItem) colorVector.get(i);
+      ColorItem item = colorVector.get(i);
       int rgb = item.rgb;
       int count = item.count;
       srgb = getKey(rgb);
@@ -292,8 +294,8 @@ public class GifEncoder extends ImageEncoder {
       ht.put(srgb, item.acc);
     }
     int iindex = 0;
-    for (Enumeration e = colorHash.elements(); e.hasMoreElements();) {
-      AdaptiveColorCollection acc = (AdaptiveColorCollection) e.nextElement();
+    for (Iterator<AdaptiveColorCollection> e = colorHash.values().iterator(); e.hasNext();) {
+      AdaptiveColorCollection acc = e.next();
       acc.index = iindex++;
       acc.setRgb(reds, grns, blus);
     }
@@ -312,7 +314,7 @@ public class GifEncoder extends ImageEncoder {
     int rgb = rgbPixels[y][x];
     int iindex;
     try{
-      iindex = ((AdaptiveColorCollection) colorHash.get(getKey(rgb))).index;
+      iindex = (colorHash.get(getKey(rgb))).index;
     } catch (Exception e) {
       iindex = 0;
     }
