@@ -45,6 +45,8 @@ import java.io.OutputStream;
  * comments, and bug fixes at:
  *
  *   david@catcode.com
+ *   
+ * Modified by Bob Hanson hansonr@stolaf.edu to include transparent background option
  *
  * @author J. David Eisenberg
  * @author http://catcode.com/pngencoder/
@@ -83,11 +85,29 @@ public class PngEncoder extends Object {
     os.write(getBytes(image, quality));
   }
 
+  public static void write(Image image, int quality, OutputStream os,
+                           int bgcolor) throws IOException {
+    os.write(getBytes(image, quality, bgcolor));
+  }
+
   public static byte[] getBytes(Image image, int quality) {
     return (new PngEncoder(image, false, PngEncoder.FILTER_NONE,
         quality)).pngEncode();
   }
 
+  public static byte[] getBytes(Image image, int quality, int bgcolor) {
+    PngEncoder pg = new PngEncoder(image, false, PngEncoder.FILTER_NONE,
+        quality);
+    pg.setTransparentColor(bgcolor);
+    return pg.pngEncode();
+  }
+
+  private Integer transparentColor;
+
+  private void setTransparentColor(int bgcolor) {
+    transparentColor = Integer.valueOf(bgcolor);
+  }
+
   /**
    * Class constructor
    *
@@ -195,6 +215,8 @@ public class PngEncoder extends Object {
     bytePos = writeBytes(pngIdBytes, 0);
     hdrPos = bytePos;
     writeHeader();
+    if (!encodeAlpha && transparentColor != null) 
+      writeTransparentColor(transparentColor.intValue());
     dataPos = bytePos;
     if (writeImageData()) {
       writeEnd();
@@ -399,16 +421,14 @@ public class PngEncoder extends Object {
    */
   protected void writeHeader() {
 
-    int startPos;
-
-    startPos = bytePos = writeInt4(13, bytePos);
+    int startPos = bytePos = writeInt4(13, bytePos);
     bytePos = writeString("IHDR", bytePos);
     width = image.getWidth(null);
     height = image.getHeight(null);
     bytePos = writeInt4(width, bytePos);
     bytePos = writeInt4(height, bytePos);
     bytePos = writeByte(8, bytePos);      // bit depth
-    bytePos = writeByte((encodeAlpha)
+    bytePos = writeByte((encodeAlpha)     // color type
                         ? 6
                         : 2, bytePos);    // direct model
     bytePos = writeByte(0, bytePos);      // compression method
@@ -420,6 +440,24 @@ public class PngEncoder extends Object {
     bytePos = writeInt4((int) crcValue, bytePos);
   }
 
+  /**
+   * Write a PNG "tRNS" chunk into the pngBytes array.
+   * @param icolor 
+   */
+  private void writeTransparentColor(int icolor) {
+    
+    crc.reset();
+    int startPos = bytePos = writeInt4(6, bytePos);
+    bytePos = writeString("tRNS", bytePos);
+    bytePos = writeInt2((icolor >> 16) & 0xFF, bytePos);
+    bytePos = writeInt2((icolor >> 8) & 0xFF, bytePos);
+    bytePos = writeInt2(icolor & 0xFF, bytePos);
+    crc.update(pngBytes, startPos, bytePos - startPos);
+    crcValue = crc.getValue();
+    bytePos = writeInt4((int) crcValue, bytePos);
+  }
+
+
   /**
    * Perform "sub" filtering on the given row.
    * Uses temporary array leftBytes to store the original values
@@ -508,9 +546,11 @@ public class PngEncoder extends Object {
                                        scrunch);
     try {
       while (rowsLeft > 0) {
-        nRows = Math.min(32767 / (width * (bytesPerPixel + 1)), rowsLeft);
-
-        // nRows = rowsLeft;
+        //         nRows = Math.min(32767 / (width * (bytesPerPixel + 1)), rowsLeft);
+        // What if bytesPerPixel == 3 and width > 8191 ?
+        // then nRows = 0 and we have an infinite loop
+        //
+        nRows = Math.min(32767 / (width * (bytesPerPixel)) + 1, rowsLeft);
 
         int[] pixels = new int[width * nRows];
 
@@ -586,7 +626,6 @@ public class PngEncoder extends Object {
       crc.update("IDAT".getBytes());
       bytePos = writeBytes(compressedLines, nCompressed, bytePos);
       crc.update(compressedLines, 0, nCompressed);
-
       crcValue = crc.getValue();
       bytePos = writeInt4((int) crcValue, bytePos);
       scrunch.finish();
