@@ -1,7 +1,7 @@
 /* $RCSfile$
  *  * $Author: hansonr $
- * $Date: 2010-08-16 19:01:26 +0200 (lun., 16 août 2010) $
- * $Revision: 14005 $
+ * $Date: 2011-12-06 15:14:53 +0100 (mar., 06 déc. 2011) $
+ * $Revision: 16561 $
  *
  * Copyright (C) 2003-2006  Miguel, Jmol Development, www.jmol.org
  *
@@ -23,10 +23,9 @@
  */
 package org.jmol.g3d;
 
-import java.awt.Component;
-import java.awt.Image;
 import java.util.BitSet;
 import java.util.Hashtable;
+import java.util.Map;
 
 import javax.vecmath.Matrix4f;
 import javax.vecmath.Point3i;
@@ -35,7 +34,9 @@ import javax.vecmath.Tuple3f;
 import javax.vecmath.Vector3f;
 import javax.vecmath.Matrix3f;
 
+import org.jmol.api.ApiPlatform;
 import org.jmol.api.JmolRendererInterface;
+import org.jmol.constant.EnumPalette;
 import org.jmol.modelset.Atom;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
@@ -185,6 +186,10 @@ final public class Graphics3D implements JmolRendererInterface {
   
   boolean twoPass = false;
   boolean isPass2;
+  public boolean isPass2() {
+    return isPass2;
+  }
+  
   boolean addAllPixels;
   boolean haveTranslucentObjects;
   boolean translucentCoverOnly = false;
@@ -214,11 +219,13 @@ final public class Graphics3D implements JmolRendererInterface {
   short colixCurrent;
   int[] shadesCurrent;
   int argbCurrent;
+  //boolean isTranslucent;
   boolean isScreened;
   int translucencyMask;
   int argbNoisyUp, argbNoisyDn;
 
   Font3D font3dCurrent;
+  ApiPlatform apiPlatform;
 
   public final static byte ENDCAPS_NONE = 0;
   public final static byte ENDCAPS_OPEN = 1;
@@ -226,8 +233,6 @@ final public class Graphics3D implements JmolRendererInterface {
   public final static byte ENDCAPS_SPHERICAL = 3;
   public final static byte ENDCAPS_OPENEND = 4;
   
-  public final static short INHERIT_ALL = 0;
-  public final static short USE_PALETTE = 2;
   public final static short BLACK = 4;
   public final static short ORANGE = 5;
   public final static short PINK = 6;
@@ -252,11 +257,14 @@ final public class Graphics3D implements JmolRendererInterface {
 
   /**
    * Allocates a g3d object
+   * @param apiPlatform 
+   * 
+   * @param isDataOnly 
    *
-   * @param awtComponent the java.awt.Component where the image will be drawn
    */
-  public Graphics3D(Component awtComponent) {
-    platform = Platform3D.createInstance(awtComponent);
+  public Graphics3D(ApiPlatform apiPlatform, boolean isDataOnly) {
+    this.apiPlatform = apiPlatform;
+    platform = (isDataOnly ? null : Platform3D.createInstance(apiPlatform));
     this.line3d = new Line3D(this);
     this.circle3d = new Circle3D(this);
     this.sphere3d = new Sphere3D(this);
@@ -428,7 +436,7 @@ final public class Graphics3D implements JmolRendererInterface {
     }
   }
   
-  public Image getScreenImage() {
+  public Object getScreenImage() {
     return platform.imagePixelBuffer;
   }
 
@@ -476,7 +484,7 @@ final public class Graphics3D implements JmolRendererInterface {
     return depth;
   }
 
-  public Image backgroundImage;
+  public Object backgroundImage;
   
   public void setBackgroundTransparent(boolean TF) {
     if (platform != null)
@@ -496,7 +504,7 @@ final public class Graphics3D implements JmolRendererInterface {
     // when background [0,0,1]. 
   }
 
-  public void setBackgroundImage(Image image) {
+  public void setBackgroundImage(Object image) {
     backgroundImage = image;
   }
 
@@ -554,6 +562,10 @@ final public class Graphics3D implements JmolRendererInterface {
     }
   }
 
+  public void setTempZSlab(int zSlab) {
+    this.zSlab = zSlab;
+  }
+  
   Pixel pixel;
   int zShadeR;
   int zShadeG;
@@ -706,14 +718,25 @@ final public class Graphics3D implements JmolRendererInterface {
   }
 
   private int currentShadeIndex;
+  private int lastRawColor;
   
+  public void setColor(int argb) {
+    argbCurrent = argbNoisyUp = argbNoisyDn = argb;
+  }
+  
+  public static boolean isColixLastAvailable(short colix) {
+    return (colix > 0 && (colix & UNMASK_CHANGEABLE_TRANSLUCENT) == UNMASK_CHANGEABLE_TRANSLUCENT);
+  }
+
+
   /**
    * sets current color from colix color index
    * @param colix the color index
    * @return true or false if this is the right pass
    */
   public boolean setColix(short colix) {
-    if (colix == colixCurrent && currentShadeIndex == -1)
+    boolean isLast = isColixLastAvailable(colix); 
+    if (!isLast && colix == colixCurrent && currentShadeIndex == -1)
       return true;
     int mask = colix & TRANSLUCENT_MASK;
     if (mask == TRANSPARENT)
@@ -726,13 +749,19 @@ final public class Graphics3D implements JmolRendererInterface {
     if (isPass2) {
       translucencyMask = (mask << ALPHA_SHIFT) | 0xFFFFFF;
     }
-//    if (isPass2) {
-  //    System.out.println("G3d mask " + Integer.toHexString(mask)+ " " +Integer.toHexString(TRANSPARENT)+ " " +Integer.toHexString(TRANSLUCENT_SCREENED));
-    //}
     colixCurrent = colix;
+    if (isLast) {
+      if (argbCurrent != lastRawColor) {
+        if (argbCurrent == 0)
+          argbCurrent = 0xFFFFFFFF;
+        lastRawColor = argbCurrent;
+        Colix3D.allocateColix(argbCurrent);
+        Colix3D.getShades(argbCurrent, inGreyscaleMode);
+      }
+    }
     shadesCurrent = getShades(colix);
-    currentShadeIndex = -1; 
-    argbCurrent = argbNoisyUp = argbNoisyDn = getColorArgbOrGray(colix);
+    currentShadeIndex = -1;
+    setColor(getColorArgbOrGray(colix));
     return true;
   }
 
@@ -771,6 +800,7 @@ final public class Graphics3D implements JmolRendererInterface {
   }
   
   class ShadePixel extends Pixel {
+    @Override
     void addPixel(int offset, int z, int p) {
       if (z > zDepth)
         return;
@@ -815,6 +845,25 @@ final public class Graphics3D implements JmolRendererInterface {
     }
   }
 
+  public void volumeRender(int diameter, int x, int y, int z) {
+    if (diameter == 1) {
+      plotPixelClipped(x, y, z);
+      return;
+    }
+    if (isClippedZ(z))
+      return;
+    int r = (diameter + 1) / 2;
+    boolean isClipped = x < r || x + r >= width || y < r || y + r >= height;
+    if (isClipped && isClippedXY(diameter, x, y))
+      return;
+    if (isClipped)
+      circle3d.plotFilledCircleCenteredClipped(x, y, z, diameter);
+    else
+      circle3d.plotFilledCircleCenteredUnclipped(x, y, z, diameter);
+  }
+
+
+  
   /**
    * fills a solid sphere
    *
@@ -837,6 +886,19 @@ final public class Graphics3D implements JmolRendererInterface {
           null, null, -1, null);
   }
 
+  private int saveAmbient, saveDiffuse;
+
+  public void volumeRender(boolean TF) {
+    if (TF) {
+      saveAmbient = Shade3D.ambientPercent;
+      saveDiffuse = Shade3D.diffusePercent;
+      setAmbientPercent(100);
+      setDiffusePercent(0);
+    } else {
+      setAmbientPercent(saveAmbient);
+      setDiffusePercent(saveDiffuse);
+    }
+  }
   /**
    * fills a solid sphere
    *
@@ -1051,16 +1113,16 @@ final public class Graphics3D implements JmolRendererInterface {
         antialiasThisFrame);    
   }
   
-  public void drawImage(Image image, int x, int y, int z, int zSlab, 
+  public void drawImage(Object objImage, int x, int y, int z, int zSlab, 
                         short bgcolix, int width, int height) {
-    if (image == null || width == 0 || height == 0)
+    if (objImage == null || width == 0 || height == 0)
       return;
     if (isClippedZ(zSlab))
       return;
-    plotImage(x, y, z, image, null, bgcolix, width, height);
+    plotImage(x, y, z, objImage, null, bgcolix, width, height);
   }
 
-  public void plotImage(int x, int y, int z, Image image, JmolRendererInterface jmolRenderer,
+  public void plotImage(int x, int y, int z, Object image, JmolRendererInterface jmolRenderer,
                         short bgcolix, int width, int height) {
     setColix(bgcolix);
     if (bgcolix == 0)
@@ -1230,11 +1292,11 @@ final public class Graphics3D implements JmolRendererInterface {
  }
 
   public void fillConeScreen(byte endcap, int screenDiameter,
-                       Point3i screenBase, Point3i screenTip) {
+                       Point3i screenBase, Point3i screenTip, boolean isBarb) {
     // dipoles, mesh, vectors
     cylinder3d.renderCone(colixCurrent, !addAllPixels, endcap, screenDiameter,
                           screenBase.x, screenBase.y, screenBase.z,
-                          screenTip.x, screenTip.y, screenTip.z, false);
+                          screenTip.x, screenTip.y, screenTip.z, false, isBarb);
   }
 
   public void fillConeSceen(byte endcap, int screenDiameter,
@@ -1242,7 +1304,7 @@ final public class Graphics3D implements JmolRendererInterface {
     // cartoons, rockets
     cylinder3d.renderCone(colixCurrent, !addAllPixels, endcap, screenDiameter,
                           screenBase.x, screenBase.y, screenBase.z,
-                          screenTip.x, screenTip.y, screenTip.z, true);
+                          screenTip.x, screenTip.y, screenTip.z, true, false);
   }
 
   public void drawHermite(int tension,
@@ -1342,14 +1404,15 @@ final public class Graphics3D implements JmolRendererInterface {
     triangle3d.fillTriangle(screenA, screenB, screenC, false);
   }
 
-  public void fillTriangle(Point3i screenA, short colixA, short normixA,
-                           Point3i screenB, short colixB, short normixB,
-                           Point3i screenC, short colixC, short normixC,
-                           float factor) {
+  public void fillTriangle(Point3i screenA, short colixA,
+                                   short normixA, Point3i screenB,
+                                   short colixB, short normixB,
+                                   Point3i screenC, short colixC,
+                                   short normixC, float factor) {
     // isosurface test showing triangles
     boolean useGouraud;
-    if (!isPass2 && normixA == normixB && normixA == normixC
-        && colixA == colixB && colixA == colixC) {
+    if (!isPass2 && normixA == normixB && normixA == normixC && colixA == colixB
+        && colixA == colixC) {
       setTriangleColixAndShadeIndex(colixA, normix3d.getShadeIndex(normixA));
       useGouraud = false;
     } else {
@@ -1360,7 +1423,8 @@ final public class Graphics3D implements JmolRendererInterface {
           getShades(colixC)[normix3d.getShadeIndex(normixC)]);
       useGouraud = true;
     }
-    triangle3d.fillTriangle(screenA, screenB, screenC, factor, useGouraud);
+    triangle3d.fillTriangle(screenA, screenB, screenC, factor,
+        useGouraud);
   }
 
   public void fillTriangle(Point3i screenA, short colixA, short normixA,
@@ -1368,16 +1432,16 @@ final public class Graphics3D implements JmolRendererInterface {
                            Point3i screenC, short colixC, short normixC) {
     // mesh, isosurface
     boolean useGouraud;
-    if (!isPass2 && normixA == normixB && normixA == normixC
-        && colixA == colixB && colixA == colixC) {
+    if (!isPass2 && normixA == normixB && normixA == normixC &&
+        colixA == colixB && colixA == colixC) {
       setTriangleColixAndShadeIndex(colixA, normix3d.getShadeIndex(normixA));
       useGouraud = false;
     } else {
       if (!setTriangleTranslucency(colixA, colixB, colixC))
         return;
       triangle3d.setGouraud(getShades(colixA)[normix3d.getShadeIndex(normixA)],
-          getShades(colixB)[normix3d.getShadeIndex(normixB)],
-          getShades(colixC)[normix3d.getShadeIndex(normixC)]);
+                            getShades(colixB)[normix3d.getShadeIndex(normixB)],
+                            getShades(colixC)[normix3d.getShadeIndex(normixC)]);
       useGouraud = true;
     }
     triangle3d.fillTriangle(screenA, screenB, screenC, useGouraud);
@@ -1391,8 +1455,7 @@ final public class Graphics3D implements JmolRendererInterface {
     setColorNoisy(shadeIndex);
   }
 
-  private boolean setTriangleTranslucency(short colixA, short colixB,
-                                          short colixC) {
+  private boolean setTriangleTranslucency(short colixA, short colixB, short colixC) {
     if (!isPass2)
       return true;
     int maskA = colixA & TRANSLUCENT_MASK;
@@ -1402,9 +1465,9 @@ final public class Graphics3D implements JmolRendererInterface {
     maskB &= ~TRANSPARENT;
     maskC &= ~TRANSPARENT;
     //if (maskA == 0 && maskB == 0 && maskC == 0)
-    //return false;
+      //return false;
     int mask = ((maskA + maskB + maskC) / 3) & TRANSLUCENT_MASK;
-    // System.out.println(mask >> TRANSLUCENT_SHIFT);
+   // System.out.println(mask >> TRANSLUCENT_SHIFT);
     translucencyMask = (mask << ALPHA_SHIFT) | 0xFFFFFF;
     return true;
   }
@@ -1444,7 +1507,7 @@ final public class Graphics3D implements JmolRendererInterface {
                  screenD, colixD, normixD);
   }
 
-  public void drawSurface(MeshSurface meshSurface, Point3f[] vertices, Point3f offset) {
+  public void drawSurface(MeshSurface meshSurface, short colix) {
     // Export3D only
   }
   
@@ -1915,23 +1978,29 @@ final public class Graphics3D implements JmolRendererInterface {
        ....
      0x0017  ...gold
      0x00?? additional colors used from JavaScript list or specified by user
+     
+     0x0177 last available colix
 
      Bob Hanson 3/2007
      
   */
-  private final static short CHANGEABLE_MASK       = (short)0x8000; // negative
+  
   private final static short UNMASK_CHANGEABLE_TRANSLUCENT =0x07FF;
-  private final static int   TRANSLUCENT_SHIFT        = 11;
+  private final static short CHANGEABLE_MASK          = (short)0x8000; // negative
+  public final static int    LAST_AVAILABLE_COLIX     = UNMASK_CHANGEABLE_TRANSLUCENT;
+  private final static int   TRANSLUCENT_SHIFT        = 11; 
   private final static int   ALPHA_SHIFT              = 24 - TRANSLUCENT_SHIFT;
   private final static int   TRANSLUCENT_MASK         = 0xF << TRANSLUCENT_SHIFT; //0x7800
-  private final static int   TRANSLUCENT_SCREENED     = TRANSLUCENT_MASK;
+  private final static int   TRANSLUCENT_SCREENED     = TRANSLUCENT_MASK;  
   private final static int   TRANSPARENT              =  8 << TRANSLUCENT_SHIFT;  //0x4000
   final static int           TRANSLUCENT_50           =  4 << TRANSLUCENT_SHIFT;  //0x2000
   public final static short  OPAQUE_MASK              = ~TRANSLUCENT_MASK;
 
 
-  private final static short INHERIT_COLOR       = 1;
-  final static short         UNUSED_OPTION3      = 3;
+  public final static short  INHERIT_ALL         = 0;
+  public final static short INHERIT_COLOR       = 1;
+  public final static short  USE_PALETTE         = 2;
+  final static short         RAW_RGB             = 3;
   final static short         SPECIAL_COLIX_MAX   = 4;
 
   /**
@@ -2083,8 +2152,7 @@ final public class Graphics3D implements JmolRendererInterface {
         return 0;
       }
     }
-    Integer boxedArgb = (Integer) mapJavaScriptColors.get(strColor
-        .toLowerCase());
+    Integer boxedArgb = mapJavaScriptColors.get(strColor.toLowerCase());
     return (boxedArgb == null ? 0 : boxedArgb.intValue());
   }
   
@@ -2100,7 +2168,7 @@ final public class Graphics3D implements JmolRendererInterface {
     if (translucentLevel == 0) //opaque
       return (short) (colix & ~TRANSLUCENT_MASK);
     if (translucentLevel < 0) //screened
-      return (short) (colix | TRANSLUCENT_MASK);
+      return (short) (colix & ~TRANSLUCENT_MASK | TRANSLUCENT_SCREENED);
     if (Float.isNaN(translucentLevel) || translucentLevel >= 255 || translucentLevel == 1.0)
       return (short) ((colix & ~TRANSLUCENT_MASK) | TRANSPARENT);
     int iLevel = (int) (translucentLevel < 1 ? translucentLevel * 256
@@ -2111,17 +2179,17 @@ final public class Graphics3D implements JmolRendererInterface {
   }
 
   public final static int getColixTranslucencyLevel(short colix) {
-    int logAlpha = (colix >> TRANSLUCENT_SHIFT) & 15;
+    int logAlpha = (colix >> TRANSLUCENT_SHIFT) & 0xF;
     switch (logAlpha) {
     case 0:
       return 0;
-    case 1:
-    case 2:
-    case 3:
-    case 4:
-    case 5:
-    case 6:
-    case 7:
+    case 1: //  32
+    case 2: //  64
+    case 3: //  96
+    case 4: // 128
+    case 5: // 160
+    case 6: // 192
+    case 7: // 224
       return logAlpha << 5;
     case 15:
       return -1;
@@ -2130,7 +2198,7 @@ final public class Graphics3D implements JmolRendererInterface {
     }
   }
   
-  public static float translucencyFractionalFromColix(short colix) {
+  public static float getColixTranslucencyFractional(short colix) {
     int translevel = getColixTranslucencyLevel(colix);
     return (
           translevel == -1 ? 0.5f 
@@ -2143,13 +2211,16 @@ final public class Graphics3D implements JmolRendererInterface {
   public static short getColix(Object obj) {
     if (obj == null)
       return INHERIT_ALL;
-    if (obj instanceof Byte)
-      return (((Byte) obj).byteValue() == 0 ? INHERIT_ALL
+    if (obj instanceof EnumPalette)
+      return (((EnumPalette) obj) == EnumPalette.NONE ? INHERIT_ALL
           : USE_PALETTE);
     if (obj instanceof Integer)
       return Colix3D.getColix(((Integer) obj).intValue());
     if (obj instanceof String)
       return getColix((String) obj);
+    if (obj instanceof Byte)
+      return (((Byte) obj).byteValue() == 0 ? INHERIT_ALL
+          : USE_PALETTE);
     if (Logger.debugging) {
       Logger.debug("?? getColix(" + obj + ")");
     }
@@ -2160,31 +2231,27 @@ final public class Graphics3D implements JmolRendererInterface {
     if (colix == INHERIT_ALL)
       colix = INHERIT_COLOR;
     colix &= ~TRANSLUCENT_MASK;
-    if (!isTranslucent)
-      return colix;
-    return applyColorTranslucencyLevel(colix, translucentLevel);
+    return (isTranslucent ? applyColorTranslucencyLevel(colix, translucentLevel) : colix);
   }
 
+  public final static short copyColixTranslucency(short colixFrom, short colixTo) {
+    return getColixTranslucent(colixTo, isColixTranslucent(colixFrom), getColixTranslucencyLevel(colixFrom));  
+  }
+  
   public int getColorArgbOrGray(short colix) {
     if (colix < 0)
       colix = changeableColixMap[colix & UNMASK_CHANGEABLE_TRANSLUCENT];
-    if (! inGreyscaleMode)
-      return Colix3D.getArgb(colix);
-    return Colix3D.getArgbGreyscale(colix);
+    return (inGreyscaleMode ? Colix3D.getArgbGreyscale(colix) : Colix3D.getArgb(colix));
   }
 
   int[] getShades(short colix) {
     if (colix < 0)
       colix = changeableColixMap[colix & UNMASK_CHANGEABLE_TRANSLUCENT];
-    if (! inGreyscaleMode)
-      return Colix3D.getShades(colix);
-    return Colix3D.getShadesGreyscale(colix);
+    return (inGreyscaleMode ? Colix3D.getShadesGreyscale(colix) : Colix3D.getShades(colix));
   }
 
   public final static short getChangeableColixIndex(short colix) {
-    if (colix >= 0)
-      return -1;
-    return (short)(colix & UNMASK_CHANGEABLE_TRANSLUCENT);
+    return (colix >= 0 ? -1 : (short)(colix & UNMASK_CHANGEABLE_TRANSLUCENT));
   }
 
   public final static boolean isColixTranslucent(short colix) {
@@ -2461,12 +2528,20 @@ final public class Graphics3D implements JmolRendererInterface {
   }
     
   // {"Plain", "Bold", "Italic", "BoldItalic"};
+  public static int getFontStyleID(String fontStyle) {
+    return Font3D.getFontStyleID(fontStyle);
+  }
+  
   public Font3D getFont3D(String fontFace, String fontStyle, float fontSize) {
+    int iStyle = Font3D.getFontStyleID(fontStyle);
+    if (iStyle < 0)
+      iStyle = 0;
     return Font3D.getFont3D(Font3D.getFontFaceID(fontFace),
-                            Font3D.getFontStyleID(fontStyle), fontSize, fontSize, platform);
+                            iStyle, fontSize, fontSize, platform);
   }
 
   public Font3D getFont3DScaled(Font3D font, float scale) {
+    // TODO: problem here is that we are assigning a bold font, then not DEassigning it
     float newScale = font.fontSizeNominal * scale;
     return (newScale == font.fontSize ? font : Font3D.getFont3D(
         font.idFontFace,
@@ -2656,6 +2731,7 @@ final public class Graphics3D implements JmolRendererInterface {
     "yellowtint",           // F6F675
   };
 
+//#FFFFC3 hover
   private final static int[] colorArgbs = {
     0xFF000000, // black
     // plus the PE chain colors
@@ -2824,10 +2900,10 @@ final public class Graphics3D implements JmolRendererInterface {
     0xFFF6F675, // yellowtint
   };
 
-  private static final Hashtable mapJavaScriptColors = new Hashtable();
+  private static final Map<String, Integer> mapJavaScriptColors = new Hashtable<String, Integer>();
   static {
     for (int i = colorNames.length; --i >= 0; )
-      mapJavaScriptColors.put(colorNames[i], new Integer(colorArgbs[i]));
+      mapJavaScriptColors.put(colorNames[i], Integer.valueOf(colorArgbs[i]));
   }
 
   /* ***************************************************************
@@ -2900,7 +2976,7 @@ final public class Graphics3D implements JmolRendererInterface {
     return false;
   }
 
-  public boolean initializeExporter(String type, Viewer viewer, Graphics3D g3d,
+  public boolean initializeExporter(String type, Viewer viewer, double privateKey, Graphics3D g3d,
                                     Object output) {
     return false;
   }
@@ -2918,4 +2994,10 @@ final public class Graphics3D implements JmolRendererInterface {
     return false;
   }
 
+  public double getPrivateKey() {
+    // exporter only
+    return 0;
+  }
+
+
 }
