@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Date: 2010-08-18 05:56:43 +0200 (mer., 18 août 2010) $
+ * $Revision: 14027 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -24,12 +24,14 @@
 
 package org.jmol.g3d;
 
+
 import javax.vecmath.Point3i;
 import javax.vecmath.Tuple3f;
 import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
-import java.util.List;
+
 import java.util.ArrayList;
+import java.util.List;
 
 import org.jmol.api.JmolRendererInterface;
 
@@ -304,6 +306,19 @@ public class Hermite3D {
     }
   }
  
+  /**
+   * 
+   * @param fill   NOT USED
+   * @param tension
+   * @param p0
+   * @param p1
+   * @param p2
+   * @param p3
+   * @param p4
+   * @param p5
+   * @param p6
+   * @param p7
+   */
   private void renderParallelPair(boolean fill, int tension,
                 //top strand segment
                 Point3i p0, Point3i p1, Point3i p2, Point3i p3,
@@ -313,7 +328,7 @@ public class Hermite3D {
     // only used for meshRibbon, so fill = false 
     Point3i[] endPoints = {p2, p1, p6, p5};
     // stores all points for top+bottom strands of 1 segment
-    List points = new ArrayList(10);
+    List<Point3i> points = new ArrayList<Point3i>(10);
     int whichPoint = 0;
 
     int numTopStrandPoints = 2; //first and last points automatically included
@@ -418,10 +433,10 @@ public class Hermite3D {
        Point3i b2 = null;
        int top = 1;
        for (;top < numTopStrandPoints && (top + numTopStrandPoints) < size; top++) {
-         t1 = (Point3i) points.get(top - 1);
-         b1 = (Point3i) points.get(numTopStrandPoints + (top - 1));
-         t2 = (Point3i) points.get(top);
-         b2 = (Point3i) points.get(numTopStrandPoints + top);
+         t1 = (Point3i) points.elementAt(top - 1);
+         b1 = (Point3i) points.elementAt(numTopStrandPoints + (top - 1));
+         t2 = (Point3i) points.elementAt(top);
+         b2 = (Point3i) points.elementAt(numTopStrandPoints + top);
 
          g3d.fillTriangle(t1, b1, t2);
          g3d.fillTriangle(b2, t2, b1);
@@ -437,8 +452,8 @@ public class Hermite3D {
      */
        for (int top = 0;
             top < numTopStrandPoints && (top + numTopStrandPoints) < size; top++)
-         g3d.drawLine((Point3i) points.get(top),
-             (Point3i) points.get(top + numTopStrandPoints));
+         g3d.drawLine(points.get(top),
+             points.get(top + numTopStrandPoints));
      //}
 
   }
