@@ -1,7 +1,7 @@
 /* $RCSfile$
- * $Author: hansonr $
- * $Date: 2009-12-25 15:18:12 +0100 (ven., 25 déc. 2009) $
- * $Revision: 12004 $
+ * $Author: nicove $
+ * $Date: 2010-08-07 01:30:20 +0200 (sam., 07 août 2010) $
+ * $Revision: 13880 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -44,6 +44,7 @@ package org.jmol.g3d;
 
 import java.util.BitSet;
 import java.util.Hashtable;
+import java.util.Map;
 
 import org.jmol.util.Logger;
 
@@ -93,13 +94,13 @@ final class Line3D {
     //if (--test > 0 || ((100-test) % 100 == 0)) System.out.println(test+" "+dx + " " + dy + " " + lineBits);
   }
   
-  Hashtable lineCache = new Hashtable();
+  Map<Float, BitSet> lineCache = new Hashtable<Float, BitSet>();
   Float slopeKey;
   boolean getCachedLine() {
     slopeKey = new Float(slope);
     if (!lineCache.containsKey(slopeKey))
       return false;
-    lineBits = (BitSet) lineCache.get(slopeKey);
+    lineBits = lineCache.get(slopeKey);
     nFound++;
     if (nFound == 1000000)
       if (Logger.debugging) {
