@@ -33,9 +33,13 @@ import org.jmol.api.AtomIndexIterator;
 import org.jmol.atomdata.AtomData;
 import org.jmol.atomdata.AtomDataServer;
 import org.jmol.atomdata.RadiusData;
+import org.jmol.g3d.Graphics3D;
 
 import java.util.BitSet;
+
+import javax.vecmath.Matrix3f;
 import javax.vecmath.Point3f;
+import javax.vecmath.Vector3f;
 
 /* ***************************************************************
  * 
@@ -170,7 +174,7 @@ public final class EnvelopeCalculation {
    
   public final static float SURFACE_DISTANCE_FOR_CALCULATION = 3f;
 
-  public final static int MAX_LEVEL = 3;//Geodesic.standardLevel;
+  public final static int MAX_LEVEL = Geodesic.standardLevel;
   
   private float maxRadius = 0;
   private boolean modelZeroBased;
@@ -232,10 +236,43 @@ public final class EnvelopeCalculation {
     mads = null;
   }
 
-  public void reCalculate(BitSet bs) {
-    calculate(null, maxRadius, bs, bsIgnore, disregardNeighbors, onlySelectedDots, isSurface, multiModel); 
+  /**
+   * problem prior to 12.3.18 was that dots once on the deodesic were not being moved.
+   * this isn't perfect, but it's reasonably good. Mostly, you should recreate dots
+   * after rotateSelected. This isn't a problem until after a state is saved and
+   * reloaded, since only then with atomData.radiusData be null.
+   * 
+   * @param bs
+   * @param m
+   */
+  public void reCalculate(BitSet bs, Matrix3f m) {
+    if (atomData.radiusData != null) {    
+      calculate(null, maxRadius, bs, bsIgnore, disregardNeighbors,
+          onlySelectedDots, isSurface, multiModel);
+      return;
+    }
+    if (dotsConvexMaps == null || dotsConvexMax == 0)
+      return;
+    Vector3f pt = new Vector3f();
+    BitSet bsTemp = new BitSet();
+    for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
+      if (i >= dotsConvexMax)
+        return;
+      FastBitSet map = dotsConvexMaps[i];
+      if (map == null || map.isEmpty())
+        continue;
+      FastBitSet bsNew = new FastBitSet();
+      for (int j = map.nextSetBit(0); j >= 0; j = map.nextSetBit(j + 1)) {
+        pt.set(Geodesic.getVertexVector(j));
+        m.transform(pt);
+        bsNew.set(Graphics3D.getNormix(pt, bsTemp));
+      }         
+      dotsConvexMaps[i] = bsNew;
+    }
+    
   }  
 
+
   
   private BitSet bsIgnore;
   private boolean onlySelectedDots;
