@@ -21,7 +21,7 @@ public class Visua extends Applet implements Runnable {
    ZoneVisu My_ZoneVisu;
    Thread My_Thread;
    private GraphCharacteristics _graphDataUtils; // All data pertaining to the graphs' features
-   public List texte;     // variable qui sont utilisees par des sous classes
+   public Vector texte;     // variable qui sont utilisees par des sous classes
    public int nbLignes;
    int shitty_starting_constant=66666; //what is this? figure it out
    public double Firstx;
@@ -245,10 +245,10 @@ public class Visua extends Applet implements Runnable {
       URL url=new URL(getDocumentBase(),tam);
       InputStream stream=url.openStream();
       DataInputStream fichier=new DataInputStream(stream);
-      texte=new ArrayList();
+      texte=new Vector();
       String s;
       while ((s=fichier.readLine())!=null) {
-        texte.add(s);
+        texte.addElement(s);
        }
       My_ZoneVisu.Nb_Clickable_Peaks=texte.size();
      } catch (Exception e) { return false; }
@@ -260,7 +260,7 @@ public class Visua extends Applet implements Runnable {
      StringTokenizer mon_token;
      while (cpt_tokens<My_ZoneVisu.Nb_Clickable_Peaks) {
        do {
-       String mysub=(String) texte.get(cpt_tokens);
+       String mysub=(String) texte.elementAt(cpt_tokens);
        mon_token=new StringTokenizer(mysub," ");
        cpt_tokens++;
           } while (cpt_tokens<My_ZoneVisu.Nb_Clickable_Peaks && mon_token.hasMoreTokens()==false);
@@ -284,7 +284,7 @@ public class Visua extends Applet implements Runnable {
     int nbp=Nbpoints;
     if (Datatype.compareTo("XYDATA")==0) {
                               // se place sur la premiere ligne de donnees
-      while (StringDataUtils.jcampSubString(((String) texte.get(indice)),0,8).compareTo("##XYDATA")!=0) indice++;
+      while (StringDataUtils.jcampSubString(((String) texte.elementAt(indice)),0,8).compareTo("##XYDATA")!=0) indice++;
       indice++;
       int indicetableau=0;
       String Un_Nombre;
@@ -293,7 +293,7 @@ public class Visua extends Applet implements Runnable {
       while (nbp>0 && indice<nbLignes) {
           StringTokenizer mon_token;
           do {
-            String mysub=(String) texte.get(indice);
+            String mysub=(String) texte.elementAt(indice);
             indice++;
             mon_token=new StringTokenizer(mysub," ");
             } while (indice<nbLignes && mon_token.hasMoreTokens()==false);
@@ -329,7 +329,7 @@ public class Visua extends Applet implements Runnable {
     }
    if (Datatype.compareTo("PEAK TABLE")==0) {
                               // se place sur la premiere ligne de donnees
-      while (((String) texte.get(indice)).substring(0,6).compareTo("##PEAK")!=0) indice++;
+      while (((String) texte.elementAt(indice)).substring(0,6).compareTo("##PEAK")!=0) indice++;
       indice++;
       int indicetableau=0;
       String Un_Nombre;
@@ -338,7 +338,7 @@ public class Visua extends Applet implements Runnable {
       while (nbp>0 && indice<nbLignes) {
           StringTokenizer mon_token;
           do {
-            String mysub=(String) texte.get(indice);
+            String mysub=(String) texte.elementAt(indice);
             indice++;
             mon_token=new StringTokenizer(mysub," ");
             } while (indice<nbLignes && mon_token.hasMoreTokens()==false);
@@ -503,10 +503,10 @@ public class Visua extends Applet implements Runnable {
       InputStream stream = url.openStream();
 //      DataInputStream fichier = new DataInputStream(stream);
       BufferedReader fichier = new BufferedReader(new InputStreamReader(stream));
-      texte=new ArrayList();
+      texte=new Vector();
       String s;
       while ((s=fichier.readLine())!=null) {
-        texte.add(s);
+        texte.addElement(s);
        }
       nbLignes=texte.size();
      } catch (Exception e) { return false; }
@@ -516,7 +516,7 @@ public class Visua extends Applet implements Runnable {
      try {
        StringTokenizer mon_token;
        do {
-         uneligne=(String) texte.get(My_Counter);
+         uneligne=(String) texte.elementAt(My_Counter);
          My_Counter++;
          mon_token=new StringTokenizer(uneligne," ");
        } while (My_Counter<nbLignes && mon_token.hasMoreTokens()==false);
@@ -607,7 +607,7 @@ public class Visua extends Applet implements Runnable {
 	   }
 	   pw = new PrintWriter(new FileWriter(filName2));
 	   for (int ii=0; ii<texte.size(); ii++) {
-		newLine = (String)texte.get(ii);
+		newLine = (String)texte.elementAt(ii);
 		pw.println(newLine);
 	   }
 	   pw.close();
