@@ -27,6 +27,7 @@ import java.util.BitSet;
 
 import javax.vecmath.Point3f;
 import javax.vecmath.Point3i;
+import javax.vecmath.Point4f;
 import javax.vecmath.Vector3f;
 
 import org.jmol.jvxl.api.VertexDataServer;
@@ -48,12 +49,13 @@ public class MarchingCubes extends TriangleData {
    * inputs: surfaceReader: interface to other methods
    *         volumeData, containing all information relating to origin, axes, etc.
    *         
+   *         MODE_PLANE volume data will be read progressively, by yz planes, starting from x = 0.
+   *            This is a very efficient way to read the data -- only two active planes, 
+   *            which are swapped.
    *         MODE_CUBE volumeData.voxelData possibly with 3D voxel data
-   *         MODE_BITSET bsVoxels -- optional alternative, JVXL-type BitSet indicating which vertices are inside and which outside
-   *            in order for(x 0 to nX){for(y 0 to nY){for(z 0 to nZ){}}}
-   *         MODE_GETXYZ If BOTH voxelData and bsVoxels are null, 
-   *            then we assume we can get the voxelData on the fly
-   *            using surfaceReader.getValue(x, y, z)  
+   *         MODE_JVXL bsVoxels -- optional alternative, JVXL-type BitSet indicating 
+   *            which vertices are inside and which outside
+   *            in the order for(x 0 to nX){for(y 0 to nY){for(z 0 to nZ){}}}
    * 
    * surfaceReader.getSurfacePointIndex gets the actual Point3f vertex points and
    * returns the fraction information. It is exported here, because the JVXL reader may use this
@@ -84,7 +86,6 @@ public class MarchingCubes extends TriangleData {
   protected int yzCount;
   
   protected boolean colorDensity;
-  protected float fractionOutside;
   protected boolean integrateSquared = true;
   protected BitSet bsVoxels;
   protected BitSet bsExcludedVertices;
@@ -93,6 +94,8 @@ public class MarchingCubes extends TriangleData {
 
   protected StringBuffer edgeData = new StringBuffer();
   
+  private boolean excludePartialCubes = true; // original way
+  
   public BitSet getBsVoxels() {
     return bsVoxels;
   }
@@ -106,6 +109,10 @@ public class MarchingCubes extends TriangleData {
 
     // If just creating a JVXL file, see org.openscience.jmol.jvxl.simplewriter.SimpleMarchingCubes.java
     //
+  
+    // setting this false could upset reading 
+    // older Jmol version files -- will need a flag IN the file for this if we do it
+    excludePartialCubes = true; 
     
     this.surfaceReader = surfaceReader;
     this.bsVoxels = bsVoxels;
@@ -113,8 +120,10 @@ public class MarchingCubes extends TriangleData {
     bsExcludedVertices =  (bsExcluded[0] == null ? bsExcluded[0] = new BitSet() : bsExcluded[0]);
     bsExcludedPlanes =    (bsExcluded[2] == null ? bsExcluded[2] = new BitSet() : bsExcluded[2]);
     bsExcludedTriangles = (bsExcluded[3] == null ? bsExcluded[3] = new BitSet() : bsExcluded[3]);
-    mode = (volumeData.voxelData != null ? MODE_CUBE 
-        : bsVoxels != null ? MODE_BITSET : MODE_GETXYZ);
+    // TODO -- need not be setting up planes for simple value-readers
+    
+    mode = (volumeData.getVoxelData() != null || volumeData.mappingPlane != null ? MODE_CUBE 
+        : bsVoxels != null ? MODE_JVXL : MODE_PLANES);
     setParameters(volumeData, params);
   }
 
@@ -131,21 +140,21 @@ public class MarchingCubes extends TriangleData {
     cubeCountX = volumeData.voxelCounts[0] - 1;
     cubeCountY = (nY = volumeData.voxelCounts[1]) - 1;
     cubeCountZ = (nZ = volumeData.voxelCounts[2]) - 1;
-    yzCount = nY * nZ;
+    yzCount = volumeData.getYzCount();
     if (bsVoxels == null)
       bsVoxels = new BitSet();
     edgeVertexPointers = (isXLowToHigh ? edgeVertexPointersLowToHigh : edgeVertexPointersHighToLow);
     edgeVertexPlanes =  (isXLowToHigh ? edgeVertexPlanesLowToHigh : edgeVertexPlanesHighToLow);
     isoPointIndexPlanes = new int[2][yzCount][3];
-    yzPlanes = (mode == MODE_GETXYZ ? new float[2][yzCount] : null);
+    yzPlanes = (mode == MODE_PLANES ? new float[2][yzCount] : null);
     setLinearOffsets();
     calcVoxelVertexVectors();
   }
 
   protected int mode;
   protected final static int MODE_CUBE = 1;
-  protected final static int MODE_BITSET = 2;
-  protected final static int MODE_GETXYZ = 3;
+  protected final static int MODE_JVXL = 2;
+  protected final static int MODE_PLANES = 3;
 
   protected final float[] vertexValues = new float[8];
 
@@ -188,8 +197,17 @@ public class MarchingCubes extends TriangleData {
     return plane;
   }
 
+  private Point4f mappingPlane;
+  private boolean allInside;
+  private boolean isInside;
+  private Point3i offset;
+  private float[][][] voxelData;
+  
   public String getEdgeData() {
 
+    if (cubeCountX < 0 || cubeCountY < 0 || cubeCountZ < 0)
+      return "";
+    mappingPlane = volumeData.mappingPlane;
 
     // Logger.startTimer();
 
@@ -207,47 +225,90 @@ public class MarchingCubes extends TriangleData {
 
     int x0, x1, xStep, ptStep, pt, ptX;
     if (isXLowToHigh) {
-      x0 = 0;
-      x1 = cubeCountX;
-      xStep = 1;
-      ptStep = yzCount;
-      pt = ptX = (yzCount - 1) - nZ - 1;
+      // used for progressive plane readers
       // we are starting at the top corner, in the next to last
       // cell on the next to last row of the first plane
+      x0 = 0;
+      x1 = cubeCountX + (colorDensity ? 1 : 0);
+      if (colorDensity) {
+        x1 = cubeCountX + 1;
+        ptX = yzCount - 1;
+      } else {
+        x1 = cubeCountX;
+        ptX = (yzCount - 1) - nZ - 1;        
+      }
+      xStep = 1;
+      ptStep = yzCount;
     } else {
-      x0 = cubeCountX - 1;
+      // NOTE: isXLowToHigh FALSE is not compatible with progressive plane reading.
+      //       One should never need to do that anyway, because one can always
+      //       define the cube axes such that they go "from low to high in X"
+      // we are starting at the top corner, in the next to last
+      // cell on the next to last row of the next to last plane(!)
+      if (colorDensity) {
+        x0 = cubeCountX;
+        ptX = (cubeCountX + 1) * yzCount - 1;
+      } else {
+        x0 = cubeCountX - 1;
+        ptX = (cubeCountX * yzCount - 1) - nZ - 1;
+      }
       x1 = -1;
       xStep = -1;
       ptStep = -yzCount;
-      pt = ptX = (cubeCountX * yzCount - 1) - nZ - 1;
-      // we are starting at the top corner, in the next to last
-      // cell on the next to last row of the next to last plane(!)
     }
-    if (cubeCountX < 0 || cubeCountY < 0 || cubeCountZ < 0)
-      return "";
-    int cellIndex0 = cubeCountY * cubeCountZ - 1;
-    int cellIndex = cellIndex0;
+    pt = ptX;
     resetIndexPlane(isoPointIndexPlanes[1]);
-    if (mode == MODE_GETXYZ)
-      surfaceReader.getPlane(x0);
-
+    voxelData = null;
+    int y1 = cubeCountY + (colorDensity ? 1 : 0);
+    int z1 = cubeCountZ + (colorDensity ? 1 : 0);
+    switch (mode) {
+    case MODE_PLANES:
+      getPlane(x0, false);
+      break;
+    case MODE_CUBE:
+      voxelData = volumeData.getVoxelData();
+      break;
+    }
+    allInside = (colorDensity && (cutoff == 0 
+        || mode == MODE_JVXL && bsVoxels.cardinality() == 0));
+    boolean colorDensityAll = (colorDensity && cutoff == 0);
     float v = 0;
-    int pti = 0;
-    boolean allInside = (colorDensity && (cutoff == 0 || bsVoxels.cardinality() == 0));
-    
-    for (int x = x0; x != x1; x += xStep, ptX += ptStep, pt = ptX, cellIndex = cellIndex0) {
+    for (int x = x0; x != x1; x += xStep, ptX += ptStep, pt = ptX) {
 
       // we swap planes of grid data when
       // obtaining the grid data point by point
 
-      if (mode == MODE_GETXYZ) {
+      if (mode == MODE_PLANES) {
         // for a progressive reader, we read the next two planes
         // for x = 0, 2, 4, 6...
-        if (x + xStep != x1)
-          surfaceReader.getPlane(x + xStep);
-        float[] plane = yzPlanes[0];
-        yzPlanes[0] = yzPlanes[1];
-        yzPlanes[1] = plane;
+        // say we have 50 points; then we have 49 cubes
+        // we definely what planes 0-49, but we will be processing those
+        // [0 1] [1 2] [2 3] ... [48 49]
+        // so our last x is 48, and there is no problem with (x + xStep) being == x1.
+        // fixed in Jmol 12.1.51 -- was "x + xStep != x1"
+        if (x + xStep <= x1)
+          getPlane(x + xStep, true);
+      }
+
+      // now scan the plane of cubicals
+
+      if (bsExcludedPlanes.get(x) && bsExcludedPlanes.get(x + xStep))
+        continue;
+      
+      if (colorDensity) {
+        // MODE_JVXL not allowed here
+        for (int y = y1; --y >= 0;)
+          for (int z = z1; --z >= 0; pt--) {
+            // 0 cutoff read as "show grid points only"
+            v = getValue(x, y, z, pt, 0);
+            if (colorDensityAll || isInside) {
+              // xyz corner is inside, so add this point
+              // TODO - we are missing the outside edges of these 
+              // in the Y and Z direction 
+              addVertex(x, y, z, pt, v);
+            }
+          }
+        continue;
       }
 
       // we swap the edge vertex index planes
@@ -256,76 +317,25 @@ public class MarchingCubes extends TriangleData {
       isoPointIndexPlanes[0] = isoPointIndexPlanes[1];
       isoPointIndexPlanes[1] = resetIndexPlane(indexPlane);
 
-      // now scan the plane of cubicals
-      
-      if (bsExcludedPlanes.get(x) && bsExcludedPlanes.get(x + xStep))
-        continue;
-      int xCount = 0;
-
-      for (int y = cubeCountY; --y >= 0; pt--) {        
-        for (int z = cubeCountZ; --z >= 0; pt--, cellIndex--) {
-
+      boolean noValues = true;
+      for (int y = y1; --y >= 0; pt--) {
+        for (int z = z1; --z >= 0; pt--) {
+          int insideMask = 0;
           // create the bitset mask indicating which vertices are inside.
           // 0xFF here means "all inside"; 0x00 means "all outside"
-
-          int insideMask = 0;
           for (int i = 8; --i >= 0;) {
-
-            // cubeVertexOffsets just gets us the specific grid point relative
-            // to our base x,y,z cube position
-
-            boolean isInside;
-            Point3i offset = cubeVertexOffsets[i];
-            pti = pt + linearOffsets[i];
-            switch (mode) {
-            case MODE_GETXYZ:
-              v = vertexValues[i] = getValue(x + offset.x, y + offset.y, z
-                  + offset.z, pti, yzPlanes[yzPlanePts[i]]);
-              isInside = bsVoxels.get(pti);
-              break;
-            case MODE_BITSET:
-              isInside = (allInside || bsVoxels.get(pti));
-              v = vertexValues[i] = (bsExcludedVertices.get(pti) ? Float.NaN
-                  : isInside 
-                  ? 1 
-                      : 0);
-              break;
-            default:
-            case MODE_CUBE:
-              v = vertexValues[i] = volumeData.voxelData[x + offset.x][y
-                  + offset.y][z + offset.z];
-              if (isSquared)
-                vertexValues[i] *= vertexValues[i];
-              isInside = (allInside ? true : isInside(vertexValues[i], cutoff, isCutoffAbsolute));
-              if (isInside)
-                bsVoxels.set(pti);
-            }
-            if (isInside) {
+            v = getValue(x, y, z, pt, i);
+            if (isInside)
               insideMask |= Pwr2[i];
-            } else {
-              fractionOutside += (integrateSquared ? vertexValues[i]
-                  * vertexValues[i] : vertexValues[i]);
-            }
-
-            //if (Float.isNaN(v))
-              //bsExcludedVertices.set(pti);
-          }
-          if (!Float.isNaN(v)) {
-            xCount++;
-          }
-          if (colorDensity && cutoff == 0) {
-            // 0 cutoff read as "show grid points only"
-            addVertex(x, y, z, pti, v);
-              continue;            
           }
+          // last value for i is 0, the x,y,z point itself
+          // so we need only check it for noValues -- on THIS plane
+          if (noValues && !Float.isNaN(v))
+            noValues = false;
           if (insideMask == 0) {
             ++outsideCount;
             continue;
           }
-          if (colorDensity && (insideMask & 1) == 1) {
-            // xyz corner is inside, so add this point
-            addVertex(x, y, z, pti, v);
-          }
           if (insideMask == 0xFF) {
             ++insideCount;
             continue;
@@ -334,13 +344,13 @@ public class MarchingCubes extends TriangleData {
 
           // This cube is straddling the cutoff. We must check all edges
           // Note that we do not process it if it has an NaN values
-          if (processOneCubical(insideMask, x, y, z, pt) 
-              && !isContoured && !colorDensity) {
+          if (processOneCubical(insideMask, x, y, z, pt) && !isContoured
+              && !colorDensity) {
             processTriangles(insideMask);
           }
         }
       }
-      if (xCount == 0) {
+      if (noValues) {
         bsExcludedPlanes.set(x);
       }
     }
@@ -348,6 +358,71 @@ public class MarchingCubes extends TriangleData {
     return edgeData.toString();
   }
 
+  private float getValue(int x, int y, int z, int pt, int i) {
+    float v;
+
+    // cubeVertexOffsets just gets us 
+    // the specific grid point relative
+    // to our base x,y,z cube position
+
+    offset = cubeVertexOffsets[i];
+    int pti = pt + linearOffsets[i];
+    switch (mode) {
+    case MODE_PLANES:
+      v = vertexValues[i] = getValue(x + offset.x, y + offset.y, z
+          + offset.z, pti, yzPlanes[yzPlanePts[i]]);
+      isInside = (allInside || bsVoxels.get(pti));
+      break;
+    case MODE_JVXL:
+      isInside = (allInside || bsVoxels.get(pti));
+      v = vertexValues[i] = (bsExcludedVertices.get(pti) ? Float.NaN
+          : isInside ? 1 : 0);
+      break;
+    default:
+    case MODE_CUBE:
+      //if (i == 0 && y == 0 && z == 0)
+        //dumpPlane(x, null);
+      if (mappingPlane == null) {
+        v = vertexValues[i] = voxelData[x + offset.x][y
+          + offset.y][z + offset.z];
+      } else {
+        volumeData.voxelPtToXYZ(x + offset.x, y + offset.y, z
+            + offset.z, pt0);
+        v = vertexValues[i] = volumeData.distanceToMappingPlane(pt0);
+      }
+      if (isSquared)
+        vertexValues[i] *= vertexValues[i];
+      isInside = (allInside ? true : isInside(vertexValues[i], cutoff, isCutoffAbsolute));
+      if (isInside)
+        bsVoxels.set(pti);
+    }
+    return v;
+  }
+
+  private void getPlane(int i, boolean andSwap) {
+    if (i < 0 || i > cubeCountX)
+      return;
+    /*float[] p = */surfaceReader.getPlane(i);
+    //dumpPlane(i, p);
+    if (andSwap) {
+      float[] plane = yzPlanes[0];
+      yzPlanes[0] = yzPlanes[1];
+      yzPlanes[1] = plane;
+    }
+  }
+/*
+  private void dumpPlane(int n, float[] plane) {
+    float test = 0;
+    if (plane == null)
+      for (int y = 0; y <= cubeCountY; y++)
+        for (int z = 0; z <= cubeCountZ; z++)
+          test += volumeData.voxelData[n][y][z];
+    else
+      for (int i = 0; i < yzCount; i++)
+        test += plane[i];
+    System.out.println(isXLowToHigh + " test " + n + "=" + test);
+  }
+*/  
   protected void processTriangles(int insideMask) {
 
     // the inside mask serves to define the triangles necessary
@@ -472,6 +547,8 @@ public class MarchingCubes extends TriangleData {
      *  edgeVertexPlanes[1]            [0] (scanning x high to low)
      *           
      */
+    
+
 
     int edgeMask = insideMaskTable[insideMask];
     boolean isNaN = false;
@@ -493,7 +570,9 @@ public class MarchingCubes extends TriangleData {
       int index = edgePointIndexes[iEdge] = isoPointIndexPlanes[iPlane][iPt][iType];
       if (index != Integer.MIN_VALUE) {
         if (index == -1)
-          isNaN = true;
+          isNaN = excludePartialCubes; // -- problem with older Jmol files? 
+          // this says, "If any point on the cube is NaN, then 
+          //don't process the cube. 
         continue; // propagated from neighbor
       }
       // here's an edge that has to be checked.
@@ -537,7 +616,7 @@ public class MarchingCubes extends TriangleData {
       // for example because it is outside the limits of the box.
       
       if (Float.isNaN(fReturn[0]) || i < 0)
-        isNaN = true;
+        isNaN = excludePartialCubes;
     }
     return !isNaN;
   }
