@@ -113,6 +113,7 @@
 package org.jmol.jvxl.data;
 
 import java.util.Hashtable;
+import java.util.Map;
 
 import javax.vecmath.Point3i;
 import javax.vecmath.Point3f;
@@ -128,7 +129,7 @@ import org.jmol.util.XmlUtil;
 
 public class VolumeData implements VolumeDataInterface {
 
-  public SurfaceReader sr;
+  public SurfaceReader sr; // used for delivering point-specific values, particularly when mapping
   public boolean doIterate = true;
   
   public final Point3f volumetricOrigin = new Point3f();
@@ -136,16 +137,30 @@ public class VolumeData implements VolumeDataInterface {
   public final Vector3f[] volumetricVectors = new Vector3f[3];
   public final int[] voxelCounts = new int[3];
   public int nPoints;
-  public float[][][] voxelData;  
-  public Hashtable voxelMap; // alternative to voxelData for sparse (plane interesected) data
+  private float[][][] voxelData; 
+  public float[][][] getVoxelData() {
+    return voxelData;
+  }
+  
+  public void setVoxelData(float[][][] voxelData) {
+    this.voxelData = voxelData;
+  }
+
+  private Map<Integer, Float> voxelMap; // alternative to voxelData for sparse (plane interesected) data
   public final float[] volumetricVectorLengths = new float[3];
   private float maxVectorLength;
   private float minToPlaneDistance;
+  private int yzCount;
 
   public final Vector3f[] unitVolumetricVectors = new Vector3f[3];
   private final Matrix3f volumetricMatrix = new Matrix3f();
   private final Matrix3f inverseMatrix = new Matrix3f();
   private Point4f thePlane;
+
+  public boolean hasPlane() {
+    return (thePlane != null);
+  }
+  
   private float thePlaneNormalMag;
   private final Point3f ptXyzTemp = new Point3f();
   public String xmlData;
@@ -159,6 +174,23 @@ public class VolumeData implements VolumeDataInterface {
     unitVolumetricVectors[2] = new Vector3f();
   }
 
+  public Point4f mappingPlane;
+  float mappingPlaneNormalMag;
+  
+  public void setMappingPlane(Point4f plane) {
+    //if(true)return;
+    mappingPlane = plane;
+    if (plane == null)
+      return;
+    mappingPlaneNormalMag = (float) Math.sqrt(plane.x * plane.x + plane.y * plane.y + plane.z * plane.z);
+  }
+
+  public float distanceToMappingPlane(Point3f pt) {
+    return (mappingPlane.x * pt.x + mappingPlane.y * pt.y + mappingPlane.z * pt.z + mappingPlane.w)
+    / mappingPlaneNormalMag;
+  }
+
+
   public void setVolumetricOrigin(float x, float y, float z) {
     volumetricOrigin.set(x, y, z);
   }
@@ -167,6 +199,25 @@ public class VolumeData implements VolumeDataInterface {
     return origin;
   }
 
+  public float minGrid;
+  public float maxGrid;
+  public float voxelVolume;
+  
+  public int getYzCount() {
+    
+    minGrid = volumetricVectors[0].length();
+    minGrid = Math.min(minGrid, volumetricVectors[1].length());
+    minGrid = Math.min(minGrid, volumetricVectors[2].length());
+
+    maxGrid = volumetricVectors[0].length();
+    maxGrid = Math.max(maxGrid, volumetricVectors[1].length());
+    maxGrid = Math.max(maxGrid, volumetricVectors[2].length());
+
+    nPoints = voxelCounts[0] * voxelCounts[1] * voxelCounts[2];
+    
+    return yzCount = voxelCounts[1] * voxelCounts[2];
+  }
+  
   public float[] getVolumetricVectorLengths() {
     return volumetricVectorLengths;
   }
@@ -186,31 +237,43 @@ public class VolumeData implements VolumeDataInterface {
     voxelCounts[0] = nPointsX;
     voxelCounts[1] = nPointsY;
     voxelCounts[2] = nPointsZ;
-    return nPoints = nPointsX * nPointsY * nPointsZ;
+    return nPointsX * nPointsY * nPointsZ;
   }
 
-  public float[][][] getVoxelData() {
-    return voxelData;
+  public float getVoxelData(int pt) {
+    int ix = pt / yzCount;
+    pt -= ix * yzCount;
+    int iy = pt / voxelCounts[2];
+    int iz = pt - iy * voxelCounts[2];
+    return voxelData[ix][iy][iz]; 
   }
   
-  public void setVoxelData(float[][][] voxelData) {
-    this.voxelData = voxelData;
-  }
-
-  public Hashtable getVoxelMap() {
-    return voxelMap;
+  public int getPointIndex(int x, int y, int z) {
+    return x * yzCount + y * voxelCounts[2] + z;  
   }
   
-  public void setVoxelMap(Hashtable voxelMap) {
-    this.voxelMap = voxelMap;
+  public void getPoint(int ipt, Point3f pt) {
+    int ix = ipt / yzCount;
+    ipt -= ix * yzCount;
+    int iy = ipt / voxelCounts[2];
+    int iz = ipt - iy * voxelCounts[2];
+    voxelPtToXYZ(ix, iy, iz, pt);
   }
   
-  public float getVoxelValueFromMap(int x, int y, int z) {
-    Float v = (voxelMap == null ? null : (Float) voxelMap.get(x+"_" + y + "_" + z));
-    return (v == null ? Float.NaN : v.floatValue());
+  public void setVoxelData(int pt, float value) {
+    int ix = pt / yzCount;
+    pt -= ix * yzCount;
+    int iy = pt / voxelCounts[2];
+    int iz = pt - iy * voxelCounts[2];
+    voxelData[ix][iy][iz] = value; 
   }
-
-  public boolean setMatrix() {
+  
+  public void setVoxelMap() {
+    voxelMap = new Hashtable<Integer, Float>();
+    getYzCount();
+  }
+  
+  private boolean setMatrix() {
     for (int i = 0; i < 3; i++)
       volumetricMatrix.setColumn(i, volumetricVectors[i]);
     try {
@@ -233,19 +296,15 @@ public class VolumeData implements VolumeDataInterface {
 
   public float calcVoxelPlaneDistance(int x, int y, int z) {
     voxelPtToXYZ(x, y, z, ptXyzTemp);
-    return (thePlane.x * ptXyzTemp.x + thePlane.y * ptXyzTemp.y + thePlane.z
-        * ptXyzTemp.z + thePlane.w)
-        / thePlaneNormalMag;
+    return distancePointToPlane(ptXyzTemp);
   }
 
-  Point4f mappingPlane;
-  public float getToPlaneParameter(Point4f plane) {
-    mappingPlane = plane;
-    return (float) (Math.sqrt(plane.x * plane.x
-        + plane.y * plane.y + plane.z * plane.z) * minToPlaneDistance);
+  public float getToPlaneParameter() {
+    return (float) (Math.sqrt(thePlane.x * thePlane.x
+        + thePlane.y * thePlane.y + thePlane.z * thePlane.z) * minToPlaneDistance);
   }
   
-  public boolean isNearPlane(int x, int y, int z, Point4f plane, float toPlaneParameter) {
+  public boolean isNearPlane(int x, int y, int z, float toPlaneParameter) {
     voxelPtToXYZ(x, y, z, ptXyzTemp);
     return ((thePlane.x * ptXyzTemp.x + thePlane.y * ptXyzTemp.y
         + thePlane.z * ptXyzTemp.z + thePlane.w) < toPlaneParameter);
@@ -264,12 +323,14 @@ public class VolumeData implements VolumeDataInterface {
 
   public boolean setUnitVectors() {
     maxVectorLength = 0;
+    voxelVolume = 1;
     for (int i = 0; i < 3; i++) {
       float d = volumetricVectorLengths[i] = volumetricVectors[i].length();
       if (d == 0)
         return false;
       if (d > maxVectorLength)
         maxVectorLength = d;
+      voxelVolume *= d;
       unitVolumetricVectors[i].normalize(volumetricVectors[i]);
     }
     minToPlaneDistance = maxVectorLength * 2;
@@ -286,39 +347,37 @@ public class VolumeData implements VolumeDataInterface {
     pt3i.set((int) ptXyzTemp.x, (int) ptXyzTemp.y, (int) ptXyzTemp.z);
   }
 
+  public boolean isPeriodic;
+  
   public float lookupInterpolatedVoxelValue(Point3f point) {
+    if (mappingPlane != null)
+      return distanceToMappingPlane(point);
     if (sr != null) {
       float v = sr.getValueAtPoint(point);
       return (isSquared ? v * v : v);
     }
     ptXyzTemp.sub(point, volumetricOrigin);
     inverseMatrix.transform(ptXyzTemp);
-    //if (ptXyzTemp.x < -0.1f || ptXyzTemp.y < -0.1f || ptXyzTemp.z < -0.1f)
-      //return Float.NaN;
-    return getInterpolatedVoxelValue(ptXyzTemp);
-  }
-
-  private float getInterpolatedVoxelValue(Point3f pt) {
     int iMax;
-    int xDown = indexDown(pt.x, iMax = voxelCounts[0] - 1);
-    int xUp = xDown + (pt.x < 0 || xDown == iMax ? 0 : 1);
-    int yDown = indexDown(pt.y, iMax = voxelCounts[1] - 1);
-    int yUp = yDown + (pt.y < 0 || yDown == iMax ? 0 : 1);
-    int zDown = indexDown(pt.z, iMax = voxelCounts[2] - 1);
-    int zUp = zDown + (pt.z < 0 || zDown == iMax ? 0 : 1);
-    float v1 = getFractional2DValue(pt.x - xDown, pt.y - yDown,
+    int xDown = indexDown(ptXyzTemp.x, iMax = voxelCounts[0] - 1);
+    int xUp = xDown + (ptXyzTemp.x < 0 || xDown == iMax ? 0 : 1);
+    int yDown = indexDown(ptXyzTemp.y, iMax = voxelCounts[1] - 1);
+    int yUp = yDown + (ptXyzTemp.y < 0 || yDown == iMax ? 0 : 1);
+    int zDown = indexDown(ptXyzTemp.z, iMax = voxelCounts[2] - 1);
+    int zUp = zDown + (ptXyzTemp.z < 0 || zDown == iMax ? 0 : 1);
+    float v1 = getFractional2DValue(ptXyzTemp.x - xDown, ptXyzTemp.y - yDown,
         getVoxelValue(xDown, yDown, zDown), getVoxelValue(xUp, yDown, zDown),
         getVoxelValue(xDown, yUp, zDown), getVoxelValue(xUp, yUp, zDown));
-    float v2 = getFractional2DValue(pt.x - xDown, pt.y - yDown,
+    float v2 = getFractional2DValue(ptXyzTemp.x - xDown, ptXyzTemp.y - yDown,
         getVoxelValue(xDown, yDown, zUp), getVoxelValue(xUp, yDown, zUp),
         getVoxelValue(xDown, yUp, zUp), getVoxelValue(xUp, yUp, zUp));
-    return v1 + (pt.z - zDown) * (v2 - v1);
+    return v1 + (ptXyzTemp.z - zDown) * (v2 - v1);
   }
 
   public float getVoxelValue(int x, int y, int z) {
     if (voxelMap == null)
       return voxelData[x][y][z];
-    Float f = (Float) voxelMap.get(x + "_" + y + "_" + z);
+    Float f = voxelMap.get(new Integer(getPointIndex(x, y, z)));
     return (f == null ? Float.NaN : f.floatValue());
   }
 
@@ -345,6 +404,7 @@ public class VolumeData implements VolumeDataInterface {
   }
 
   public void setDataDistanceToPlane(Point4f plane) {
+    //TODO REMOVE THIS METHOD.
     setPlaneParameters(plane);
     int nx = voxelCounts[0];
     int ny = voxelCounts[1];
@@ -370,7 +430,7 @@ public class VolumeData implements VolumeDataInterface {
       for (int x = 0; x < nx; x++)
         for (int y = 0; y < ny; y++)
           for (int z = 0; z < nz; z++)
-            voxelData[x][y][z] = voxelData[x][y][z] * voxelData[x][y][z];
+            voxelData[x][y][z] *= voxelData[x][y][z];
     if (doInvert) 
       for (int x = 0; x < nx; x++)
         for (int y = 0; y < ny; y++)
@@ -426,7 +486,7 @@ public class VolumeData implements VolumeDataInterface {
   public void setVoxelMapValue(int x, int y, int z, float v) {
     if (voxelMap == null)
       return;
-    voxelMap.put(x+"_" + y + "_" + z, new Float(v));    
+    voxelMap.put(new Integer(getPointIndex(x, y, z)), Float.valueOf(v));    
   }
 
   private final Vector3f edgeVector = new Vector3f();
@@ -453,7 +513,8 @@ public class VolumeData implements VolumeDataInterface {
     int n = 0;
     ptTemp.set(pt);
     float v = lookupInterpolatedVoxelValue(ptTemp);
-    float v0 = Float.NaN;    
+    float v0 = Float.NaN;
+    
     while (++n < 10) {
       float fnew = (v - valueA) / d;
       if (fnew < 0 || fnew > 1)
@@ -472,5 +533,5 @@ public class VolumeData implements VolumeDataInterface {
     return v0;
   }
 
-
-}
+  
+}
\ No newline at end of file
