@@ -23,8 +23,6 @@
  */
 package org.jmol.jvxl.readers;
 
-import javax.vecmath.Point3f;
-
 class IsoPlaneReader extends AtomDataReader {
 
   IsoPlaneReader(SurfaceGenerator sg) {
@@ -32,21 +30,12 @@ class IsoPlaneReader extends AtomDataReader {
     precalculateVoxelData = false;
   }
 
-  protected void setup() {
-    super.setup();
-    doAddHydrogens = false;
-    getAtoms(params.mep_marginAngstroms, false, false);
-    if (xyzMin == null) {
-      setBoundingBox(new Point3f(-10, -10, -10), 0);
-      setBoundingBox(new Point3f(10, 10, 10), 0);
-    }
+  @Override
+  protected void setup(boolean isMapData) {
+    super.setup(isMapData);
     setHeader("PLANE", params.thePlane.toString());
-    setRangesAndAddAtoms(params.solvent_ptsPerAngstrom, params.solvent_gridMax, Math.min(myAtomCount, 100)); 
     params.cutoff = 0;
-  }
-
-  public float getValue(int x, int y, int z, int ptyz) {    
-    return  volumeData.calcVoxelPlaneDistance(x, y, z);
+    setVolumeForPlane();
   }
 
 }
