@@ -621,6 +621,7 @@ class IsoSolventReader extends AtomDataReader {
     Edge(int ia, int ib) {
       this.ia = Math.min(ia, ib);
       this.ib = Math.max(ia, ib);
+      //dump();
     }
 
 //    private List<Face> aFaces;
@@ -639,16 +640,16 @@ class IsoSolventReader extends AtomDataReader {
       return (nFaces > 0 ? nFaces : nInvalid > 0 ? -nInvalid : 0);
     }
 
-    /*
+/*    
         void dump() {
           System.out.println("draw e" + (nTest++) + " @{point"
               + new Point3f(atomXyz[ia]) + "} @{point" + new Point3f(atomXyz[ib])
-              + "} color green # " + getType());
-          for (int i = 0; i < aFaces.size(); i++)
-            aFaces.get(i).dump();
+              + "} color yellow # " + getType());
+   //       for (int i = 0; i < aFaces.size(); i++)
+     //       aFaces.get(i).dump();
         }
-    
-   */
+*/    
+   
     @Override
     public String toString() {
       return ia + "_" + ib;
@@ -685,7 +686,6 @@ class IsoSolventReader extends AtomDataReader {
     }
 
     protected void dump() {
-      return;
 /*
       Point3f ptA = atomXyz[ia];
       Point3f ptB = atomXyz[ib];
@@ -697,7 +697,7 @@ class IsoSolventReader extends AtomDataReader {
       dumpLine2(pS, ptA, "f", solventRadius, color, "white");
       dumpLine2(pS, ptB, "f", solventRadius, color, "white");
       dumpLine2(pS, ptC, "f", solventRadius, color, "white");
- */
+*/ 
     }
   }
 
@@ -725,16 +725,20 @@ class IsoSolventReader extends AtomDataReader {
       bs.and(bsLocale[ib]);
       for (int ic = bs.nextSetBit(ib + 1); ic >= 0; ic = bs.nextSetBit(ic + 1)) {
         if (getSolventPoints(ia, ib, ic)) {
-          noFaceSpheres.clear(ia);
-          noFaceSpheres.clear(ib);
-          noFaceSpheres.clear(ic);
-          Face f = new Face(ia, ib, ic, edge, ptS1);
-          if (validateFace(f)) {
+          Face f;
+          boolean isOK = false;
+          if (validateFace(f = new Face(ia, ib, ic, edge, ptS1))) {
             vFaces.add(f);
+            isOK = true;
           }
-          f = new Face(ib, ia, ic, edge, ptS2);
-          if (validateFace(f)) {
+          if (validateFace(f = new Face(ia, ib, ic, edge, ptS2))) {
             vFaces.add(f);
+            isOK = true;
+          }
+          if (isOK) {
+            noFaceSpheres.clear(ia);
+            noFaceSpheres.clear(ib);
+            noFaceSpheres.clear(ic);
           }
         }
       }
