@@ -118,12 +118,16 @@ import javax.vecmath.Point3f;
 import javax.vecmath.Point4f;
 import javax.vecmath.Vector3f;
 
-import java.util.Hashtable;
+import java.util.ArrayList;
 import java.util.BitSet;
+import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 
 import org.jmol.atomdata.RadiusData;
+import org.jmol.jvxl.data.VolumeData;
 import org.jmol.util.ColorEncoder;
+import org.jmol.util.ContactPair;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
 
@@ -136,12 +140,14 @@ public class Parameters {
   public final static int STATE_DATA_READ = 2;
   public final static int STATE_DATA_COLORED = 3;
 
+  
   int state = STATE_UNINITIALIZED;
 
+  public int testFlags = 0;
   boolean logMessages = false;
   boolean logCompression = false;
   boolean logCube = false;
-  boolean isSilent = false;
+  public boolean isSilent = false;
   
   float assocCutoff = 0.3f; // fraction along the edge to use as a cutoff for averaging of normals  
 
@@ -149,6 +155,8 @@ public class Parameters {
   final static int IS_SILENT = 1 << 6;
   final public static int IS_SOLVENTTYPE = 1 << 7;
   final static int HAS_MAXGRID = 1 << 8;
+  final static int IS_POINTMAPPABLE = 1 << 9;
+  final static int IS_SLABBABLE = 1 << 10;
   
   public int dataType;
   int surfaceType;
@@ -167,29 +175,34 @@ public class Parameters {
   final static int SURFACE_FUNCTIONXYZ = 9;
 
   // getSurface or mapColor:
-  final static int SURFACE_SOLVENT = 11 | IS_SOLVENTTYPE | NO_ANISOTROPY;
-  final static int SURFACE_SASURFACE = 12 | IS_SOLVENTTYPE | NO_ANISOTROPY;
-  final static int SURFACE_MOLECULARORBITAL = 13 | NO_ANISOTROPY | HAS_MAXGRID;
-  final static int SURFACE_ATOMICORBITAL = 14 | HAS_MAXGRID;
-  final static int SURFACE_MEP = 16 | NO_ANISOTROPY | HAS_MAXGRID;
-  final static int SURFACE_MLP = 17 | NO_ANISOTROPY | HAS_MAXGRID;
-  final static int SURFACE_MOLECULAR = 19 | IS_SOLVENTTYPE | NO_ANISOTROPY;
+  final static int SURFACE_SOLVENT = 11 | IS_SOLVENTTYPE | NO_ANISOTROPY | IS_SLABBABLE ;
+  final static int SURFACE_SASURFACE = 12 | IS_SOLVENTTYPE | NO_ANISOTROPY | IS_SLABBABLE;
+  final static int SURFACE_MOLECULARORBITAL = 13 | NO_ANISOTROPY | HAS_MAXGRID | IS_POINTMAPPABLE | IS_SLABBABLE;
+  final static int SURFACE_ATOMICORBITAL = 14 | HAS_MAXGRID | IS_SLABBABLE;
+  final static int SURFACE_MEP = 16 | NO_ANISOTROPY | HAS_MAXGRID | IS_SLABBABLE;
+  final static int SURFACE_MLP = 17 | NO_ANISOTROPY | HAS_MAXGRID | IS_SLABBABLE;
+  final static int SURFACE_MOLECULAR = 19 | IS_SOLVENTTYPE | NO_ANISOTROPY | IS_SLABBABLE;
+  final static int SURFACE_NCI = 20 | NO_ANISOTROPY | HAS_MAXGRID | IS_POINTMAPPABLE | IS_SLABBABLE;
+  final static int SURFACE_INTERSECT = 21 | NO_ANISOTROPY | HAS_MAXGRID | IS_SLABBABLE;
 
   // mapColor only:
 
-  final static int SURFACE_NOMAP = 20 | IS_SOLVENTTYPE | NO_ANISOTROPY;
-  final static int SURFACE_PROPERTY = 21 | IS_SOLVENTTYPE | NO_ANISOTROPY;
+  final static int SURFACE_NOMAP = 21 | IS_SOLVENTTYPE | NO_ANISOTROPY | IS_SLABBABLE;
+  final static int SURFACE_PROPERTY = 22 | IS_SOLVENTTYPE | NO_ANISOTROPY | IS_SLABBABLE;
 
   void initialize() {
     addHydrogens = false;
+    allowVolumeRender = true;
+    atomRadiusData = null;
     atomIndex = -1;
     blockCubeData = false; // Gaussian standard, but we allow for multiple surfaces one per data block
+    boundingBox = null;
+    bsExcluded = new BitSet[4];
     bsIgnore = null;
     bsSelected = null;
     bsSolvent = null;
     calculationType = "";
     center = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
-    cappingObject= null;
     colorBySign = colorByPhase = colorBySets = false;
     colorDensity = false;
     colorEncoder = null;
@@ -197,17 +210,28 @@ public class Parameters {
     colorNegLCAO = defaultColorNegativeLCAO;
     colorPos = defaultColorPositive;
     colorPosLCAO = defaultColorPositiveLCAO;
+    colorRgb = Integer.MIN_VALUE;
+    colorSchemeTranslucent = false;
+    contactPair = null;
+    contourIncrements = null;
+    contoursDiscrete = null;
+    contourColixes = null;
+    contourFromZero = true;
     cutoff = Float.MAX_VALUE;
     cutoffAutomatic = true;
     dataXYReversed = false;
     distance = Float.MAX_VALUE;
+    doFullMolecular = false;
     envelopeRadius = 10f;
     fileIndex = 1;
     readAllData = true;
     fileName = "";
-    functionXYinfo = null;
+    fullyLit = false;
+    func = null;
+    functionInfo = null;
     iAddGridPoints = false;
     insideOut = false;
+    intersection = null;
     isAngstroms = false;
     isBicolorMap = isCutoffAbsolute = isPositiveOnly = false;
     isCavity = false;
@@ -215,58 +239,54 @@ public class Parameters {
     isSquared = false;
     isContoured = false;
     isEccentric = isAnisotropic = false;
+    isPeriodic = false;
     isSilent = false;
-    iUseBitSets = false;
     logCube = logCompression = false;
     logMessages = Logger.debugging;
     mappedDataMin = Float.MAX_VALUE;
+    mep_calcType = -1;
     minSet = 0;
     modelIndex = -1;
-    boundingBox = null;
-    bsExcluded = new BitSet[4];
     nContours = 0;
-    colorSchemeTranslucent = false;
-    contourIncrements = null;
-    contoursDiscrete = null;
-    contourColixes = null;
-    mep_calcType = -1;
     pocket = null;
     propertyDistanceMax = Integer.MAX_VALUE;
     propertySmoothing = false;
+    propertySmoothingPower = 4;
     rangeDefined = false;
+    rangeAll = false;
+    rangeSelected = false;
     resolution = Float.MAX_VALUE;
     scale = Float.NaN;
     scale3d = 0;
     sigma = Float.NaN;
-    slabbingObject = null;
+    slabInfo = null;
     solventExtendedAtomRadius = 0;
     state = STATE_INITIALIZED;
+    testFlags = 0;
     thePlane = null;
     theProperty = null;
-    usePropertyForColorRange = true; // except for MEP and MLP
     thisContour = -1;
-    contourFromZero = true;
     title = null;
-    atomRadiusData = null;
-    rangeAll = false;
-    rangeSelected = false;
+    usePropertyForColorRange = true; // except for MEP and MLP
+    vertexSource = null;
+    volumeData = null;
   }
   
   String calculationType = "";
 
   //solvent/molecular-related:
-  RadiusData atomRadiusData;
+  public RadiusData atomRadiusData;
   boolean addHydrogens;
   float solventRadius;
   float solventExtendedAtomRadius;
   boolean propertySmoothing;
+  int propertySmoothingPower = 4;
   float envelopeRadius;
   float cavityRadius;
   boolean isCavity;
   Boolean pocket; //three states: TRUE, FALSE, and NULL
   int minSet;
-  public Object cappingObject;
-  public Object slabbingObject;
+  public List<Object[]> slabInfo;
   float slabPlaneOffset = Float.NaN;
 
   float[] theProperty;
@@ -275,6 +295,8 @@ public class Parameters {
   float solvent_ptsPerAngstrom = 4f;
   int solvent_gridMax = 60;
 
+  float plane_ptsPerAngstrom = 4f;
+  int plane_gridMax = 81;
 
   //defaults
   
@@ -286,7 +308,9 @@ public class Parameters {
   final static float defaultMappedDataMin = 0f;
   final static float defaultMappedDataMax = 1.0f;
   final static float defaultCutoff = 0.02f;
-  final static float defaultOrbitalCutoff = 0.14f;
+  final static float defaultOrbitalCutoff = 0.04f;
+  final static float defaultLobeCutoff = 0.14f;
+  final static float defaultOrbitalCutoffOld = 0.14f;
   public final static float defaultQMOrbitalCutoff = 0.050f; // WebMO
   final static float defaultQMElectronDensityCutoff = 0.010f;
   final static int defaultContourCount = 11; //odd is better
@@ -305,18 +329,17 @@ public class Parameters {
   boolean colorBySign;
   boolean colorByPhase;
   boolean colorBySets;
+  public int colorRgb;
   int colorNeg;
   int colorPos;
   int colorPosLCAO;
   int colorNegLCAO;
   int colorPhase;
   public boolean colorDensity;
-  public ColorEncoder colorEncoder;
 
    
   //special effects
   boolean iAddGridPoints;
-  boolean remappable;
   
   /////////////////////////////
   
@@ -414,7 +437,7 @@ public class Parameters {
   float[] anisoB;
   public void setEllipsoid(float[] bList) {
     anisoB = bList;
-    for (int i = 0; i < 6; i++)System.out.print(bList[i] + " ");System.out.println( " in Parameters setEllipsoid" + center);
+    //for (int i = 0; i < 6; i++)System.out.print(bList[i] + " ");System.out.println( " in Parameters setEllipsoid" + center);
     dataType = SURFACE_ELLIPSOID3;
     distance = 0.3f * (Float.isNaN(scale) ? 1f : scale);
     cutoff = Float.MIN_VALUE;
@@ -433,33 +456,28 @@ public class Parameters {
     dataType = SURFACE_LOBE;
     setEccentricity(v);
     if (cutoff == Float.MAX_VALUE) {
-      cutoff = defaultOrbitalCutoff;
+      cutoff = defaultLobeCutoff;
       if (isSquared)
         cutoff = cutoff * cutoff;
     }
     isSilent = !logMessages;
     script = getScriptParams() + " LOBE {" + v.x + " "
-    + v.y + " " + v.z + " " + v.w + "};";
-}
-
+        + v.y + " " + v.z + " " + v.w + "};";
+  }
+  
   private String getScriptParams() {
-    return (slabbingObject == null ? "" : " slab within "
-        + Escape.escape(slabbingObject))
-        + (cappingObject == null ? "" : " cap within "
-            + Escape.escape(cappingObject))
-        + " center "
+    return " center "
         + Escape.escape(center) + (Float.isNaN(scale) ? "" : " scale " + scale);
   }
 
-  
   void setLp(Point4f v) {
     dataType = SURFACE_LONEPAIR;
     setEccentricity(v);
     if (cutoff == Float.MAX_VALUE) {
-      cutoff = defaultOrbitalCutoff;
+      cutoff = defaultLobeCutoff;
       if (isSquared)
         cutoff = cutoff * cutoff;
-    }
+    } 
     isSilent = !logMessages;
     script = " center " + Escape.escape(center)
         + (Float.isNaN(scale) ? "" : " scale " + scale) + " LP {" + v.x + " "
@@ -470,7 +488,7 @@ public class Parameters {
     dataType = SURFACE_RADICAL;
     setEccentricity(v);
     if (cutoff == Float.MAX_VALUE) {
-      cutoff = defaultOrbitalCutoff;
+      cutoff = defaultLobeCutoff;
       if (isSquared)
         cutoff = cutoff * cutoff;
     }
@@ -489,22 +507,25 @@ public class Parameters {
     isSilent = !logMessages;
   }
     
-  void setRadius(RadiusData rd) {
-    atomRadiusData = rd;
-  }
-
   void setSolvent(String propertyName, float radius) {
     isEccentric = isAnisotropic = false;
     //anisotropy[0] = anisotropy[1] = anisotropy[2] = 1f;
-    solventRadius = radius;
-    if (solventRadius < 0)
-      solventRadius = defaultSolventRadius;
-    dataType = ("nomap" == propertyName ? SURFACE_NOMAP
-        : "molecular" == propertyName ? SURFACE_MOLECULAR
-            : "sasurface" == propertyName || solventRadius == 0f ? SURFACE_SASURFACE
-                : SURFACE_SOLVENT);
+    solventRadius = Math.abs(radius);
+    dataType = (intersection != null ? SURFACE_INTERSECT
+        : "nomap" == propertyName ? SURFACE_NOMAP
+            : "molecular" == propertyName ? SURFACE_MOLECULAR
+                : "sasurface" == propertyName || solventRadius == 0f ? SURFACE_SASURFACE
+                    : SURFACE_SOLVENT);
+
+    if (state < Parameters.STATE_DATA_READ
+        && (cutoffAutomatic || !colorDensity)
+        && (intersection == null || cutoff == Float.MAX_VALUE))
+      cutoff = 0.0f;
 
     switch (dataType) {
+    case Parameters.SURFACE_INTERSECT:
+      calculationType = "VDW intersection";
+      break;
     case Parameters.SURFACE_NOMAP:
       calculationType = "unmapped plane";
       break;
@@ -543,19 +564,20 @@ public class Parameters {
     }
   }
   
-  public List functionXYinfo;
+  public List<Object> functionInfo;
   
-  void setFunctionXY(List value) {
+  void setFunctionXY(List<Object> value) {
     dataType = SURFACE_FUNCTIONXY;
-    functionXYinfo = value;
+    functionInfo = value;
     cutoff = Float.MIN_VALUE;
     isEccentric = isAnisotropic = false;
   }
 
-  void setFunctionXYZ(List value) {
+  void setFunctionXYZ(List<Object> value) {
     dataType = SURFACE_FUNCTIONXYZ;
-    functionXYinfo = value;
-    cutoff = Float.MIN_VALUE;
+    functionInfo = value;
+    if (cutoff == Float.MAX_VALUE)
+      cutoff = Float.MIN_VALUE;
     isEccentric = isAnisotropic = false;
   }
 
@@ -564,34 +586,40 @@ public class Parameters {
   int psi_m = 1;
   float psi_Znuc = 1; // hydrogen
   float psi_ptsPerAngstrom = 5f;
+  public int psi_monteCarloCount = 0;
 
-  boolean setAtomicOrbital(float[] nlmZ) {
+  boolean setAtomicOrbital(float[] nlmZprs) {
     dataType = SURFACE_ATOMICORBITAL;
-    psi_n = (int) nlmZ[0];
-    psi_l = (int) nlmZ[1];
-    psi_m = (int) nlmZ[2];
-    psi_Znuc = nlmZ[3];
+    setEccentricity(new Point4f(0, 0, 1, 1));
+    psi_n = (int) nlmZprs[0];
+    psi_l = (int) nlmZprs[1];
+    psi_m = (int) nlmZprs[2];
+    psi_Znuc = nlmZprs[3];
+    psi_monteCarloCount = (int) nlmZprs[4];
+    distance = nlmZprs[5];
+    if (distance != 0 || thePlane != null)
+      allowVolumeRender = false;
+    randomSeed = (int) nlmZprs[6];
     psi_ptsPerAngstrom = 10;
     // quantum rule is abs(m) <= l < n
-    if (cutoff == Float.MAX_VALUE) {
-      cutoff = defaultOrbitalCutoff;
+    if (cutoff == Float.MAX_VALUE || cutoff == defaultOrbitalCutoffOld) {
+      cutoff = (psi_monteCarloCount > 0 ? 0 : defaultOrbitalCutoff);
       if (isSquared)
         cutoff = cutoff * cutoff;
     }
     isCutoffAbsolute = true;
     if (state < STATE_DATA_READ && thePlane == null && colorBySign)
-        isBicolorMap = true;
+      isBicolorMap = true;
     return (psi_Znuc > 0 && Math.abs(psi_m) <= psi_l && psi_l < psi_n);
   }  
-  
- 
+
   public final static int MEP_MAX_GRID = 40;
   int mep_gridMax = MEP_MAX_GRID;
   float mep_ptsPerAngstrom = 3f;
   float mep_marginAngstroms = 1f; // may have to adjust this
   public int mep_calcType = -1;
 
-  void setMep(float[] charges, boolean isRangeDefined, boolean isMLP) {
+  void setMep(float[] charges, boolean isMLP) {
     dataType = (isMLP ? SURFACE_MLP : SURFACE_MEP);
     theProperty = charges;
     usePropertyForColorRange = false;
@@ -625,31 +653,57 @@ public class Parameters {
   final static int QM_TYPE_UNKNOWN = 0;
   final static int QM_TYPE_GAUSSIAN = 1;
   final static int QM_TYPE_SLATER = 2;
+  final static int QM_TYPE_NCI_PRO = 3;
+  final static int QM_TYPE_NCI_SCF = 4;
+  final static int QM_TYPE_VOLUME_DATA = 5;
   
-  Hashtable moData, mo;
-  float[] moCoefficients;
-  int[][] dfCoefMaps;
+  Map<String, Object> moData;
   public final static int MO_MAX_GRID = 80;
   int qm_gridMax = MO_MAX_GRID;
   float qm_ptsPerAngstrom = 10f;
   float qm_marginAngstroms = 1f; // may have to adjust this
   int qm_nAtoms;
   int qm_moNumber = Integer.MAX_VALUE;
+  float[] qm_moLinearCombination = null;
   
-  void setMO(int iMo, boolean isRangeDefined) {
-    iUseBitSets = true;
-    qm_moNumber = Math.abs(iMo);
-    qmOrbitalType = (moData.containsKey("gaussians") ? QM_TYPE_GAUSSIAN
+  void setNci(boolean isPromolecular) {
+    fullyLit = true;
+    qm_gridMax = 200;
+    if (isPromolecular)
+      dataType = SURFACE_NCI;
+    qm_marginAngstroms = 2f;
+    qmOrbitalType = (isPromolecular ? QM_TYPE_NCI_PRO : QM_TYPE_NCI_SCF);
+
+    if (isPromolecular) {
+      if (parameters == null || parameters.length < 2)
+        parameters = new float[] { cutoff, 2 }; // default intermolecular
+    }
+
+    if (cutoff == Float.MAX_VALUE || cutoff == 0)
+      cutoff = 0.3f;
+    if (isSquared)
+      cutoff *= cutoff;
+
+    if (title == null)
+      title = new String[0];
+    moData = new Hashtable<String, Object>();
+  }
+   
+  @SuppressWarnings("unchecked")
+  void setMO(int iMo, float[] linearCombination) {
+    qm_moLinearCombination = linearCombination;
+    qm_moNumber = (linearCombination == null ? Math.abs(iMo) : (int) linearCombination[1]);
+    qmOrbitalType = (moData.containsKey("haveVolumeData") ? QM_TYPE_VOLUME_DATA 
+        : moData.containsKey("gaussians") ? QM_TYPE_GAUSSIAN
         : moData.containsKey("slaters") ? QM_TYPE_SLATER : QM_TYPE_UNKNOWN);
-    boolean isElectronDensity = (iMo <= 0);
+    boolean isElectronDensity = (iMo <= 0 && linearCombination == null);
     if (qmOrbitalType == QM_TYPE_UNKNOWN) {
       //TODO     value = moData; // must be generic surface info
       Logger
           .error("MO ERROR: No basis functions found in file for MO calculation. (GAUSSIAN 'gfprint' keyword may be missing?)");
-      mo = null;
       title = new String[] {"no basis functions found in file"};
     } else {
-      List mos = (List) (moData.get("mos"));
+      List<Object> mos = (List<Object>) moData.get("mos");
       qmOrbitalCount = mos.size();
       calculationType = (String) moData.get("calculationType");
       calculationType = "Molecular orbital #" + qm_moNumber + "/"
@@ -666,9 +720,6 @@ public class Parameters {
           title[3] = "?Symmetry = %S";
           title[4] = "?Occupancy = %O";
         }
-        mo = (Hashtable) mos.get(qm_moNumber - 1);
-        moCoefficients = (float[]) mo.get("coefficients");
-        dfCoefMaps = (int[][]) mo.get("dfCoefMaps");
       }
     }
     dataType = SURFACE_MOLECULARORBITAL;
@@ -692,15 +743,16 @@ public class Parameters {
   
   Point3f center, point;
   float distance;
+  public boolean allowVolumeRender;
   
   String script;
   
-  BitSet bsSelected;
-  BitSet bsIgnore;
-  BitSet bsSolvent;
-  
-  boolean iUseBitSets = false;
+  public BitSet bsSelected;
+  public BitSet bsIgnore;
+  public BitSet bsSolvent;
   
+  public Object func;
+
   String[] title;
   boolean blockCubeData;
   boolean readAllData;
@@ -724,7 +776,7 @@ public class Parameters {
   float mappedDataMin;
   float mappedDataMax;
   boolean isColorReversed;
-  boolean isBicolorMap;
+  public boolean isBicolorMap;
   public boolean isSquared;
 
   public Point4f thePlane;
@@ -733,20 +785,32 @@ public class Parameters {
   int nContours;
   int thisContour; 
   boolean contourFromZero;
+  float[] parameters; // additional parameters
  
-  float resolution;
+  public float resolution;
   int downsampleFactor;
   int maxSet;
   public float[] contoursDiscrete;
   public short[] contourColixes;
   Point3f contourIncrements;
   public Point3f[] boundingBox;
-  public Point3f[] mappedDataBox;
   public BitSet[] bsExcluded;
   public int contourType;
   public boolean colorSchemeTranslucent;
+  public ColorEncoder colorEncoder;
   public boolean usePropertyForColorRange = true;
-  public float propertyDistanceMax = Float.MAX_VALUE;
+  public boolean isPeriodic;
+  public boolean doFullMolecular;
+  public float propertyDistanceMax = Integer.MAX_VALUE;
+  public int randomSeed;
+  public boolean fullyLit;
+  public int[] vertexSource;
+  public BitSet[] intersection;
+  public Point3f origin;
+  public Point3f steps;
+  public Point3f points;
+  public VolumeData volumeData;
+  public ContactPair contactPair;
   
   void setMapRanges(SurfaceReader surfaceReader, boolean haveData) {
     if (!colorDensity)
@@ -756,6 +820,7 @@ public class Parameters {
     }
     if (mappedDataMin == Float.MAX_VALUE || mappedDataMin == mappedDataMax) {
       float[] minMax = surfaceReader.getMinMaxMappedValues(haveData);
+      //System.out.println("parameters - setmapranges " + Escape.escape(minMax));
       mappedDataMin = minMax[0];
       mappedDataMax = minMax[1];
     }
@@ -771,4 +836,30 @@ public class Parameters {
     }
   }
 
+  /**
+   * reset some parameters at the "MAP" keyword
+   * 
+   * @param haveSurface
+   */
+  public void resetForMapping(boolean haveSurface) {
+    if (!haveSurface) 
+      state = Parameters.STATE_DATA_READ;
+    qmOrbitalType = QM_TYPE_UNKNOWN; 
+    parameters = null;
+    colorDensity = false;
+    mappedDataMin = Float.MAX_VALUE;
+    intersection = null;
+    func = null;
+    points = null;
+    origin = null;
+    steps = null;
+    volumeData = null;
+  }
+
+    public void addSlabInfo(Object[] slabObject) {
+    if (slabInfo == null)
+      slabInfo = new ArrayList<Object[]>();
+    slabInfo.add(slabObject);
+  }
+
 }
