@@ -112,6 +112,7 @@
 
 package org.jmol.jvxl.readers;
 
+
 import java.io.BufferedInputStream;
 import java.io.BufferedReader;
 import java.io.IOException;
@@ -119,14 +120,15 @@ import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.StringReader;
 import java.util.BitSet;
-import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.Point3f;
 import javax.vecmath.Point4f;
 import javax.vecmath.Vector3f;
 
 import org.jmol.util.*;
+import org.jmol.atomdata.AtomData;
 import org.jmol.atomdata.AtomDataServer;
 import org.jmol.atomdata.RadiusData;
 import org.jmol.jvxl.data.JvxlCoder;
@@ -189,10 +191,6 @@ public class SurfaceGenerator {
     return params.state == Parameters.STATE_DATA_READ;
   }
 
-  public int getDataType() {
-    return params.dataType;
-  }
-  
   public String getFileName() {
     return params.fileName;
   }
@@ -209,6 +207,10 @@ public class SurfaceGenerator {
     return params.colorEncoder;
   }
 
+  public int[] getVertexSource() {
+    return params.vertexSource;
+  }
+
   public void setJvxlData(JvxlData jvxlData) {
     this.jvxlData = jvxlData;
     if (jvxlData != null)
@@ -256,11 +258,7 @@ public class SurfaceGenerator {
     return params.bsIgnore;
   }
   
-  public List getFunctionXYinfo() {
-    return params.functionXYinfo;
-  }
-
-  VolumeData getVolumeData() {
+  public VolumeData getVolumeData() {
     return volumeData;
   }
 
@@ -286,10 +284,6 @@ public class SurfaceGenerator {
     params.modelIndex = modelIndex;
   }
 
-  public boolean getIUseBitSets() {
-    return params.iUseBitSets;
-  }
-
   public boolean getIAddGridPoints() {
     return params.iAddGridPoints;
   }
@@ -306,7 +300,7 @@ public class SurfaceGenerator {
     return params.cutoff;
   }
   
-  public Hashtable getMoData() {
+  public Map<String, Object> getMoData() {
     return params.moData;
   }
   
@@ -317,7 +311,6 @@ public class SurfaceGenerator {
   //////////////////////////////////////////////////////////////
 
   int colorPtr;
-  private boolean rangeDefined;
 
   /**
    * setParameter is the main interface for surface generation. 
@@ -332,6 +325,14 @@ public class SurfaceGenerator {
     return setParameter(propertyName, value, null);
   }
 
+  /**
+   * 
+   * @param propertyName
+   * @param value
+   * @param bs
+   * @return TRUE if done processing
+   */
+  @SuppressWarnings("unchecked")
   public boolean setParameter(String propertyName, Object value, BitSet bs) {
 
     if ("debug" == propertyName) {
@@ -356,6 +357,12 @@ public class SurfaceGenerator {
       return false; // more to do
     }
 
+    if ("map" == propertyName) {
+      params.resetForMapping(((Boolean)value).booleanValue());
+      if (surfaceReader != null)
+        surfaceReader.minMax = null;
+      return true;
+    }
     if ("finalize" == propertyName) {
       initializeIsosurface();
       return true;
@@ -381,14 +388,24 @@ public class SurfaceGenerator {
     }
 
     if ("withinPoints" == propertyName) {
-      Object[] o = (Object[]) value;
-      params.boundingBox = (Point3f[]) o[1];
+      params.boundingBox = (Point3f[]) ((Object[]) value)[1];
       return true;
     }
-    
+
     if ("boundingBox" == propertyName) {
       Point3f[] pts = (Point3f[]) value;
-      params.boundingBox = new Point3f[] { new Point3f(pts[0]), new Point3f(pts[pts.length - 1]) };
+      params.boundingBox = new Point3f[] { new Point3f(pts[0]),
+          new Point3f(pts[pts.length - 1]) };
+      return true;
+    }
+
+    if ("func" == propertyName) {
+      params.func = value;
+      return true;
+    }
+
+    if ("intersection" == propertyName) {
+      params.intersection = (BitSet[]) value;
       return true;
     }
 
@@ -417,6 +434,11 @@ public class SurfaceGenerator {
       return true;
     }
 
+    if ("propertySmoothingPower" == propertyName) {
+      params.propertySmoothingPower = ((Integer) value).intValue();
+      return true;
+    }
+
     if ("title" == propertyName) {
       if (value == null) {
         params.title = null;
@@ -446,19 +468,22 @@ public class SurfaceGenerator {
       return true;
     }
 
+    if ("parameters" == propertyName) {
+      params.parameters = ArrayUtil.ensureLength((float[]) value, 2);
+      if (params.parameters.length > 0 && params.parameters[0] != 0)
+        params.cutoff = params.parameters[0];
+      return true;
+    }
+    
     if ("cutoffPositive" == propertyName) {
       params.cutoff = ((Float) value).floatValue();
       params.isPositiveOnly = true;
       return true;
     }
 
-    if ("cap" == propertyName) {
-      params.cappingObject = value;
-      return true;
-    }
-
-    if ("slab" == propertyName) {
-      params.slabbingObject = value;
+    if ("cap" == propertyName || "slab" == propertyName) {
+      if (value != null)
+        params.addSlabInfo((Object[]) value);
       return true;
     }
 
@@ -522,11 +547,6 @@ public class SurfaceGenerator {
 
     // / color options
 
-    if ("remappable" == propertyName) {
-      params.remappable = true;
-      return true;
-    }
-
     if ("insideOut" == propertyName) {
       params.insideOut = true;
       return true;
@@ -541,12 +561,19 @@ public class SurfaceGenerator {
 
     if ("colorRGB" == propertyName) {
       int rgb = ((Integer) value).intValue();
-      params.colorPos = params.colorPosLCAO = rgb;
-      if (colorPtr++ == 0)
+      params.colorRgb = params.colorPos = params.colorPosLCAO = rgb;
+      if (colorPtr++ == 0) {
         params.colorNeg = params.colorNegLCAO = rgb;
+      } else {
+        params.colorRgb = Integer.MAX_VALUE;
+      }
       return true;
     }
 
+    if ("monteCarloCount" == propertyName) {
+      params.psi_monteCarloCount = ((Integer) value).intValue();
+      return true;
+    }
     if ("rangeAll" == propertyName) {
       params.rangeAll = true;
       return true;
@@ -591,6 +618,27 @@ public class SurfaceGenerator {
       params.center.set((Point3f) value);
       return true;
     }
+    
+    if ("volumeData" == propertyName) {
+      params.volumeData = (VolumeData) value;
+      return true;
+    }
+      
+
+    if ("origin" == propertyName) {
+      params.origin = (Point3f) value;
+      return true;
+    }
+
+    if ("step" == propertyName) {
+      params.steps = (Point3f) value;
+      return true;
+    }
+
+    if ("point" == propertyName) {
+      params.points = (Point3f) value;
+      return true;
+    }
 
     if ("withinDistance" == propertyName) {
       params.distance = ((Float) value).floatValue();
@@ -641,7 +689,8 @@ public class SurfaceGenerator {
      */
 
     if ("radius" == propertyName) {
-      params.setRadius((RadiusData) value);
+      Logger.info("solvent probe radius set to " + value);
+      params.atomRadiusData = (RadiusData) value;
       return true;
     }
 
@@ -660,8 +709,14 @@ public class SurfaceGenerator {
       return true;
     }
 
+    if ("doFullMolecular" == propertyName) {
+      params.doFullMolecular = true;
+      return true;
+    }
+
     if ("pocket" == propertyName) {
       params.pocket = (Boolean) value;
+      params.fullyLit = params.pocket.booleanValue();
       return true;
     }
 
@@ -775,21 +830,21 @@ public class SurfaceGenerator {
 
     if ("lp" == propertyName) {
       params.setLp((Point4f) value);
-      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15);
+      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15, 0);
       generateSurface();
       return true;
     }
 
     if ("rad" == propertyName) {
       params.setRadical((Point4f) value);
-      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15);
+      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15, 0);
       generateSurface();
       return true;
     }
 
     if ("lobe" == propertyName) {
       params.setLobe((Point4f) value);
-      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15);
+      surfaceReader = new IsoShapeReader(this, 3, 2, 0, 15, 0);
       generateSurface();
       return true;
     }
@@ -800,25 +855,25 @@ public class SurfaceGenerator {
         return true;
       }
       surfaceReader = new IsoShapeReader(this, params.psi_n, params.psi_l,
-          params.psi_m, params.psi_Znuc);
+          params.psi_m, params.psi_Znuc, params.psi_monteCarloCount);
       processState();
       return true;
     }
 
     if ("functionXY" == propertyName) {
-      params.setFunctionXY((List) value);
+      params.setFunctionXY((List<Object>) value);
       if (params.isContoured)
         volumeData.setPlaneParameters(new Point4f(0, 0, 1, 0)); // xy plane
-                                                                // through
-                                                                // origin
-      if (((String) params.functionXYinfo.get(0)).indexOf("_xyz") >= 0)
+      // through
+      // origin
+      if (((String) params.functionInfo.get(0)).indexOf("_xyz") >= 0)
         getFunctionZfromXY();
       processState();
       return true;
     }
 
     if ("functionXYZ" == propertyName) {
-      params.setFunctionXYZ((List) value);
+      params.setFunctionXYZ((List<Object>) value);
       processState();
       return true;
     }
@@ -840,15 +895,12 @@ public class SurfaceGenerator {
         || "sasurface" == propertyName || "nomap" == propertyName) {
       params.setSolvent(propertyName, ((Float) value).floatValue());
       Logger.info(params.calculationType);
-      if (params.state < Parameters.STATE_DATA_READ &&
-          (params.cutoffAutomatic || !params.colorDensity))
-        params.cutoff = 0.0f;
       processState();
       return true;
     }
 
     if ("moData" == propertyName) {
-      params.moData = (Hashtable) value;
+      params.moData = (Map<String, Object>) value;
       return true;
     }
 
@@ -858,25 +910,51 @@ public class SurfaceGenerator {
     }
 
     if ("mep" == propertyName) {
-      params.setMep((float[]) value, rangeDefined, false); // mep charges
+      params.setMep((float[]) value, false); // mep charges
       processState();
       return true;
     }
 
     if ("mlp" == propertyName) {
-      params.setMep((float[]) value, rangeDefined, true); // mlp charges
+      params.setMep((float[]) value, true); // mlp charges
       processState();
       return true;
     }
 
+    if ("nci" == propertyName) {
+      boolean isPromolecular = ((Boolean) value).booleanValue(); 
+      params.setNci(isPromolecular);
+      if (isPromolecular)
+        processState();
+      return true;
+    }
+    
+    if ("calculationType" == propertyName) {
+      params.calculationType = (String) value;
+      return true;
+    }
+
     if ("charges" == propertyName) {
       params.theProperty = (float[]) value;
       return true;
     }
 
+    if ("randomSeed" == propertyName) {
+      // for any object requiring reproduction in the state
+      // and using random numbers -- AO and MO
+      params.randomSeed = ((Integer) value).intValue();
+      return true;
+    }
+    
     if ("molecularOrbital" == propertyName) {
-      int iMo = ((Integer) value).intValue();
-      params.setMO(iMo, rangeDefined);
+      int iMo = 0;
+      float[] linearCombination = null;
+      if (value instanceof Integer) {
+        iMo = ((Integer) value).intValue();
+      } else {
+        linearCombination = (float[]) value;
+      }
+      params.setMO(iMo, linearCombination);
       Logger.info(params.calculationType);
       processState();
       return true;
@@ -896,7 +974,7 @@ public class SurfaceGenerator {
       os = (OutputStream) value;
       return true;
     }
-    
+
     if ("readFile" == propertyName) {
       if ((surfaceReader = setFileData(value)) == null) {
         Logger.error("Could not set the surface data");
@@ -922,6 +1000,10 @@ public class SurfaceGenerator {
       return true;
     }
 
+    if ("periodic" == propertyName) {
+      params.isPeriodic = true;
+    }
+
     // continue with operations in calling class...
     return false;
   }
@@ -956,11 +1038,15 @@ public class SurfaceGenerator {
     case Parameters.SURFACE_PROPERTY:
       surfaceReader = new AtomPropertyMapper(this, null);
       break;
+    case Parameters.SURFACE_INTERSECT:
+      surfaceReader = new IsoIntersectReader(this);
+      break;
     case Parameters.SURFACE_SOLVENT:
     case Parameters.SURFACE_MOLECULAR:
     case Parameters.SURFACE_SASURFACE:
       surfaceReader = new IsoSolventReader(this);
       break;
+    case Parameters.SURFACE_NCI:
     case Parameters.SURFACE_MOLECULARORBITAL:
       surfaceReader = new IsoMOReader(this);
       break;
@@ -983,6 +1069,7 @@ public class SurfaceGenerator {
         surfaceReader = new IsoMlpReader(this);
       break;
     }
+    Logger.info("Using surface reader " + surfaceReader);
     return true;
   }
   
@@ -1013,11 +1100,8 @@ public class SurfaceGenerator {
     if (params.maxSet > 0)
       surfaceReader.excludeMaximumSet();
 
-    if (params.slabbingObject != null)
-      surfaceReader.slabIsosurface(params.slabbingObject, false);
-
-    if (params.cappingObject != null)
-      surfaceReader.slabIsosurface(params.cappingObject, true);
+    if (params.slabInfo != null)
+      surfaceReader.slabIsosurface(params.slabInfo);
 
     if (haveMeshDataServer)
       meshDataServer.notifySurfaceGenerationCompleted();
@@ -1026,6 +1110,9 @@ public class SurfaceGenerator {
       surfaceReader.colorIsosurface();
       params.state = Parameters.STATE_DATA_COLORED;
     }
+    if (jvxlData.jvxlDataIsColorDensity) {
+      params.state = Parameters.STATE_DATA_COLORED;
+    }
     if (params.colorBySign || params.isBicolorMap) {
       params.state = Parameters.STATE_DATA_COLORED;
       surfaceReader.applyColorScale();
@@ -1035,33 +1122,32 @@ public class SurfaceGenerator {
     surfaceReader.discardTempData(false);
     params.mappedDataMin = Float.MAX_VALUE;
     surfaceReader.closeReader();
-    if (surfaceReader.hasColorData || params.colorDensity) {
+    if (params.state != Parameters.STATE_DATA_COLORED &&
+        (surfaceReader.hasColorData || params.colorDensity)) {
       params.state = Parameters.STATE_DATA_COLORED;
       colorIsosurface();
-    } else {
-      surfaceReader = null; // resets voxel reader for mapping
     }
+    surfaceReader = null;
   }
 
   private void mapSurface() {
     if (params.state == Parameters.STATE_INITIALIZED && params.thePlane != null)
       params.state++;
-    if (++params.state != Parameters.STATE_DATA_COLORED)
+    if (++params.state < Parameters.STATE_DATA_COLORED)
       return;
     if (!setReader())
       return;    
     //if (params.dataType == Parameters.SURFACE_FUNCTIONXY)
       //params.thePlane = new Point4f(0, 0, 1, 0);
+    if (params.isPeriodic)
+      volumeData.isPeriodic = true;
     if (params.thePlane != null) {
       boolean isSquared = params.isSquared;
       params.isSquared = false;
       params.cutoff = 0;
-      surfaceReader.setMappingPlane(params.thePlane);
-      surfaceReader.createIsosurface(true);//but don't read volume data yet
-      if (params.slabbingObject != null)
-        surfaceReader.slabIsosurface(params.slabbingObject, false);
-      if (params.cappingObject != null)
-        surfaceReader.slabIsosurface(params.cappingObject, true);
+      volumeData.setMappingPlane(params.thePlane);
+      surfaceReader.createIsosurface(!params.isPeriodic);//but don't read volume data yet
+      volumeData.setMappingPlane(null);
       if (meshDataServer != null)
         meshDataServer.notifySurfaceGenerationCompleted();
       if (params.dataType == Parameters.SURFACE_NOMAP) {
@@ -1071,16 +1157,21 @@ public class SurfaceGenerator {
       }
       params.isSquared = isSquared;
       params.mappedDataMin = Float.MAX_VALUE;
-      surfaceReader.readTheVolumeData(true, params.thePlane);
-    } else if (!params.colorBySets) {
-      surfaceReader.readAndSetVolumeParameters();
+      surfaceReader.readVolumeData(true);
+    } else if (!params.colorBySets && !params.colorDensity) {
+      surfaceReader.readAndSetVolumeParameters(true);
       params.mappedDataMin = Float.MAX_VALUE;
-      surfaceReader.readTheVolumeData(true, null);
+      surfaceReader.readVolumeData(true);
     }
     colorIsosurface();
     surfaceReader.closeReader();
+    surfaceReader = null;
   }
 
+  public List<Object[]> getSlabInfo() {
+    return params.slabInfo;
+  }
+  
   void colorIsosurface() {
     surfaceReader.colorIsosurface();
     surfaceReader.jvxlUpdateInfo();
@@ -1099,6 +1190,7 @@ public class SurfaceGenerator {
     return null;
   }
 
+  @SuppressWarnings("unchecked")
   private SurfaceReader setFileData(Object value) {
     String fileType = this.fileType;
     this.fileType = null;
@@ -1106,13 +1198,15 @@ public class SurfaceGenerator {
       volumeData = (VolumeData) value;
       return new VolumeDataReader(this);
     }
-    if (value instanceof Hashtable) {
-      volumeData = (VolumeData) ((Hashtable) value).get("volumeData");
+    if (value instanceof Map) {
+      volumeData = (VolumeData) ((Map<String, Object>) value).get("volumeData");
       return new VolumeDataReader(this);
     }
     String data = null;
     if (value instanceof String) {
       data = (String) value;
+      // this will be OK, because any string will be a simple string, 
+      // not a binary file.
       value = new BufferedReader(new StringReader((String) value));
     }
     BufferedReader br = (BufferedReader) value;
@@ -1123,7 +1217,7 @@ public class SurfaceGenerator {
       // -- ah, but this does not work, because it is asynchronous!
       // -- first a message is sent back that suggests you might have to wait, 
       //    then a message that says, "Here is your map" is sent.
-      
+
       String fname = params.fileName;
       fname = fname.substring(0, fname.indexOf("/", 10));
       fname += Parser.getNextQuotedString(fileType,
@@ -1164,7 +1258,7 @@ public class SurfaceGenerator {
         // ignore
       }
       br = null;
-      return new MrcBinaryReader(this, params.fileName, data);
+      return new MrcBinaryReader(this, params.fileName);
     }
     if (fileType.equals("DSN6")) {
       try {
@@ -1181,6 +1275,12 @@ public class SurfaceGenerator {
       return new PmeshReader(this, params.fileName, br);
     if (fileType.equals("Obj"))
       return new ObjReader(this, br);
+    if (fileType.equals("Msms"))
+      return new MsmsReader(this, params.fileName, br);
+    if (fileType.equals("Kinemage"))
+      return new KinemageReader(this, br);
+    if (fileType.equals("CastepDensity"))
+      return new CastepDensityReader(this, br);
     return null;
   }
 
@@ -1204,14 +1304,13 @@ public class SurfaceGenerator {
 
   }
 
-  
   private void getFunctionZfromXY() {
-    Point3f origin = (Point3f) params.functionXYinfo.get(1);
+    Point3f origin = (Point3f) params.functionInfo.get(1);
     int[] counts = new int[3];
     int[] nearest = new int[3];
     Vector3f[] vectors = new Vector3f[3];
     for (int i = 0; i < 3; i++) {
-      Point4f info = (Point4f) params.functionXYinfo.get(i + 2);
+      Point4f info = (Point4f) params.functionInfo.get(i + 2);
       counts[i] = Math.abs((int) info.x);
       vectors[i] = new Vector3f(info.y, info.z, info.w);
     }
@@ -1222,7 +1321,7 @@ public class SurfaceGenerator {
     Point3f ptb = new Point3f();
     Point3f ptc = new Point3f();
 
-    float[][] data = (float[][]) params.functionXYinfo.get(5);
+    float[][] data = (float[][]) params.functionInfo.get(5);
     float[][] data2 = new float[nx][ny];
     float[] d;
     //int n = 0;
@@ -1244,7 +1343,7 @@ public class SurfaceGenerator {
         data2[i][j] = pt.z;
         //System.out.println("draw pt"+(++n)+" " + Escape.escape(pt) + " color red");
       }
-    params.functionXYinfo.set(5, data2);
+    params.functionInfo.set(5, data2);
   }
 
   final Vector3f vAC = new Vector3f();
@@ -1291,4 +1390,43 @@ public class SurfaceGenerator {
     result[2] = i3;
     return dist1;
   }
+
+  public void addRequiredFile(String fileName) {
+    if (meshDataServer == null)
+      return;
+    meshDataServer.addRequiredFile(fileName);    
+  }
+
+  void log(String msg) {
+    if (atomDataServer == null)
+      System.out.println(msg);
+    else
+      atomDataServer.log(msg);
+  }
+
+  void setOutputStream(BinaryDocument binaryDoc, OutputStream os) {
+    if (meshDataServer == null)
+      return;
+     meshDataServer.setOutputStream(binaryDoc, os);    
+  }
+
+  public boolean isFullyLit() {
+    return (params.thePlane != null || params.fullyLit);
+  }
+
+  BitSet bsVdw;
+  
+  public BitSet geVdwBitSet() {
+    return bsVdw;
+  }
+  
+  void fillAtomData(AtomData atomData, int mode) {
+    if ((mode & AtomData.MODE_FILL_RADII) != 0 
+        && atomData.bsSelected != null) {
+      if (bsVdw == null)
+        bsVdw = new BitSet();
+      bsVdw.or(atomData.bsSelected);
+    }
+    atomDataServer.fillAtomData(atomData, mode);
+  }
 }
