@@ -25,14 +25,15 @@ package org.jmol.jvxl.readers;
 
 import java.io.OutputStream;
 import java.util.BitSet;
+import java.util.List;
 
 import javax.vecmath.Matrix3f;
 import javax.vecmath.Point3f;
 import javax.vecmath.Point3i;
-import javax.vecmath.Point4f;
 import javax.vecmath.Vector3f;
 
 import org.jmol.util.*;
+import org.jmol.api.QuantumPlaneCalculationInterface;
 import org.jmol.g3d.Graphics3D;
 import org.jmol.jvxl.data.*;
 import org.jmol.jvxl.api.MeshDataServer;
@@ -192,11 +193,12 @@ public abstract class SurfaceReader implements VertexDataServer {
   protected VolumeData volumeData;
   private String edgeData;
 
+  protected boolean haveSurfaceAtoms = false;
   protected boolean allowSigma = false;
   protected boolean isProgressive = false;
   protected boolean isXLowToHigh = false; //can be overridden in some readers by --progressive
   private float assocCutoff = 0.3f;
-  protected Point4f mappingPlane;
+  protected boolean isQuiet;
   
   boolean vertexDataOnly;
   boolean hasColorData;
@@ -250,33 +252,34 @@ public abstract class SurfaceReader implements VertexDataServer {
   
   abstract protected void closeReader();
   
-//  boolean mustCalcPoint = true; // for now
-
+  /**
+   * 
+   * @param os
+   */
   protected void setOutputStream(OutputStream os) {
     // only for file readers
   }
  
-  void setVolumeData(VolumeData v) {
+  protected void newVoxelDataCube() {
+    volumeData.setVoxelData(voxelData = new float[nPointsX][nPointsY][nPointsZ]);
+  }
+
+  protected void setVolumeData(VolumeData v) {
     nBytes = 0;
     volumetricOrigin = v.volumetricOrigin;
     volumetricVectors = v.volumetricVectors;
     voxelCounts = v.voxelCounts;
-    voxelData = v.voxelData;
+    voxelData = v.getVoxelData();
     volumeData = v;
     
 /*    if (mustCalcPoint)
       v.setDataSource(this);
 */  }
 
-  protected abstract boolean readVolumeParameters();
+  protected abstract boolean readVolumeParameters(boolean isMapData);
 
   protected abstract boolean readVolumeData(boolean isMapData);
 
-  boolean readTheVolumeData(boolean isMapData, Point4f plane) {
-    mappingPlane = plane;
-    return readVolumeData(isMapData);  
-  }
-  
   ////////////////////////////////////////////////////////////////
   // CUBE/APBS/JVXL file reading stuff
   ////////////////////////////////////////////////////////////////
@@ -300,6 +303,7 @@ public abstract class SurfaceReader implements VertexDataServer {
   protected boolean jvxlDataIsColorMapped;
   protected boolean jvxlDataIsPrecisionColor;
   protected boolean jvxlDataIs2dContour;
+  protected boolean jvxlDataIsColorDensity;
   protected float jvxlCutoff;
   protected int jvxlNSurfaceInts;
   protected char cJvxlEdgeNaN;
@@ -310,18 +314,30 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.jvxlUpdateInfo(params.title, nBytes);
   }
 
-  boolean readAndSetVolumeParameters() {
-    return (readVolumeParameters() && (vertexDataOnly || volumeData.setUnitVectors()));
+  boolean readAndSetVolumeParameters(boolean isMapData) {
+    if (!readVolumeParameters(isMapData))
+        return false;
+    if (vertexDataOnly)
+      return true;
+    //if (volumeData.sr != null)
+      //return true;
+    return (volumeData.setUnitVectors());
+    
+//        && (vertexDataOnly 
+//            || isMapData && params.thePlane != null 
+//            || volumeData.sr != null
+//            || volumeData.setUnitVectors()));
   }
 
   boolean createIsosurface(boolean justForPlane) {
     resetIsosurface();
     jvxlData.cutoff = Float.NaN;
-    if (!readAndSetVolumeParameters())
+    if (!readAndSetVolumeParameters(justForPlane))
       return false;
     if (!justForPlane && !Float.isNaN(params.sigma) && !allowSigma) {
       if (params.sigma > 0)
-        Logger.error("Reader does not support SIGMA option -- using cutoff 1.6");
+        Logger
+            .error("Reader does not support SIGMA option -- using cutoff 1.6");
       params.cutoff = 1.6f;
     }
     // negative sigma just ignores the error message
@@ -331,6 +347,7 @@ public abstract class SurfaceReader implements VertexDataServer {
     nPointsX = voxelCounts[0];
     nPointsY = voxelCounts[1];
     nPointsZ = voxelCounts[2];
+    jvxlData.isSlabbable = ((params.dataType & Parameters.IS_SLABBABLE) != 0);
     jvxlData.insideOut = params.insideOut;
     jvxlData.dataXYReversed = params.dataXYReversed;
     jvxlData.isBicolorMap = params.isBicolorMap;
@@ -338,33 +355,40 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.nPointsY = nPointsY;
     jvxlData.nPointsZ = nPointsZ;
     jvxlData.jvxlVolumeDataXml = volumeData.xmlData;
+    jvxlData.voxelVolume = volumeData.voxelVolume;
     if (justForPlane) {
-      float[][][] voxelDataTemp =  volumeData.voxelData;
-      volumeData.setDataDistanceToPlane(params.thePlane);
+      //float[][][] voxelDataTemp =  volumeData.voxelData;
+      volumeData.setMappingPlane(params.thePlane);
+      //volumeData.setDataDistanceToPlane(params.thePlane);
       if (meshDataServer != null)
         meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_VERTICES, null);
       params.setMapRanges(this, false);
       generateSurfaceData();
-      volumeData.voxelData = voxelDataTemp;
+      volumeData.setMappingPlane(null);
+
+      //if (volumeData != null)
+      //  volumeData.voxelData = voxelDataTemp;
     } else {
-      if (!readTheVolumeData(false, null))
+      if (!readVolumeData(false))
         return false;
       generateSurfaceData();
     }
-    String s = jvxlFileHeaderBuffer.toString();
-    int i = s.indexOf('\n', s.indexOf('\n',s.indexOf('\n') + 1) + 1) + 1;
-    jvxlData.jvxlFileTitle = s.substring(0, i);
-    jvxlData.jvxlFileHeader = s;
-    if (xyzMin == null)
+    if (jvxlFileHeaderBuffer != null) {
+      String s = jvxlFileHeaderBuffer.toString();
+      int i = s.indexOf('\n', s.indexOf('\n', s.indexOf('\n') + 1) + 1) + 1;
+      jvxlData.jvxlFileTitle = s.substring(0, i);
+    }
+    if (params.contactPair == null)
       setBoundingBox();
     if (!params.isSilent)
-      Logger.info("boundbox corners " + Escape.escape(xyzMin) + " " + Escape.escape(xyzMax));
-    jvxlData.boundingBox = new Point3f[] {xyzMin, xyzMax};
+      Logger.info("boundbox corners " + Escape.escape(xyzMin) + " "
+          + Escape.escape(xyzMax));
+    jvxlData.boundingBox = new Point3f[] { xyzMin, xyzMax };
     jvxlData.dataMin = dataMin;
     jvxlData.dataMax = dataMax;
     jvxlData.cutoff = (isJvxl ? jvxlCutoff : params.cutoff);
     jvxlData.isCutoffAbsolute = params.isCutoffAbsolute;
-    jvxlData.pointsPerAngstrom = 1f/volumeData.volumetricVectorLengths[0];
+    jvxlData.pointsPerAngstrom = 1f / volumeData.volumetricVectorLengths[0];
     jvxlData.jvxlColorData = "";
     jvxlData.jvxlPlane = params.thePlane;
     jvxlData.jvxlEdgeData = edgeData;
@@ -373,8 +397,8 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.colorDensity = params.colorDensity;
     if (jvxlData.vContours != null)
       params.nContours = jvxlData.vContours.length;
-    jvxlData.nContours = (params.contourFromZero 
-        ? params.nContours : -1 - params.nContours);
+    jvxlData.nContours = (params.contourFromZero ? params.nContours : -1
+        - params.nContours);
     jvxlData.nEdges = edgeCount;
     jvxlData.edgeFractionBase = edgeFractionBase;
     jvxlData.edgeFractionRange = edgeFractionRange;
@@ -382,14 +406,16 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.colorFractionRange = colorFractionRange;
     jvxlData.jvxlDataIs2dContour = jvxlDataIs2dContour;
     jvxlData.jvxlDataIsColorMapped = jvxlDataIsColorMapped;
+    jvxlData.jvxlDataIsColorDensity = jvxlDataIsColorDensity;
     jvxlData.isXLowToHigh = isXLowToHigh;
     jvxlData.vertexDataOnly = vertexDataOnly;
     jvxlData.saveVertexCount = 0;
 
-    if (jvxlDataIsColorMapped) {
+    if (jvxlDataIsColorMapped || jvxlData.nVertexColors > 0) {
       if (meshDataServer != null) {
         meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_VERTICES, null);
-        meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_COLOR_INDEXES, null);
+        meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_COLOR_INDEXES,
+            null);
       }
       jvxlData.jvxlColorData = readColorData();
       updateSurfaceData();
@@ -452,6 +478,12 @@ public abstract class SurfaceReader implements VertexDataServer {
     return true;
   }
 
+  /**
+   * 
+   * @param n
+   * @param nPoints
+   * @throws Exception
+   */
   protected void gotoData(int n, int nPoints) throws Exception {
     //only for file reader
   }
@@ -466,16 +498,36 @@ public abstract class SurfaceReader implements VertexDataServer {
   ////////////////////////////////////////////////////////////////
 
   protected MarchingSquares marchingSquares;
-  private MarchingCubes marchingCubes;
+  protected MarchingCubes marchingCubes;
+
+  protected float[][] yzPlanes;
+  protected int yzCount;
+
+  protected QuantumPlaneCalculationInterface qpc;
+  
+  public float[] getPlane(int x) {
+    if (yzCount == 0)
+      initPlanes();
+    if (qpc != null)
+      qpc.getPlane(x, yzPlanes[x % 2]);
+    return yzPlanes[x % 2];
+  }
+
+  protected void initPlanes() {
+    yzCount = nPointsY * nPointsZ;
+    if (!isQuiet)
+      Logger.info("reading data progressively -- yzCount = " + yzCount);
+    yzPlanes = new float[2][];
+    yzPlanes[0] = new float[yzCount];
+    yzPlanes[1] = new float[yzCount];
+  }
 
   public float getValue(int x, int y, int z, int ptyz) {
-    return volumeData.voxelData[x][y][z];
+    if (yzPlanes == null)
+      return voxelData[x][y][z];
+    return yzPlanes[x % 2][ptyz];
   }
 
-  public void getPlane(int x) {
-    // implementation specific -- for progressive readers
-    // MrcBinaryReader does this
-  }
   private void generateSurfaceData() {
     edgeData = "";
     if (vertexDataOnly) {
@@ -511,8 +563,13 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.jvxlExcluded = params.bsExcluded;
     if (isJvxl)
       edgeData = jvxlEdgeDataRead;
+    postProcessVertices();
   }
 
+  protected void postProcessVertices() {
+    // optional
+  }
+  
   /////////////////  MarchingReader Interface Methods ///////////////////
 
   protected final Point3f ptTemp = new Point3f();
@@ -523,7 +580,7 @@ public abstract class SurfaceReader implements VertexDataServer {
                                   Point3f pointA, Vector3f edgeVector,
                                   boolean isContourType, float[] fReturn) {
     float thisValue = getSurfacePointAndFraction(cutoff, isCutoffAbsolute, valueA,
-        valueB, pointA, edgeVector, fReturn, ptTemp);
+        valueB, pointA, edgeVector, x, y, z, vA, vB, fReturn, ptTemp);
     /* 
      * from MarchingCubes
      * 
@@ -533,10 +590,9 @@ public abstract class SurfaceReader implements VertexDataServer {
      * we take every point.
      * 
      */
-
+        
     if (marchingSquares != null && params.isContoured)
-      return marchingSquares.addContourVertex(x, y, z, offset,
-          ptTemp, cutoff);
+      return marchingSquares.addContourVertex(ptTemp, cutoff);
     int assocVertex = (assocCutoff > 0 ? (fReturn[0] < assocCutoff ? vA
         : fReturn[0] > 1 - assocCutoff ? vB : MarchingSquares.CONTOUR_POINT)
         : MarchingSquares.CONTOUR_POINT);
@@ -553,12 +609,30 @@ public abstract class SurfaceReader implements VertexDataServer {
     return n;
   }
 
+  /**
+   * 
+   * @param cutoff
+   * @param isCutoffAbsolute
+   * @param valueA
+   * @param valueB
+   * @param pointA
+   * @param edgeVector
+   * @param x TODO
+   * @param y TODO
+   * @param z TODO
+   * @param vA
+   * @param vB
+   * @param fReturn
+   * @param ptReturn
+   * @return          fractional distance from A to B
+   */
   protected float getSurfacePointAndFraction(float cutoff, boolean isCutoffAbsolute,
                                    float valueA, float valueB, Point3f pointA,
-                                   Vector3f edgeVector, float[] fReturn,
-                                   Point3f ptReturn) {
+                                   Vector3f edgeVector, int x,
+                                   int y, int z, int vA, int vB, float[] fReturn, Point3f ptReturn) {
 
     //JvxlReader may or may not call this
+    //IsoSolventReader overrides this for nonlinear Marching Cubes (12.1.29)
 
     float diff = valueB - valueA;
     float fraction = (cutoff - valueA) / diff;
@@ -621,7 +695,7 @@ public abstract class SurfaceReader implements VertexDataServer {
           params.valueMappedToBlue);
       jvxlData.saveVertexCount = marchingSquares.contourVertexCount;
       contourVertexCount = marchingSquares
-         .generateContourData(jvxlDataIs2dContour, (params.isSquared ? 1e-8f : 1e-4f));      
+          .generateContourData(jvxlDataIs2dContour, (params.isSquared ? 1e-8f : 1e-4f));      
       jvxlData.contourValuesUsed = marchingSquares.getContourValues();
       minMax = marchingSquares.getMinMax();
       if (meshDataServer != null)
@@ -645,13 +719,18 @@ public abstract class SurfaceReader implements VertexDataServer {
       meshData.vertexColixes = new short[meshData.vertexCount];
     } else {
       meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_VERTICES, null);
-      meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_COLOR_INDEXES, null);
+      if (params.contactPair == null)
+        meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_COLOR_INDEXES,
+            null);
     }
     //colorBySign is true when colorByPhase is true, but not vice-versa
     //old: boolean saveColorData = !(params.colorByPhase && !params.isBicolorMap && !params.colorBySign); //sorry!
-    boolean saveColorData = (params.colorDensity || params.isBicolorMap || params.colorBySign || !params.colorByPhase);
+    boolean saveColorData = (params.colorDensity || params.isBicolorMap
+        || params.colorBySign || !params.colorByPhase);
+    if (params.contactPair != null)
+      saveColorData = false;
     // colors mappable always now
-    jvxlData.isJvxlPrecisionColor = true;//(jvxlDataIsPrecisionColor || params.isContoured || params.remappable);
+    jvxlData.isJvxlPrecisionColor = true;
     jvxlData.vertexCount = (contourVertexCount > 0 ? contourVertexCount
         : meshData.vertexCount);
     jvxlData.minColorIndex = -1;
@@ -659,37 +738,46 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.contourValues = params.contoursDiscrete;
     jvxlData.isColorReversed = params.isColorReversed;
     if (!params.colorDensity)
-    if (params.isBicolorMap && !params.isContoured || params.colorBySign) {
-      jvxlData.minColorIndex = Graphics3D.getColix(params.isColorReversed ? params.colorPos
-              : params.colorNeg);
-      jvxlData.maxColorIndex = Graphics3D.getColix(params.isColorReversed ? params.colorNeg
-              : params.colorPos);
-    }
+      if (params.isBicolorMap && !params.isContoured || params.colorBySign) {
+        jvxlData.minColorIndex = Graphics3D
+        .getColixTranslucent(Graphics3D.getColix(params.isColorReversed ? params.colorPos
+            : params.colorNeg), jvxlData.translucency != 0, jvxlData.translucency);
+        jvxlData.maxColorIndex = Graphics3D
+        .getColixTranslucent(Graphics3D.getColix(params.isColorReversed ? params.colorNeg
+            : params.colorPos), jvxlData.translucency != 0, jvxlData.translucency);
+      }
     jvxlData.isTruncated = (jvxlData.minColorIndex >= 0 && !params.isContoured);
-    boolean useMeshDataValues = jvxlDataIs2dContour ||
-    //      !jvxlDataIs2dContour && (params.isContoured && jvxlData.jvxlPlane != null || 
-    vertexDataOnly || params.colorDensity || params.isBicolorMap && !params.isContoured;
+    boolean useMeshDataValues = jvxlDataIs2dContour
+        ||
+        //      !jvxlDataIs2dContour && (params.isContoured && jvxlData.jvxlPlane != null || 
+        hasColorData || vertexDataOnly || params.colorDensity || params.isBicolorMap
+        && !params.isContoured;
     if (!useMeshDataValues) {
+      if (haveSurfaceAtoms && meshData.vertexSource == null)
+        meshData.vertexSource = new int[meshData.vertexCount];
       float min = Float.MAX_VALUE;
       float max = -Float.MAX_VALUE;
       float value;
       initializeMapping();
-      for (int i = meshData.vertexCount; --i >= 0;) {
+      for (int i = meshData.vertexCount; --i >= meshData.mergeVertexCount0;) {
         /* right, so what we are doing here is setting a range within the 
          * data for which we want red-->blue, but returning the actual
          * number so it can be encoded more precisely. This turned out to be
          * the key to making the JVXL contours work.
          *  
-         */        
-        if (params.colorBySets)
+         */
+        if (params.colorBySets) {
           value = meshData.vertexSets[i];
-        else if (params.colorByPhase)
+        } else if (params.colorByPhase) {
           value = getPhase(meshData.vertices[i]);
         //else if (jvxlDataIs2dContour)
         //marchingSquares
-          //    .getInterpolatedPixelValue(meshData.vertices[i]);
-        else
+        //    .getInterpolatedPixelValue(meshData.vertices[i]);
+        } else {
           value = volumeData.lookupInterpolatedVoxelValue(meshData.vertices[i]);
+          if (haveSurfaceAtoms)
+            meshData.vertexSource[i] = getSurfaceAtomIndex();
+        }
         if (value < min)
           min = value;
         if (value > max && value != Float.MAX_VALUE)
@@ -705,15 +793,19 @@ public abstract class SurfaceReader implements VertexDataServer {
     jvxlData.mappedDataMax = params.mappedDataMax;
     jvxlData.valueMappedToRed = params.valueMappedToRed;
     jvxlData.valueMappedToBlue = params.valueMappedToBlue;
-    colorData();
-
+    if (params.contactPair == null)
+      colorData();
+    
     JvxlCoder.jvxlCreateColorData(jvxlData,
         (saveColorData ? meshData.vertexValues : null));
 
+    if (haveSurfaceAtoms && meshDataServer != null)
+      meshDataServer.fillMeshData(meshData, MeshData.MODE_PUT_VERTICES, null);
+
     if (meshDataServer != null && params.colorBySets)
       meshDataServer.fillMeshData(meshData, MeshData.MODE_PUT_SETS, null);
   }
-  
+
   private void colorData() {
 
     float[] vertexValues = meshData.vertexValues;
@@ -735,11 +827,10 @@ public abstract class SurfaceReader implements VertexDataServer {
         else if (value > 0)
           vertexColixes[i] = maxColorIndex;
       } else {
-        if (value < valueRed)
+        if (value <= valueRed)
           value = valueRed;
         if (value >= valueBlue)
-          value = valueBlue;
-          
+          value = valueBlue;         
         vertexColixes[i] = params.colorEncoder.getColorIndex(value);
       }
     }
@@ -830,7 +921,7 @@ public abstract class SurfaceReader implements VertexDataServer {
         : meshData.vertexCount);
     Point3f[] vertexes = meshData.vertices;
     boolean useVertexValue = (haveData || jvxlDataIs2dContour || vertexDataOnly || params.colorDensity);
-    for (int i = 0; i < vertexCount; i++) {
+    for (int i = meshData.mergeVertexCount0; i < vertexCount; i++) {
       float v;
       if (useVertexValue)
         v = meshData.vertexValues[i];
@@ -860,6 +951,10 @@ public abstract class SurfaceReader implements VertexDataServer {
     meshData.updateInvalidatedVertices(params.bsExcluded[1]);
   }
 
+  /**
+   * 
+   * @param doExclude
+   */
   public void selectPocket(boolean doExclude) {
     // solvent reader implements this
   }
@@ -892,12 +987,10 @@ public abstract class SurfaceReader implements VertexDataServer {
       meshDataServer.fillMeshData(meshData, MeshData.MODE_PUT_SETS, null);
   }
   
-  public void slabIsosurface(Object slabbingObject, boolean andCap) {
+  public void slabIsosurface(List<Object[]> slabInfo) {
     if (meshDataServer != null)
       meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_VERTICES, null);
-    if (meshData.polygonIndexes == null)
-      return;
-    meshData.slabPolygons(slabbingObject, andCap);
+    meshData.slabPolygons(slabInfo, true);
     if (meshDataServer != null)
       meshDataServer.fillMeshData(meshData, MeshData.MODE_PUT_VERTICES, null);
   }
@@ -932,28 +1025,10 @@ public abstract class SurfaceReader implements VertexDataServer {
     volumetricOrigin.set(center);
   }
 
-  protected void setBoundingBox(Point3f pt, float margin) {
-    if (xyzMin == null) {
-      xyzMin = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
-      xyzMax = new Point3f(-Float.MAX_VALUE, -Float.MAX_VALUE, -Float.MAX_VALUE);
-    }
-    if (pt.x - margin < xyzMin.x)
-      xyzMin.x = pt.x - margin;
-    if (pt.x + margin > xyzMax.x)
-      xyzMax.x = pt.x + margin;
-    if (pt.y - margin < xyzMin.y)
-      xyzMin.y = pt.y - margin;
-    if (pt.y + margin > xyzMax.y)
-      xyzMax.y = pt.y + margin;
-    if (pt.z - margin < xyzMin.z)
-      xyzMin.z = pt.z - margin;
-    if (pt.z + margin > xyzMax.z)
-      xyzMax.z = pt.z + margin;
-  }
-
   private void setBoundingBox() {
     if (meshDataServer != null)
       meshDataServer.fillMeshData(meshData, MeshData.MODE_GET_VERTICES, null);
+    xyzMin = null;
     for (int i = 0; i < meshData.vertexCount; i++) {
       Point3f p = meshData.vertices[i];
       if (!Float.isNaN(p.x))
@@ -961,15 +1036,14 @@ public abstract class SurfaceReader implements VertexDataServer {
     }
   }
 
-  public void setMappingPlane(Point4f thePlane) {
-    mappingPlane = thePlane;
+  protected void setBoundingBox(Point3f pt, float margin) {
+    if (xyzMin == null) {
+      xyzMin = new Point3f(Float.MAX_VALUE, Float.MAX_VALUE, Float.MAX_VALUE);
+      xyzMax = new Point3f(-Float.MAX_VALUE, -Float.MAX_VALUE, -Float.MAX_VALUE);
+    }
+    BoxInfo.addPoint(pt, xyzMin, xyzMax, margin);
   }
 
-  /**
-   * 
-   * @param pt
-   * @return   value
-   */
   /**
    * 
    * @param pt
@@ -988,5 +1062,9 @@ public abstract class SurfaceReader implements VertexDataServer {
     // release any iterators
   }
 
+  public int getSurfaceAtomIndex() {
+    // atomPropertyMapper
+    return -1;
+  }
 
 }
