@@ -24,12 +24,12 @@
 
 package org.jmol.minimize;
 
+import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
-import org.jmol.api.JmolEdge;
 import org.jmol.api.MinimizerInterface;
 import org.jmol.i18n.GT;
 import org.jmol.minimize.forcefield.ForceField;
@@ -40,6 +40,7 @@ import org.jmol.util.ArrayUtil;
 import org.jmol.util.BitSetUtil;
 import org.jmol.util.Elements;
 import org.jmol.util.Escape;
+import org.jmol.util.JmolEdge;
 import org.jmol.util.Logger;
 import org.jmol.util.Parser;
 
@@ -64,7 +65,7 @@ public class Minimizer implements MinimizerInterface {
   private int steps = 50;
   private double crit = 1e-3;
 
-  private static List atomTypes;
+  private static List<String[]> atomTypes;
   private ForceField pFF;
   private String ff = "UFF";
   private BitSet bsTaint, bsSelected, bsAtoms;
@@ -72,7 +73,7 @@ public class Minimizer implements MinimizerInterface {
   private BitSet bsFixed;
   private BitSet bsAromatic;
   
-  public List constraints;
+  public List<Object[]> constraints;
   
   private boolean isSilent;
   
@@ -116,7 +117,7 @@ public class Minimizer implements MinimizerInterface {
     return null;
   }
   
-  private Hashtable constraintMap;
+  private Map<String, Object[]> constraintMap;
 
   private void addConstraint(Object[] c) {
     if (c == null)
@@ -128,8 +129,8 @@ public class Minimizer implements MinimizerInterface {
       return;
     }
     if (constraints == null) {
-      constraints = new ArrayList();
-      constraintMap = new Hashtable();
+      constraints = new ArrayList<Object[]>();
+      constraintMap = new Hashtable<String, Object[]>();
     }
     if (atoms[1] > atoms[nAtoms]) {
         ArrayUtil.swap(atoms, 1, nAtoms);
@@ -137,7 +138,7 @@ public class Minimizer implements MinimizerInterface {
           ArrayUtil.swap(atoms, 2, 3);
     }
     String id = Escape.escape(atoms);
-    Object[] c1 = (Object[]) constraintMap.get(id);
+    Object[] c1 = constraintMap.get(id);
     if (c1 != null) {
       c1[2] = c[2]; // just set target value
       return;
@@ -235,7 +236,7 @@ public class Minimizer implements MinimizerInterface {
 
     if (constraints != null) {
       for (int i = constraints.size(); --i >= 0;) {
-        Object[] constraint = (Object[]) constraints.get(i);
+        Object[] constraint = constraints.get(i);
         int[] aList = (int[]) constraint[0];
         int[] minList = (int[]) constraint[1];
         int nAtoms = aList[0] = Math.abs(aList[0]);
@@ -286,7 +287,7 @@ public class Minimizer implements MinimizerInterface {
     Logger.info("minimize: creating bonds...");
 
     // add all bonds
-    List bondInfo = new ArrayList();
+    List<int[]> bondInfo = new ArrayList<int[]>();
     bondCount = 0;
     for (int i = bsAtoms.nextSetBit(0); i >= 0; i = bsAtoms.nextSetBit(i + 1)) {
       Bond[] bonds = atoms[i].getBonds();
@@ -307,8 +308,7 @@ public class Minimizer implements MinimizerInterface {
               bondOrder = 1;
             }
             bondCount++;
-            bondInfo
-                .add(new int[] { atomMap[i], atomMap[i2], bondOrder });
+            bondInfo.add(new int[] { atomMap[i], atomMap[i2], bondOrder });
           }
         }
     }
@@ -316,8 +316,7 @@ public class Minimizer implements MinimizerInterface {
 
     minBonds = new MinBond[bondCount];
     for (int i = 0; i < bondCount; i++) {
-      MinBond bond = minBonds[i] = new MinBond(atomIndexes = (int[]) bondInfo
-          .get(i), false, false);
+      MinBond bond = minBonds[i] = new MinBond(atomIndexes = bondInfo.get(i), false, false);
       int atom1 = atomIndexes[0];
       int atom2 = atomIndexes[1];
       minAtoms[atom1].bonds.add(bond);
@@ -340,7 +339,7 @@ public class Minimizer implements MinimizerInterface {
     int nElements = atomTypes.size();
     bsElements.clear(0);
     for (int i = 0; i < nElements; i++) {
-      String[] data = ((String[]) atomTypes.get(i));
+      String[] data = atomTypes.get(i);
       String smarts = data[0];
       if (smarts == null)
         continue;
@@ -587,7 +586,7 @@ Token[keyword(0x880001) value=")"]
   
   public void getAngles() {
 
-    List vAngles = new ArrayList();
+    List<int[]> vAngles = new ArrayList<int[]>();
 
     for (int ib = 0; ib < atomCount; ib++) {
       MinAtom atomB = minAtoms[ib];
@@ -609,13 +608,13 @@ Token[keyword(0x880001) value=")"]
     
     angles = new int[vAngles.size()][];
     for (int i = vAngles.size(); --i >= 0; )
-      angles[i] = (int[]) vAngles.get(i);
+      angles[i] = vAngles.get(i);
     Logger.info(angles.length + " angles");
   }
 
   public void getTorsions() {
 
-    List vTorsions = new ArrayList();
+    List<int[]> vTorsions = new ArrayList<int[]>();
 
     // extend all angles a-b-c by one, but only
     // when when c > b -- other possibility will be found
@@ -657,7 +656,7 @@ Token[keyword(0x880001) value=")"]
 
     torsions = new int[vTorsions.size()][];
     for (int i = vTorsions.size(); --i >= 0;)
-      torsions[i] = (int[]) vTorsions.get(i);
+      torsions[i] = vTorsions.get(i);
 
     Logger.info(torsions.length + " torsions");
 
@@ -685,7 +684,7 @@ Token[keyword(0x880001) value=")"]
     return pFF;
   }
   
-  public List getAtomTypes() {
+  public List<String[]> getAtomTypes() {
     getForceField();
     return (pFF == null ? null : pFF.getAtomTypes());
   }
@@ -827,12 +826,13 @@ Token[keyword(0x880001) value=")"]
     endMinimization();
   }
   
-  class MinimizationThread extends Thread implements Runnable {
+  class MinimizationThread extends Thread {
     
     MinimizationThread() {
       this.setName("MinimizationThread");
     }
     
+    @Override
     public void run() {
       long startTime = System.currentTimeMillis();
       long lastRepaintTime = startTime;
