@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-11-05 17:22:54 +0100 (sam., 05 nov. 2011) $
- * $Revision: 16457 $
+ * $Date: 2012-05-04 00:21:25 +0200 (ven., 04 mai 2012) $
+ * $Revision: 17072 $
 
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
@@ -866,23 +866,22 @@ final public class Atom extends Point3fi implements JmolNode {
   String getIdentity(boolean allInfo) {
     StringBuffer info = new StringBuffer();
     String group3 = getGroup3(true);
-    String seqcodeString = getSeqcodeString();
-    char chainID = getChainID();
     if (group3 != null && group3.length() > 0) {
       info.append("[");
       info.append(group3);
       info.append("]");
-    }
-    if (seqcodeString != null)
-      info.append(seqcodeString);
-    if (chainID != 0 && chainID != ' ') {
-      info.append(":");
-      info.append(chainID);
-    }
-    if (!allInfo)
-      return info.toString();
-    if (info.length() > 0)
+      String seqcodeString = getSeqcodeString();
+      if (seqcodeString != null)
+        info.append(seqcodeString);
+      char chainID = getChainID();
+      if (chainID != 0 && chainID != ' ') {
+        info.append(":");
+        info.append(chainID);
+      }
+      if (!allInfo)
+        return info.toString();
       info.append(".");
+    }
     info.append(getAtomName());
     if (info.length() == 0) {
       // since atomName cannot be null, this is unreachable
