@@ -25,21 +25,24 @@
 
 package org.jmol.modelset;
 
+
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.List;
 
 import org.jmol.util.ArrayUtil;
 import org.jmol.util.BitSetUtil;
+import org.jmol.util.JmolEdge;
+import org.jmol.util.JmolMolecule;
 import org.jmol.util.Logger;
 
 import org.jmol.viewer.JmolConstants;
-import org.jmol.api.JmolEdge;
-import org.jmol.api.JmolMolecule;
+import org.jmol.modelset.Bond.BondSet;
 import org.jmol.script.Token;
 
 abstract public class BondCollection extends AtomCollection {
 
+  @Override
   protected void releaseModelSet() {
     bonds = null;
     freeBonds = null;
@@ -50,6 +53,11 @@ abstract public class BondCollection extends AtomCollection {
   protected JmolMolecule[] molecules = new JmolMolecule[4];
   protected int moleculeCount;
 
+  protected void resetMolecules() {
+    molecules = null;
+    moleculeCount = 0;
+  }
+
   protected Bond[] bonds;
   protected int bondCount;
   
@@ -89,7 +97,7 @@ abstract public class BondCollection extends AtomCollection {
   }
 
   public int getBondOrder(int i) {
-    return bonds[i].getOrder();
+    return bonds[i].order;
   }
 
   public short getBondColix1(int i) {
@@ -131,11 +139,17 @@ abstract public class BondCollection extends AtomCollection {
     return bs;
   }
 
-  public Bond bondAtoms(Atom atom1, Atom atom2, int order, short mad, BitSet bsBonds, float energy, boolean isNew) {
+  public Bond bondAtoms(Atom atom1, Atom atom2, int order, short mad, BitSet bsBonds, float energy, boolean addGroup, boolean isNew) {
     // this method used when a bond must be flagged as new
     Bond bond = getOrAddBond(atom1, atom2, order, mad, bsBonds, energy, true);
-    if (isNew)
+    if (isNew) {
       bond.order |= JmolEdge.BOND_NEW;
+      if (addGroup) {
+        // for adding hydrogens
+        atom1.group = atom2.group;
+        atom1.group.addAtoms(atom1.index);
+      }
+    }
     return bond;
   }
 
@@ -172,7 +186,7 @@ abstract public class BondCollection extends AtomCollection {
 
   protected Bond bondMutually(Atom atom, Atom atomOther, int order, short mad, float energy) {
     Bond bond;
-    if ((order & JmolEdge.BOND_HYDROGEN_MASK) != 0) {
+    if (Bond.isHydrogen(order)) {
       bond = new HBond(atom, atomOther, order, mad, (short) 0, energy);
     } else {
       bond = new Bond(atom, atomOther, order, mad, (short) 0);
@@ -231,12 +245,12 @@ abstract public class BondCollection extends AtomCollection {
   protected BitSet bsHBondsRasmol;
 
   void getRasmolHydrogenBonds(Model m, BitSet bsA, BitSet bsB,
-                              List vHBonds, boolean nucleicOnly,
+                              List<Bond> vHBonds, boolean nucleicOnly,
                               int nMax, boolean dsspIgnoreHydrogens) {
     boolean doAdd = (vHBonds == null);
     Polymer bp, bp1;
     if (doAdd)
-      vHBonds = new ArrayList();
+      vHBonds = new ArrayList<Bond>();
     if (nMax < 0)
       nMax = Integer.MAX_VALUE;
     boolean asDSSP = (bsB == null);
@@ -287,7 +301,7 @@ abstract public class BondCollection extends AtomCollection {
     return setBond(bondCount++, bondMutually(atom1, atom2, order, (short) 1, energy)).index;
   }
 
-  protected short getBondOrder(float bondingRadiusA,
+  protected static short getBondOrder(float bondingRadiusA,
                              float bondingRadiusB, float distance2,
                              float minBondDistance2, float bondTolerance) {
     if (bondingRadiusA == 0 || bondingRadiusB == 0 || distance2 < minBondDistance2)
@@ -340,8 +354,8 @@ abstract public class BondCollection extends AtomCollection {
    * @param order
    * @return if hydrogen bond, default to 1; otherwise 0 (general default)
    */
-  protected short getDefaultMadFromOrder(int order) {
-    return (short) ((order & JmolEdge.BOND_HYDROGEN_MASK) != 0 ? 1
+  public short getDefaultMadFromOrder(int order) {
+    return (short) (Bond.isHydrogen(order) ? 1
         : (order & JmolEdge.BOND_STRUT) != 0 ? (int) (viewer
             .getStrutDefaultRadius() * 2000) : defaultCovalentMad);
   }
@@ -362,7 +376,7 @@ abstract public class BondCollection extends AtomCollection {
     BitSet bsDelete = new BitSet();
     int nDeleted = 0;
     int newOrder = order |= JmolEdge.BOND_NEW;
-    if (!matchNull && (order & JmolEdge.BOND_HYDROGEN_MASK) != 0)
+    if (!matchNull && Bond.isHydrogen(order))
       order = JmolEdge.BOND_HYDROGEN_MASK;
     BitSet bsBonds;
     if (isBonds) {
@@ -408,8 +422,7 @@ abstract public class BondCollection extends AtomCollection {
     int iDst = bsBond.nextSetBit(0);
     if (iDst < 0)
       return;
-    molecules = null;
-    moleculeCount = 0;
+    resetMolecules();
     int modelIndexLast = -1;
     int n = bsBond.cardinality();
     for (int iSrc = iDst; iSrc < bondCount; ++iSrc) {
@@ -432,8 +445,9 @@ abstract public class BondCollection extends AtomCollection {
     bondCount = iDst;
     BitSet[] sets = (BitSet[]) viewer.getShapeProperty(
         JmolConstants.SHAPE_STICKS, "sets");
-    for (int i = 0; i < sets.length; i++)
-      BitSetUtil.deleteBits(sets[i], bsBond);
+    if (sets != null)
+      for (int i = 0; i < sets.length; i++)
+        BitSetUtil.deleteBits(sets[i], bsBond);
     BitSetUtil.deleteBits(bsHBondsRasmol, bsBond);
     BitSetUtil.deleteBits(bsAromatic, bsBond);
   }
@@ -768,9 +782,12 @@ abstract public class BondCollection extends AtomCollection {
       }
   }
 
-  protected BitSet getAtomBits(int tokType, Object specInfo) {
+  @Override
+  protected BitSet getAtomBitsMaybeDeleted(int tokType, Object specInfo) {
     BitSet bs;
     switch (tokType) {
+    default:
+      return super.getAtomBitsMaybeDeleted(tokType, specInfo);
     case Token.isaromatic:
       bs = new BitSet();
       for (int i = bondCount; --i >= 0;)
@@ -788,7 +805,6 @@ abstract public class BondCollection extends AtomCollection {
       }
       return bs;
     }
-    return super.getAtomBits(tokType, specInfo);
   }
  
   public BitSet setBondOrder(int bondIndex, char type) {
@@ -802,7 +818,7 @@ abstract public class BondCollection extends AtomCollection {
       break;
     case 'p':
     case 'm':
-      bondOrder = JmolConstants.getBondOrderNumberFromOrder(
+      bondOrder = JmolEdge.getBondOrderNumberFromOrder(
           bond.getCovalentOrder()).charAt(0)
           - '0' + (type == 'p' ? 1 : -1);
       if (bondOrder > 3)
@@ -855,5 +871,16 @@ abstract public class BondCollection extends AtomCollection {
     if (bs.nextSetBit(0) >= 0)
       viewer.deleteAtoms(bs, false);
   }
+  
+  protected boolean haveHiddenBonds;
+  
+  public void displayBonds(BondSet bs, boolean isDisplay) {
+    if (!isDisplay)
+      haveHiddenBonds = true;
+    for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1))
+      if (i < bondCount && bonds[i].mad != 0)
+        bonds[i].setShapeVisibility(isDisplay);
+  }
+
 }
 
