@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-02-24 23:48:47 +0100 (jeu., 24 févr. 2011) $
- * $Revision: 15225 $
+ * $Date: 2011-09-20 22:53:47 +0200 (mar., 20 sept. 2011) $
+ * $Revision: 16086 $
 
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
@@ -25,25 +25,36 @@
 
 package org.jmol.modelset;
 
-import org.jmol.util.ArrayUtil;
 import org.jmol.util.BitSetUtil;
 import org.jmol.util.Escape;
+import org.jmol.util.JmolEdge;
+import org.jmol.util.JmolMolecule;
+import org.jmol.util.Logger;
+import org.jmol.util.Measure;
+import org.jmol.util.Quaternion;
 
 import org.jmol.viewer.JmolConstants;
+import org.jmol.viewer.StateManager;
+import org.jmol.viewer.Viewer;
+import org.jmol.modelset.Bond.BondSet;
 import org.jmol.script.Token;
 import org.jmol.api.Interface;
-import org.jmol.api.JmolEdge;
+import org.jmol.api.JmolAdapter;
 import org.jmol.api.SymmetryInterface;
-import org.jmol.atomdata.AtomData;
 import org.jmol.atomdata.RadiusData;
 import org.jmol.shape.Shape;
 
+import java.util.ArrayList;
 import java.util.BitSet;
-import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 
+import javax.vecmath.Matrix3f;
+import javax.vecmath.Matrix4f;
 import javax.vecmath.Point3f;
+import javax.vecmath.Point4f;
 import javax.vecmath.Tuple3f;
+import javax.vecmath.Vector3f;
 
 /*
  * An abstract class always created using new ModelLoader(...)
@@ -67,10 +78,16 @@ import javax.vecmath.Tuple3f;
  * Bob Hanson, 5/2007, 10/2007
  * 
  */
-abstract public class ModelSet extends ModelCollection {
+ public class ModelSet extends ModelCollection {
 
   ////////////////////////////////////////////////////////////////
 
+  public ModelSet(Viewer viewer, String name) {
+    this.viewer = viewer;
+    modelSetName = name;
+  }
+
+  @Override
   protected void releaseModelSet() {
     models = null;
     closest[0] = null;
@@ -157,7 +174,7 @@ abstract public class ModelSet extends ModelCollection {
   }
 
   public void setTrajectory(int modelIndex) {
-    if (modelIndex < 0 || !models[modelIndex].isTrajectory)
+    if (modelIndex < 0 || !isTrajectory(modelIndex))
       return;
     // The user has used the MODEL command to switch to a new set of atom coordinates
     // Or has specified a trajectory in a select, display, or hide command.
@@ -168,18 +185,25 @@ abstract public class ModelSet extends ModelCollection {
       return;
     int baseModel = models[modelIndex].trajectoryBaseIndex;
     models[baseModel].selectedTrajectory = modelIndex;
-    Point3f[] trajectory = (Point3f[]) trajectorySteps.get(modelIndex);
+    Point3f[] trajectory = trajectorySteps.get(modelIndex);
+    Vector3f[] vibrations = (vibrationSteps == null ? null : vibrationSteps.get(modelIndex));
     BitSet bs = new BitSet();
+    Vector3f vib = new Vector3f();
     int iMax = iFirst + getAtomCountInModel(baseModel);
-    for (int pt = 0, i = iFirst; i < iMax && pt < trajectory.length && trajectory[pt]!= null; i++) {
-      atoms[i].setFractionalCoord(trajectory[pt++], true);
+    for (int pt = 0, i = iFirst; i < iMax && pt < trajectory.length && trajectory[pt]!= null; i++, pt++) {
+      atoms[i].setFractionalCoord(trajectory[pt], true);
       atoms[i].modelIndex = (short) modelIndex;
+      if (vibrationSteps != null) {
+        if (vibrations != null && vibrations[pt] != null)
+          vib = vibrations[pt];
+          setVibrationVector(i, vib.x, vib.y, vib.z);
+      }
       bs.set(i);
     }
     // Clear the Binary Search so that select within(),
     // isosurface, and dots will work properly
     initializeBspf();
-    bspf.clearBspt(baseModel);
+    validateBspf(baseModel, false);
     // Recalculate critical points for cartoons and such
     // note that models[baseModel] and models[modelIndex]
     // point to the same model. So there is only one copy of 
@@ -189,7 +213,7 @@ abstract public class ModelSet extends ModelCollection {
     recalculateLeadMidpointsAndWingVectors(baseModel);
     // Recalculate all measures that involve trajectories
 
-    shapeManager.refreshShapeTrajectories(baseModel, bs);
+    shapeManager.refreshShapeTrajectories(baseModel, bs, null);
 
     if (models[baseModel].hasRasmolHBonds) {
       clearRasmolHydrogenBonds(baseModel, null);
@@ -245,10 +269,14 @@ abstract public class ModelSet extends ModelCollection {
    * @param specInfo  
    * @return bitset; null only if we mess up with name
    */
+  @Override
   public BitSet getAtomBits(int tokType, Object specInfo) {
     switch (tokType) {
     case Token.spec_model:
-      return getSpecModel(((Integer) specInfo).intValue());
+      int modelNumber = ((Integer) specInfo).intValue();
+      int modelIndex = getModelNumberIndex(modelNumber, true, true);
+      return (modelIndex < 0 && modelNumber > 0 ? new BitSet()
+          : viewer.getModelUndeletedAtomsBitSet(modelIndex));
     }
     return super.getAtomBits(tokType, specInfo);
   }
@@ -257,12 +285,6 @@ abstract public class ModelSet extends ModelCollection {
     return (String) viewer.getShapeProperty(JmolConstants.SHAPE_LABELS, "label", i);
   }
   
-  private BitSet getSpecModel(int modelNumber) {
-    int modelIndex = getModelNumberIndex(modelNumber, true, true);
-    return (modelIndex < 0 && modelNumber > 0 ? new BitSet()
-        : viewer.getModelUndeletedAtomsBitSet(modelIndex));
-  }
-
   protected final Atom[] closest = new Atom[1];
 
   public int findNearestAtomIndex(int x, int y, BitSet bsNot) {
@@ -281,7 +303,7 @@ abstract public class ModelSet extends ModelCollection {
   }
 
   /*
-  private Hashtable userProperties;
+  private Map userProperties;
 
   void putUserProperty(String name, Object property) {
     if (userProperties == null)
@@ -293,43 +315,22 @@ abstract public class ModelSet extends ModelCollection {
   }
 */  
 
-  ////////////////// atomData filling ////////////
-
-  public void fillAtomData(AtomData atomData, int mode) {
-    if (mode == AtomData.MODE_GET_ATTACHED_HYDROGENS) {
-      int[] nH = new int[1];
-      atomData.hAtomRadius = viewer.getVanderwaalsMar(1) / 1000f;
-      atomData.hAtoms = calculateHydrogens(atomData.bsSelected, nH, false, true, null);
-      atomData.hydrogenAtomCount = nH[0];
-      return;
-    }
-    if(atomData.modelIndex < 0)
-      atomData.firstAtomIndex = (atomData.bsSelected == null ? 0 : Math.max(0, atomData.bsSelected.nextSetBit(0)));
-    else
-      atomData.firstAtomIndex = models[atomData.modelIndex].firstAtomIndex;
-    atomData.lastModelIndex = atomData.firstModelIndex = (atomCount == 0 ? 0 : atoms[atomData.firstAtomIndex].modelIndex);
-    atomData.modelName = getModelNumberDotted(atomData.firstModelIndex);
-    super.fillAtomData(atomData, mode);
-  }
-
-
   ///////// atom and shape selecting /////////
-  
-  
+
   public String calculateStructures(BitSet bsAtoms, boolean asDSSP,
                                     boolean dsspIgnoreHydrogen,
                                     boolean setStructure) {
     BitSet bsAllAtoms = new BitSet();
-    BitSet bsDefined = BitSetUtil.copyInvert(modelsOf(bsAtoms, bsAllAtoms),
+    BitSet bsModelsExcluded = BitSetUtil.copyInvert(modelsOf(bsAtoms, bsAllAtoms),
         modelCount);
     if (!setStructure)
-      return calculateStructuresAllExcept(bsDefined, asDSSP, true,
+      return calculateStructuresAllExcept(bsModelsExcluded, asDSSP, true,
           dsspIgnoreHydrogen, false, false);
     for (int i = 0; i < modelCount; i++)
-      if (!bsDefined.get(i))
+      if (!bsModelsExcluded.get(i))
         addBioPolymerToModel(null, models[i]);
-    calculatePolymers(0, bsDefined);
-    String ret = calculateStructuresAllExcept(bsDefined, asDSSP, true,
+    calculatePolymers(null, 0, 0, bsModelsExcluded);
+    String ret = calculateStructuresAllExcept(bsModelsExcluded, asDSSP, true,
         dsspIgnoreHydrogen, true, false);
     viewer.resetBioshapes(bsAllAtoms);
     setStructureIds();
@@ -341,8 +342,9 @@ abstract public class ModelSet extends ModelCollection {
         false, false, null, 0, 0);
   }
 
-  public Hashtable getPointGroupInfo(BitSet bsAtoms) {
-    return (Hashtable) calculatePointGroupForFirstModel(bsAtoms, false,
+  @SuppressWarnings("unchecked")
+  public Map<String, Object> getPointGroupInfo(BitSet bsAtoms) {
+    return (Map<String, Object>) calculatePointGroupForFirstModel(bsAtoms, false,
         false, true, null, 0, 0);
   }
   
@@ -405,6 +407,15 @@ abstract public class ModelSet extends ModelCollection {
     return bsModels;
   }
 
+  public String getDefaultStructure(BitSet bsAtoms, BitSet bsAllAtoms) {
+    BitSet bsModels = modelsOf(bsAtoms, bsAllAtoms);
+    StringBuffer ret = new StringBuffer();
+    for (int i = bsModels.nextSetBit(0); i >= 0; i = bsModels.nextSetBit(i + 1)) 
+      if (models[i].isPDB && models[i].defaultStructure != null)
+        ret.append(models[i].defaultStructure);
+    return ret.toString();
+  }
+
 
   
   ///// super-overloaded methods ///////
@@ -420,10 +431,11 @@ abstract public class ModelSet extends ModelCollection {
       shapeManager.setShapeSize(JmolConstants.SHAPE_STICKS, Integer.MIN_VALUE, null, bsAromatic);
   }
 
+  @Override
   public int[] makeConnections(float minDistance, float maxDistance, int order,
                                int connectOperation, BitSet bsA, BitSet bsB,
-                               BitSet bsBonds, boolean isBonds, float energy) {
-    if (connectOperation == JmolConstants.CONNECT_AUTO_BOND
+                               BitSet bsBonds, boolean isBonds, boolean addGroup, float energy) {
+    if (connectOperation == Token.auto
         && order != JmolEdge.BOND_H_REGULAR) {
       String stateScript = "connect ";
       if (minDistance != JmolConstants.DEFAULT_MIN_CONNECT_DISTANCE)
@@ -435,21 +447,21 @@ abstract public class ModelSet extends ModelCollection {
     }
     moleculeCount = 0;
     return super.makeConnections(minDistance, maxDistance, order,
-        connectOperation, bsA, bsB, bsBonds, isBonds, energy);
+        connectOperation, bsA, bsB, bsBonds, isBonds, addGroup, energy);
   }
   
+  @SuppressWarnings("unchecked")
   public void setPdbConectBonding(int baseAtomIndex, int baseModelIndex,
                                   BitSet bsExclude) {
     short mad = viewer.getMadBond();
     for (int i = baseModelIndex; i < modelCount; i++) {
-      List vConnect = (List) getModelAuxiliaryInfo(i, "PDB_CONECT_bonds");
+      List<int[]> vConnect = (List<int[]>) getModelAuxiliaryInfo(i, "PDB_CONECT_bonds");
       if (vConnect == null)
         continue;
       int nConnect = vConnect.size();
       setModelAuxiliaryInfo(i, "initialBondCount", Integer.valueOf(nConnect));
-      int[] atomInfo = (int[]) getModelAuxiliaryInfo(i,
-          "PDB_CONECT_firstAtom_count_max");
-      int firstAtom = atomInfo[0] + baseAtomIndex;
+      int[] atomInfo = (int[]) getModelAuxiliaryInfo(i, "PDB_CONECT_firstAtom_count_max");
+      int firstAtom = atomInfo[0] +  baseAtomIndex;
       int atomMax = firstAtom + atomInfo[1];
       int max = atomInfo[2];
       int[] serialMap = new int[max + 1];
@@ -458,7 +470,7 @@ abstract public class ModelSet extends ModelCollection {
         if ((iSerial = atomSerials[iAtom]) > 0)
           serialMap[iSerial] = iAtom + 1;
       for (int iConnect = 0; iConnect < nConnect; iConnect++) {
-        int[] pair = (int[]) vConnect.get(iConnect);
+        int[] pair = vConnect.get(iConnect);
         int sourceSerial = pair[0];
         int targetSerial = pair[1];
         short order = (short) pair[2];
@@ -481,11 +493,14 @@ abstract public class ModelSet extends ModelCollection {
     }
   }
   
+  @Override
   public void deleteAllBonds() {
     moleculeCount = 0;
-    for (int i = stateScripts.size(); --i >= 0;) 
-      if (((StateScript) stateScripts.get(i)).isConnect())
+    for (int i = stateScripts.size(); --i >= 0;) { 
+      if (stateScripts.get(i).isConnect()) {
         stateScripts.remove(i);
+      }
+    }
     super.deleteAllBonds();
   }
 
@@ -504,8 +519,8 @@ abstract public class ModelSet extends ModelCollection {
     StringBuffer commands = new StringBuffer();
     String cmd;
     for (int i = 0; i < len; i++) {
-      StateScript ss = (StateScript) stateScripts.get(i); 
-      if (!ss.postDefinitions && (cmd = ss.toString()).length() > 0) {
+      StateScript ss = stateScripts.get(i); 
+      if (ss.inDefinedStateBlock && (cmd = ss.toString()).length() > 0) {
         commands.append("  ").append(cmd).append("\n");
         haveDefs = true;
       }
@@ -524,7 +539,8 @@ abstract public class ModelSet extends ModelCollection {
     return cmd + commands.toString();
   }
   
-  public String getState(StringBuffer sfunc, boolean isAll, boolean withProteinStructure) {
+  public String getState(StringBuffer sfunc, boolean isAll,
+                         boolean withProteinStructure) {
     StringBuffer commands = new StringBuffer();
     if (isAll && sfunc != null) {
       sfunc.append("  _setModelState;\n");
@@ -538,31 +554,49 @@ abstract public class ModelSet extends ModelCollection {
 
       int len = stateScripts.size();
       for (int i = 0; i < len; i++) {
-        StateScript ss = (StateScript) stateScripts.get(i);
-        if (ss.postDefinitions && (cmd = ss.toString()).length() > 0)
+        StateScript ss = stateScripts.get(i);
+        if (!ss.inDefinedStateBlock && (cmd = ss.toString()).length() > 0) {
           commands.append("  ").append(cmd).append("\n");
+        }
       }
 
-      StringBuffer sb = new StringBuffer();
+      StringBuilder sb = new StringBuilder();
       for (int i = 0; i < bondCount; i++)
         if (!models[bonds[i].atom1.modelIndex].isModelKit)
           if (bonds[i].isHydrogen()
               || (bonds[i].order & JmolEdge.BOND_NEW) != 0) {
             Bond bond = bonds[i];
-            sb.append(bond.atom1.index)
-                .append('\t').append(bond.atom2.index)
-                .append('\t').append(bond.order & ~JmolEdge.BOND_NEW)
-                .append('\t').append(bond.mad / 1000f) // DARN! diameter, not radius... oh well...
-                .append('\t').append(bond.getEnergy())
-                .append('\t').append(JmolConstants.getBondOrderNameFromOrder(bond.order))
-                .append(";\n");
+            int index = bond.atom1.index;
+            if (bond.atom1.getGroup().isAdded(index))
+              index = -1 - index;
+            sb.append(index).append('\t').append(bond.atom2.index).append('\t')
+                .append(bond.order & ~JmolEdge.BOND_NEW).append('\t').append(
+                    bond.mad / 1000f).append('\t').append(bond.getEnergy())
+                .append('\t').append(
+                    JmolEdge.getBondOrderNameFromOrder(bond.order)).append(
+                    ";\n");
           }
       if (sb.length() > 0)
-        commands.append("data \"connect_atoms\"\n")
-            .append(sb).append("end \"connect_atoms\";\n");
+        commands.append("data \"connect_atoms\"\n").append(sb).append(
+            "end \"connect_atoms\";\n");
       commands.append("\n");
     }
 
+    // bond visibility
+
+    if (haveHiddenBonds) {
+      BondSet bs = new BondSet();
+      for (int i = bondCount; --i >= 0;)
+        if (bonds[i].mad != 0
+            && (bonds[i].shapeVisibilityFlags & Bond.myVisibilityFlag) == 0)
+          bs.set(i);
+      if (bs.isEmpty())
+        haveHiddenBonds = false;
+      else
+        commands.append("  hide ").append(Escape.escape(bs, false)).append(
+            ";\n");
+    }
+
     // shape construction
 
     viewer.setModelVisibility();
@@ -570,24 +604,48 @@ abstract public class ModelSet extends ModelCollection {
     // unnecessary. Removed in 11.5.35 -- oops!
 
     if (withProteinStructure)
-      commands.append(getProteinStructureState(null, isAll, false, false));
+      commands.append(getProteinStructureState(null, isAll, false, 0));
 
     viewer.getShapeState(commands, isAll);
 
-    boolean needOrientations =  false;
-    for (int i = 0; i < modelCount; i++)
-      if (models[i].isJmolDataFrame) {
-        needOrientations = true;
-        break;
+    if (isAll) {
+      boolean needOrientations = false;
+      for (int i = 0; i < modelCount; i++)
+        if (models[i].isJmolDataFrame) {
+          needOrientations = true;
+          break;
+        }
+      for (int i = 0; i < modelCount; i++) {
+        String t = frameTitles[i];
+        if (t != null && t.length() > 0)
+          commands.append("  frame " + getModelNumberDotted(i)
+              + "; frame title " + Escape.escape(t) + ";\n");
+        if (needOrientations && models[i].orientation != null
+            && !isTrajectorySubFrame(i))
+          commands.append("  frame " + getModelNumberDotted(i) + "; "
+              + models[i].orientation.getMoveToText(false) + "\n");
       }
-    for (int i = 0; i < modelCount; i++) {
-      String t = frameTitles[i];
-      if (t != null && t.length() > 0)
-        commands.append("  frame " + getModelNumberDotted(i)
-            + "; frame title " + Escape.escape(t) + ";\n");
-      if (needOrientations && models[i].orientation != null && !isTrajectorySubFrame(i))
-        commands.append("  frame " + getModelNumberDotted(i) + "; "
-            + models[i].orientation.getMoveToText(false) + "\n");
+
+      if (unitCells != null) {
+        for (int i = 0; i < modelCount; i++) {
+          SymmetryInterface symmetry = getUnitCell(i);
+          if (symmetry == null)
+            continue;
+          commands.append("  frame ").append(getModelNumberDotted(i));
+          Point3f pt = symmetry.getFractionalOffset();
+          if (pt != null)
+            commands.append("; set unitcell ").append(Escape.escape(pt));
+          pt = symmetry.getUnitCellMultiplier();
+          if (pt != null)
+            commands.append("; set unitcell ").append(Escape.escape(pt));
+          commands.append(";\n");
+        }
+        if (viewer.getObjectMad(StateManager.OBJ_UNITCELL) == 0)
+          commands.append("  unitcell OFF;\n");
+      }
+      commands.append("  set fontScaling " + viewer.getFontScaling() + ";\n");
+      if (viewer.isModelKitMode())
+        commands.append("  set modelKitMode true;\n");
     }
     if (sfunc != null)
       commands.append("\n}\n\n");
@@ -641,7 +699,7 @@ abstract public class ModelSet extends ModelCollection {
 
     // zero out reproducible arrays
 
-    bspf = null;
+    validateBspf(false);
 
     // create a new models array,
     // and pre-calculate Model.bsAtoms and Model.atomCount
@@ -663,6 +721,7 @@ abstract public class ModelSet extends ModelCollection {
     deleteBonds(bsBonds, true);
 
     // main deletion cycle
+    
     for (int i = 0, mpt = 0; i < oldModelCount; i++) {
       if (!bsModels.get(i)) {
         mpt++;
@@ -674,10 +733,6 @@ abstract public class ModelSet extends ModelCollection {
       nAtomsDeleted += nAtoms;
       BitSet bs = oldModels[i].bsAtoms;
       int firstAtomIndex = oldModels[i].firstAtomIndex;
-      int firstGroupIndex = atoms[firstAtomIndex].getGroupIndex();
-      int nGroups = atoms[firstAtomIndex + nAtoms - 1].getGroupIndex() - firstGroupIndex + 1;
-      groups = (Group[]) ArrayUtil.deleteElements(groups, firstGroupIndex, nGroups);
-      groupCount -= nGroups;
 
       // delete from symmetry set
       BitSetUtil.deleteBits(bsSymmetry, bs);
@@ -701,6 +756,7 @@ abstract public class ModelSet extends ModelCollection {
     return bsDeleted;
   }
 
+  @Override
   public void setAtomProperty(BitSet bs, int tok, int iValue, float fValue,
                               String sValue, float[] values, String[] list) {
     switch (tok) {
@@ -725,9 +781,7 @@ abstract public class ModelSet extends ModelCollection {
           fValue = 0;
         mar = (int) (fValue * 2000);
       } else {
-        rd = new RadiusData();
-        rd.values = values;
-        rd.value = Integer.MAX_VALUE;
+        rd = new RadiusData(values);
       }
       shapeManager
           .setShapeSize(JmolConstants.shapeTokenIndex(tok), mar, rd, bs);
@@ -736,10 +790,11 @@ abstract public class ModelSet extends ModelCollection {
     super.setAtomProperty(bs, tok, iValue, fValue, sValue, values, list);
   }
   
+  @SuppressWarnings("unchecked")
   public Object getFileData(int modelIndex) {
     if (modelIndex < 0)
       return "";
-    Hashtable fileData = (Hashtable) getModelAuxiliaryInfo(modelIndex, "fileData");
+    Map<String, Object> fileData = (Map<String, Object>) getModelAuxiliaryInfo(modelIndex, "fileData");
     if (fileData != null)
       return fileData;
     if (!getModelAuxiliaryInfoBoolean(modelIndex, "isCIF"))
@@ -758,6 +813,7 @@ abstract public class ModelSet extends ModelCollection {
    * @param bs2
    * @return     number of struts found
    */
+  @Override
   public int calculateStruts(BitSet bs1, BitSet bs2) {
     viewer.setModelVisibility();
     return super.calculateStruts(bs1, bs2);
@@ -771,23 +827,24 @@ abstract public class ModelSet extends ModelCollection {
    * @param pts
    * @return            BitSet of new atoms
    */
-  public BitSet addHydrogens(List vConnections, Point3f[] pts) {
+  public BitSet addHydrogens(List<Atom> vConnections, Point3f[] pts) {
     int modelIndex = modelCount - 1;
     BitSet bs = new BitSet();
-    if (models[modelIndex].isTrajectory || models[modelIndex].getGroupCount() > 1)
+    if (isTrajectory(modelIndex) || models[modelIndex].getGroupCount() > 1) {
       return bs; // can't add atoms to a trajectory or a system with multiple groups!
+    }
     growAtomArrays(atomCount + pts.length);
     RadiusData rd = viewer.getDefaultRadiusData();
     short mad = getDefaultMadFromOrder(1);
     for (int i = 0, n = models[modelIndex].atomCount + 1; i < vConnections.size(); i++, n++) {
-      Atom atom1 = (Atom) vConnections.get(i);
+      Atom atom1 = vConnections.get(i);
       // hmm. atom1.group will not be expanded, though...
       // something like within(group,...) will not select these atoms!
       Atom atom2 = addAtom(modelIndex, atom1.group, (short) 1, "H"
           + n, n, n, pts[i].x, pts[i].y, pts[i].z);
       atom2.setMadAtom(viewer, rd);
       bs.set(atom2.index);
-      bondAtoms(atom1, atom2, JmolEdge.BOND_COVALENT_SINGLE, mad, null, 0, false);
+      bondAtoms(atom1, atom2, JmolEdge.BOND_COVALENT_SINGLE, mad, null, 0, false, false);
     }
     // must reset the shapes to give them new atom counts and arrays
     shapeManager.loadDefaultShapes(this);
@@ -796,9 +853,13 @@ abstract public class ModelSet extends ModelCollection {
 
   public void setAtomCoordRelative(Tuple3f offset, BitSet bs) {
     setAtomCoordRelative(bs, offset.x, offset.y, offset.z);
-    recalculatePositionDependentQuantities(bs);
+    mat4.setIdentity();
+    vTemp.set(offset);
+    mat4.setTranslation(vTemp);
+    recalculatePositionDependentQuantities(bs, mat4);
   }
 
+  @Override
   public void setAtomCoord(BitSet bs, int tokType, Object xyzValues) {
     super.setAtomCoord(bs, tokType, xyzValues);
     switch(tokType) {
@@ -808,35 +869,201 @@ abstract public class ModelSet extends ModelCollection {
     case Token.vibxyz:
       break;
     default:
-      recalculatePositionDependentQuantities(bs);
+      recalculatePositionDependentQuantities(bs, null);
     }
   }
 
-  public void recalculatePositionDependentQuantities(BitSet bs) {
+  public void invertSelected(Point3f pt, Point4f plane, int iAtom,
+                             BitSet invAtoms, BitSet bs) {
+    if (pt != null) {
+      for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
+        float x = (pt.x - atoms[i].x) * 2;
+        float y = (pt.y - atoms[i].y) * 2;
+        float z = (pt.z - atoms[i].z) * 2;
+        setAtomCoordRelative(i, x, y, z);
+      }
+      return;
+    }
+    if (plane != null) {
+      // ax + by + cz + d = 0
+      Vector3f norm = new Vector3f(plane.x, plane.y, plane.z);
+      norm.normalize();
+      float d = (float) Math.sqrt(plane.x * plane.x + plane.y * plane.y
+          + plane.z * plane.z);
+      for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
+        float twoD = -Measure.distanceToPlane(plane, d, atoms[i]) * 2;
+        float x = norm.x * twoD;
+        float y = norm.y * twoD;
+        float z = norm.z * twoD;
+        setAtomCoordRelative(i, x, y, z);
+      }
+      return;
+    }
+    if (iAtom >= 0) {
+      Atom thisAtom = atoms[iAtom];
+      // stereochemical inversion at iAtom
+      Bond[] bonds = thisAtom.bonds;
+      if (bonds == null)
+        return;
+      BitSet bsAtoms = new BitSet();
+      List<Point3f> vNot = new ArrayList<Point3f>();
+      BitSet bsModel = viewer.getModelUndeletedAtomsBitSet(thisAtom.modelIndex);
+      for (int i = 0; i < bonds.length; i++) {
+        Atom a = bonds[i].getOtherAtom(thisAtom);
+        if (invAtoms.get(a.index)) {
+            bsAtoms.or(JmolMolecule.getBranchBitSet(atoms, a.index, bsModel, null, iAtom, true, true));
+        } else {
+          vNot.add(a);
+        }
+      }
+      if (vNot.size() == 0)
+        return;
+      pt = Measure.getCenterAndPoints(vNot)[0];
+      Vector3f v = new Vector3f(thisAtom);
+      v.sub(pt);
+      Quaternion q = new Quaternion(v, 180);
+      moveAtoms(null, q.getMatrix(), null, bsAtoms, thisAtom, true);
+    }
+  }
+
+  private final Matrix3f matTemp = new Matrix3f();
+  private final Matrix3f matInv = new Matrix3f();
+  private final Matrix4f mat4 = new Matrix4f();
+  private final Matrix4f mat4t = new Matrix4f();
+  private final Vector3f vTemp = new Vector3f();
+
+  public void moveAtoms(Matrix3f mNew, Matrix3f matrixRotate,
+                        Vector3f translation, BitSet bs, Point3f center,
+                        boolean isInternal) {
+    if (mNew == null) {
+      matTemp.set(matrixRotate);
+    } else {
+      matInv.set(matrixRotate);
+      matInv.invert();
+      ptTemp.set(0, 0, 0);
+      matTemp.mul(mNew, matrixRotate);
+      matTemp.mul(matInv, matTemp);
+    }
+    if (isInternal) {
+      vTemp.set(center);
+      mat4.setIdentity();
+      mat4.setTranslation(vTemp);
+      mat4t.set(matTemp);
+      mat4.mul(mat4t);
+      mat4t.setIdentity();
+      vTemp.scale(-1);
+      mat4t.setTranslation(vTemp);
+      mat4.mul(mat4t);
+    } else {
+      mat4.set(matTemp);
+    }
+    for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
+      if (isInternal) {
+        mat4.transform(atoms[i]);
+      } else {
+        ptTemp.add(atoms[i]);
+        mat4.transform(atoms[i]);
+        ptTemp.sub(atoms[i]);
+      }
+      taint(i, TAINT_COORD);
+    }
+    if (!isInternal) {
+      ptTemp.scale(1f / bs.cardinality());
+      if (translation == null)
+        translation = new Vector3f();
+      translation.add(ptTemp);
+    }
+    if (translation != null) {
+      for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1))
+        atoms[i].add(translation);
+      mat4t.setIdentity();
+      mat4t.setTranslation(translation);
+      mat4.mul(mat4t, mat4);
+    }
+    recalculatePositionDependentQuantities(bs, mat4);
+  }
+
+  
+  /*
+ 
+ static {
+
+    Point3f pt = new Point3f(-1, 2, 3);
+    Point3f center = new Point3f(.2f,.4f,.5f);
+    Matrix3f matTemp = (new Quaternion(.2f, .3f, .4f, .5f)).getMatrix();
+    
+    Matrix4f mat4 = new Matrix4f();
+    Matrix4f mat4t = new Matrix4f();
+    Vector3f vTemp = new Vector3f();
+
+    vTemp.set(center);
+    mat4.setIdentity();
+    mat4.setTranslation(vTemp);
+    mat4t.set(matTemp);
+    mat4.mul(mat4t);
+    mat4t.setIdentity();
+    vTemp.scale(-1);
+    mat4t.setTranslation(vTemp);
+    mat4.mul(mat4t);
+
+    Point3f pt1 = new Point3f(pt);
+    System.out.println(pt);    
+    pt1.sub(center);
+    matTemp.transform(pt1);
+    pt1.add(center);
+    System.out.println(pt1);
+    //mat4.transform(pt);
+
+    vTemp.set(2,3,4);
+    pt1.set(pt);
+    mat4.transform(pt1);
+    pt1.add(vTemp);
+    System.out.println(pt1);
+    
+    
+    mat4t.setIdentity();
+    mat4t.setTranslation(vTemp);
+    mat4.mul(mat4t, mat4);
+    pt1.set(pt);
+    mat4.transform(pt1);
+    System.out.println(pt1);
+    
+    
+    
+    
+    // mat4 == (1) rot, then (2) trans
+    
+    System.out.println("HHH MODELSET");
+  }
+*/
+  public void recalculatePositionDependentQuantities(BitSet bs, Matrix4f mat) {
     if (getHaveStraightness())
       calculateStraightness();
     recalculateLeadMidpointsAndWingVectors(-1);
     BitSet bsModels = getModelBitSet(bs, false);
     for (int i = bsModels.nextSetBit(0); i >= 0; i = bsModels.nextSetBit(i + 1))
-      shapeManager.refreshShapeTrajectories(i, viewer.getModelUndeletedAtomsBitSet(i));
+      shapeManager.refreshShapeTrajectories(i, bs, mat);
   }
 
   public void connect(float[][] connections) {
     // array of [index1 index2 order diameter energy]
-    molecules = null;
-    moleculeCount = 0;
+    resetMolecules();
     BitSet bsDelete = new BitSet();
     for (int i = 0; i < connections.length; i++) {
       float[] f = connections[i];
+      if (f == null || f.length < 2)
+        continue;
       int index1 = (int) f[0];
+      boolean addGroup = (index1 < 0);
+      if (addGroup)
+        index1 = -1 - index1;
       int index2 = (int) f[1];
-      if (index1 < 0 || index2 < 0 || index1 >= atomCount || index2 >= atomCount)
+      if (index2 < 0 || index1 >= atomCount || index2 >= atomCount)
         continue;
       int order = (f.length > 2 ? (int) f[2] : JmolEdge.BOND_COVALENT_SINGLE);
       if (order < 0)
         order &= 0xFFFF; // 12.0.1 was saving struts as negative numbers
       short mad = (f.length > 3 ? (short) (1000f * connections[i][3]) : getDefaultMadFromOrder(order));
-      // sad to say I used DIAMETER here, not radius....
       if (order == 0 || mad == 0 && order != JmolEdge.BOND_STRUT && !Bond.isHydrogen(order)) {
         Bond b = atoms[index1].getBond(atoms[index2]);
         if (b != null)
@@ -844,11 +1071,112 @@ abstract public class ModelSet extends ModelCollection {
         continue; 
       }
       float energy = (f.length > 4 ? f[4] : 0);
-      bondAtoms(atoms[index1], atoms[index2], order, mad, null, energy, true);
+      bondAtoms(atoms[index1], atoms[index2], order, mad, null, energy, addGroup, true);
     }
     if (bsDelete.nextSetBit(0) >= 0)
       deleteBonds(bsDelete, false);
   }
+  
+  public void createAtomDataSet(int tokType, Object atomSetCollection,
+                                BitSet bsSelected) {
+    if (atomSetCollection == null)
+      return;
+    // must be one of JmolConstants.LOAD_ATOM_DATA_TYPES
+    JmolAdapter adapter = viewer.getModelAdapter();
+    Point3f pt = new Point3f();
+    Point3f v = new Point3f();
+    float tolerance = viewer.getLoadAtomDataTolerance();
+    if (unitCells != null)
+      for (int i = bsSelected.nextSetBit(0); i >= 0; i = bsSelected
+          .nextSetBit(i + 1))
+        if (atoms[i].getAtomSymmetry() != null) {
+          tolerance = -tolerance;
+          break;
+        }
+    int i = -1;
+    int n = 0;
+    boolean loadAllData = (BitSetUtil.cardinalityOf(bsSelected) == viewer
+        .getAtomCount());
+    for (JmolAdapter.AtomIterator iterAtom = adapter
+        .getAtomIterator(atomSetCollection); iterAtom.hasNext();) {
+      float x = iterAtom.getX();
+      float y = iterAtom.getY();
+      float z = iterAtom.getZ();
+      if (Float.isNaN(x + y + z))
+        continue;
+
+      if (tokType == Token.xyz) {
+        // we are loading selected coordinates only
+        i = bsSelected.nextSetBit(i + 1);
+        if (i < 0)
+          break;
+        n++;
+        if (Logger.debugging)
+          Logger.debug("atomIndex = " + i + ": " + atoms[i]
+              + " --> (" + x + "," + y + "," + z);
+        setAtomCoord(i, x, y, z);
+        continue;
+      }
+      pt.set(x, y, z);
+      BitSet bs = new BitSet(atomCount);
+      getAtomsWithin(tolerance, pt, bs, -1);
+      bs.and(bsSelected);
+      if (loadAllData) {
+        n = BitSetUtil.cardinalityOf(bs);
+        if (n == 0) {
+          Logger.warn("createAtomDataSet: no atom found at position " + pt);
+          continue;
+        } else if (n > 1 && Logger.debugging) {
+          Logger.debug("createAtomDataSet: " + n + " atoms found at position "
+              + pt);
+        }
+      }
+      switch (tokType) {
+      case Token.vibxyz:
+        float vx = iterAtom.getVectorX();
+        float vy = iterAtom.getVectorY();
+        float vz = iterAtom.getVectorZ();
+        if (Float.isNaN(vx + vy + vz))
+          continue;
+        v.set(vx, vy, vz);
+        if (Logger.debugging)
+          Logger.info("xyz: " + pt + " vib: " + v);
+        setAtomCoord(bs, Token.vibxyz, v);
+        break;
+      case Token.occupancy:
+        // [0 to 100], default 100
+        setAtomProperty(bs, tokType, iterAtom.getOccupancy(), 0, null, null,
+            null);
+        break;
+      case Token.partialcharge:
+        // anything but NaN, default NaN
+        setAtomProperty(bs, tokType, 0, iterAtom.getPartialCharge(), null,
+            null, null);
+        break;
+      case Token.temperature:
+        // anything but NaN but rounded to 0.01 precision and stored as a short (-32000 - 32000), default NaN
+        setAtomProperty(bs, tokType, 0, iterAtom.getBfactor(), null, null, null);
+        break;
+      }
+    }
+    //finally:
+    switch (tokType) {
+    case Token.vibxyz:
+      String vibName = adapter.getAtomSetName(atomSetCollection, 0);
+      Logger.info("_vibrationName = " + vibName);
+      viewer.setStringProperty("_vibrationName", vibName);
+      break;
+    case Token.xyz:
+      Logger.info(n + " atom positions read");
+      recalculateLeadMidpointsAndWingVectors(-1);
+      break;
+    }    
+  }
+
+  public boolean allowSpecAtom() {
+    // old Chime scripts use *.C for _C
+    return modelCount != 1 || models[0].isPDB;
+  }
 
 }
 
