@@ -30,13 +30,14 @@ import java.util.List;
 import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
 
+import org.jmol.constant.EnumStructure;
 import org.jmol.modelset.Atom;
+import org.jmol.modelset.LabelToken;
 import org.jmol.modelset.ModelSet;
 
 import org.jmol.util.Logger;
 import org.jmol.util.Measure;
 import org.jmol.util.OutputStringBuffer;
-import org.jmol.viewer.JmolConstants;
 import org.jmol.viewer.Viewer;
 
 
@@ -47,7 +48,7 @@ public class AlphaPolymer extends BioPolymer {
     //System.out.println("new AlphaPolymer " + monomers[0].getChainID() + " " + monomers[0].getSeqcodeString() + " " + monomers[monomers.length - 1].getSeqcodeString());
   }
 
-  
+  @Override
   protected Point3f getControlPoint(int i, Vector3f v) {
     if (!monomers[i].isSheet())
       return leadPoints[i];
@@ -58,17 +59,18 @@ public class AlphaPolymer extends BioPolymer {
     return pt;
   }
 
-  
+  @Override
   public void getPdbData(Viewer viewer, char ctype, char qtype, int mStep, int derivType,
-                         boolean isDraw, BitSet bsAtoms, 
-                         OutputStringBuffer pdbATOM, StringBuffer pdbCONECT, 
-                         BitSet bsSelected, boolean addHeader, boolean bothEnds, BitSet bsWritten) {
-    getPdbData(viewer, this, ctype, qtype, mStep, derivType, isDraw, bsAtoms, pdbATOM, 
-        pdbCONECT, bsSelected, addHeader, bothEnds, bsWritten);
+                         BitSet bsAtoms, BitSet bsSelected, 
+                         boolean bothEnds, boolean isDraw, boolean addHeader, 
+                         LabelToken[] tokens, OutputStringBuffer pdbATOM, 
+                         StringBuffer pdbCONECT, BitSet bsWritten) {
+    getPdbData(viewer, this, ctype, qtype, mStep, derivType, bsAtoms, bsSelected, bothEnds, 
+        isDraw, addHeader, tokens, pdbATOM, pdbCONECT, bsWritten);
   }
 
-  
-  public void addSecondaryStructure(byte type,
+  @Override
+  public void addSecondaryStructure(EnumStructure type,
                                     String structureID, int serialID, int strandCount,
                              char startChainID, int startSeqcode,
                              char endChainID, int endSeqcode) {
@@ -80,12 +82,10 @@ public class AlphaPolymer extends BioPolymer {
     addSecondaryStructure(type, structureID, serialID, strandCount, indexStart, indexEnd);
   }
 
-  protected void addSecondaryStructure(byte type, 
+  protected void addSecondaryStructure(EnumStructure type, 
                              String structureID, int serialID, int strandCount,
                              int indexStart, int indexEnd) {
 
-    //System.out.println("alphapolymer add2ndryStruct " + monomers[indexStart].getModelIndex() + " " + type + " " + indexStart + " " + indexEnd);
-
     //these two can be the same if this is a carbon-only polymer
     if (indexEnd < indexStart) {
       Logger.error("AlphaPolymer:addSecondaryStructure error: " +
@@ -95,17 +95,19 @@ public class AlphaPolymer extends BioPolymer {
     }
     int structureCount = indexEnd - indexStart + 1;
     ProteinStructure proteinstructure = null;
+    if (type == null)
+      System.out.println("alhapoly null type");
     switch(type) {
-    case JmolConstants.PROTEIN_STRUCTURE_HELIX:
-    case JmolConstants.PROTEIN_STRUCTURE_HELIX_ALPHA:
-    case JmolConstants.PROTEIN_STRUCTURE_HELIX_310:
-    case JmolConstants.PROTEIN_STRUCTURE_HELIX_PI:
+    case HELIX:
+    case HELIX_ALPHA:
+    case HELIX_310:
+    case HELIX_PI:
       proteinstructure = new Helix(this, indexStart, structureCount, 0, type);
       break;
-    case JmolConstants.PROTEIN_STRUCTURE_SHEET:
+    case SHEET:
       proteinstructure = new Sheet(this, indexStart, structureCount, 0, type);
       break;
-    case JmolConstants.PROTEIN_STRUCTURE_TURN:
+    case TURN:
       proteinstructure = new Turn(this, indexStart, structureCount, 0);
       break;
     default:
@@ -126,62 +128,64 @@ public class AlphaPolymer extends BioPolymer {
   ///////////////////////////////////////////////////////////
   /**
    * 
-   * Algorithm of George Phillips   phillips@biochem.wisc.edu
+   * Algorithm of George Phillips phillips@biochem.wisc.edu
    * 
-   * originally a contribution to pyMol as struts.py; 
-   * adapted here by Bob Hanson for Jmol 1/2010
+   * originally a contribution to pyMol as struts.py; adapted here by Bob Hanson
+   * for Jmol 1/2010
    * 
-   * Return a vector of support posts for rapid prototyping models 
-   * along the lines of George Phillips for Pymol except on actual molecular
-   * segments (biopolymers), not PDB chains (which may or may not be
-   * continuous). 
+   * Return a vector of support posts for rapid prototyping models along the
+   * lines of George Phillips for Pymol except on actual molecular segments
+   * (biopolymers), not PDB chains (which may or may not be continuous).
    * 
    * Like George, we go from thresh-4 to thresh in units of 1 Angstrom, but we
    * do not require this threshold to be an integer. In addition, we prevent
-   * double-creation of struts by tracking where struts are, and we do not
-   * look for any addtional end struts if there is a strut already to an atom
-   * at a particular biopolymer end. The three parameters are:
+   * double-creation of struts by tracking where struts are, and we do not look
+   * for any addtional end struts if there is a strut already to an atom at a
+   * particular biopolymer end. The three parameters are:
    * 
-   * set strutDefaultRadius 0.3
-   * set strutSpacingMinimum 6
-   * set strutLengthMaximum 7.0
+   * set strutDefaultRadius 0.3 set strutSpacingMinimum 6 set strutLengthMaximum
+   * 7.0
    * 
-   * Struts will be introduced by: 
+   * Struts will be introduced by:
    * 
    * calculate struts {atom set A} {atom set B}
    * 
-   * where the two atom sets are optional and default to the currently selected set.
+   * where the two atom sets are optional and default to the currently selected
+   * set.
    * 
    * They can be manipulated using the STRUTS command much like any "bond"
    * 
-   * struts 0.3
-   * color struts opaque pink
-   * connect {atomno=3} {atomno=4} strut
+   * struts 0.3 color struts opaque pink connect {atomno=3} {atomno=4} strut
    * 
-   * structs only
+   * struts only
    * 
    * command
    * 
    * @param modelSet
-   * @param atoms
    * @param bs1
    * @param bs2
    * @param vCA
    * @param thresh
    * @param delta
-   * @param allowMultiple 
+   * @param allowMultiple
    * @return vector of pairs of atoms
    * 
    */
-  
-  public List calculateStruts(ModelSet modelSet, Atom[] atoms, BitSet bs1,
-                                BitSet bs2, List vCA, float thresh, int delta, boolean allowMultiple) {
-    List vStruts = new ArrayList(); // the output vector
+  @Override
+  public List<Atom[]> calculateStruts(ModelSet modelSet, BitSet bs1,
+                                      BitSet bs2, List<Atom> vCA, float thresh,
+                                      int delta, boolean allowMultiple) {
+    return calculateStrutsStatic(modelSet, bs1, bs2, vCA, thresh, delta,
+        allowMultiple);
+  }
+    
+  private List<Atom[]> calculateStrutsStatic(ModelSet modelSet, BitSet bs1, BitSet bs2,
+                                             List<Atom> vCA, float thresh,
+                                             int delta, boolean allowMultiple) {
+    List<Atom[]> vStruts = new ArrayList<Atom[]>(); // the output vector
     float thresh2 = thresh * thresh; // use distance squared for speed
 
-    //TODO  CHECK IMPLEMENT BITSETS
-    
-    int n = vCA.size();
+    int n = vCA.size();  // the set of alpha carbons
     int nEndMin = 3;
 
     // We set bitsets that indicate that there is no longer any need to
@@ -195,12 +199,12 @@ public class AlphaPolymer extends BioPolymer {
     // check for a strut. We are going to set struts within 3 residues
     // of the ends of biopolymers, so we track those positions as well.
     
-    Atom a1 = (Atom) vCA.get(0);
+    Atom a1 = vCA.get(0);
     Atom a2;
     int nBiopolymers = modelSet.getBioPolymerCountInModel(a1.modelIndex);
     int[][] biopolymerStartsEnds = new int[nBiopolymers][nEndMin * 2];
     for (int i = 0; i < n; i++) {
-      a1 = (Atom) vCA.get(i);
+      a1 = vCA.get(i);
       int polymerIndex = a1.getPolymerIndexInModel();
       int monomerIndex = a1.getMonomerIndex();
       int bpt = monomerIndex;
@@ -220,10 +224,10 @@ public class AlphaPolymer extends BioPolymer {
 
     float[] d2 = new float[n * (n - 1) / 2];
     for (int i = 0; i < n; i++) {
-      a1 = (Atom) vCA.get(i);
+      a1 = vCA.get(i);
       for (int j = i + 1; j < n; j++) {
         int ipt = strutPoint(i, j, n);
-        a2 = (Atom) vCA.get(j);
+        a2 = vCA.get(j);
         int resno1 = a1.getResno();
         int polymerIndex1 = a1.getPolymerIndexInModel();
         int resno2 = a2.getResno();
@@ -318,17 +322,17 @@ public class AlphaPolymer extends BioPolymer {
     return vStruts;
   }
 
-  private int strutPoint(int i, int j, int n) {
+  private static int strutPoint(int i, int j, int n) {
     return (j < i ? j * (2 * n - j - 1) / 2 + i - j - 1
      : i * (2 * n - i - 1) / 2 + j - i - 1);
   }
 
-  private void setStrut(int i, int j, int n, List vCA, BitSet bs1, BitSet bs2, 
-                        List vStruts,
+  private static void setStrut(int i, int j, int n, List<Atom> vCA, BitSet bs1, BitSet bs2, 
+                        List<Atom[]> vStruts,
                         BitSet bsStruts, BitSet bsNotAvailable,
                         BitSet bsNearbyResidues, int delta) {
-    Atom a1 = (Atom) vCA.get(i);
-    Atom a2 = (Atom) vCA.get(j);
+    Atom a1 = vCA.get(i);
+    Atom a2 = vCA.get(j);
     if (!bs1.get(a1.index) || !bs2.get(a2.index))
       return;
     vStruts.add(new Atom[] { a1, a2 });
@@ -369,30 +373,25 @@ public class AlphaPolymer extends BioPolymer {
    * href='http://csb.stanford.edu/levitt/Levitt_JMB77_Secondary_structure.pdf'>
    * http://csb.stanford.edu/levitt/Levitt_JMB77_Secondary_structure.pdf
    * </a>
-   * @param alphaOnly 
    */
-  
+  @Override
   public void calculateStructures(boolean alphaOnly) {
     // alphaOnly parameter is here so this can be 
     // caught by AminoPolymer and discarded if desired
     if (monomerCount < 4)
       return;
     float[] angles = calculateAnglesInDegrees();
-    byte[] codes = calculateCodes(angles);
+    Code[] codes = calculateCodes(angles);
     checkBetaSheetAlphaHelixOverlap(codes, angles);
-    byte[] tags = calculateRunsFourOrMore(codes);
+    EnumStructure[] tags = calculateRunsFourOrMore(codes);
     extendRuns(tags);
     searchForTurns(codes, angles, tags);
     addStructuresFromTags(tags);
   }
 
-  private final static byte CODE_NADA        = 0;
-  private final static byte CODE_RIGHT_HELIX = 1;
-  private final static byte CODE_BETA_SHEET  = 2;
-  private final static byte CODE_LEFT_HELIX  = 3;
-
-  private final static byte CODE_LEFT_TURN  = 4;
-  private final static byte CODE_RIGHT_TURN = 5;
+  private enum Code {
+    NADA, RIGHT_HELIX, BETA_SHEET, LEFT_HELIX, LEFT_TURN, RIGHT_TURN;
+  }
   
   private float[] calculateAnglesInDegrees() {
     float[] angles = new float[monomerCount];
@@ -405,48 +404,43 @@ public class AlphaPolymer extends BioPolymer {
     return angles;
   }
 
-  private byte[] calculateCodes(float[] angles) {
-    byte[] codes = new byte[monomerCount];
+  private Code[] calculateCodes(float[] angles) {
+    Code[] codes = new Code[monomerCount];
     for (int i = monomerCount - 1; --i >= 2; ) {
       float degrees = angles[i];
       codes[i] = ((degrees >= 10 && degrees < 120)
-                  ? CODE_RIGHT_HELIX
+                  ? Code.RIGHT_HELIX
                   : ((degrees >= 120 || degrees < -90)
-                     ? CODE_BETA_SHEET
+                     ? Code.BETA_SHEET
                      : ((degrees >= -90 && degrees < 0)
-                        ? CODE_LEFT_HELIX
-                        : CODE_NADA)));
+                        ? Code.LEFT_HELIX
+                        : Code.NADA)));
     }
     return codes;
   }
 
-  private void checkBetaSheetAlphaHelixOverlap(byte[] codes, float[] angles) {
+  private void checkBetaSheetAlphaHelixOverlap(Code[] codes, float[] angles) {
     for (int i = monomerCount - 2; --i >= 2; )
-      if (codes[i] == CODE_BETA_SHEET &&
+      if (codes[i] == Code.BETA_SHEET &&
           angles[i] <= 140 &&
-          codes[i - 2] == CODE_RIGHT_HELIX &&
-          codes[i - 1] == CODE_RIGHT_HELIX &&
-          codes[i + 1] == CODE_RIGHT_HELIX &&
-          codes[i + 2] == CODE_RIGHT_HELIX)
-        codes[i] = CODE_RIGHT_HELIX;
+          codes[i - 2] == Code.RIGHT_HELIX &&
+          codes[i - 1] == Code.RIGHT_HELIX &&
+          codes[i + 1] == Code.RIGHT_HELIX &&
+          codes[i + 2] == Code.RIGHT_HELIX)
+        codes[i] = Code.RIGHT_HELIX;
   }
 
-  private final static byte TAG_NADA  = JmolConstants.PROTEIN_STRUCTURE_NONE;
-  private final static byte TAG_TURN  = JmolConstants.PROTEIN_STRUCTURE_TURN;
-  private final static byte TAG_SHEET = JmolConstants.PROTEIN_STRUCTURE_SHEET;
-  private final static byte TAG_HELIX = JmolConstants.PROTEIN_STRUCTURE_HELIX;
-
-  private byte[] calculateRunsFourOrMore(byte[] codes) {
-    byte[] tags = new byte[monomerCount];
-    byte tag = TAG_NADA;
-    byte code = CODE_NADA;
+  private EnumStructure[] calculateRunsFourOrMore(Code[] codes) {
+    EnumStructure[] tags = new EnumStructure[monomerCount];
+    EnumStructure tag = EnumStructure.NONE;
+    Code code = Code.NADA;
     int runLength = 0;
     for (int i = 0; i < monomerCount; ++i) {
       // throw away the sheets ... their angle technique does not work well
-      if (codes[i] == code && code != CODE_NADA && code != CODE_BETA_SHEET) {
+      if (codes[i] == code && code != Code.NADA && code != Code.BETA_SHEET) {
         ++runLength;
         if (runLength == 4) {
-          tag = (code == CODE_BETA_SHEET ? TAG_SHEET : TAG_HELIX);
+          tag = (code == Code.BETA_SHEET ? EnumStructure.SHEET : EnumStructure.HELIX);
           for (int j = 4; --j >= 0; )
             tags[i - j] = tag;
         } else if (runLength > 4)
@@ -459,40 +453,40 @@ public class AlphaPolymer extends BioPolymer {
     return tags;
   }
 
-  private void extendRuns(byte[] tags) {
+  private void extendRuns(EnumStructure[] tags) {
     for (int i = 1; i < monomerCount - 4; ++i)
-      if (tags[i] == TAG_NADA && tags[i + 1] != TAG_NADA)
+      if (tags[i] == EnumStructure.NONE && tags[i + 1] != EnumStructure.NONE)
         tags[i] = tags[i + 1];
     
     tags[0] = tags[1];
     tags[monomerCount - 1] = tags[monomerCount - 2];
   }
 
-  private void searchForTurns(byte[] codes, float[] angles, byte[] tags) {
+  private void searchForTurns(Code[] codes, float[] angles, EnumStructure[] tags) {
     for (int i = monomerCount - 1; --i >= 2; ) {
-      codes[i] = CODE_NADA;
-      if (tags[i] == TAG_NADA) {
+      codes[i] = Code.NADA;
+      if (tags[i] == null || tags[i] == EnumStructure.NONE) {
         float angle = angles[i];
         if (angle >= -90 && angle < 0)
-          codes[i] = CODE_LEFT_TURN;
+          codes[i] = Code.LEFT_TURN;
         else if (angle >= 0 && angle < 90)
-          codes[i] = CODE_RIGHT_TURN;
+          codes[i] = Code.RIGHT_TURN;
       }
     }
 
     for (int i = monomerCount - 1; --i >= 0; ) {
-      if (codes[i] != CODE_NADA &&
+      if (codes[i] != Code.NADA &&
           codes[i + 1] == codes[i] &&
-          tags[i] == TAG_NADA)
-        tags[i] = TAG_TURN;
+          tags[i] == EnumStructure.NONE)
+        tags[i] = EnumStructure.TURN;
     }
   }
 
-  private void addStructuresFromTags(byte[] tags) {
+  private void addStructuresFromTags(EnumStructure[] tags) {
     int i = 0;
     while (i < monomerCount) {
-      byte tag = tags[i];
-      if (tag == TAG_NADA) {
+      EnumStructure tag = tags[i];
+      if (tag == null || tag == EnumStructure.NONE) {
         ++i;
         continue;
       }
