@@ -83,8 +83,8 @@ public abstract class BioPolymer extends Polymer {
       Monomer current;
       if (!(group instanceof Monomer)
           || (current = (Monomer) group).bioPolymer != null || previous != null
-          && previous.getClass() != current.getClass() 
-          || checkConnections && !current.isConnectedAfter(previous))
+          && (previous.getClass() != current.getClass() 
+          || checkConnections && !current.isConnectedAfter(previous)))
         break;
       previous = current;
       count++;
