@@ -282,13 +282,9 @@ public class NucleicMonomer extends PhosphorusMonomer {
 
   @Override
   boolean isConnectedAfter(Monomer possiblyPreviousMonomer) {
-    if (possiblyPreviousMonomer == null)
-      return true;
     Atom myPhosphorusAtom = getAtomFromOffsetIndex(NP);
     if (myPhosphorusAtom == null)
       return false;
-    if (! (possiblyPreviousMonomer instanceof NucleicMonomer))
-      return false;
     NucleicMonomer other = (NucleicMonomer)possiblyPreviousMonomer;
     if (other.getAtomFromOffsetIndex(O3Pr).isBonded(myPhosphorusAtom))
       return true;
