@@ -23,14 +23,14 @@
  */
 package org.jmol.modelsetbio;
 
-import org.jmol.viewer.JmolConstants;
+import org.jmol.constant.EnumStructure;
 
 class Turn extends ProteinStructure {
 
   Turn(AlphaPolymer apolymer, int monomerIndex, int monomerCount, int id) {
-    super(apolymer, JmolConstants.PROTEIN_STRUCTURE_TURN,
+    super(apolymer, EnumStructure.TURN,
           monomerIndex, monomerCount, id);
-    this.subtype = JmolConstants.PROTEIN_STRUCTURE_TURN;
+    this.subtype = EnumStructure.TURN;
   }
 }
 
