@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Date: 2011-10-13 15:47:18 +0200 (jeu., 13 oct. 2011) $
+ * $Revision: 16347 $
  *
  * Copyright (C) 2000-2005  The Jmol Development Team
  *
@@ -23,17 +23,17 @@
  */
 package org.jmol.popup;
 
-import org.jmol.api.*;
+import org.jmol.api.JmolPopupInterface;
+import org.jmol.api.JmolViewer;
 import org.jmol.i18n.GT;
 import org.jmol.util.Elements;
-import org.jmol.util.Logger;
 import org.jmol.viewer.JmolConstants;
 
 import java.awt.Component;
+import java.awt.Container;
 import java.util.BitSet;
-import java.util.Hashtable;
-import java.util.Enumeration;
 import java.util.List;
+import java.util.Map;
 
 import javax.swing.JCheckBoxMenuItem;
 import javax.swing.JMenu;
@@ -41,44 +41,30 @@ import javax.swing.JMenuBar;
 import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
 
-public class JmolPopup extends SimplePopup {
+public class JmolPopup extends SimplePopup implements JmolPopupInterface {
   
   private int aboutComputedMenuBaseCount;
 
   private static String strMenuStructure;
   
   private JMenu mainMenu;
-  
-
+ 
   final private static int MENUITEM_HEIGHT = 20;
   final private static int MAX_ITEMS = 25;
   final private static int TITLE_MAX_WIDTH = 20;
 
-  static public JmolPopup newJmolPopup(JmolViewer viewer, boolean doTranslate, String menu, boolean asPopup) {
-    strMenuStructure = menu;
-    GT.setDoTranslate(true);
-    JmolPopup popup;
-    try {
-      popup = (JmolPopup) new JmolPopup(viewer, asPopup);
-    } catch (Exception e) {
-      Logger.error("JmolPopup not loaded");
-      GT.setDoTranslate(doTranslate);
-      return null;
-    }
-    try {
-      popup.updateComputedMenus();
-    } catch (NullPointerException e) {
-      // ignore -- the frame just wasn't ready yet;
-      // updateComputedMenus() will be called again when the frame is ready; 
-    }
-    GT.setDoTranslate(doTranslate);
-    return popup;
+  public JmolPopup() {
+    // required by reflection
   }
 
-  private JmolPopup(JmolViewer viewer, boolean asPopup) {
-    super(viewer);
+  public void initialize(JmolViewer viewer, boolean doTranslate, String menu,
+                         boolean asPopup) {
+    set(viewer);
+    strMenuStructure = menu;
+    GT.setDoTranslate(true);
     this.asPopup = asPopup;
-    PopupResource bundle = new MainPopupResourceBundle(strMenuStructure, menuText);
+    PopupResource bundle = new MainPopupResourceBundle(strMenuStructure,
+        menuText);
     String title = "popupMenu";
     if (asPopup) {
       swingPopup = new JPopupMenu("Jmol");
@@ -87,8 +73,20 @@ public class JmolPopup extends SimplePopup {
       mainMenu = new JMenu("Jmol");
       build(title, mainMenu, bundle);
     }
+    try {
+      updateComputedMenus();
+    } catch (NullPointerException e) {
+      // ignore -- the frame just wasn't ready yet;
+      // updateComputedMenus() will be called again when the frame is ready; 
+    }
+    GT.setDoTranslate(doTranslate);
   }
 
+  public Container getJMenu() {
+    return swingPopup;
+  }
+
+
   public String getMenu(String title) {
     int pt = title.indexOf("|"); 
     if (pt >= 0) {
@@ -101,6 +99,7 @@ public class JmolPopup extends SimplePopup {
     return (new MainPopupResourceBundle(strMenuStructure, null)).getMenu(title);
   }
   
+  @Override
   protected void checkSpecialMenu(String item, Object subMenu, String word) {
     // these will need tweaking:
     if ("aboutComputedMenu".equals(item)) {
@@ -111,15 +110,15 @@ public class JmolPopup extends SimplePopup {
     }
   }
 
+  @SuppressWarnings("unchecked")
   public void updateComputedMenus() {
     updateMode = UPDATE_ALL;
     getViewerData();
-    //System.out.println("jmolPopup updateComputedMenus " + modelSetFileName + " " + modelSetName + " " + atomCount);
     updateSelectMenu();
     updateFileMenu();
     updateElementsComputedMenu(viewer.getElementsPresentBitSet(modelIndex));
     updateHeteroComputedMenu(viewer.getHeteroList(modelIndex));
-    updateSurfMoComputedMenu((Hashtable) modelInfo.get("moData"));
+    updateSurfMoComputedMenu((Map<String, Object>) modelInfo.get("moData"));
     updateFileTypeDependentMenus();
     updatePDBComputedMenus();
     updateMode = UPDATE_CONFIG;
@@ -169,7 +168,7 @@ public class JmolPopup extends SimplePopup {
       String entryName = elementSymbol + " - " + elementName;
       addMenuItem(menu, entryName, "SELECT " + elementName, null);
     }
-    for (int i = JmolConstants.firstIsotope; i < Elements.altElementMax; ++i) {
+    for (int i = Elements.firstIsotope; i < Elements.altElementMax; ++i) {
       int n = Elements.elementNumberMax + i;
       if (elementsPresentBitSet.get(n)) {
         n = Elements.altElementNumberFromIndex(i);
@@ -182,7 +181,7 @@ public class JmolPopup extends SimplePopup {
     enableMenu(menu, true);
   }
 
-  void updateHeteroComputedMenu(Hashtable htHetero) {
+  void updateHeteroComputedMenu(Map<String, String> htHetero) {
     Object menu = htMenus.get("PDBheteroComputedMenu");
     if (menu == null)
       return;
@@ -190,11 +189,10 @@ public class JmolPopup extends SimplePopup {
     enableMenu(menu, false);
     if (htHetero == null)
       return;
-    Enumeration e = htHetero.keys();
     int n = 0;
-    while (e.hasMoreElements()) {
-      String heteroCode = (String) e.nextElement();
-      String heteroName = (String) htHetero.get(heteroCode);
+    for (Map.Entry<String, String> hetero : htHetero.entrySet()) {
+      String heteroCode = hetero.getKey();
+      String heteroName = hetero.getValue();
       if (heteroName.length() > 20)
         heteroName = heteroName.substring(0, 20) + "...";
       String entryName = heteroCode + " - " + heteroName;
@@ -204,18 +202,21 @@ public class JmolPopup extends SimplePopup {
     enableMenu(menu, (n > 0));
   }
 
-  void updateSurfMoComputedMenu(Hashtable moData) {
-    Object menu = htMenus.get("surfMoComputedMenu");
+  @SuppressWarnings("unchecked")
+  void updateSurfMoComputedMenu(Map<String, Object> moData) {
+    Object menu = htMenus.get("surfMoComputedMenuText");
     if (menu == null)
       return;
-    enableMenu(menu, false);
     removeAll(menu);
-    if (moData == null)
-      return;
-    List mos = (List) (moData.get("mos"));
+    List<Map<String, Object>> mos = (moData == null ? null : (List<Map<String, Object>>) (moData.get("mos")));
     int nOrb = (mos == null ? 0 : mos.size());
-    if (nOrb == 0)
+    String text = getMenuText("surfMoComputedMenuText");
+    if (nOrb == 0) {
+      setLabel(menu, GT._(text, ""));
+      enableMenu(menu, false);
       return;
+    }
+    setLabel(menu, GT._(text, nOrb));
     enableMenu(menu, true);
     Object subMenu = menu;
     int nmod = (nOrb % MAX_ITEMS);
@@ -233,7 +234,7 @@ public class JmolPopup extends SimplePopup {
         htMenus.put(id, subMenu);
         pt = 1;
       }
-      Hashtable mo = (Hashtable) mos.get(i);
+      Map<String, Object> mo = mos.get(i);
       String entryName = "#" + (i + 1) + " " 
           + (mo.containsKey("type") ? (String)mo.get("type") + " " : "")
           + (mo.containsKey("symmetry") ? (String)mo.get("symmetry") + " ": "")
@@ -333,6 +334,7 @@ public class JmolPopup extends SimplePopup {
     updateSYMMETRYShowComputedMenu();
   }
 
+  @SuppressWarnings("unchecked")
   private void updateSYMMETRYShowComputedMenu() {
     Object menu = htMenus.get("SYMMETRYShowComputedMenu");
     if (menu == null)
@@ -341,7 +343,7 @@ public class JmolPopup extends SimplePopup {
     enableMenu(menu, false);
     if (!isSymmetry || modelIndex < 0)
       return;
-    Hashtable info = (Hashtable) viewer.getProperty("DATA_API", "spaceGroupInfo", null);
+    Map<String, Object> info = (Map<String, Object>) viewer.getProperty("DATA_API", "spaceGroupInfo", null);
     if (info == null)
       return;
     Object[][] infolist = (Object[][]) info.get("operations");
@@ -405,8 +407,8 @@ public class JmolPopup extends SimplePopup {
     if (menu == null)
       return;
     enableMenu(menu, (modelCount > 1));
-    setLabel(menu, (modelIndex < 0 ? GT._(getMenuText("allModelsText"), modelCount, true)
-        : getModelLabel()));
+    setLabel(menu, (modelIndex < 0 ? GT._(getMenuText("allModelsText"),
+        modelCount, true) : getModelLabel()));
     removeAll(menu);
     if (modelCount < 2)
       return;
@@ -428,8 +430,15 @@ public class JmolPopup extends SimplePopup {
       }
       String script = "" + viewer.getModelNumberDotted(i);
       String entryName = viewer.getModelName(i);
-      if (!entryName.equals(script))
+      if (!entryName.equals(script)) {
+        int ipt = entryName.indexOf(";PATH");
+        if (ipt >= 0)
+          entryName = entryName.substring(0, ipt);
+        if (entryName.indexOf("Model[") == 0
+            && (ipt = entryName.indexOf("]:")) >= 0)
+          entryName = entryName.substring(ipt + 2);
         entryName = script + ": " + entryName;
+      }
       if (entryName.length() > 50)
         entryName = entryName.substring(0, 45) + "...";
       addCheckboxMenuItem(subMenu, entryName, "model " + script + " ##", null,
@@ -460,6 +469,7 @@ public class JmolPopup extends SimplePopup {
     }
   }
 
+  @SuppressWarnings("unchecked")
   private void updateModelSetComputedMenu() {
     Object menu = htMenus.get("modelSetMenu");
     if (menu == null)
@@ -505,19 +515,19 @@ public class JmolPopup extends SimplePopup {
       }
       removeAll(submenu);
       enableMenu(submenu, false);
-      List biomolecules;
+      List<Map<String, Object>> biomolecules;
       if (modelIndex >= 0
-          && (biomolecules = (List) viewer.getModelAuxiliaryInfo(modelIndex,
+          && (biomolecules = (List<Map<String, Object>>) viewer.getModelAuxiliaryInfo(modelIndex,
               "biomolecules")) != null) {
         enableMenu(submenu, true);
         int nBiomolecules = biomolecules.size();
         for (int i = 0; i < nBiomolecules; i++) {
           String script = (isMultiFrame ? ""
               : "save orientation;load \"\" FILTER \"biomolecule " + (i + 1) + "\";restore orientation;");
-          int nAtoms = ((Integer) ((Hashtable) biomolecules.get(i)).get("atomCount")).intValue();
+          int nAtoms = ((Integer) biomolecules.get(i).get("atomCount")).intValue();
           String entryName = GT._(getMenuText(isMultiFrame ? "biomoleculeText"
-              : "loadBiomoleculeText"), new Object[] { new Integer(i + 1),
-              new Integer(nAtoms) });
+              : "loadBiomoleculeText"), new Object[] { Integer.valueOf(i + 1),
+              Integer.valueOf(nAtoms) });
           addMenuItem(submenu, entryName, script, null);
         }
       }
@@ -597,12 +607,20 @@ public class JmolPopup extends SimplePopup {
     String id = getId(menu);
     GT.Language[] languages = GT.getLanguageList();
     for (int i = 0; i < languages.length; i++) {
-      if (languages[i].display) {
+      if (language.equals(languages[i].code)) {
+        languages[i].forceDisplay();
+      }
+      if (languages[i].shouldDisplay()) {
         String code = languages[i].code;
         String name = languages[i].language;
+        String nativeName = languages[i].nativeLanguage;
+        String menuLabel = code + " - " + GT._(name, true);
+        if ((nativeName != null) && (!nativeName.equals(GT._(name, true)))) {
+          menuLabel += " - " + nativeName; 
+        }
         addCheckboxMenuItem(
             menu,
-            GT._(name, true) + " (" + code + ")",
+            menuLabel,
             "language = \"" + code + "\" ##" + name,
             id + "." + code,
             language.equals(code), false);
@@ -623,6 +641,7 @@ public class JmolPopup extends SimplePopup {
    * @param what option to set
    * @param TF   true or false
    */
+  @Override
   protected void setCheckBoxValue(JMenuItem item, String what, boolean TF) {    
     super.setCheckBoxValue(item, what, TF);
     if (what.indexOf("#CONFIG") >= 0) {
@@ -633,6 +652,7 @@ public class JmolPopup extends SimplePopup {
     }
   }
 
+  @Override
   protected void updateForShow() {
     getViewerData();
     updateMode = UPDATE_SHOW;
@@ -643,7 +663,7 @@ public class JmolPopup extends SimplePopup {
   }
 
   public void show(int x, int y) {
-    super.show(x, y, false);
+    show(x, y, false);
     if (x < 0) {
       getViewerData();
       setFrankMenu(currentMenuItemId);
@@ -655,7 +675,7 @@ public class JmolPopup extends SimplePopup {
       }
     }
     restorePopupMenu();
-    if (asPopup)
+    if (asPopup && thisx >= 0)
       showPopupMenu(thisx, thisy);
   }
 
@@ -682,7 +702,7 @@ public class JmolPopup extends SimplePopup {
       String strMenu = id.substring(i, iNew);
       Object menu = htMenus.get(strMenu);
       frankList[nFrankList++] = new Object[] { getParent(menu), menu,
-          new Integer(getPosition(menu)) };
+          Integer.valueOf(getPosition(menu)) };
       addMenuSubMenu(frankPopup, menu);
       i = iNew + 1;
     }
@@ -706,6 +726,7 @@ public class JmolPopup extends SimplePopup {
     return -1;
   }
 
+  @Override
   void restorePopupMenu() {
     if (nFrankList < 2)
       return;
