@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-12-22 19:26:50 +0100 (mer., 22 déc. 2010) $
- * $Revision: 14875 $
+ * $Date: 2012-01-31 03:44:48 +0100 (mar., 31 janv. 2012) $
+ * $Revision: 16704 $
  *
  * Copyright (C) 2000-2005  The Jmol Development Team
  *
@@ -26,6 +26,7 @@ package org.jmol.popup;
 import java.util.Properties;
 
 import org.jmol.i18n.GT;
+import org.jmol.util.TextFormat;
 
 class MainPopupResourceBundle extends PopupResource {
 
@@ -33,6 +34,7 @@ class MainPopupResourceBundle extends PopupResource {
     super(menuStructure, menuText);
   }
 
+  @Override
   protected void buildStructure(String menuStructure) {
     addItems(menuContents);
     addItems(structureContents);
@@ -49,7 +51,7 @@ class MainPopupResourceBundle extends PopupResource {
       {   "@AXESCOLOR", "gray salmon maroon olive slateblue gold orchid"},
       
       {   "popupMenu",
-          "FRAMESbyModelComputedMenu configurationComputedMenu - selectMenuText viewMenu renderMenu colorMenu - surfaceMenu UNITCELLMenu - "
+          "FRAMESbyModelComputedMenu configurationComputedMenu - selectMenuText viewMenu renderMenu colorMenu - surfaceMenu FILEUNITMenu - "
               + "zoomMenu spinMenu VIBRATIONMenu "
               + "FRAMESanimateMenu - "
               + "measureMenu pickingMenu - showConsole showMenu fileMenu computationMenu - "
@@ -96,7 +98,7 @@ class MainPopupResourceBundle extends PopupResource {
               + "bond100 bond150 bond200 bond250 bond300" },
 
       {   "hbondMenu",
-          "PDBhbondCalc hbondNone hbondWireframe - "
+          "hbondCalc hbondNone hbondWireframe - "
               + "PDBhbondSidechain PDBhbondBackbone - "
               + "hbond100 hbond150 hbond200 hbond250 hbond300" },
 
@@ -205,10 +207,10 @@ class MainPopupResourceBundle extends PopupResource {
 
 
       {   "surfaceMenu",
-          "surfDots surfVDW surfSolventAccessible14 surfSolvent14 surfMolecular CHARGEsurfMEP surfMoComputedMenu - surfOpaque surfTranslucent surfOff" },
+          "surfDots surfVDW surfSolventAccessible14 surfSolvent14 surfMolecular CHARGEsurfMEP surfMoComputedMenuText - surfOpaque surfTranslucent surfOff" },
 
-      {   "UNITCELLMenu",
-          "SYMMETRYShowComputedMenu SYMMETRYhide UNITCELLone UNITCELLnine UNITCELLnineRestricted UNITCELLninePoly" },
+      {   "FILEUNITMenu",
+          "SYMMETRYShowComputedMenu SYMMETRYhide FILEMOLload FILEUNITone FILEUNITnine FILEUNITnineRestricted FILEUNITninePoly" },
 
       {   "[set_axes]Menu", 
           "off#axes dotted - byPixelMenu byAngstromMenu" },
@@ -319,7 +321,7 @@ class MainPopupResourceBundle extends PopupResource {
       { "bond250", "wireframe .25" },
       { "bond300", "wireframe .3" },
 
-      { "PDBhbondCalc", "hbonds calculate" },
+      { "hbondCalc", "hbonds calculate" },
       { "hbondNone", "hbonds off" },
       { "hbondWireframe", "hbonds on" },
       { "PDBhbondSidechain", "set hbonds sidechain" },
@@ -503,13 +505,15 @@ class MainPopupResourceBundle extends PopupResource {
       { "surfTranslucent", "mo translucent;isosurface translucent" },
       { "surfOff", "mo delete;isosurface delete;select *;dots off" },
       { "SYMMETRYhide", "draw sym_* delete" },
-      { "UNITCELLone",
+      { "FILEMOLload",
+      "save orientation;load \"\";restore orientation;center" },
+      { "FILEUNITone",
           "save orientation;load \"\" {1 1 1} ;restore orientation;center" },
-      { "UNITCELLnine",
+      { "FILEUNITnine",
           "save orientation;load \"\" {444 666 1} ;restore orientation;center" },
-      { "UNITCELLnineRestricted",
+      { "FILEUNITnineRestricted",
           "save orientation;load \"\" {444 666 1} ;restore orientation; unitcell on; display cell=555;center visible;zoom 200" },
-      { "UNITCELLninePoly",
+      { "FILEUNITninePoly",
           "save orientation;load \"\" {444 666 1} ;restore orientation; unitcell on; display cell=555; polyhedra 4,6 (displayed);center (visible);zoom 200" },
 
       { "1p", "on" },
@@ -524,11 +528,11 @@ class MainPopupResourceBundle extends PopupResource {
       { "100a", "1.0" },
   };
   
+  @Override
   protected String[] getWordContents() {
     
     boolean wasTranslating = GT.getDoTranslate();
-    if (!wasTranslating)
-      GT.setDoTranslate(true);
+    GT.setDoTranslate(true);
     String[] words = new String[] {
         "modelSetMenu", GT._("No atoms loaded"),
         
@@ -540,7 +544,7 @@ class MainPopupResourceBundle extends PopupResource {
         "PDBnucleicResiduesComputedMenu", GT._("By Residue Name"),
         "PDBcarboResiduesComputedMenu", GT._("By Residue Name"),
         "PDBheteroComputedMenu", GT._("By HETATM"),
-        "surfMoComputedMenu", GT._("Molecular Orbitals"),
+        "surfMoComputedMenuText", GT._("Molecular Orbitals ({0})"),
         "SYMMETRYSelectComputedMenu", GT._("Symmetry"),
         "SYMMETRYShowComputedMenu", GT._("Space Group"),
         "SYMMETRYhide", GT._("Hide Symmetry"),
@@ -572,7 +576,7 @@ class MainPopupResourceBundle extends PopupResource {
         "front", GT._("Front"),
         "left", GT._("Left"),
         "right", GT._("Right"),
-        "top", GT._("Top"),
+        "top", TextFormat.split(GT._("Top[as in \"view from the top, from above\" - (translators: remove this bracketed part]"), '[')[0],
         "bottom", GT._("Bottom"),
         "back", GT._("Back"),
 
@@ -601,7 +605,7 @@ class MainPopupResourceBundle extends PopupResource {
         "nonWaterSolvent",
             GT._("Nonaqueous Solvent") + " (solvent and not water)",
         "exceptWater", GT._("Nonaqueous HETATM") + " (hetero and not water)",
-        "Ligand", GT._("Ligand") + " (hetero and not solvent)",
+        "Ligand", GT._("Ligand"),
 
         "allCarbo", GT._("All"),
         "PDBcarboMenu", GT._("Carbohydrate"),
@@ -636,7 +640,7 @@ class MainPopupResourceBundle extends PopupResource {
 
         "hbondMenu", GT._("Hydrogen Bonds"),
         "hbondNone", GT._("Off"),
-        "PDBhbondCalc", GT._("Calculate"),
+        "hbondCalc", GT._("Calculate"),
         "hbondWireframe", GT._("On"),
         "PDBhbondSidechain", GT._("Set H-Bonds Side Chain"),
         "PDBhbondBackbone", GT._("Set H-Bonds Backbone"),
@@ -897,11 +901,12 @@ class MainPopupResourceBundle extends PopupResource {
         "surfTranslucent", GT._("Make Translucent"),
         "surfOff", GT._("Off"),
 
-        "UNITCELLMenu", GT._("Symmetry"),
-        "UNITCELLone", GT._("Reload {0}", "{1 1 1}"),
-        "UNITCELLnine", GT._("Reload {0}", "{444 666 1}"),
-        "UNITCELLnineRestricted", GT._("Reload {0} + Display {1}", new Object[] { "{444 666 1}", "555" } ),
-        "UNITCELLninePoly", GT._("Reload + Polyhedra"),
+        "FILEUNITMenu", GT._("Symmetry"),
+        "FILEMOLload", GT._("Reload {0}", "(molecular)"),
+        "FILEUNITone", GT._("Reload {0}", "{1 1 1}"),
+        "FILEUNITnine", GT._("Reload {0}", "{444 666 1}"),
+        "FILEUNITnineRestricted", GT._("Reload {0} + Display {1}", new Object[] { "{444 666 1}", "555" } ),
+        "FILEUNITninePoly", GT._("Reload + Polyhedra"),
         
 
         "[set_axes]Menu", GT._("Axes"), 
@@ -942,12 +947,11 @@ class MainPopupResourceBundle extends PopupResource {
         "APPLETtranslationUrl", GT._("Translations")
     };
  
-    if (!wasTranslating)
-      GT.setDoTranslate(wasTranslating);
-
+    GT.setDoTranslate(wasTranslating);
     return words;
   }
   
+  @Override
   public String getMenu(String title) {
     return "# Jmol.mnu " + title + "\n\n" +
            "# Part I -- Menu Structure\n" +
