@@ -33,12 +33,12 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
-import java.util.Enumeration;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 import java.util.StringTokenizer;
 import java.util.Hashtable;
-import java.util.List;
-import java.util.ArrayList;
 
 import javax.swing.AbstractButton;
 import javax.swing.ButtonGroup;
@@ -70,22 +70,26 @@ public class SimplePopup {
   protected String nullModelSetName, modelSetName;
   protected String modelSetFileName, modelSetRoot;
   
-  protected Hashtable modelSetInfo, modelInfo;
+  protected Map<String, Object> modelSetInfo;
+  protected Map<String, Object> modelInfo;
   protected JPopupMenu frankPopup;
 
-  protected Hashtable htMenus = new Hashtable();
-  protected List NotPDB = new ArrayList();
-  protected List PDBOnly = new ArrayList();
-  protected List UnitcellOnly = new ArrayList();
-  protected List SingleModelOnly = new ArrayList();
-  protected List FramesOnly = new ArrayList();
-  protected List VibrationOnly = new ArrayList();
-  protected List SymmetryOnly = new ArrayList();
-  protected List SignedOnly = new ArrayList();
-  protected List AppletOnly = new ArrayList();
-  protected List ChargesOnly = new ArrayList();
-  protected List TemperatureOnly = new ArrayList();
+  protected Map<String, Object> htMenus = new Hashtable<String, Object>();
+  protected List<Object> NotPDB = new ArrayList<Object>();
+  protected List<Object> PDBOnly = new ArrayList<Object>();
+  protected List<Object> FileUnitOnly = new ArrayList<Object>();
+  protected List<Object> FileMolOnly = new ArrayList<Object>();
+  protected List<Object> UnitcellOnly = new ArrayList<Object>();
+  protected List<Object> SingleModelOnly = new ArrayList<Object>();
+  protected List<Object> FramesOnly = new ArrayList<Object>();
+  protected List<Object> VibrationOnly = new ArrayList<Object>();
+  protected List<Object> SymmetryOnly = new ArrayList<Object>();
+  protected List<Object> SignedOnly = new ArrayList<Object>();
+  protected List<Object> AppletOnly = new ArrayList<Object>();
+  protected List<Object> ChargesOnly = new ArrayList<Object>();
+  protected List<Object> TemperatureOnly = new ArrayList<Object>();
 
+  protected boolean fileHasUnitCell;
   protected boolean isPDB;
   protected boolean isSymmetry;
   protected boolean isUnitCell;
@@ -104,10 +108,18 @@ public class SimplePopup {
 
   protected JPopupMenu swingPopup;
 
+  SimplePopup() {
+    // required by reflection
+  }
+  
   SimplePopup(JmolViewer viewer) {
+    set(viewer);
+  }
+
+  protected void set(JmolViewer viewer) {
     this.viewer = viewer;
     asPopup = true;
-    display = viewer.getDisplay();
+    display = (Component) viewer.getDisplay();
     mil = new MenuItemListener();
     cmil = new CheckboxMenuItemListener();
   }
@@ -115,7 +127,7 @@ public class SimplePopup {
   private boolean isHorizontal = false;
   
   public SimplePopup(JmolViewer viewer, String title, PopupResource bundle, boolean isHorizontal) {
-    this(viewer);
+    set(viewer);
     this.isHorizontal = isHorizontal;
     build(title, swingPopup = new JPopupMenu(title), bundle);
   }
@@ -137,26 +149,28 @@ public class SimplePopup {
 
   protected int thisx, thisy;
 
-  public void show(int x, int y) {
-    show(x, y, true);
-  }
-
   protected void updateForShow() {
     // depends upon implementation
   }
   
+  /**
+   * @param x 
+   * @param y 
+   * @param doPopup  
+   */
   public void show(int x, int y, boolean doPopup) {
     thisx = x;
     thisy = y;
     updateForShow();
-    for (Enumeration keys = htCheckbox.keys(); keys.hasMoreElements();) {
-      String key = (String) keys.nextElement();
-      Object item = htCheckbox.get(key);
+    for (Map.Entry<String, JMenuItem> entry : htCheckbox.entrySet()) {
+      String key = entry.getKey();
+      Object item = entry.getValue();
       String basename = key.substring(0, key.indexOf(":"));
       boolean b = viewer.getBooleanProperty(basename);
       setCheckBoxState(item, b);
     }
-    showPopupMenu(thisx, thisy);
+    if (doPopup)
+      showPopupMenu(thisx, thisy);
   }
 
   static protected void addCurrentItem(StringBuffer sb, char type, int level, String name, String label, String script, String flags) {
@@ -181,7 +195,7 @@ public class SimplePopup {
     return (str == null ? key : str);
   }
 
-  boolean checkBoolean(Hashtable info, String key) {
+  boolean checkBoolean(Map<String, Object> info, String key) {
     Object value = (info == null ? null : info.get(key));
     return !(value == null || value instanceof Boolean && !((Boolean) (value)).booleanValue());
   }
@@ -206,11 +220,12 @@ public class SimplePopup {
     modelSetInfo = viewer.getModelSetAuxiliaryInfo();
     modelInfo = viewer.getModelAuxiliaryInfo(modelIndex);
     if (modelInfo == null)
-      modelInfo = new Hashtable();
+      modelInfo = new Hashtable<String, Object>();
     isPDB = checkBoolean(modelSetInfo, "isPDB");
+    isMultiFrame = (modelCount > 1);
     isSymmetry = checkBoolean(modelInfo, "hasSymmetry");
     isUnitCell = checkBoolean(modelInfo, "notionalUnitcell");
-    isMultiFrame = (modelCount > 1);
+    fileHasUnitCell = (isPDB && isUnitCell || checkBoolean(modelInfo, "fileHasUnitCell"));
     isLastFrame = (modelIndex == modelCount - 1);
     altlocs = viewer.getAltLocListInModel(modelIndex);
     isMultiConfiguration = (altlocs.length() > 0);
@@ -226,6 +241,10 @@ public class SimplePopup {
       enableMenu(PDBOnly.get(i), isPDB);
     for (int i = 0; i < UnitcellOnly.size(); i++)
       enableMenu(UnitcellOnly.get(i), isUnitCell);
+    for (int i = 0; i < FileUnitOnly.size(); i++)
+      enableMenu(FileUnitOnly.get(i), isUnitCell || fileHasUnitCell);
+    for (int i = 0; i < FileMolOnly.size(); i++)
+      enableMenu(FileMolOnly.get(i), isUnitCell || fileHasUnitCell);
     for (int i = 0; i < SingleModelOnly.size(); i++)
       enableMenu(SingleModelOnly.get(i), isLastFrame);
     for (int i = 0; i < FramesOnly.size(); i++)
@@ -327,6 +346,10 @@ public class SimplePopup {
         TemperatureOnly.add(newMenu);
       } else if (item.indexOf("UNITCELL") >= 0) {
         UnitcellOnly.add(newMenu);
+      } else if (item.indexOf("FILEUNIT") >= 0) {
+        FileUnitOnly.add(newMenu);
+      } else if (item.indexOf("FILEMOL") >= 0) {
+        FileMolOnly.add(newMenu);
       } 
       
       if (item.indexOf("!FRAMES") >= 0) {
@@ -352,13 +375,18 @@ public class SimplePopup {
     }
   }
 
+  /**
+   * @param item  
+   * @param subMenu 
+   * @param word 
+   */
   protected void checkSpecialMenu(String item, Object subMenu, String word) {
     // special considerations here
   }
 
-  protected Hashtable htCheckbox = new Hashtable();
+  protected Map<String, JMenuItem> htCheckbox = new Hashtable<String, JMenuItem>();
 
-  void rememberCheckbox(String key, Object checkboxMenuItem) {
+  void rememberCheckbox(String key, JMenuItem checkboxMenuItem) {
     htCheckbox.put(key + "::" + htCheckbox.size(), checkboxMenuItem);
   }
 
@@ -403,6 +431,12 @@ public class SimplePopup {
     viewer.evalStringQuiet(what);
   }
 
+  /**
+   * @param item  
+   * @param name 
+   * @param what 
+   * @return   option
+   */
   protected String setCheckBoxOption(JMenuItem item, String name, String what) {
     return null;
   }
@@ -460,7 +494,7 @@ public class SimplePopup {
     if (script == null || script.length() == 0)
       return;
     if (script.equals("MAIN")) {
-      show(thisx, thisy);
+      show(thisx, thisy, true);
       return;
     }
     String id = getId(source);
@@ -479,9 +513,9 @@ public class SimplePopup {
     return addMenuItem(menuItem, entry, "", null);
   }
 
-  Object addCheckboxMenuItem(Object menu, String entry, String basename,
+  JMenuItem addCheckboxMenuItem(Object menu, String entry, String basename,
                              String id, boolean isRadio) {
-    Object item = addCheckboxMenuItem(menu, entry, basename, id, false, isRadio);
+    JMenuItem item = addCheckboxMenuItem(menu, entry, basename, id, false, isRadio);
     rememberCheckbox(basename, item);
     return item;
   }
@@ -552,6 +586,10 @@ public class SimplePopup {
     ((JMenuItem) item).setSelected(state);
   }
 
+  /**
+   * @param name  
+   * @return  icon
+   */
   protected ImageIcon getIcon(String name) {
     return null; // subclassed
   }
@@ -572,7 +610,7 @@ public class SimplePopup {
       b.setActionCommand(script);
   }
   
-  Object addCheckboxMenuItem(Object menu, String entry, String basename,
+  JMenuItem addCheckboxMenuItem(Object menu, String entry, String basename,
                              String id, boolean state, boolean isRadio) {
     JMenuItem jm;
     if (isRadio) {
@@ -593,6 +631,10 @@ public class SimplePopup {
     return jm;
   }
 
+  /**
+   * @param menu  
+   * @return new menu
+   */
   Object cloneMenu(Object menu) {
     return null;
   }
