@@ -1,6 +1,6 @@
 /* $Author: hansonr $
- * $Date: 2012-03-14 12:05:59 +0100 (mer., 14 mars 2012) $
- * $Revision: 16883 $
+ * $Date: 2012-06-24 02:39:28 +0200 (dim., 24 juin 2012) $
+ * $Revision: 17325 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -88,26 +88,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
   String scriptExtensions;
   
   private ScriptFunction thisFunction;
-  
- 
-  /**
-   * return a structure that is only the first part of the process - identifying lines and commands
-   * for the scriptEditor
-   * 
-   * @param script
-   * @return        ScriptContext
-   */
-  
-  ScriptContext parseScriptForTokens(String script) {
-    this.script = script;
-    filename = null;
-    isCheckOnly = true;
-    isSilent = true;
-    logMessages = false;
-    preDefining = false;
-    return parseScript(false);
-  }
- 
+   
   private ScriptContext parseScript(boolean doFull) {
     if (!compile0(doFull))
       handleError();
@@ -129,7 +110,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return sc;
   }
 
-  ScriptContext compile(String filename, String script, boolean isPredefining,
+  synchronized ScriptContext compile(String filename, String script, boolean isPredefining,
                   boolean isSilent, boolean debugScript, boolean isCheckOnly) {
     this.isCheckOnly = isCheckOnly;
     this.filename = filename;
@@ -618,13 +599,17 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return (nTokens == 0 ? OK2 : CONTINUE);
   }
 
+  private boolean isComment;
+  
   private int processTokenList(short iLine, boolean doCompile) {
     if (nTokens > 0 || comment != null) {
       if (nTokens == 0) {
         // just a comment
         ichCurrentCommand = ichToken;
-        if (comment != null)
+        if (comment != null) {
+          isComment = true;
           addTokenToPrefix(new Token(Token.nada, comment));
+        }
       } else if (setBraceCount > 0 && endOfLine && ichToken < cchScript) {
         return CONTINUE;
       }
@@ -740,13 +725,15 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     }
     if (endOfLine) {
       if (flowContext != null && flowContext.checkForceEndIf()) {
-        forceFlowEnd(flowContext.token);
+        if (!isComment)
+          forceFlowEnd(flowContext.token);
         isEndOfCommand = true;
         cchToken = 0;
         ichCurrentCommand = ichToken;
         lineCurrent--;
         return CONTINUE;
       }
+      isComment = false;
       isShowCommand = false;
       ++lineCurrent;
     }
