@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-02-10 19:00:32 +0100 (jeu., 10 févr. 2011) $
- * $Revision: 15145 $
+ * $Date: 2011-08-05 02:05:49 +0200 (ven., 05 août 2011) $
+ * $Revision: 15933 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -24,6 +24,7 @@
 
 package org.jmol.shape;
 
+import org.jmol.constant.EnumPalette;
 import org.jmol.g3d.*;
 import org.jmol.modelset.Atom;
 import org.jmol.modelset.LabelToken;
@@ -35,6 +36,7 @@ import org.jmol.viewer.JmolConstants;
 
 import java.util.Hashtable;
 import java.util.BitSet;
+import java.util.Map;
 
 public class Labels extends AtomShape {
 
@@ -44,9 +46,11 @@ public class Labels extends AtomShape {
   byte[] fids;
   int[] offsets;
 
-  private Hashtable atomLabels = new Hashtable();
+  private Map<Integer, Text> atomLabels = new Hashtable<Integer, Text>();
   private Text text;
 
+  private Map<Integer, float[]> labelBoxes;
+
   private BitSet bsFontSet, bsBgColixSet;
 
   private int defaultOffset;
@@ -67,6 +71,7 @@ public class Labels extends AtomShape {
   
   //labels
 
+  @Override
   public void initShape() {
     super.initShape();
     defaultFontId = zeroFontId = g3d.getFont3D(JmolConstants.DEFAULT_FONTFACE,
@@ -78,8 +83,11 @@ public class Labels extends AtomShape {
     translucentAllowed = false;
   }
 
+  @Override
   public void setProperty(String propertyName, Object value, BitSet bsSelected) {
     isActive = true;
+    
+    //System.out.println(propertyName + " Labels " + value);
 
     if ("setDefaults" == propertyName) {
       setDefaults = ((Boolean) value).booleanValue();
@@ -88,7 +96,7 @@ public class Labels extends AtomShape {
 
     if ("color" == propertyName) {
       isActive = true;
-      byte pid = JmolConstants.pidOf(value);
+      byte pid = EnumPalette.pidOf(value);
       short colix = Graphics3D.getColix(value);
       if (!setDefaults)
         for (int i = atomCount; --i >= 0;)
@@ -123,10 +131,20 @@ public class Labels extends AtomShape {
     }
 
     if ("label" == propertyName) {
-      setLabel((String) value, bsSelected);
+      setScaling();
+      String strLabel = (String) value;
+      LabelToken[][] tokens = (strLabel == null || strLabel.length() == 0 ? nullToken : new LabelToken[][] { null });
+      for (int i = atomCount; --i >= 0;)
+        if (bsSelected.get(i)) 
+          setLabel(tokens, strLabel, i);
       return;
     }
 
+    if ("clearBoxes" == propertyName) {
+      labelBoxes = null;
+      return;
+    }
+    
     if ("translucency" == propertyName || "bgtranslucency" == propertyName) {
       // no translucency
       return;
@@ -252,6 +270,12 @@ public class Labels extends AtomShape {
           : ((Boolean) value).booleanValue() ? 1 : -1);
       if (mads == null)
         mads = new short[atomCount];
+      String strLabelPDB = null;
+      LabelToken[] tokensPDB = null;
+      String strLabelUNK = null;
+      LabelToken[] tokensUNK = null;
+      String strLabel;
+      LabelToken[] tokens;
       for (int atomIndex = atomCount; --atomIndex >= 0;) {
         if (bsSelected.get(atomIndex)) {
           Atom atom = atoms[atomIndex];
@@ -264,9 +288,23 @@ public class Labels extends AtomShape {
           } else {
             if (bsSizeSet == null)
               bsSizeSet = new BitSet();
-            String strLabel = viewer.getStandardLabelFormat();
             strings = ArrayUtil.ensureLength(strings, atomIndex + 1);
-            strings[atomIndex] = LabelToken.formatLabel(viewer, atom, strLabel);
+            if (atom.getGroup3(false).equals("UNK")) {
+              if (strLabelUNK == null) {
+                strLabelUNK = viewer.getStandardLabelFormat(1);
+                tokensUNK = LabelToken.compile(viewer, strLabelUNK, '\0', null);
+              }              
+              strLabel = strLabelUNK;
+              tokens = tokensUNK;
+            } else {
+              if (strLabelPDB == null) {
+                strLabelPDB = viewer.getStandardLabelFormat(2);
+                tokensPDB = LabelToken.compile(viewer, strLabelPDB, '\0', null);
+              }
+              strLabel = strLabelPDB;
+              tokens = tokensPDB;
+            }
+            strings[atomIndex] = LabelToken.formatLabel(viewer, atom, tokens, '\0', null);
             formats[atomIndex] = strLabel;
             bsSizeSet.set(atomIndex);
             if ((bsBgColixSet == null || !bsBgColixSet.get(atomIndex))
@@ -285,11 +323,13 @@ public class Labels extends AtomShape {
     }
 
     if (propertyName.startsWith("label:")) {
-      setLabel(propertyName.substring(6), ((Integer)value).intValue());
+      setScaling();
+      setLabel(new LabelToken[1][], propertyName.substring(6), ((Integer)value).intValue());
       return;
     }
 
     if (propertyName == "deleteModelAtoms") {
+      labelBoxes = null;
       int firstAtomDeleted = ((int[]) ((Object[]) value)[2])[1];
       int nAtomsDeleted = ((int[]) ((Object[]) value)[2])[2];
       fids = (byte[]) ArrayUtil.deleteElements(fids, firstAtomDeleted,
@@ -311,67 +351,62 @@ public class Labels extends AtomShape {
 
   }
 
-  private void setLabel(String value, BitSet bsSelected) {
+  private final static LabelToken[][] nullToken = new LabelToken[][] { null };
+  private boolean isScaled;
+  private float scalePixelsPerMicron;
+  
+  private void setScaling() {
     isActive = true;
     if (bsSizeSet == null)
       bsSizeSet = new BitSet();
-    String strLabel = (String) value;
-    boolean isScaled = viewer.getFontScaling();
-    float scalePixelsPerMicron = (isScaled ? viewer
+    isScaled = viewer.getFontScaling();
+    scalePixelsPerMicron = (isScaled ? viewer
         .getScalePixelsPerAngstrom(false) * 10000f : 0);
-    for (int i = atomCount; --i >= 0;)
-      if (bsSelected.get(i)) 
-        setLabel(strLabel, i, isScaled, scalePixelsPerMicron);
-  }
-
-  private void setLabel(String value, int i) {
-    isActive = true;
-    if (bsSizeSet == null)
-      bsSizeSet = new BitSet();
-    String strLabel = (String) value;
-    boolean isScaled = viewer.getFontScaling();
-    float scalePixelsPerMicron = (isScaled ? viewer
-        .getScalePixelsPerAngstrom(false) * 10000f : 0);
-    setLabel(strLabel, i, isScaled, scalePixelsPerMicron);
   }
   
-  private void setLabel(String strLabel, int i, boolean isScaled, float scalePixelsPerMicron) {
-      Atom atom = atoms[i];
-      String label = LabelToken.formatLabel(viewer, atom, strLabel);
-      atom.setShapeVisibility(myVisibilityFlag, label != null);
-      if (strings == null || i >= strings.length)
-        strings = ArrayUtil.ensureLength(strings, i + 1);
-      if (formats == null || i >= formats.length)
-        formats = ArrayUtil.ensureLength(formats, i + 1);
-      strings[i] = label;
-      formats[i] = strLabel;
-      bsSizeSet.set(i, (strLabel != null));
-      text = getLabel(i);
-      if (isScaled) {
-        text = new Text(g3d, null, label, (short) 0, (short) 0, 0, 0, 0, 0,
-            0, scalePixelsPerMicron);
-        putLabel(i, text);
-      } else if (text != null) {
-        text.setText(label);
-      }
-      if (defaultOffset != zeroOffset)
-        setOffsets(i, defaultOffset, false);
-      if (defaultAlignment != Object2d.ALIGN_LEFT)
-        setAlignment(i, defaultAlignment);
-      if ((defaultZPos & FRONT_FLAG) != 0)
-        setFront(i, true);
-      else if ((defaultZPos & GROUP_FLAG) != 0)
-        setGroup(i, true);
-      if (defaultPointer != Object2d.POINTER_NONE)
-        setPointer(i, defaultPointer);
-      if (defaultColix != 0 || defaultPaletteID != 0)
-        setColix(i, defaultColix, defaultPaletteID);
-      if (defaultBgcolix != 0)
-        setBgcolix(i, defaultBgcolix);
-      if (defaultFontId != zeroFontId)
-        setFont(i, defaultFontId);
- }
+  private void setLabel(LabelToken[][] temp, String strLabel, int i) {
+    Atom atom = atoms[i];
+    LabelToken[] tokens = temp[0];
+    if (tokens == null)
+      tokens = temp[0] = LabelToken.compile(viewer, strLabel, '\0', null);
+    String label = (tokens == null ? null : LabelToken.formatLabel(viewer,
+        atom, tokens, '\0', null));
+    atom.setShapeVisibility(myVisibilityFlag, label != null);
+    if (strings == null || i >= strings.length)
+      strings = ArrayUtil.ensureLength(strings, i + 1);
+    if (formats == null || i >= formats.length)
+      formats = ArrayUtil.ensureLength(formats, i + 1);
+    strings[i] = label;
+    formats[i] = (strLabel != null && strLabel.indexOf("%{") >= 0 ? label
+        : strLabel);
+    bsSizeSet.set(i, (strLabel != null));
+    text = getLabel(i);
+    if (isScaled) {
+      text = new Text(g3d, null, label, (short) 0, (short) 0, 0, 0, 0, 0, 0,
+          scalePixelsPerMicron);
+      putLabel(i, text);
+    } else if (text != null) {
+      text.setText(label);
+    }
+    if (defaultOffset != zeroOffset)
+      setOffsets(i, defaultOffset, false);
+    if (defaultAlignment != Object2d.ALIGN_LEFT)
+      setAlignment(i, defaultAlignment);
+    if ((defaultZPos & FRONT_FLAG) != 0)
+      setFront(i, true);
+    else if ((defaultZPos & GROUP_FLAG) != 0)
+      setGroup(i, true);
+    if (defaultPointer != Object2d.POINTER_NONE)
+      setPointer(i, defaultPointer);
+    if (defaultColix != 0 || defaultPaletteID != 0)
+      setColix(i, defaultColix, defaultPaletteID);
+    if (defaultBgcolix != 0)
+      setBgcolix(i, defaultBgcolix);
+    if (defaultFontId != zeroFontId)
+      setFont(i, defaultFontId);
+  }
 
+  @Override
   public Object getProperty(String property, int index) {
     if (property.equals("offsets"))
       return offsets;
@@ -385,15 +420,27 @@ public class Labels extends AtomShape {
 
   void putLabel(int i, Text text) {
     if (text == null)
-      atomLabels.remove(new Integer(i));
+      atomLabels.remove(Integer.valueOf(i));
     else
-      atomLabels.put(new Integer(i), text);
+      atomLabels.put(Integer.valueOf(i), text);
   }
 
   Text getLabel(int i) {
-    return (Text) atomLabels.get(new Integer(i));
+    return atomLabels.get(Integer.valueOf(i));
   }
 
+  void putBox(int i, float[] boxXY) {
+    if (labelBoxes == null)
+      labelBoxes = new Hashtable<Integer, float[]>(); 
+    labelBoxes.put(Integer.valueOf(i), boxXY);
+  }
+
+  float[] getBox(int i) {
+    if (labelBoxes == null)
+      return null;
+    return labelBoxes.get(Integer.valueOf(i));
+  }
+  
   private void setColix(int i, short colix, byte pid) {
     setColixAndPalette(colix, pid, i);
     // text is only created by labelsRenderer
@@ -509,6 +556,7 @@ public class Labels extends AtomShape {
     }
   }
 
+  @Override
   public void setModelClickability() {
     if (strings == null)
       return;
@@ -539,12 +587,13 @@ public class Labels extends AtomShape {
     return s.toString();
   }
 
+  @Override
   public String getShapeState() {
     if (!isActive || bsSizeSet == null)
       return "";
-    Hashtable temp = new Hashtable();
-    Hashtable temp2 = new Hashtable();
-    Hashtable temp3 = new Hashtable();
+    Map<String, BitSet> temp = new Hashtable<String, BitSet>();
+    Map<String, BitSet> temp2 = new Hashtable<String, BitSet>();
+    Map<String, BitSet> temp3 = new Hashtable<String, BitSet>();
     for (int i = bsSizeSet.nextSetBit(0); i >= 0; i = bsSizeSet
         .nextSetBit(i + 1)) {
       setStateInfo(temp, i, "label " + Escape.escape(formats[i]));
@@ -590,18 +639,24 @@ public class Labels extends AtomShape {
         + getShapeCommands(null, temp3);
   }
 
-  int pickedAtom = -1;
-  int pickedOffset = 0;
+  private int pickedAtom = -1;
+  private int pickedOffset = 0;
+  private int pickedX;
+  private int pickedY;
+  
+  @Override
   public synchronized boolean checkObjectDragged(int prevX, int prevY, int x,
                                                  int y, int modifiers,
                                                  BitSet bsVisible) {
-    if (viewer.getPickingMode() != ActionManager.PICKING_LABEL)
+    if (viewer.getPickingMode() != ActionManager.PICKING_LABEL || labelBoxes == null)
       return false;
     // mouse down ?
     if (prevX == Integer.MIN_VALUE) {
-      int iAtom = viewer.findNearestAtomIndex(x, y);
-      if (iAtom >= 0 && atoms[iAtom].isVisible(myVisibilityFlag)) {
+      int iAtom = findNearestLabel(x, y);
+      if (iAtom >= 0) {
         pickedAtom = iAtom;
+        pickedX = x;
+        pickedY = y;
         pickedOffset = (offsets == null 
             || pickedAtom >= offsets.length ? 0 
                 : offsets[pickedAtom]) >> FLAG_OFFSET;
@@ -620,16 +675,35 @@ public class Labels extends AtomShape {
     return true;
   }
                          
-  private void move2D(int pickedAtom, int x, int y) {
-    if (g3d.isAntialiased()) {
-      x <<= 1;
-      y <<= 1;
+  private int findNearestLabel(int x, int y) {
+    if (labelBoxes == null)
+      return -1;
+    float dmin = Float.MAX_VALUE;
+    int imin = -1;
+    float zmin = Float.MAX_VALUE;
+    for (Map.Entry<Integer, float[]> entry : labelBoxes.entrySet()) {
+      if (!atoms[entry.getKey().intValue()].isVisible(myVisibilityFlag))
+        continue;
+      float[] boxXY = entry.getValue();
+      float dx = x - boxXY[0];
+      float dy = y - boxXY[1];
+      if (dx <= 0 || dy <= 0 || dx >= boxXY[2] || dy >= boxXY[3] || boxXY[4] > zmin)
+        continue;
+      zmin = boxXY[4];
+      float d = Math.min(Math.abs(dx - boxXY[2]/2), Math.abs(dy - boxXY[3]/2));
+      if (d <= dmin) {
+        dmin = d;
+        imin = entry.getKey().intValue();
+      }
     }
-    Atom atom = atoms[pickedAtom];
+    return imin;
+  }
+
+  private void move2D(int pickedAtom, int x, int y) {
     int xOffset = Object2d.getXOffset(pickedOffset);
     int yOffset = -Object2d.getYOffset(pickedOffset);
-    xOffset += x - atom.screenX;
-    yOffset += atom.screenY - y;
+    xOffset += x - pickedX;
+    yOffset += pickedY - y;
     int offset = Object2d.getOffset(xOffset, yOffset);
     if (offset == 0)
       offset = Short.MAX_VALUE;
