@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Date: 2012-05-28 19:31:14 +0200 (lun., 28 mai 2012) $
+ * $Revision: 17225 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -25,6 +25,7 @@
 package org.jmol.shape;
 
 import org.jmol.api.JmolMeasurementClient;
+import org.jmol.atomdata.RadiusData;
 import org.jmol.g3d.*;
 import org.jmol.modelset.Atom;
 import org.jmol.modelset.Measurement;
@@ -37,10 +38,11 @@ import org.jmol.modelset.TickInfo;
 import org.jmol.viewer.JmolConstants;
 import org.jmol.script.Token;
 
+import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.List;
-import java.util.ArrayList;
 import java.util.Hashtable;
+import java.util.Map;
 
 
 public class Measures extends Shape implements JmolMeasurementClient {
@@ -50,12 +52,13 @@ public class Measures extends Shape implements JmolMeasurementClient {
   private String strFormat;
   private boolean mustBeConnected = false;
   private boolean mustNotBeConnected = false;
-  private float[] rangeMinMax = {Float.MAX_VALUE, Float.MAX_VALUE};
+  private RadiusData radiusData;
+  private Boolean intramolecular;
 
   private Atom[] atoms;
 
   int measurementCount = 0;
-  final List measurements = new ArrayList();
+  final List<Measurement> measurements = new ArrayList<Measurement>();
   MeasurementPending measurementPending;
   
   short mad = (short)-1;
@@ -66,31 +69,35 @@ public class Measures extends Shape implements JmolMeasurementClient {
   TickInfo tickInfo;
   TickInfo defaultTickInfo;
   
+  @Override
   protected void initModelSet() {
     for (int i = measurements.size(); --i >= 0; ) {
-      Measurement m = (Measurement) measurements.get(i);
+      Measurement m = measurements.get(i);
       if (m != null)
         m.modelSet = modelSet;
     }
     atoms = modelSet.atoms;
   }
   
+  @Override
   public void initShape() {
     super.initShape();
     font3d = g3d.getFont3D(JmolConstants.MEASURE_DEFAULT_FONTSIZE);
   }
 
+  @Override
   protected void setSize(int size, BitSet bsSelected) {
     mad = (short)size;
   }
 
+  @Override
   public void setProperty(String propertyName, Object value, BitSet bsIgnored) {
     // the following can be used with "select measures ({bitset})"
     
     Measurement mt;
     if ("clearModelIndex" == propertyName) {
       for (int i = 0; i < measurementCount; i++)
-        ((Measurement) measurements.get(i)).setModelIndex((short) 0);
+        measurements.get(i).setModelIndex((short) 0);
       return;
     }
     
@@ -124,7 +131,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
     if ((isRefresh = ("refresh" == propertyName)) 
         || "refreshTrajectories" == propertyName) {
       for (int i = measurements.size(); --i >= 0;)
-        if ((mt = (Measurement) measurements.get(i)) != null 
+        if ((mt = measurements.get(i)) != null 
             && (isRefresh || mt.isTrajectory()))
           mt.refresh();
       return;
@@ -159,7 +166,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
         int[] v = new int[len + 1];
         v[0] = len;
         System.arraycopy(maps[i], 0, v, 1, len);
-        toggleOn((int[]) v);
+        toggleOn(v);
       }
     } else if ("measure" == propertyName) {
       MeasurementData md = (MeasurementData) value;
@@ -168,10 +175,10 @@ public class Measures extends Shape implements JmolMeasurementClient {
         defaultTickInfo = md.tickInfo;
         return;
       }
-      rangeMinMax[0] = md.rangeMinMax[0];
-      rangeMinMax[1] = md.rangeMinMax[1];
+      radiusData = md.radiusData;
       mustBeConnected = md.mustBeConnected;
       mustNotBeConnected = md.mustNotBeConnected;
+      intramolecular = md.intramolecular;
       strFormat = md.strFormat;
       if (md.isAll) {
         if (tickInfo != null)
@@ -214,7 +221,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
       int nAtomsDeleted = ((int[])((Object[])value)[2])[2];
       int atomMax = firstAtomDeleted + nAtomsDeleted;
       for (int i = measurementCount; --i >= 0;) {
-        mt = (Measurement) measurements.get(i);
+        mt = measurements.get(i);
         int[] indices = mt.getCountPlusIndices();
         for (int j = 1; j <= indices[0]; j++) {
           int iAtom = indices[j];
@@ -265,7 +272,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
     
   }
 
-  private Measurement setSingleItem(List vector) {
+  private Measurement setSingleItem(List<Object> vector) {
     Point3fi[] points = new Point3fi[4];
     int[] indices = new int[5];
     indices[0] = vector.size();
@@ -284,18 +291,19 @@ public class Measures extends Shape implements JmolMeasurementClient {
     return new Measurement(modelSet, indices, points, tickInfo == null ? defaultTickInfo : tickInfo);
   }
 
+  @Override
   public Object getProperty(String property, int index) {
     if ("pending".equals(property))
       return measurementPending;
     if ("count".equals(property))
-      return new Integer(measurementCount);
+      return Integer.valueOf(measurementCount);
     if ("countPlusIndices".equals(property))
       return (index < measurementCount ? 
-          ((Measurement) measurements.get(index)).getCountPlusIndices() : null);
+          measurements.get(index).getCountPlusIndices() : null);
     if ("stringValue".equals(property))
-      return (index < measurementCount ? ((Measurement) measurements.get(index)).getString() : null);
+      return (index < measurementCount ? measurements.get(index).getString() : null);
     if ("pointInfo".equals(property))
-      return ((Measurement) measurements.get(index / 10)).getLabel(index % 10, false, false);
+      return measurements.get(index / 10).getLabel(index % 10, false, false);
     if ("info".equals(property))
       return getAllInfo();
     if ("infostring".equals(property))
@@ -318,7 +326,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
         this.colix = colix;
     Measurement mt;
     for (int i = measurements.size(); --i >= 0; )
-      if ((mt = (Measurement) measurements.get(i)) != null
+      if ((mt = measurements.get(i)) != null
           && (bsSelected != null && bsSelected.get(i) || bsSelected == null
               && (colix == Graphics3D.INHERIT_ALL || mt.getColix() == Graphics3D.INHERIT_ALL))) {
         mt.setColix(colix);
@@ -331,27 +339,27 @@ public class Measures extends Shape implements JmolMeasurementClient {
       format = null;
     for (int i = measurements.size(); --i >= 0;)
       if (bsSelected == null || bsSelected.get(i))
-        ((Measurement) measurements.get(i)).formatMeasurement(format, null, false);
+        measurements.get(i).formatMeasurement(format, null, false);
   }
   
   private void showHide(boolean isHide) {
     for (int i = measurements.size(); --i >= 0;)
       if (bsSelected == null || bsSelected.get(i))
-        ((Measurement) measurements.get(i)).setHidden(isHide);
+        measurements.get(i).setHidden(isHide);
   }
 
   private void showHide(Measurement m, boolean isHide) {
     int i = find(m);
     if (i >= 0)
-      ((Measurement) measurements.get(i)).setHidden(isHide);
+      measurements.get(i).setHidden(isHide);
   }
   
   private void toggle(Measurement m) {
-    rangeMinMax[0] = Float.MAX_VALUE;
+    radiusData = null;
     //toggling one that is hidden should be interpreted as DEFINE
     int i = find(m);
     Measurement mt;
-    if (i >= 0 && !(mt = (Measurement) measurements.get(i)).isHidden()) // delete it and all like it
+    if (i >= 0 && !(mt = measurements.get(i)).isHidden()) // delete it and all like it
       define(i, mt, true, false, false);
     else // define OR turn on if measureAllModels
       define(-1, m, false, true, false);
@@ -359,7 +367,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
   }
 
   private void toggleOn(int[] indices) {
-    rangeMinMax[0] = Float.MAX_VALUE;
+    radiusData = null;
     //toggling one that is hidden should be interpreted as DEFINE
     bsSelected = new BitSet();
     define(Integer.MIN_VALUE, new Measurement(modelSet, indices, null, defaultTickInfo), false, true, true);
@@ -368,11 +376,11 @@ public class Measures extends Shape implements JmolMeasurementClient {
   }
 
   private void delete(Measurement m) {
-    rangeMinMax[0] = Float.MAX_VALUE;
+    radiusData = null;
     //toggling one that is hidden should be interpreted as DEFINE
     int i = find(m);
     if (i >= 0)
-      define(i, (Measurement) measurements.get(i), true, false, false);
+      define(i, measurements.get(i), true, false, false);
     setIndices();
   }
 
@@ -405,22 +413,22 @@ public class Measures extends Shape implements JmolMeasurementClient {
     }
     // we create a set of atoms involving all atoms with the
     // same atom number in each model
-    List points = new ArrayList();
+    List<Object> points = new ArrayList<Object>();
     int nPoints = m.getCount();
     for (int i = 1; i <= nPoints; i++) {
       int atomIndex = m.getAtomIndex(i);
       points.add(atomIndex >= 0 ? (Object) viewer.getAtomBits(
-          Token.atomno, new Integer(atoms[atomIndex].getAtomNumber()))
+          Token.atomno, Integer.valueOf(atoms[atomIndex].getAtomNumber()))
           : (Object) m.getAtom(i));
     }
     MeasurementData md = new MeasurementData(points, 
                    tokAction,
-                   rangeMinMax, 
+                   radiusData, 
                    strFormat, null,
                    tickInfo,
                    mustBeConnected,
                    mustNotBeConnected,
-                   true);
+                   intramolecular, true);
     define(md, (isDelete ? Token.delete 
         : Token.define
         ));
@@ -432,7 +440,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
 
   private void setIndices() {
     for (int i = 0; i < measurementCount; i++)
-      ((Measurement) measurements.get(i)).setIndex(i);
+      measurements.get(i).setIndex(i);
   }
   
   private int tokAction;
@@ -450,11 +458,10 @@ public class Measures extends Shape implements JmolMeasurementClient {
       if (tokAction == Token.delete) {
         deleteMeasurement(iThis);
       } else if (strFormat != null) {
-        ((Measurement) measurements.get(iThis)).formatMeasurement(strFormat,
+        measurements.get(iThis).formatMeasurement(strFormat,
             null, true);
       } else {
-        ((Measurement) measurements.get(iThis))
-            .setHidden(tokAction == Token.off);
+        measurements.get(iThis).setHidden(tokAction == Token.off);
       }
     } else if (tokAction == Token.define || tokAction == Token.opToggle) {
       m.tickInfo = (tickInfo == null ? defaultTickInfo : tickInfo);
@@ -464,13 +471,12 @@ public class Measures extends Shape implements JmolMeasurementClient {
 
   private void defineMeasurement(int i, Measurement m, boolean doSelect) {
     float value = m.getMeasurement();
-    if (rangeMinMax[0] != Float.MAX_VALUE
-        && (value < rangeMinMax[0] || value > rangeMinMax[1]))
+    if (radiusData != null && !m.isInRange(radiusData, value))
       return;
     if (i == Integer.MIN_VALUE)
       i = find(m);
     if (i >= 0) {
-      ((Measurement) measurements.get(i)).setHidden(false);
+      measurements.get(i).setHidden(false);
       if (doSelect)
         bsSelected.set(i);
       return;
@@ -479,11 +485,11 @@ public class Measures extends Shape implements JmolMeasurementClient {
         strFormat, measurementCount);
     measurements.add(measureNew);
     viewer.setStatusMeasuring("measureCompleted", measurementCount++,
-        measureNew.toVector().toString(), measureNew.getValue());
+        measureNew.toVector(false).toString(), measureNew.getValue());
   }
 
   private void deleteMeasurement(int i) {
-    String msg = ((Measurement) measurements.get(i)).toVector().toString();
+    String msg = measurements.get(i).toVector(true).toString();
     measurements.remove(i);
     measurementCount--;
     viewer.setStatusMeasuring("measureDeleted", i, msg, 0);
@@ -495,16 +501,16 @@ public class Measures extends Shape implements JmolMeasurementClient {
       return;
     if (measurementPending.getCount() > 1)
       viewer.setStatusMeasuring("measurePending",
-          measurementPending.getCount(), measurementPending.toVector().toString(), measurementPending.getValue());
+          measurementPending.getCount(), measurementPending.toVector(false).toString(), measurementPending.getValue());
   }
 
   private void reformatDistances() {
     for (int i = measurementCount; --i >= 0; )
-      ((Measurement) measurements.get(i)).reformatDistanceIfSelected();    
+      measurements.get(i).reformatDistanceIfSelected();    
   }
   
-  private List getAllInfo() {
-    List info = new ArrayList();
+  private List<Map<String, Object>> getAllInfo() {
+    List<Map<String, Object>> info = new ArrayList<Map<String,Object>>();
     for (int i = 0; i< measurementCount; i++) {
       info.add(getInfo(i));
     }
@@ -519,15 +525,15 @@ public class Measures extends Shape implements JmolMeasurementClient {
     return info;
   }
   
-  private Hashtable getInfo(int index) {
-    Measurement m = (Measurement) measurements.get(index);
+  private Map<String, Object> getInfo(int index) {
+    Measurement m = measurements.get(index);
     int count = m.getCount();
-    Hashtable info = new Hashtable();
-    info.put("index", new Integer(index));
+    Map<String, Object> info = new Hashtable<String, Object>();
+    info.put("index", Integer.valueOf(index));
     info.put("type", (count == 2 ? "distance" : count == 3 ? "angle"
         : "dihedral"));
     info.put("strMeasurement", m.getString());
-    info.put("count", new Integer(count));
+    info.put("count", Integer.valueOf(count));
     info.put("value", new Float(m.getValue()));
     TickInfo tickInfo = m.getTickInfo();
     if (tickInfo != null) {
@@ -539,13 +545,13 @@ public class Measures extends Shape implements JmolMeasurementClient {
       if (!Float.isNaN(tickInfo.first))
         info.put("tickStart", new Float(tickInfo.first));
     }
-    List atomsInfo = new ArrayList();
+    List<Map<String, Object>> atomsInfo = new ArrayList<Map<String,Object>>();
     for (int i = 1; i <= count; i++) {
-      Hashtable atomInfo = new Hashtable();
+      Map<String, Object> atomInfo = new Hashtable<String, Object>();
       int atomIndex = m.getAtomIndex(i);
-      atomInfo.put("_ipt", new Integer(atomIndex));
+      atomInfo.put("_ipt", Integer.valueOf(atomIndex));
       atomInfo.put("coord", Escape.escape(m.getAtom(i)));
-      atomInfo.put("atomno", new Integer(atomIndex < 0 ? -1 : atoms[atomIndex].getAtomNumber()));
+      atomInfo.put("atomno", Integer.valueOf(atomIndex < 0 ? -1 : atoms[atomIndex].getAtomNumber()));
       atomInfo.put("info", (atomIndex < 0 ? "<point>" : atoms[atomIndex].getInfo()));
       atomsInfo.add(atomInfo);
     }
@@ -554,14 +560,14 @@ public class Measures extends Shape implements JmolMeasurementClient {
   }
 
   private String getInfoAsString(int index) {
-    return ((Measurement) measurements.get(index)).getInfoAsString(null);
+    return measurements.get(index).getInfoAsString(null);
   }
   
   void setVisibilityInfo() {
     BitSet bsModels = viewer.getVisibleFramesBitSet();
     out:
     for (int i = measurementCount; --i >= 0; ) {
-      Measurement m = ((Measurement) measurements.get(i));
+      Measurement m = measurements.get(i);
       m.setVisible(false);
       if(mad == 0 || m.isHidden())
         continue;
@@ -580,7 +586,8 @@ public class Measures extends Shape implements JmolMeasurementClient {
     }
   }
   
- public String getShapeState() {
+ @Override
+public String getShapeState() {
     StringBuffer commands = new StringBuffer("");
     appendCmd(commands, "measures delete");
     for (int i = 0; i < measurementCount; i++)
@@ -588,10 +595,10 @@ public class Measures extends Shape implements JmolMeasurementClient {
     appendCmd(commands, "select *; set measures " + viewer.getMeasureDistanceUnits());
     appendCmd(commands, getFontCommand("measures", font3d));
     int nHidden = 0;
-    Hashtable temp = new Hashtable();
+    Map<String, BitSet> temp = new Hashtable<String, BitSet>();
     BitSet bs = new BitSet(measurementCount);
     for (int i = 0; i < measurementCount; i++) {
-      Measurement m = ((Measurement) measurements.get(i));
+      Measurement m = measurements.get(i);
       if (m.isHidden()) {
         nHidden++;
         bs.set(i);
@@ -615,7 +622,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
       commands.append(";\n");
     }
     if (mad >= 0)
-      commands.append(" set measurements " + (mad / 2000f));
+      commands.append(" set measurements " + (mad / 2000f)).append(";\n");
     String s = getShapeCommands(temp, null, "select measures");
     if (s != null && s.length() != 0) {
       commands.append(s);
@@ -626,7 +633,7 @@ public class Measures extends Shape implements JmolMeasurementClient {
   }
   
   private String getState(int index) {
-    Measurement m = ((Measurement) measurements.get(index));
+    Measurement m = measurements.get(index);
     int count = m.getCount();
     StringBuffer sb = new StringBuffer("measure");
     TickInfo tickInfo = m.getTickInfo();
