@@ -24,13 +24,15 @@
 
 package org.jmol.shape;
 
-import org.jmol.g3d.*;
-import org.jmol.viewer.JmolConstants;
 import org.jmol.viewer.StateManager;
+import org.jmol.g3d.Graphics3D;
+import org.jmol.script.ScriptVariable;
 import org.jmol.script.Token;
 
 import java.util.BitSet;
 import java.util.Hashtable;
+import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.Point3f;
 
@@ -46,42 +48,46 @@ public abstract class MeshCollection extends Shape {
   public int meshCount;
   public Mesh[] meshes = new Mesh[4];
   public Mesh currentMesh;
-  public int modelCount;
   public boolean isFixed;  
-  public String script;
   public int nUnnamed;
   public short colix;
   public String myType;
   public boolean explicitID;
-  public String actualID;
   protected String previousMeshID;
   protected Mesh linkedMesh;
-  protected boolean iHaveModelIndex;
   protected int modelIndex;
-  protected boolean allowContourLines;
-  protected boolean haveContours = false;
-  
+
+  protected float displayWithinDistance2;
+  protected boolean isDisplayWithinNot;
+  protected List<Point3f> displayWithinPoints;
+  protected BitSet bsDisplay;
+
   public String[] title;
-  protected boolean allowMesh = true;
   
-  protected Mesh pickedMesh = null;
+  protected Mesh pickedMesh;
   protected int pickedModel;
   protected int pickedVertex;
+  
+  protected int[] connections;
 
   private Mesh setMesh(String thisID) {
     linkedMesh = null;
     if (thisID == null || TextFormat.isWild(thisID)) {
+      if (thisID != null)
+        previousMeshID = thisID;
       currentMesh = null;
       return null;
     }
     currentMesh = getMesh(thisID);
     if (currentMesh == null) {
       allocMesh(thisID, null);
-    } else if (thisID.equals(JmolConstants.PREVIOUS_MESH_ID)) {
+    } else if (thisID.equals(MeshCollection.PREVIOUS_MESH_ID)) {
       linkedMesh = currentMesh.linkedMesh;
     }
     if (currentMesh.thisID == null) {
-      currentMesh.thisID = myType + (++nUnnamed);
+      if (nUnnamed == 0 || getMesh(myType + nUnnamed) != null)
+        nUnnamed++;
+      currentMesh.thisID = myType + nUnnamed;
       if (htObjects != null)
         htObjects.put(currentMesh.thisID.toUpperCase(), currentMesh);
     }
@@ -89,7 +95,9 @@ public abstract class MeshCollection extends Shape {
     return currentMesh;
   }
 
-  protected Hashtable htObjects;
+  protected Map<String, Mesh> htObjects;
+  protected int color;
+  public final static String PREVIOUS_MESH_ID = "+PREVIOUS_MESH+";
   
   public void allocMesh(String thisID, Mesh m) {
     // this particular version is only run from privately;
@@ -97,6 +105,7 @@ public abstract class MeshCollection extends Shape {
     int index = meshCount++;
     meshes = (Mesh[])ArrayUtil.ensureLength(meshes, meshCount * 2);
     currentMesh = meshes[index] = (m == null ? new Mesh(thisID, g3d, colix, index) : m);
+    currentMesh.color = color;
     currentMesh.index = index;
     if (thisID != null && htObjects != null)
       htObjects.put(thisID.toUpperCase(), currentMesh);
@@ -109,6 +118,7 @@ public abstract class MeshCollection extends Shape {
    * @param shape 
    * 
    */
+  @Override
   public void merge(Shape shape) {
     MeshCollection mc = (MeshCollection) shape;
     for (int i = 0; i < mc.meshCount; i++) {
@@ -127,12 +137,15 @@ public abstract class MeshCollection extends Shape {
     currentMesh = null;
   }
 
+  @Override
   public void initShape() {
     super.initShape();
     colix = Graphics3D.ORANGE;
-    modelCount = viewer.getModelCount();
+    color = 0xFFFFFFFF;
   }
   
+ @SuppressWarnings("unchecked")
+ @Override
  public void setProperty(String propertyName, Object value, BitSet bs) {
 
    if (propertyName == "setXml") {
@@ -159,7 +172,8 @@ public abstract class MeshCollection extends Shape {
 
     if ("variables" == propertyName) {
       if (currentMesh != null && currentMesh.scriptCommand != null && !currentMesh.scriptCommand.startsWith("{"))
-        currentMesh.scriptCommand = "{\n" + StateManager.getVariableList((Hashtable) value, 0, false) + "\n" + currentMesh.scriptCommand;
+        currentMesh.scriptCommand = "{\n" 
+          + StateManager.getVariableList((Map<String, ScriptVariable>) value, 0, false, false) + "\n" + currentMesh.scriptCommand;
       return;
     }
 
@@ -212,17 +226,20 @@ public abstract class MeshCollection extends Shape {
       if (value == null)
         return;
       colix = Graphics3D.getColix(value);
-      setTokenProperty(Token.color, false);
+      color = ((Integer) value).intValue();
+      if (currentMesh != null)
+        currentMesh.color = color;
+      setTokenProperty(Token.color, false, false);
       return;
     }
 
     if ("translucency" == propertyName) {
-      setTokenProperty(Token.translucent, (((String) value).equals("translucent")));
+      setTokenProperty(Token.translucent, (((String) value).equals("translucent")), false);
       return;
     }
 
     if ("hidden" == propertyName) {
-      value = new Integer(((Boolean)value).booleanValue() ? Token.off: Token.on);
+      value = Integer.valueOf(((Boolean)value).booleanValue() ? Token.off: Token.on);
       propertyName = "token";
       //continue
     }
@@ -232,6 +249,7 @@ public abstract class MeshCollection extends Shape {
       int tok2 = 0;
       boolean test = true;
       switch (tok) {
+      case Token.display:
       case Token.on:
       case Token.frontlit:
       case Token.backlit:
@@ -250,8 +268,8 @@ public abstract class MeshCollection extends Shape {
         break;
       case Token.nocontourlines:
         test = false;
-        // TODO leave this as is for now; probably TRUE is correct...
-        tok = Token.contourlines; //(true || allowContourLines ? Token.contourlines : Token.mesh);
+        // TODO leave this as is for now; probably correct...
+        tok = Token.contourlines;//(allowContourLines ? Token.contourlines : Token.mesh);
         tok2 = Token.mesh;
         break;
       case Token.mesh:
@@ -281,11 +299,9 @@ public abstract class MeshCollection extends Shape {
       default:
         Logger.error("PROBLEM IN MESHCOLLECTION: token? " + Token.nameOf(tok));
       }
-      setTokenProperty(tok, test);
-      if (tok2 != 0) {
-        if (currentMesh.havePlanarContours && currentMesh.drawTriangles != currentMesh.showContourLines)
-          setTokenProperty(tok2, test);
-      }
+      setTokenProperty(tok, test, false);
+      if (tok2 != 0)
+          setTokenProperty(tok2, test, true);
       return;
     }
     super.setProperty(propertyName, value, bs);
@@ -294,12 +310,12 @@ public abstract class MeshCollection extends Shape {
   protected void checkExplicit(String id) {
     if (explicitID) // not twice
       return;
-    explicitID = (id != null && !id.equals(JmolConstants.PREVIOUS_MESH_ID));
+    explicitID = (id != null && !id.equals(MeshCollection.PREVIOUS_MESH_ID));
     if (explicitID)
       previousMeshID = id;
   } 
   
-  private void setTokenProperty(int tokProp, boolean bProp) {
+  private void setTokenProperty(int tokProp, boolean bProp, boolean testD) {
     if (currentMesh == null) {
       String key = (explicitID && previousMeshID != null
           && TextFormat.isWild(previousMeshID) ? previousMeshID.toUpperCase()
@@ -309,16 +325,23 @@ public abstract class MeshCollection extends Shape {
       for (int i = 0; i < meshCount; i++)
         if (key == null
             || TextFormat.isMatch(meshes[i].thisID.toUpperCase(), key, true, true))
-          setMeshTokenProperty(meshes[i], tokProp, bProp);
+          setMeshTokenProperty(meshes[i], tokProp, bProp, testD);
     } else {
-      setMeshTokenProperty(currentMesh, tokProp, bProp);
+      setMeshTokenProperty(currentMesh, tokProp, bProp, testD);
       if (linkedMesh != null)
-        setMeshTokenProperty(linkedMesh, tokProp, bProp);
+        setMeshTokenProperty(linkedMesh, tokProp, bProp, testD);
     }
   }
  
-  private void setMeshTokenProperty(Mesh m, int tokProp, boolean bProp) {
+  private void setMeshTokenProperty(Mesh m, int tokProp, boolean bProp, boolean testD) {
+    if (testD && (!m.havePlanarContours || m.drawTriangles == m.showContourLines))
+      return;
     switch (tokProp) {
+    case Token.display:
+      m.bsDisplay = bsDisplay;
+      if (bsDisplay == null && displayWithinPoints != null) 
+        m.setShowWithin(displayWithinPoints, displayWithinDistance2, isDisplayWithinNot);
+      return;
     case Token.on:
       m.visible = bProp;
       return;
@@ -327,34 +350,34 @@ public abstract class MeshCollection extends Shape {
       return;
     case Token.translucent:
       m.setTranslucent(bProp, translucentLevel);
+      if (m.bsSlabGhost != null)
+        m.resetSlab();
       return;
-    case Token.frontlit:
-    case Token.backlit:
-    case Token.fullylit:
-      m.setLighting(tokProp);
-      return;
-    case Token.dots:
-      m.showPoints = bProp;
-      return;
-    case Token.mesh:
-      m.drawTriangles = bProp;
-      return;
-    case Token.fill:
-      m.fillTriangles = bProp;
-      return;
-    case Token.triangles:
-      m.showTriangles = bProp;
-      return;
-    case Token.contourlines:
-      m.showContourLines = bProp;
-      return;
-    case Token.frontonly:
-      m.frontOnly = bProp;
-      return;
+    default:
+      m.setTokenProperty(tokProp, bProp);
     }
   }
-  
+
+  @SuppressWarnings("unchecked")
+  @Override
   public boolean getProperty(String property, Object[] data) {
+    if (property == "getNames") {
+      Map<String, Token> map = (Map<String, Token>) data[0];
+      boolean withDollar = ((Boolean) data[1]).booleanValue();
+      for (int i = meshCount; --i >= 0;)
+        if (meshes[i] != null && meshes[i].vertexCount != 0)
+          map.put((withDollar ? "$" : "") + meshes[i].thisID, Token.tokenOr); // just a placeholder
+      return true;
+    }
+    if (property == "getVertices") {
+      Mesh m = getMesh((String) data[0]);
+      if (m == null)
+        return false;
+      data[1] = m.vertices;
+      data[2] = m.getVisibleVertexBitSet();
+      return true;
+
+    }
     if (property == "checkID") {
       String key = ((String) data[0]).toUpperCase();
       boolean isWild = TextFormat.isWild(key);
@@ -383,27 +406,35 @@ public abstract class MeshCollection extends Shape {
     return false;
   }
 
+  @Override
   public Object getProperty(String property, int index) {
-   Mesh m;
+    Mesh m;
     if (property == "count") {
       int n = 0;
       for (int i = 0; i < meshCount; i++)
         if ((m = meshes[i]) != null && m.vertexCount > 0)
           n++;
-      return new Integer(n);
+      return Integer.valueOf(n);
     }
     if (property == "ID")
       return (currentMesh == null ? null : currentMesh.thisID);
-    if (property == "list") {
+    if (property.startsWith("list")) {
+      clean();
       StringBuffer sb = new StringBuffer();
       int k = 0;
+      String id = (property.equals("list") ? null : property.substring(5));
       for (int i = 0; i < meshCount; i++) {
-         if ((m = meshes[i]) == null || m.vertexCount == 0)
+        m = meshes[i];
+        if (id != null && !id.equalsIgnoreCase(m.thisID))
           continue;
         sb.append((++k)).append(" id:" + m.thisID).append(
             "; model:" + viewer.getModelNumberDotted(m.modelIndex)).append(
             "; vertices:" + m.vertexCount).append(
-            "; polygons:" + m.polygonCount).append("; visible:" + m.visible);
+            "; polygons:" + m.polygonCount)
+            .append("; visible:" + m.visible);
+        Object range = getProperty("dataRange", 0);
+        if (range != null)
+            sb.append("; dataRange:").append(Escape.escape(range));
         if (m.title != null) {
           String s = "";
           for (int j = 0; j < m.title.length; j++)
@@ -413,11 +444,19 @@ public abstract class MeshCollection extends Shape {
           sb.append(s);
         }
         sb.append('\n');
+        if (id != null) {
+          Object info = getProperty("jvxlFileInfo", 0);
+          if (info != null)
+            sb.append(info).append('\n');
+        }
       }
       return sb.toString();
     }
     if (property == "vertices")
       return getVertices(currentMesh);
+    if (property == "getInfo")
+      return (currentMesh == null ? null : currentMesh.getInfo());
+
     return null;
   }
 
@@ -473,8 +512,9 @@ public abstract class MeshCollection extends Shape {
     return (i < 0 ? null : meshes[i]);
   }
   
+  @Override
   public int getIndexFromName(String thisID) {
-    if (JmolConstants.PREVIOUS_MESH_ID.equals(thisID))
+    if (MeshCollection.PREVIOUS_MESH_ID.equals(thisID))
       return (previousMeshID == null ? meshCount - 1
           : getIndexFromName(previousMeshID));
     if (TextFormat.isWild(thisID)) {
@@ -486,76 +526,35 @@ public abstract class MeshCollection extends Shape {
       }
     } else {
       if (htObjects != null) {
-        Mesh m = (Mesh)(htObjects.get(thisID.toUpperCase()));
+        Mesh m = htObjects.get(thisID.toUpperCase());
         return (m == null ? -1 : m.index);
       }
       for (int i = meshCount; --i >= 0;) {
-        if (meshes[i] != null && thisID.equalsIgnoreCase(meshes[i].thisID))
+        if (meshes[i] != null && meshes[i].vertexCount != 0 && thisID.equalsIgnoreCase(meshes[i].thisID))
           return i;
       }
     }
     return -1;
   }
   
-  public void setModelIndex(int atomIndex, int modelIndex) {
-    if (currentMesh == null)
-      return;
-    currentMesh.visible = true; 
-    if ((currentMesh.atomIndex = atomIndex) >= 0)
-      currentMesh.modelIndex = viewer.getAtomModelIndex(atomIndex);
-    else if (isFixed)
-      currentMesh.modelIndex = -1;
-    else if (modelIndex >= 0)
-      currentMesh.modelIndex = modelIndex;
-    else
-      currentMesh.modelIndex = viewer.getCurrentModelIndex();
-    currentMesh.scriptCommand = script;
-  }
-
-/*
-private String encapsulateData(String cmd, List data, String ext) {
-  String name = ((String) data.get(0)).toLowerCase();
-  Object array = data.get(5);
-  if (array instanceof float[][] && name.indexOf("data2d_") != 0)
-    name = "data2d_" + name;
-  else if (array instanceof float[][][] && name.indexOf("data3d_") != 0)
-    name = "data3d_" + name;    
-  cmd = Escape.encapsulateData(name, array) 
-      + "  " + cmd + "# DATA" + ext + "=\"" + name + "\"";
-  return cmd;
-}
-*/
-
-public void setVisibilityFlags(BitSet bs) {
+  @Override
+  public void setVisibilityFlags(BitSet bs) {
     /*
      * set all fixed objects visible; others based on model being displayed
      * 
      */
+    BitSet bsDeleted = viewer.getDeletedAtoms();
     for (int i = meshCount; --i >= 0;) {
       Mesh mesh = meshes[i];
-      mesh.visibilityFlags = (mesh.visible && mesh.isValid
+      mesh.visibilityFlags = (mesh.visible
+          && mesh.isValid
           && (mesh.modelIndex < 0 || bs.get(mesh.modelIndex)
-          && (mesh.atomIndex < 0 || !modelSet.isAtomHidden(mesh.atomIndex))
-          ) ? myVisibilityFlag
+              && (mesh.atomIndex < 0 || !modelSet.isAtomHidden(mesh.atomIndex)
+                  && !(bsDeleted != null && bsDeleted.get(mesh.atomIndex)))) ? myVisibilityFlag
           : 0);
     }
   }
  
-  protected void getModelIndex(String script) {
-    //pmesh and isosurface state
-    int i;
-    iHaveModelIndex = false;
-    modelIndex = -1;
-    if (script == null || (i = script.indexOf("MODEL({")) < 0)
-      return;
-    int j = script.indexOf("})", i);
-    if (j < 0)
-      return;
-    BitSet bs = Escape.unescapeBitset(script.substring(i + 3, j + 1));
-    modelIndex = (bs == null ? -1 : bs.nextSetBit(0));
-    iHaveModelIndex = (modelIndex >= 0);
-  }
-  
   protected void setStatusPicked(int flag, Point3f v) {
     // for draw and isosurface
     viewer.setStatusAtomPicked(flag, "[\"" + myType + "\"," + Escape.escape(pickedMesh.thisID) + "," +
@@ -563,6 +562,20 @@ public void setVisibilityFlags(BitSet bs) {
         + (pickedMesh.title == null ? "\"\"" 
                : Escape.escape(pickedMesh.title[0]))+"]");
   }
+
+  protected Map<String, Object> getPickedPoint(Point3f v, int modelIndex) {
+    Map<String, Object> map = new Hashtable<String, Object>();
+    if (v != null) {
+      map.put("pt", v);
+      map.put("modelIndex", Integer.valueOf(modelIndex));
+      map.put("model", viewer.getModelNumberDotted(modelIndex));
+      map.put("id", pickedMesh.thisID);
+      map.put("vertex", Integer.valueOf(pickedVertex + 1));
+      map.put("type", myType);
+    }
+    return map;
+  }
+
 }
 
  
\ No newline at end of file
