@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-08-31 12:27:56 +0200 (mar., 31 août 2010) $
- * $Revision: 14209 $
+ * $Date: 2011-10-16 16:59:03 +0200 (dim., 16 oct. 2011) $
+ * $Revision: 16360 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -30,6 +30,7 @@ import javax.vecmath.Point3f;
 import javax.vecmath.Point3i;
 import javax.vecmath.Vector3f;
 
+import org.jmol.constant.EnumStructure;
 import org.jmol.g3d.Graphics3D;
 import org.jmol.modelset.Atom;
 import org.jmol.modelsetbio.AlphaMonomer;
@@ -37,10 +38,10 @@ import org.jmol.modelsetbio.CarbohydratePolymer;
 import org.jmol.modelsetbio.Monomer;
 import org.jmol.modelsetbio.NucleicPolymer;
 import org.jmol.modelsetbio.ProteinStructure;
+import org.jmol.script.Token;
 import org.jmol.shape.MeshRenderer;
 import org.jmol.shape.Mesh;
 import org.jmol.util.Logger;
-import org.jmol.viewer.JmolConstants;
 
 import java.util.BitSet;
 
@@ -76,16 +77,17 @@ abstract class BioShapeRenderer extends MeshRenderer {
   protected Vector3f[] wingVectors;
   protected short[] mads;
   protected short[] colixes;
-  protected byte[] structureTypes;
+  protected EnumStructure[] structureTypes;
 
   protected abstract void renderBioShape(BioShape bioShape);
 
+  @Override
   protected void render() {
     if (shape == null)
       return;
     
     invalidateMesh = false;
-    boolean TF = exportType != Graphics3D.EXPORT_NOT || viewer.getHighResolution();
+    boolean TF = (isExport || viewer.getHighResolution());
     if (TF != isHighRes)
       invalidateMesh = true;
     isHighRes = TF;
@@ -134,7 +136,7 @@ abstract class BioShapeRenderer extends MeshRenderer {
   private void freeTempArrays() {
     if (haveControlPointScreens)
       viewer.freeTempScreens(controlPointScreens);
-    viewer.freeTempBytes(structureTypes);
+    viewer.freeTempEnum(structureTypes);
   }
 
   private boolean initializePolymer(BioShape bioShape) {
@@ -150,9 +152,9 @@ abstract class BioShapeRenderer extends MeshRenderer {
 
     bsVisible.clear();
     boolean haveVisible = false;
+    if (invalidateMesh)
+      bioShape.falsifyMesh();
     for (int i = monomerCount; --i >= 0;) {
-      if (invalidateMesh)
-        bioShape.falsifyMesh(i, false);
       if ((monomers[i].shapeVisibilityFlags & myVisibilityFlag) == 0
           || modelSet.isAtomHidden(leadAtomIndices[i]))
         continue;
@@ -185,17 +187,17 @@ abstract class BioShapeRenderer extends MeshRenderer {
   }
 
   private void setStructureTypes() {
-    structureTypes = viewer.allocTempBytes(monomerCount + 1);
+    structureTypes = viewer.allocTempEnum(monomerCount + 1);
     for (int i = monomerCount; --i >= 0;) {
       structureTypes[i] = monomers[i].getProteinStructureType();
-      if (structureTypes[i] == JmolConstants.PROTEIN_STRUCTURE_TURN)
-        structureTypes[i] = JmolConstants.PROTEIN_STRUCTURE_NONE;
+      if (structureTypes[i] == EnumStructure.TURN)
+        structureTypes[i] = EnumStructure.NONE;
     }
     structureTypes[monomerCount] = structureTypes[monomerCount - 1];
   }
 
   protected boolean isHelix(int i) {
-    return structureTypes[i] == JmolConstants.PROTEIN_STRUCTURE_HELIX;
+    return structureTypes[i] == EnumStructure.HELIX;
   }
 
   protected void calcScreenControlPoints() {
@@ -395,13 +397,13 @@ abstract class BioShapeRenderer extends MeshRenderer {
       }
     }
 
-    calc1Screen(controlPoints[i], wingVectors[i], (short) madBeg, .0007f,
+    calc1Screen(controlPoints[i], wingVectors[i], madBeg, .0007f,
         screenArrowTop);
-    calc1Screen(controlPoints[i], wingVectors[i], (short) madBeg, -.0007f,
+    calc1Screen(controlPoints[i], wingVectors[i], madBeg, -.0007f,
         screenArrowBot);
-    calc1Screen(controlPoints[i], wingVectors[i], (short) madBeg, 0.001f,
+    calc1Screen(controlPoints[i], wingVectors[i], madBeg, 0.001f,
         screenArrowTopPrev);
-    calc1Screen(controlPoints[i], wingVectors[i], (short) madBeg, -0.001f,
+    calc1Screen(controlPoints[i], wingVectors[i], madBeg, -0.001f,
         screenArrowBotPrev);
     g3d.drawHermite(true, ribbonBorder, isNucleic ? 4 : 7,
         screenArrowTopPrev, screenArrowTop, controlPointScreens[iNext],
@@ -416,6 +418,14 @@ abstract class BioShapeRenderer extends MeshRenderer {
   }
 
   //  rockets --not satisfactory yet
+  /**
+   * @param i            IGNORED 
+   * @param pointBegin   IGNORED
+   * @param pointEnd     IGNORED
+   * @param screenPtBegin 
+   * @param screenPtEnd 
+   * 
+   */
   protected void renderCone(int i, Point3f pointBegin, Point3f pointEnd,
                   Point3f screenPtBegin, Point3f screenPtEnd) {
     int coneDiameter = mad + (mad >> 2);
@@ -582,7 +592,7 @@ abstract class BioShapeRenderer extends MeshRenderer {
       for (int k = hermiteLevel * 2; --k >= 0;)
         mesh.addQuad(nPoints - k - 1, nPoints - nPer + (nPer - k) % nPer,
             nPoints - nPer + k + 1, nPoints - nPer + k + 2);
-    mesh.initialize(JmolConstants.FRONTLIT, null, null);
+    mesh.initialize(Token.frontlit, null, null);
     //System.out.sprintln("mesh "+ mesh.thisID + " " + mesh.vertexCount+" "+mesh.vertices.length + " " + mesh.polygonCount + " " + mesh.polygonIndexes.length);
     mesh.setVisibilityFlags(1);
     return (meshReady[i] = true);
