@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-08-24 19:54:10 +0200 (mer., 24 août 2011) $
- * $Revision: 15989 $
+ * $Date: 2012-04-02 19:18:37 +0200 (lun., 02 avr. 2012) $
+ * $Revision: 16956 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -39,6 +39,9 @@ import java.util.BitSet;
 import java.util.Hashtable;
 import java.util.Map;
 
+import javax.vecmath.Matrix3f;
+import javax.vecmath.Matrix4f;
+
 public class Dots extends AtomShape {
 
   public EnvelopeCalculation ec;
@@ -135,7 +138,13 @@ public class Dots extends AtomShape {
     }
 
     if ("refreshTrajectories" == propertyName) {
-      ec.reCalculate(bs);
+      bs = (BitSet) ((Object[]) value)[1];
+      Matrix4f m4 = (Matrix4f) ((Object[]) value)[2];
+      if (m4 == null) // end of compare command
+        return;
+      Matrix3f m = new Matrix3f();
+      m4.get(m);
+      ec.reCalculate(bs, m);
       return;
     }
 
