@@ -25,6 +25,7 @@
 package org.jmol.shapespecial;
 
 import org.jmol.api.AtomIndexIterator;
+import org.jmol.constant.EnumPalette;
 import org.jmol.g3d.Graphics3D;
 import org.jmol.modelset.Atom;
 import org.jmol.modelset.Bond;
@@ -34,13 +35,11 @@ import org.jmol.util.ArrayUtil;
 import org.jmol.util.Logger;
 
 import java.util.BitSet;
-import java.util.Hashtable;
 import javax.vecmath.Point3i;
 import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
 
 import org.jmol.util.Measure;
-import org.jmol.viewer.JmolConstants;
 
 public class Polyhedra extends AtomShape {
 
@@ -72,6 +71,7 @@ public class Polyhedra extends AtomShape {
   private BitSet bsVertices;
   private BitSet bsVertexCount;
 
+  @Override
   public void setProperty(String propertyName, Object value, BitSet bs) {
 
     if ("init" == propertyName) {
@@ -170,13 +170,11 @@ public class Polyhedra extends AtomShape {
       drawEdges = EDGES_FRONT;
       return;
     }
+
     if (propertyName.indexOf("color") == 0) {
       // from polyhedra command, we may not be using the prior select
       // but from Color we need to identify the centers.
-      if ("colorThis" == propertyName && iHaveCenterBitSet)
-        bs = centers;
-      else
-        andBitSet(bs);
+      bs = ("colorThis" == propertyName && iHaveCenterBitSet ? centers : andBitSet(bs));
       propertyName = "color";
       //allow super
     }
@@ -184,10 +182,9 @@ public class Polyhedra extends AtomShape {
     if (propertyName.indexOf("translucency") == 0) {
       // from polyhedra command, we may not be using the prior select
       // but from Color we need to identify the centers.
-      if ("translucencyThis" == propertyName && iHaveCenterBitSet)
-        bs = centers;
-      else
-        andBitSet(bs);
+      bs = ("translucentThis".equals(value) && iHaveCenterBitSet ? centers : andBitSet(bs));
+      if (value.equals("translucentThis"))
+        value = "translucent";
       //allow super
     }
 
@@ -212,16 +209,17 @@ public class Polyhedra extends AtomShape {
     super.setProperty(propertyName, value, bs);
   }
 
-  private void andBitSet(BitSet bs) {
+  private BitSet andBitSet(BitSet bs) {
     BitSet bsCenters = new BitSet();
     for (int i = polyhedronCount; --i >= 0;)
       bsCenters.set(polyhedrons[i].centralAtom.getIndex());
-    bs.and(bsCenters);
+    bsCenters.and(bs);
+    return bsCenters;
   }
 
   private void deletePolyhedra() {
     int newCount = 0;
-    byte pid = JmolConstants.pidOf(null);
+    byte pid = EnumPalette.pidOf(null);
     for (int i = 0; i < polyhedronCount; ++i) {
       Polyhedron p = polyhedrons[i];
       int iAtom = p.centralAtom.getIndex();
@@ -245,7 +243,7 @@ public class Polyhedra extends AtomShape {
 
   private void buildPolyhedra() {
     boolean useBondAlgorithm = radius == 0 || bondedOnly;
-    AtomIndexIterator iter = modelSet.getSelectedAtomIterator(null, false, false, false);
+    AtomIndexIterator iter = modelSet.getSelectedAtomIterator(null, false, false, false, false);
     for (int i = centers.nextSetBit(0); i >= 0; i = centers.nextSetBit(i + 1)) {
       Polyhedron p = (haveBitSetVertices ? constructBitSetPolyhedron(i)
           : useBondAlgorithm ? constructBondsPolyhedron(i)
@@ -567,7 +565,7 @@ public class Polyhedra extends AtomShape {
         this.planes[i] = planes[i];
     }
 
-    protected String getState(Hashtable temp) {
+    protected String getState() {
       BitSet bs = new BitSet();
       for (int i = 0; i < ptCenter; i++)
         bs.set(((Atom) vertices[i]).getIndex());
@@ -581,6 +579,7 @@ public class Polyhedra extends AtomShape {
     }
   }
 
+  @Override
   public void setVisibilityFlags(BitSet bs) {
     /*
      * set all fixed objects visible; others based on model being displayed note
@@ -595,11 +594,11 @@ public class Polyhedra extends AtomShape {
     }
   }
 
+  @Override
   public String getShapeState() {
-    Hashtable temp = new Hashtable();
     StringBuffer s = new StringBuffer();
     for (int i = 0; i < polyhedronCount; i++)
-      s.append(polyhedrons[i].getState(temp));
+      s.append(polyhedrons[i].getState());
     if (drawEdges == EDGES_FRONT)
       appendCmd(s, "polyhedra frontedges");
     else if (drawEdges == EDGES_ALL)
