@@ -828,9 +828,9 @@ public class Isosurface extends MeshCollection implements MeshDataServer {
 
   private void getMeshCommand(StringBuffer sb, int i) {
     IsosurfaceMesh imesh = (IsosurfaceMesh) meshes[i];
-    String cmd = imesh.scriptCommand;
-    if (cmd == null)
+    if (imesh == null || imesh.scriptCommand == null)
       return;
+    String cmd = imesh.scriptCommand;
     int modelCount = viewer.getModelCount();
     if (modelCount > 1)
       appendCmd(sb, "frame " + viewer.getModelNumberDotted(imesh.modelIndex));
@@ -1273,9 +1273,11 @@ public class Isosurface extends MeshCollection implements MeshDataServer {
   }
 
   public void notifySurfaceMappingCompleted() {
-    if (!thisMesh.isMerged)
+    if (!thisMesh.isMerged) {
       thisMesh.initialize(sg.isFullyLit() ? Token.fullylit
           : Token.frontlit, null, sg.getPlane());
+      thisMesh.setJvxlDataRendering();
+    }
     setBsVdw();
     thisMesh.isColorSolid = false;
     thisMesh.colorDensity = jvxlData.colorDensity;
