@@ -640,12 +640,7 @@ public class IsosurfaceMesh extends Mesh {
         jvxlData.translucency);
     if (jvxlData.meshColor != null)
       meshColix = Graphics3D.getColix(jvxlData.meshColor);
-    if (jvxlData.rendering != null) {
-      String[] tokens = Parser.getTokens(jvxlData.rendering);
-      for (int i = 0; i < tokens.length; i++)
-        setTokenProperty(Token.getTokFromName(tokens[i]), true);
-    }
-      
+    setJvxlDataRendering();
     isColorSolid = !jvxlData.isBicolorMap;
     if (colorEncoder != null) {
       if (jvxlData.colorScheme != null) {
@@ -668,6 +663,14 @@ public class IsosurfaceMesh extends Mesh {
     }    
   }
 
+  void setJvxlDataRendering() {
+    if (jvxlData.rendering != null) {
+      String[] tokens = Parser.getTokens(jvxlData.rendering);
+      for (int i = 0; i < tokens.length; i++)
+        setTokenProperty(Token.getTokFromName(tokens[i]), true);
+    }
+  }
+
   /**
    * remaps colors based on a new color scheme or translucency level
    * 
