@@ -24,9 +24,5 @@
 package org.jmol.shapesurface;
 
 public class PmeshRenderer extends IsosurfaceRenderer {
-  
-  protected void initRenderer() {
-    super.initRenderer();
-  }
-
+  // no changes to renderer
 }
