@@ -26,16 +26,17 @@ package org.jmol.smiles;
 
 import java.util.ArrayList;
 import java.util.BitSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Hashtable;
-import java.util.Enumeration;
+import java.util.Map;
 
 import javax.vecmath.Point3f;
 
-import org.jmol.api.JmolMolecule;
-import org.jmol.api.JmolNode;
-import org.jmol.api.JmolEdge;
 import org.jmol.util.Elements;
+import org.jmol.util.JmolEdge;
+import org.jmol.util.JmolMolecule;
+import org.jmol.util.JmolNode;
 import org.jmol.util.Logger;
 import org.jmol.smiles.SmilesSearch.VTemp;
 
@@ -71,8 +72,8 @@ public class SmilesGenerator {
   
   // outputs
 
-  private Hashtable htRingsSequence = new Hashtable();
-  private Hashtable htRings = new Hashtable();
+  private Map<String, Object[]> htRingsSequence = new Hashtable<String, Object[]>();
+  private Map<String, Object[]> htRings = new Hashtable<String, Object[]>();
   private BitSet bsIncludingH;
 
   // generation of SMILES strings
@@ -102,27 +103,27 @@ public class SmilesGenerator {
     BitSet bsIgnore = new BitSet();
     String lastComponent = null;
     String s;
-    List vLinks = new ArrayList();
+    List<Integer> vLinks = new ArrayList<Integer>();
     try {
       int len = 0;
       for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
         JmolNode a = atoms[i];
         String ch = a.getGroup1('?');
-        String groupType = a.getGroupType();
+        String bioStructureName = a.getBioStructureTypeName();
         boolean unknown = (ch.equals("?"));
         if (end != null) {
           if (sb.length() > 0)
             sb.append(end);
           end = null;
           len = 0;
-          if (groupType.length() > 0) {
+          if (bioStructureName.length() > 0) {
             char id = a.getChainID();
             if (id != '\0') {
-              s = "//* chain " + id + " " + groupType + " " + a.getResno() + " *// ";
+              s = "//* chain " + id + " " + bioStructureName + " " + a.getResno() + " *// ";
               len = s.length();
               sb.append(s);
             }
-            sb.append("~").append(groupType.charAt(0)).append("~");
+            sb.append("~").append(bioStructureName.charAt(0)).append("~");
             len++;
           } else {
             s = getSmilesComponent(a, bs, true);
@@ -130,9 +131,9 @@ public class SmilesGenerator {
               end = "";
             } else {
               lastComponent = s;
-              groupType = a.getGroup3(true);
-              if (groupType != null)
-                sb.append("//* ").append(groupType).append(" *//");
+              String groupName = a.getGroup3(true);
+              if (groupName != null)
+                sb.append("//* ").append(groupName).append(" *//");
               sb.append(s);
               end = ".\n";
             }
@@ -144,7 +145,7 @@ public class SmilesGenerator {
           len = 2;
         }
         if (unknown) {
-          addBracketedBioName(sb, a, groupType.length() > 0 ? ".0" : null);
+          addBracketedBioName(sb, a, bioStructureName.length() > 0 ? ".0" : null);
         } else {
           sb.append(ch);
         }
@@ -154,14 +155,14 @@ public class SmilesGenerator {
           a.getCrossLinkLeadAtomIndexes(vLinks);
           for (int j = 0; j < vLinks.size(); j++) {
             sb.append(":");
-            s = getRingCache(i0, ((Integer) vLinks.get(j)).intValue(),
+            s = getRingCache(i0, vLinks.get(j).intValue(),
                 htRingsSequence);
             sb.append(s);
             len += 1 + s.length();
           }
           vLinks.clear();
         }
-        a.setGroupBits(bsIgnore);
+        a.getGroupBits(bsIgnore);
         bs.andNot(bsIgnore);
         int i2 = a.getOffsetResidueAtom("0", 1);
         if (i2 < 0 || !bs.get(i2)) {
@@ -207,22 +208,23 @@ public class SmilesGenerator {
 
   /**
    * 
-   * creates a valid SMILES string from a model.
-   * TODO: stereochemistry other than square planar and tetrahedral
+   * creates a valid SMILES string from a model. TODO: stereochemistry other
+   * than square planar and tetrahedral
    * 
    * @param atom
    * @param bs
-   * @param allowConnectionsToOutsideWorld 
-   * @return        SMILES
+   * @param allowConnectionsToOutsideWorld
+   * @return SMILES
    * @throws InvalidSmilesException
    */
-  private String getSmilesComponent(JmolNode atom, BitSet bs, boolean allowConnectionsToOutsideWorld)
+  private String getSmilesComponent(JmolNode atom, BitSet bs,
+                                    boolean allowConnectionsToOutsideWorld)
       throws InvalidSmilesException {
 
     if (atom.getElementNumber() == 1 && atom.getEdges().length > 0)
       atom = atoms[atom.getBondedAtomIndex(0)]; // don't start with H
-    bsSelected = JmolMolecule.getBranchBitSet(atoms, (BitSet) bs.clone(), atom
-        .getIndex(), -1, true, false);
+    bsSelected = JmolMolecule.getBranchBitSet(atoms, atom.getIndex(),
+        (BitSet) bs.clone(), null, -1, true, false);
     bs.andNot(bsSelected);
     bsIncludingH = (BitSet) bsSelected.clone();
     for (int j = bsSelected.nextSetBit(0); j >= 0; j = bsSelected
@@ -247,20 +249,19 @@ public class SmilesGenerator {
     }
     bsToDo = (BitSet) bsSelected.clone();
     StringBuffer sb = new StringBuffer();
-    
-    for (int i = bsToDo.nextSetBit(0); i >= 0 ; i = bsToDo.nextSetBit(i + 1))
+
+    for (int i = bsToDo.nextSetBit(0); i >= 0; i = bsToDo.nextSetBit(i + 1))
       if (atoms[i].getCovalentBondCount() > 4) {
         getSmiles(sb, atoms[i], allowConnectionsToOutsideWorld, false);
-        atom = null;       
+        atom = null;
       }
     if (atom != null)
       while ((atom = getSmiles(sb, atom, allowConnectionsToOutsideWorld, true)) != null) {
       }
     while (bsToDo.cardinality() > 0 || !htRings.isEmpty()) {
-      Enumeration e = htRings.keys();
-      if (e.hasMoreElements()) {
-        atom = atoms[((Integer) ((Object[]) htRings.get(e.nextElement()))[1])
-            .intValue()];
+      Iterator<Object[]> e = htRings.values().iterator();
+      if (e.hasNext()) {
+        atom = atoms[((Integer) e.next()[1]).intValue()];
         if (!bsToDo.get(atom.getIndex()))
           break;
       } else {
@@ -290,7 +291,7 @@ public class SmilesGenerator {
   private char getBondStereochemistry(JmolEdge bond, JmolNode atomFrom) {
     if (bond == null)
       return '\0';
-    int i = bond.getIndex();
+    int i = bond.index;
     boolean isFirst = (atomFrom == null || bond.getAtomIndex1() == atomFrom
         .getIndex());
     return (bsBondsUp.get(i) ? (isFirst ? '/' : '\\')
@@ -316,7 +317,7 @@ public class SmilesGenerator {
       JmolEdge[] bonds = atom1.getEdges();
       for (int k = 0; k < bonds.length; k++) {
         JmolEdge bond = bonds[k];
-        int index = bond.getIndex();
+        int index = bond.index;
         if (bsDone.get(index))
           continue;
         JmolNode atom2 = bond.getOtherAtom(atom1);
@@ -359,7 +360,7 @@ public class SmilesGenerator {
         if (b0 == null) {
           i0 = 0;
           b0 = edges[i0][0];
-          bsBondsUp.set(b0.getIndex());
+          bsBondsUp.set(b0.index);
         }
         
         // The character '/' or '\\' is assigned based on a
@@ -380,23 +381,25 @@ public class SmilesGenerator {
         
         char c0 = getBondStereochemistry(b0, atom12[i0]);
         a0 = b0.getOtherAtom(atom12[i0]);
+        if (a0 == null)
+          continue;
         for (int j = 0; j < 2; j++)
           for (int jj = 0; jj < 2; jj++) {
             JmolEdge b1 = edges[j][jj];
             if (b1 == null || b1 == b0)
               continue;
-            int bi = b1.getIndex();
+            int bi = b1.index;
             JmolNode a1 = b1.getOtherAtom(atom12[j]);
+            if (a1 == null)
+              continue;
             char c1 = getBondStereochemistry(b1, atom12[j]);
 
             //   c1 is FROM the double bond:
             //    
-            //    a     b
-            //     \   /
-            //      C=C       /a /b  \c \d
-            //     /   \
-            //    c     d
-
+            //     a0    a1
+            //      \   /
+            //    [i0]=[j]       /a /b  \c \d
+            //   
             boolean isOpposite = SmilesSearch.isDiaxial(atom12[i0], atom12[j],
                 a0, a1, vTemp, 0);
             if (c1 == '\0' || (c1 != c0) == isOpposite) {
@@ -419,8 +422,7 @@ public class SmilesGenerator {
   }
 
   private JmolNode getSmiles(StringBuffer sb, JmolNode atom,
-                             boolean allowConnectionsToOutsideWorld,
-                             boolean allowBranches) {
+                             boolean allowConnectionsToOutsideWorld, boolean allowBranches) {
     int atomIndex = atom.getIndex();
 
     if (!bsToDo.get(atomIndex))
@@ -435,7 +437,7 @@ public class SmilesGenerator {
     int nSp2Atoms = 0;
     int atomicNumber = atom.getElementNumber();
     int nH = 0;
-    List v = new ArrayList();
+    List<JmolEdge> v = new ArrayList<JmolEdge>();
     JmolEdge bond0 = null;
     JmolEdge bondPrev = null;
     JmolEdge[] bonds = atom.getEdges();
@@ -490,11 +492,12 @@ public class SmilesGenerator {
 
     // add the bond to the previous atom
 
+    //System.out.println(" " + atom);
+
     String strBond = null;
     if (sp2Atoms == null)
       sp2Atoms = new JmolNode[5];
-    if (bondPrev == null) {
-    } else {
+    if (bondPrev != null) {
       strBond = SmilesBond.getBondOrderString(bondPrev.getCovalentOrder());
       if (prevSp2Atoms == null)
         sp2Atoms[nSp2Atoms++] = prevAtom;
@@ -508,12 +511,12 @@ public class SmilesGenerator {
     BitSet bsBranches = new BitSet();
     if (allowBranches)
       for (int i = 0; i < v.size(); i++) {
-        JmolEdge bond = (JmolEdge) v.get(i);
+        JmolEdge bond = v.get(i);
         JmolNode a = bond.getOtherAtom(atom);
         int n = a.getCovalentBondCount() - a.getCovalentHydrogenCount();
         int order = bond.getCovalentOrder();
         if (order == 1 && n == 1 && i < v.size() - (bond0 == null ? 1 : 0)) {
-          bsBranches.set(bond.getIndex());
+          bsBranches.set(bond.index);
         } else if ((order > 1 || n > nMax)
             && !htRings.containsKey(getRingKey(a.getIndex(), atomIndex))) {
           nMax = (order > 1 ? 1000 + order : n);
@@ -544,8 +547,8 @@ public class SmilesGenerator {
 
     StringBuffer sMore = new StringBuffer();
     for (int i = 0; i < v.size(); i++) {
-      JmolEdge bond = (JmolEdge) v.get(i);
-      if (!bsBranches.get(bond.getIndex()))
+      JmolEdge bond = v.get(i);
+      if (!bsBranches.get(bond.index))
         continue;
       JmolNode a = bond.getOtherAtom(atom);
       StringBuffer s2 = new StringBuffer();
@@ -576,16 +579,14 @@ public class SmilesGenerator {
         || SmilesSearch.isRingBond(ringSets, atomIndex, index2)) {
       nSp2Atoms = -1;
     }
-    if (nSp2Atoms < 0) {
+    if (nSp2Atoms < 0)
       sp2Atoms = null;
-    }
 
     // output section
 
     if (strBond != null || chBond != '\0') {
-      if (chBond != '\0') {
+      if (chBond != '\0')
         strBond = "" + chBond;
-      }
       sb.append(strBond);
     }
 
@@ -595,17 +596,16 @@ public class SmilesGenerator {
     if (!allowBranches && (v.size() == 5 || v.size() == 6))
       atat = sortInorganic(atom, v);
     for (int i = 0; i < v.size(); i++) {
-      JmolEdge bond = (JmolEdge) v.get(i);
+      JmolEdge bond = v.get(i);
       if (bond == bond0)
         continue;
       JmolNode a = bond.getOtherAtom(atom);
       String s = getRingCache(atomIndex, a.getIndex(), htRings);
-      strBond = SmilesBond.getBondOrderString(bond.getOrder());
+      strBond = SmilesBond.getBondOrderString(bond.order);
       if (!deferStereo) {
         chBond = getBondStereochemistry(bond, atom);
-        if (chBond != '\0') {
+        if (chBond != '\0')
           strBond = "" + chBond;
-        }
       }
 
       sMore.append(strBond);
@@ -637,16 +637,18 @@ public class SmilesGenerator {
     int charge = atom.getFormalCharge();
     int isotope = atom.getIsotopeNumber();
     String atomName = atom.getAtomName();
-    String groupType = atom.getGroupType();
+    String groupType = atom.getBioStructureTypeName();
     // for bioSMARTS we provide the connecting atom if 
     // present. For example, in 1BLU we have 
     // .[CYS.SG#16] could match either the atom number or the element number 
+    if (Logger.debugging)
+      sb.append("\n//* " + atom + " *//\t");
     if (isExtension && groupType.length() != 0 && atomName.length() != 0)
       addBracketedBioName(sb, atom, "." + atomName);
     else
-      sb.append(SmilesAtom.getAtomLabel(atomicNumber, isotope, valence, charge,
-          nH, isAromatic, atat != null ? atat : checkStereoPairs(atom,
-              atomIndex, stereo, stereoFlag)));
+      sb.append(SmilesAtom
+          .getAtomLabel(atomicNumber, isotope, valence, charge, nH, isAromatic,
+              atat != null ? atat : checkStereoPairs(atom, atomIndex, stereo, stereoFlag)));
     sb.append(sMore);
 
     // check the next bond
@@ -676,17 +678,18 @@ public class SmilesGenerator {
    * We must sort the bond vector such that a diaxial pair is
    * first and last. Then we assign stereochemistry based on what
    * is left. The assignment is not made if there are no diaxial groups
-   * or with octahedral if there are fewer than three.
+   * or with octahedral if there are fewer than three or trigonal bipyramidal
+   * with no axial ligands.
    * 
    * @param atom
    * @param v
    * @return  "@" or "@@" or ""
    */
-  private String sortInorganic(JmolNode atom, List v) {
+  private String sortInorganic(JmolNode atom, List<JmolEdge> v) {
     int atomIndex = atom.getIndex();
     int n = v.size();
-    List axialPairs = new ArrayList();
-    List bonds = new ArrayList();
+    List<JmolEdge[]> axialPairs = new ArrayList<JmolEdge[]>();
+    List<JmolEdge> bonds = new ArrayList<JmolEdge>();
     JmolNode a1, a2;
     JmolEdge bond1, bond2;
     BitSet bsDone = new BitSet();
@@ -695,7 +698,7 @@ public class SmilesGenerator {
     boolean isOK = true; // AX6 or AX5
     String s = "";
     for (int i = 0; i < n; i++) {
-      bond1 = (JmolEdge) v.get(i);
+      bond1 = v.get(i);
       stereo[0] = a1 = bond1.getOtherAtom(atom);
       if (i == 0)
         s = addStereoCheck(atomIndex, stereo, 0, "");
@@ -708,7 +711,7 @@ public class SmilesGenerator {
       for (int j = i + 1; j < n; j++) {
         if (bsDone.get(j))
           continue;
-        bond2 = (JmolEdge) v.get(j);
+        bond2 = v.get(j);
         a2 = bond2.getOtherAtom(atom);
         if (SmilesSearch.isDiaxial(atom, atom, a1, a2, vTemp, -0.95f)) {
           axialPairs.add(new JmolEdge[] { bond1, bond2 });
@@ -725,31 +728,35 @@ public class SmilesGenerator {
     // AX6 or AX5 are fine as is
     // can't proceed if octahedral and not all axial pairs
     // or trigonal bipyramidal and no axial pair.
-
+    
     if (isOK || n == 6 && nPairs != 3 || n == 5 && nPairs == 0)
       return "";
-
-    // now sort them into the ligand vector in the proper order
-    pair0 = (JmolEdge[]) axialPairs.get(0);
+    pair0 = axialPairs.get(0);
     bond1 = pair0[0];
     stereo[0] = bond1.getOtherAtom(atom);
+    
+    // now sort them into the ligand vector in the proper order
+    
     v.clear();
     v.add(bond1);
     if (nPairs > 1)
-      bonds.add(((JmolEdge[]) axialPairs.get(1))[0]);
+      bonds.add(axialPairs.get(1)[0]);
     if (nPairs == 3)
-      bonds.add(((JmolEdge[]) axialPairs.get(2))[0]);
+      bonds.add(axialPairs.get(2)[0]);
     if (nPairs > 1)
-      bonds.add(((JmolEdge[]) axialPairs.get(1))[1]);
+      bonds.add(axialPairs.get(1)[1]);
     if (nPairs == 3)
-      bonds.add(((JmolEdge[]) axialPairs.get(2))[1]);
+      bonds.add(axialPairs.get(2)[1]);
     for (int i = 0; i < bonds.size(); i++) {
-      bond1 = (JmolEdge) bonds.get(i);
+      bond1 = bonds.get(i);
       v.add(bond1);
       stereo[i + 1] = bond1.getOtherAtom(atom);
     }
     v.add(pair0[1]);
-    return (isOK ? "" : getStereoFlag(atom, stereo, n, vTemp));
+    
+    // now deterimine the stereochemistry
+    
+    return getStereoFlag(atom, stereo, n, vTemp);
   }
 
   private String checkStereoPairs(JmolNode atom, int atomIndex,
@@ -849,13 +856,13 @@ public class SmilesGenerator {
     return s + sa;
   }
 
-  private String getRingCache(int i0, int i1, Hashtable ht) {
+  private String getRingCache(int i0, int i1, Map<String, Object[]> ht) {
     String key = getRingKey(i0, i1);
-    Object[] o = (Object[]) ht.get(key);
+    Object[] o = ht.get(key);
     String s = (o == null ? null : (String) o[0]);
     if (s == null) {
       ht.put(key, new Object[] {
-          s = SmilesParser.getRingPointer(++nPairs), new Integer(i1) });
+          s = SmilesParser.getRingPointer(++nPairs), Integer.valueOf(i1) });
       if (Logger.debugging)
         Logger.info("adding for " + i0 + " ring key " + nPairs + ": " + key);
     } else {
@@ -866,11 +873,11 @@ public class SmilesGenerator {
     return s;//  + " _" + key + "_ \n";
   }
 
-  private void dumpRingKeys(StringBuffer sb, Hashtable ht) {
+  private void dumpRingKeys(StringBuffer sb, Map<String, Object[]> ht) {
     Logger.info(sb.toString() + "\n\n");
-    Enumeration e = ht.keys();
-    while (e.hasMoreElements()) {
-      Logger.info("unmatched ring key: " + e.nextElement());
+    Iterator<String> e = ht.keySet().iterator();
+    while (e.hasNext()) {
+      Logger.info("unmatched ring key: " + e.next());
     }
   }
 
