@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Date: 2012-04-11 18:09:00 +0200 (mer., 11 avr. 2012) $
+ * $Revision: 16998 $
  *
  * Copyright (C) 2005  The Jmol Development Team
  *
@@ -24,9 +24,11 @@
 
 package org.jmol.smiles;
 
-import java.util.List;
+
 import java.util.ArrayList;
+import java.util.List;
 import java.util.Hashtable;
+import java.util.Map;
 
 import org.jmol.util.Elements;
 import org.jmol.util.TextFormat;
@@ -113,7 +115,7 @@ public class SmilesParser {
   private boolean isSmarts;
   private boolean isBioSequence;
   private char bioType;
-  private Hashtable ringBonds = new Hashtable();
+  private Map<Integer, SmilesBond> ringBonds = new Hashtable<Integer, SmilesBond>();
   private int braceCount;
   private int branchLevel;
 
@@ -152,7 +154,7 @@ public class SmilesParser {
     }
     if (pattern.indexOf("$") >= 0)
       pattern = parseVariables(pattern);
-    if (isSmarts && pattern.indexOf("[$(") >= 0)
+    if (isSmarts && pattern.indexOf("[$") >= 0)
       pattern = parseVariableLength(pattern);
     if (pattern.indexOf("||") >= 0) {
       SmilesSearch search = new SmilesSearch();
@@ -206,17 +208,34 @@ public class SmilesParser {
     } else {
       int pt = -1;
       int[] ret = new int[1];
-      // [$(...)n] or [$(...)m-n]
+      // [$n(...)] or [$m-n(...)]
       boolean isOK = true;
       String bracketed = null;
-      while ((pt = pattern.indexOf("[$(", pt + 1)) >= 0) {
+      while ((pt = pattern.indexOf("[$", pt + 1)) >= 0) {
         int pt0 = pt;
+        int min = Integer.MIN_VALUE;
+        int max = Integer.MIN_VALUE;
+        pt = getDigits(pattern, pt + 2, ret);
+        min = ret[0];
+        if (min != Integer.MIN_VALUE) {
+          if (getChar(pattern, pt) == '-') {
+            pt = getDigits(pattern, pt + 1, ret);
+            max = ret[0];
+          }
+        } 
+        if (getChar(pattern, pt) != '(')
+          continue;
+        bracketed = getSubPattern(pattern, pt0, '[');
+        if (!bracketed.endsWith(")"))
+          continue;
+        int pt1 = pt0 + bracketed.length() + 2;
+        String repeat = getSubPattern(pattern, pt, '(');
+        int pt2 = pt;
         bracketed = getSubPattern(pattern, pt, '[');
-        int pt1 = pt + bracketed.length() + 2;
-        String repeat = getSubPattern(bracketed, 1, '(');
-        pt += 4 + repeat.length();
+        pt += 1 + repeat.length();
         if (repeat.indexOf(':') >= 0 && repeat.indexOf('|') < 0) {
           // must enclose first ":" in ()
+          // what is this??
           int parenCount = 0;
           int n = repeat.length();
           int ptColon = -1;
@@ -244,30 +263,17 @@ public class SmilesParser {
             repeat = repeat.substring(0, ptColon) + "("
                 + repeat.substring(ptColon, n) + ")" + repeat.substring(n);
         }
-        if (!Character.isDigit(getChar(pattern, pt))) {
+        if (min == Integer.MIN_VALUE) {
           int ptOr = repeat.indexOf("|");
           if (ptOr >= 0)
-            return parseVariableLength(pattern.substring(0, pt0 + ptOr + 3) + ")1"
-                + pattern.substring(pt) + "||"
-                + pattern.substring(0, pt0 + 3) + repeat.substring(ptOr + 1) + ")1"
-                + pattern.substring(pt));
+            return parseVariableLength(pattern.substring(0, pt0) + "[$1" + pattern.substring(pt2, pt2 + ptOr + 1) + ")]"
+                + pattern.substring(pt1) + "||"
+                + pattern.substring(0, pt0) + "[$1(" + pattern.substring(pt2 + ptOr + 2)
+                + pattern.substring(pt1));
           continue;
         }
-        pt = getDigits(pattern, pt, ret);
-        int min = ret[0];
-        int max = min;
-        if (getChar(pattern, pt) == '-') {
-          if (!Character.isDigit(getChar(pattern, pt + 1))) {
-            isOK = false;
-            break;
-          }
-          pt = getDigits(pattern, pt + 1, ret);
-          max = ret[0];
-        }
-        if (pt != pt1 - 1) {
-          isOK = false;
-          break;
-        }
+        if (max == Integer.MIN_VALUE)
+          max = min;
         if (repeat.indexOf("|") >= 0)
           repeat = "[$(" + repeat + ")]";
         for (int i = min; i <= max; i++) {
@@ -283,9 +289,7 @@ public class SmilesParser {
         throw new InvalidSmilesException("bad variable expression: "
             + bracketed);
     }
-    if (haveInternalOr)
-      return parseVariableLength(sout.substring(2));
-    return (sout.length() < 2 ? pattern : sout.substring(2));
+    return (haveInternalOr ? parseVariableLength(sout.substring(2)) : sout.length() < 2 ? pattern : sout.substring(2));
   }
 
   private SmilesSearch getSearch(SmilesSearch parent, String pattern,
@@ -293,7 +297,7 @@ public class SmilesParser {
                                  boolean ignoreStereochemistry)
       throws InvalidSmilesException {
     // First pass
-    htMeasures = new Hashtable();
+    htMeasures = new Hashtable<String, SmilesMeasure>();
     SmilesSearch molecule = new SmilesSearch();
     molecule.setParent(parent);
     molecule.isSmarts = isSmarts;
@@ -432,8 +436,11 @@ public class SmilesParser {
     SmilesBond bond = null;
     while (pattern != null && pattern.length() != 0) {
       int index = 0;
-      if (currentAtom == null || bond != null && bond.bondType == SmilesBond.TYPE_NONE)
+      if (currentAtom == null || bond != null && bond.order == SmilesBond.TYPE_NONE) {
+        if (isBioSequence)
+          molecule.parent.needAromatic = false;
         index = checkBioType(pattern, 0);
+      }
       ch = getChar(pattern, index);
       boolean haveOpen = checkBrace(molecule, ch, '{');
       if (haveOpen)
@@ -471,16 +478,16 @@ public class SmilesParser {
             .substring(pt, index), null, currentAtom, false, isBranchAtom);
 
 
-        if (haveOpen && bond.bondType != SmilesBond.TYPE_UNKNOWN)
+        if (haveOpen && bond.order != SmilesBond.TYPE_UNKNOWN)
           index = pt;
         ch = getChar(pattern, index);
         if (checkBrace(molecule, ch, '{'))
           ch = getChar(pattern, ++index);
-        if (ch == '~' && bond.bondType == SmilesBond.TYPE_NONE) {
+        if (ch == '~' && bond.order == SmilesBond.TYPE_NONE) {
           index = checkBioType(pattern, index);
           ch = getChar(pattern, index);
         }
-        if (ch == '\0' && bond.bondType == SmilesBond.TYPE_NONE)
+        if (ch == '\0' && bond.order == SmilesBond.TYPE_NONE)
           return;
         boolean isRing = (Character.isDigit(ch) || ch == '%');
         boolean isAtom = (!isRing && (ch == '_' || ch == '[' || ch == '*' || Character
@@ -523,8 +530,8 @@ public class SmilesParser {
               subPattern += ".0";
             currentAtom = parseAtom(molecule, null, subPattern, currentAtom,
                 bond, ch == '[', false, isBranchAtom);
-            if (bond.bondType != SmilesBond.TYPE_UNKNOWN
-                && bond.bondType != SmilesBond.TYPE_NONE)
+            if (bond.order != SmilesBond.TYPE_UNKNOWN
+                && bond.order != SmilesBond.TYPE_NONE)
               bond.set(null, currentAtom);
             break;
           default:
@@ -582,7 +589,7 @@ public class SmilesParser {
     return index;
   }
 
-  Hashtable htMeasures = new Hashtable();
+  Map<String, SmilesMeasure> htMeasures = new Hashtable<String, SmilesMeasure>();
   
   private void parseMeasure(SmilesSearch molecule, String strMeasure,
                             SmilesAtom currentAtom)
@@ -597,7 +604,7 @@ public class SmilesParser {
       int len = id.length();
       if (len == 1)
         id += "0";
-      SmilesMeasure m = (SmilesMeasure) htMeasures.get(id);
+      SmilesMeasure m = htMeasures.get(id);
       if ((m == null) == (pt < 0))
         break;
       try {
@@ -620,8 +627,8 @@ public class SmilesParser {
           float min = (pt + 1 == pt2 ? 0 : Float.parseFloat(s));
           s = strMeasure.substring(pt2 + 1);
           float max = (s.length() == 0 ? Float.MAX_VALUE : Float.parseFloat(s));
-          molecule.measures.add(m = new SmilesMeasure(molecule, index, type,
-              min, max, isNot));
+          m = new SmilesMeasure(molecule, index, type, min, max, isNot);
+          molecule.measures.add(m);
           if (index > 0)
             htMeasures.put(id, m);
           else if (index == 0 && Logger.debugging)
@@ -683,8 +690,8 @@ public class SmilesParser {
   }
 
   private String parseVariables(String pattern) throws InvalidSmilesException {
-    List keys = new ArrayList();
-    List values = new ArrayList();
+    List<String> keys = new ArrayList<String>();
+    List<String> values = new ArrayList<String>();
     int index;
     int ipt = 0;
     int iptLast = -1;
@@ -882,8 +889,8 @@ public class SmilesParser {
               if (isPrimitive)
                 atomSet.hasSymbol = true;
               // indicates we have already assigned an atom number
-              if (!symbol.equals("*"))
-                molecule.parent.needAromatic = true;
+              //if (!symbol.equals("*"))
+                //molecule.parent.needAromatic = true;
               index += symbol.length();
             } else {
               index = getDigits(pattern, index + 1, ret);
@@ -960,19 +967,19 @@ public class SmilesParser {
 
     // Final check
 
-    if (currentAtom != null && bond.bondType == SmilesBond.TYPE_NONE) {
+    if (currentAtom != null && bond.order == SmilesBond.TYPE_NONE) {
       newAtom.notBondedIndex = currentAtom.index;
     }
-    if (currentAtom != null && bond.bondType != SmilesBond.TYPE_NONE) {
-      if (bond.bondType == SmilesBond.TYPE_UNKNOWN)
-        bond.bondType = (isBioSequence && isBranchAtom ? SmilesBond.TYPE_BIO_PAIR
+    if (currentAtom != null && bond.order != SmilesBond.TYPE_NONE) {
+      if (bond.order == SmilesBond.TYPE_UNKNOWN)
+        bond.order = (isBioSequence && isBranchAtom ? SmilesBond.TYPE_BIO_PAIR
             : isSmarts || currentAtom.isAromatic() && newAtom.isAromatic() ? SmilesBond.TYPE_ANY
                 : SmilesBond.TYPE_SINGLE);
       if (!isBracketed)
         bond.set(null, newAtom);
       if (branchLevel == 0 && 
-          (bond.bondType == SmilesBond.TYPE_AROMATIC 
-              || bond.bondType == SmilesBond.TYPE_BIO_PAIR))
+          (bond.order == SmilesBond.TYPE_AROMATIC 
+              || bond.order == SmilesBond.TYPE_BIO_PAIR))
         branchLevel++;
     }
     // if (Logger.debugging)
@@ -999,8 +1006,8 @@ public class SmilesParser {
 
     // Ring management
 
-    Integer r = new Integer(ringNum);
-    SmilesBond bond0 = (SmilesBond) ringBonds.get(r);
+    Integer r = Integer.valueOf(ringNum);
+    SmilesBond bond0 = ringBonds.get(r);
     if (bond0 == null) {
       ringBonds.put(r, bond);
       return;
@@ -1013,22 +1020,22 @@ public class SmilesParser {
     // (4) or "SINGLE"
     // we must check for C1......C/1....
     // in which case the "/" is referring to "second to first" not "first to second"
-    switch (bond.bondType) {
+    switch (bond.order) {
     case SmilesBond.TYPE_UNKNOWN:
-      bond.bondType = (bond0.bondType != SmilesBond.TYPE_UNKNOWN ? bond0.bondType
+      bond.order = (bond0.order != SmilesBond.TYPE_UNKNOWN ? bond0.order
           : isSmarts || currentAtom.isAromatic()
               && bond0.getAtom1().isAromatic() ? SmilesBond.TYPE_ANY
               : SmilesBond.TYPE_SINGLE);
       break;
     case SmilesBond.TYPE_DIRECTIONAL_1:
-      bond.bondType = SmilesBond.TYPE_DIRECTIONAL_2;
+      bond.order = SmilesBond.TYPE_DIRECTIONAL_2;
       break;
     case SmilesBond.TYPE_DIRECTIONAL_2:
-      bond.bondType = SmilesBond.TYPE_DIRECTIONAL_1;
+      bond.order = SmilesBond.TYPE_DIRECTIONAL_1;
       break;
     }
-    if (bond0.bondType != SmilesBond.TYPE_UNKNOWN
-        && bond0.bondType != bond.bondType)
+    if (bond0.order != SmilesBond.TYPE_UNKNOWN
+        && bond0.order != bond.order)
       throw new InvalidSmilesException("Incoherent bond type for ring");
     bond0.set(bond);
     currentAtom.bondCount--;
@@ -1172,8 +1179,8 @@ public class SmilesParser {
         molecule.haveBondStereochemistry = true;
         break;
       case SmilesBond.TYPE_AROMATIC:
-        if (!isBioSequence)
-          molecule.parent.needAromatic = true;
+        //if (!isBioSequence)
+          //molecule.parent.needAromatic = true;
         break;
       case SmilesBond.TYPE_DOUBLE:
       case SmilesBond.TYPE_SINGLE:
@@ -1335,6 +1342,12 @@ public class SmilesParser {
     return (i < 10 ? "" + i : i < 100 ? "%" + i : "%(" + i + ")");
   }
 
+  /**
+   * 
+   * @param pattern
+   * @param isSmarts
+   * @return  comments and white space removed, also ^^ to '
+   */
   private static String cleanPattern(String pattern, boolean isSmarts) {
     pattern = pattern.replaceAll("\\s", "").replaceAll("\\^\\^","'");
     int i = 0;
