@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-09-19 23:17:35 +0200 (lun., 19 sept. 2011) $
- * $Revision: 16082 $
+ * $Date: 2012-05-20 04:27:04 +0200 (dim., 20 mai 2012) $
+ * $Revision: 17165 $
  *
  * Copyright (C) 2005  The Jmol Development Team
  *
@@ -240,7 +240,7 @@ public class SmilesSearch extends JmolMolecule {
       search.bsRequired = null;
       search.firstMatchOnly = false;
       search.matchAllAtoms = false;
-      search.bsFound = bsFound;
+      //search.bsFound = bsFound;      // removed in 12.2.26
     } else if (isRingCheck) {
       search.bsRequired = null;
       search.isSilent = true;
