@@ -130,6 +130,7 @@ class PointGroup {
   private String name = "C_1?";
   private Operation principalAxis;
   private Operation principalPlane;
+  private float scale;
 
   String getName() {
     return name;
@@ -896,6 +897,7 @@ class PointGroup {
     Operation op;
     if (scaleFactor == 0)
       scaleFactor = 1;
+    scale = scaleFactor;
     int[][] nType = new int[4][2];
     for (int i = 1; i < maxAxis; i++)
       for (int j = nAxes[i]; --j >= 0;)
@@ -1065,8 +1067,11 @@ class PointGroup {
     return info;
   }
 
-  boolean isDrawType(String type, int index) {
-    return (drawInfo != null && drawType.equals(type == null ? "" : type) && drawIndex == index);
+  boolean isDrawType(String type, int index, float scale) {
+    return (drawInfo != null 
+        && drawType.equals(type == null ? "" : type) 
+        && drawIndex == index
+        && this.scale == scale);
   }
   
 }
