@@ -25,11 +25,13 @@ package org.jmol.util;
 
 import java.util.ArrayList;
 import java.util.Hashtable;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import javax.vecmath.Point3f;
+
 import org.jmol.viewer.JmolConstants;
+import org.jmol.constant.EnumPalette;
 import org.jmol.g3d.Graphics3D;
 import org.jmol.util.ArrayUtil;
 
@@ -50,8 +52,8 @@ import org.jmol.util.ArrayUtil;
 
    public ColorEncoder(ColorEncoder propertyColorEncoder) {
     if (propertyColorEncoder == null) {
-      schemes = new Hashtable();
-      argbsCpk = JmolConstants.argbsCpk;
+      schemes = new Hashtable<String, int[]>();
+      argbsCpk = EnumPalette.argbsCpk;
       argbsRoygb = JmolConstants.argbsRoygbScale;
       argbsRwb = JmolConstants.argbsRwbScale;
       argbsShapely = JmolConstants.argbsShapely;
@@ -74,6 +76,7 @@ import org.jmol.util.ArrayUtil;
   private final static String BYRESIDUE_SHAPELY = BYRESIDUE_PREFIX + "_shapely"; 
   private final static String BYRESIDUE_AMINO = BYRESIDUE_PREFIX + "_amino"; 
   
+  public final static int CUSTOM = -1;
   public final static int ROYGB = 0;
   public final static int BGYOR = 1;
   public final static int JMOL = 2;
@@ -106,7 +109,7 @@ import org.jmol.util.ArrayUtil;
     for (int i = 0; i < colorSchemes.length; i++)
       if (colorSchemes[i].equalsIgnoreCase(colorScheme))
         return (i >= ALT ? i - ALT : i < -USER ? i : -i);
-    return -1;
+    return CUSTOM;
   }
 
   private final static String fixName(String name) {
@@ -126,10 +129,11 @@ import org.jmol.util.ArrayUtil;
   private int[] argbsAmino;
   private int ihalf;
   private static int[] rasmolScale;
-  public Map schemes;
+  public Map<String, int[]> schemes;
 
 
   public int currentPalette = ROYGB;
+  public int currentSegmentCount = 1;
   public boolean isTranslucent = false;
   public float lo;
   public float hi;
@@ -177,7 +181,7 @@ import org.jmol.util.ArrayUtil;
           argbsRwb = JmolConstants.argbsRwbScale;
           break;
         case JMOL:
-          argbsCpk = JmolConstants.argbsCpk;
+          argbsCpk = EnumPalette.argbsCpk;
           break;
         case RASMOL:
           getRasmolScale();
@@ -223,7 +227,7 @@ import org.jmol.util.ArrayUtil;
         argbsShapely = thisScale;
         break;
       }
-    return -1;
+    return CUSTOM;
   }
 
   /**
@@ -301,12 +305,12 @@ import org.jmol.util.ArrayUtil;
     colorScheme = fixName(colorScheme);
     int ipt = getSchemeIndex(colorScheme);
     if (schemes.containsKey(colorScheme)) {
-      setThisScheme(colorScheme, (int[]) schemes.get(colorScheme));
+      setThisScheme(colorScheme, schemes.get(colorScheme));
       return ipt; // -1 means custom -- use "thisScale", otherwise a scheme number
     }
     
     // return a positive value for a known scheme or ROYGB if a default is ok, or MAX_VALUE
-    return (ipt != -1 ? ipt : defaultToRoygb ? ROYGB 
+    return (ipt != CUSTOM ? ipt : defaultToRoygb ? ROYGB 
         : Integer.MAX_VALUE);
   }
 
@@ -325,19 +329,19 @@ import org.jmol.util.ArrayUtil;
      c = quantizeRgb(val, lo, hi, rgbRed, rgbGreen, rgbBlue);
      break;
      */
-    case -1:
+    case CUSTOM:
       return thisScale;      
     case ROYGB:
       return propertyColorEncoder.argbsRoygb;
     case BGYOR:
       return ArrayUtil.arrayCopy(propertyColorEncoder.argbsRoygb, 0, -1, true);
     case LOW:
-      return ArrayUtil.arrayCopy(propertyColorEncoder.argbsRoygb, 0, ihalf, false);
+      return ArrayUtil.arrayCopy(propertyColorEncoder.argbsRoygb, 0, propertyColorEncoder.ihalf, false);
     case HIGH:
-      int[] a = ArrayUtil.arrayCopy(argbsRoygb, propertyColorEncoder.argbsRoygb.length - 2 * propertyColorEncoder.ihalf, -1, false);
+      int[] a = ArrayUtil.arrayCopy(propertyColorEncoder.argbsRoygb, propertyColorEncoder.argbsRoygb.length - 2 * propertyColorEncoder.ihalf, -1, false);
       b = new int[propertyColorEncoder.ihalf];
-      for (int i = propertyColorEncoder.ihalf, j = propertyColorEncoder.argbsRoygb.length - 1; --i >= 0;j -= 2)
-        b[i] = a[j];
+      for (int i = b.length, j = a.length; --i >= 0 && --j >= 0;)
+        b[i] = a[j--];
       return b;
     case BW:
       return getPaletteBW();
@@ -380,56 +384,14 @@ import org.jmol.util.ArrayUtil;
     return colix;
   }
 
-  public int getArgbFromPalette(float val, float lo, float hi, int palette) {
-    if (Float.isNaN(val))
-      return GRAY;
+  public int getPaletteColorCount(int palette) {
     switch (palette) {
-    case -1:
-      if (isColorIndex) {
-        lo = 0;
-        hi = thisScale.length;
-      }
-      return thisScale[quantize(val, lo, hi, thisScale.length)];
-    case BW:
-      return getPaletteBW()[quantize(val, lo, hi, propertyColorEncoder.paletteBW.length)];
-    case WB:
-      return getPaletteWB()[quantize(val, lo, hi, propertyColorEncoder.paletteWB.length)];
-    case ROYGB:
-      return propertyColorEncoder.argbsRoygb[quantize(val, lo, hi, propertyColorEncoder.argbsRoygb.length)];
-    case BGYOR:
-      return propertyColorEncoder.argbsRoygb[quantize(-val, -hi, -lo, propertyColorEncoder.argbsRoygb.length)];
-    case LOW:
-      return propertyColorEncoder.argbsRoygb[quantize(val, lo, hi, propertyColorEncoder.ihalf)];
-    case HIGH:
-      return propertyColorEncoder.argbsRoygb[propertyColorEncoder.ihalf + quantize(val, lo, hi, propertyColorEncoder.ihalf) * 2];
-    case RWB:
-      return propertyColorEncoder.argbsRwb[quantize(val, lo, hi, propertyColorEncoder.argbsRwb.length)];
-    case BWR:
-      return propertyColorEncoder.argbsRwb[quantize(-val, -hi, -lo, propertyColorEncoder.argbsRwb.length)];
-    case USER:
-      return (propertyColorEncoder.userScale.length == 0 ? GRAY : propertyColorEncoder.userScale[quantize(val, lo, hi, userScale.length)]);
-    case RESU:
-      return (propertyColorEncoder.userScale.length == 0 ? GRAY : propertyColorEncoder.userScale[quantize(-val, -hi, -lo, userScale.length)]);
-    case JMOL:
-      return propertyColorEncoder.argbsCpk[colorIndex((int)val, argbsCpk.length)];
-    case RASMOL:
-      return getRasmolScale()[colorIndex((int)val, rasmolScale.length)];
-    case SHAPELY:
-      return propertyColorEncoder.argbsShapely[colorIndex((int)val, propertyColorEncoder.argbsShapely.length)];
-    case AMINO:
-      return propertyColorEncoder.argbsAmino[colorIndex((int)val, propertyColorEncoder.argbsAmino.length)];
-    default:
-      return GRAY;
-    }
-  }
-  
-  private int getSegmentCount(int palette) {
-    switch (palette) {
-    case -1:
+    case CUSTOM:
       return thisScale.length;
     case BW:
     case WB:
-      return propertyColorEncoder.paletteWB.length;
+      getPaletteBW();
+      return propertyColorEncoder.paletteBW.length;
     case ROYGB:
     case BGYOR:
       return propertyColorEncoder.argbsRoygb.length;
@@ -443,9 +405,9 @@ import org.jmol.util.ArrayUtil;
     case RESU:
       return propertyColorEncoder.userScale.length;
     case JMOL:
-      return propertyColorEncoder.argbsCpk.length;
+      return argbsCpk.length;
     case RASMOL:
-      return rasmolScale.length;
+      return getRasmolScale().length;
     case SHAPELY:
       return propertyColorEncoder.argbsShapely.length;
     case AMINO:
@@ -454,6 +416,50 @@ import org.jmol.util.ArrayUtil;
       return 0;
     }
   }
+  
+  public int getArgbFromPalette(float val, float lo, float hi, int palette) {
+    if (Float.isNaN(val))
+      return GRAY;
+    int n = getPaletteColorCount(palette);
+    switch (palette) {
+    case CUSTOM:
+      if (isColorIndex) {
+        lo = 0;
+        hi = thisScale.length;
+      }
+      return thisScale[quantize(val, lo, hi, n)];
+    case BW:
+      return getPaletteBW()[quantize(val, lo, hi, n)];
+    case WB:
+      return getPaletteWB()[quantize(val, lo, hi, n)];
+    case ROYGB:
+      return propertyColorEncoder.argbsRoygb[quantize(val, lo, hi, n)];
+    case BGYOR:
+      return propertyColorEncoder.argbsRoygb[quantize(-val, -hi, -lo, n)];
+    case LOW:
+      return propertyColorEncoder.argbsRoygb[quantize(val, lo, hi, n)];
+    case HIGH:
+      return propertyColorEncoder.argbsRoygb[propertyColorEncoder.ihalf + quantize(val, lo, hi, n) * 2];
+    case RWB:
+      return propertyColorEncoder.argbsRwb[quantize(val, lo, hi, n)];
+    case BWR:
+      return propertyColorEncoder.argbsRwb[quantize(-val, -hi, -lo, n)];
+    case USER:
+      return (propertyColorEncoder.userScale.length == 0 ? GRAY : propertyColorEncoder.userScale[quantize(val, lo, hi, n)]);
+    case RESU:
+      return (propertyColorEncoder.userScale.length == 0 ? GRAY : propertyColorEncoder.userScale[quantize(-val, -hi, -lo, n)]);
+    case JMOL:
+      return propertyColorEncoder.argbsCpk[colorIndex((int)val, n)];
+    case RASMOL:
+      return getRasmolScale()[colorIndex((int)val, n)];
+    case SHAPELY:
+      return propertyColorEncoder.argbsShapely[colorIndex((int)val, n)];
+    case AMINO:
+      return propertyColorEncoder.argbsAmino[colorIndex((int)val, n)];
+    default:
+      return GRAY;
+    }
+  }
 
   private void setThisScheme(String name, int[] scale) {
     thisName = name;
@@ -477,26 +483,27 @@ import org.jmol.util.ArrayUtil;
         : getColorIndexFromPalette(val, lo, hi, currentPalette, isTranslucent));
   }
 
-  public Map getColorKey() {
-    Hashtable info = new Hashtable();
+  public Map<String, Object> getColorKey() {
+    Map<String, Object> info = new Hashtable<String, Object>();
+    int segmentCount = getPaletteColorCount(currentPalette);
+    List<Point3f> colors = new ArrayList<Point3f>(segmentCount);
+/*    
     boolean isReverse = isReversed;
-    int segmentCount = getSegmentCount(currentPalette);
+    
     switch (currentPalette) {
     case BGYOR:
     case BWR:
     case RESU:
       isReverse = !isReverse;
       break;
-    default:
-      break;
     }
-    List colors = new ArrayList(segmentCount);
+    */
     float[] values = new float[segmentCount + 1];
-    float quanta = (hi - lo) / segmentCount;
-    float f = quanta * (isReversed ? -0.5f : 0.5f);
+    float quantum = (hi - lo) / segmentCount;
+    float f = quantum * (isReversed ? -0.5f : 0.5f);
 
     for (int i = 0; i < segmentCount; i++) {
-      values[i] = (isReversed ? hi - i * quanta : lo + i * quanta);
+      values[i] = (isReversed ? hi - i * quantum : lo + i * quantum);
       colors.add(Graphics3D.colorPointFromInt2(getArgb(values[i] + f)));
     }
     values[segmentCount] = (isReversed ? lo : hi);
@@ -523,7 +530,7 @@ import org.jmol.util.ArrayUtil;
   public void setRange(float lo, float hi, boolean isReversed) {
     if (hi == Float.MAX_VALUE) {
       lo = 1; 
-      hi = getSegmentCount(currentPalette) + 1;
+      hi = getPaletteColorCount(currentPalette) + 1;
     }
     this.lo = Math.min(lo, hi);
     this.hi = Math.max(lo, hi);
@@ -536,7 +543,7 @@ import org.jmol.util.ArrayUtil;
   
   public String getColorSchemeName(int i) {
     int absi = Math.abs(i);
-    return (i == -1 ? thisName : absi < colorSchemes.length && absi >= 0 ? colorSchemes[absi] : null);  
+    return (i == CUSTOM ? thisName : absi < colorSchemes.length && absi >= 0 ? colorSchemes[absi] : null);  
   }
 
   // legitimate static methods:
@@ -553,12 +560,12 @@ import org.jmol.util.ArrayUtil;
   public final static synchronized int[] getRasmolScale() {
     if (rasmolScale != null)
       return rasmolScale;
-    rasmolScale = new int[JmolConstants.argbsCpk.length];
-    int argb = JmolConstants.argbsCpkRasmol[0] | 0xFF000000;
+    rasmolScale = new int[EnumPalette.argbsCpk.length];
+    int argb = EnumPalette.argbsCpkRasmol[0] | 0xFF000000;
     for (int i = rasmolScale.length; --i >= 0;)
       rasmolScale[i] = argb;
-    for (int i = JmolConstants.argbsCpkRasmol.length; --i >= 0;) {
-      argb = JmolConstants.argbsCpkRasmol[i];
+    for (int i = EnumPalette.argbsCpkRasmol.length; --i >= 0;) {
+      argb = EnumPalette.argbsCpkRasmol[i];
       rasmolScale[argb >> 24] = argb | 0xFF000000;
     }
     return rasmolScale;
@@ -575,6 +582,24 @@ import org.jmol.util.ArrayUtil;
     return propertyColorEncoder.paletteWB = b;
   }
 
+  public static int[] getPaletteAtoB(int color1, int color2, int n) {
+    if (n < 2)
+      n = JmolConstants.argbsRoygbScale.length;
+    int[] b = new int[n];
+    float red1 = (((color1 & 0xFF0000) >> 16) & 0xFF) / 255f;
+    float green1 = (((color1 & 0xFF00) >> 8) & 0xFF) / 255f;
+    float blue1 = (color1 & 0xFF) / 255f;
+    float red2 = (((color2 & 0xFF0000) >> 16) & 0xFF) / 255f;
+    float green2 = (((color2 & 0xFF00) >> 8) & 0xFF) / 255f;
+    float blue2 = (color2 & 0xFF) / 255f;
+    float dr = (red2 - red1) / (n - 1);
+    float dg = (green2 - green1) / (n - 1);
+    float db = (blue2 - blue1) / (n - 1);
+    for (int i = 0; i < n; i++)
+      b[i] = Graphics3D.colorTriadToInt(red1 + dr * i, green1 + dg * i, blue1
+          + db * i);
+    return b;
+  }
   private int[] getPaletteBW() {
     if (propertyColorEncoder.paletteBW != null) 
       return propertyColorEncoder.paletteBW;
@@ -586,6 +611,18 @@ import org.jmol.util.ArrayUtil;
     return propertyColorEncoder.paletteBW = b;
   }
 
+  /**
+   * gets the value at the color boundary for this color range fraction 
+   * @param x
+   * @param isLowEnd
+   * @return quantized value
+   */
+  public float quantize(float x, boolean isLowEnd) {
+    int n = getPaletteColorCount(currentPalette);
+    x = (((int) (x * n)) + (isLowEnd ? 0f : 1f)) / n;
+    return (x <= 0 ? lo : x >= 1 ? hi : lo + (hi - lo) * x);
+  }
+  
   public final static int quantize(float val, float lo, float hi, int segmentCount) {
     /* oy! Say you have an array with 10 values, so segmentCount=10
      * then we expect 0,1,2,...,9  EVENLY
@@ -656,13 +693,78 @@ import org.jmol.util.ArrayUtil;
 */
 
   public int getState(StringBuffer s) {
-    Iterator e = schemes.keySet().iterator();
     int n = 0;
-    while (e.hasNext()) {
-      String name = (String) e.next();
-      if (name.length() > 0 & n++ >= 0) 
-        s.append("color \"" + name + "=" + getColorSchemeList((int[])schemes.get(name)) + "\";\n");
+    for (Map.Entry<String, int[]> entry : schemes.entrySet()) {
+      String name = entry.getKey();
+      if (name.length() > 0 & n++ >= 0)
+        s.append("color \"" + name + "="
+            + getColorSchemeList(entry.getValue()) + "\";\n");
     }
     return n;
   }
+
+  public String getColorScheme() {
+    return (isTranslucent ? "translucent " : "")
+        + (currentPalette < 0 ? getColorSchemeList(getColorSchemeArray(currentPalette))
+            : getColorSchemeName(currentPalette));
+  }
+
+  static public void RGBtoHSL(float r, float g, float b, float[] ret) {
+    //http://tips4java.wordpress.com/2009/07/05/hsl-color/
+ 
+    //  Get RGB values in the range 0 - 1
+
+   
+      //  Minimum and Maximum RGB values are used in the HSL calculations
+
+      r /= 255;
+      g /= 255;
+      b /= 255;
+      if (r > 1)
+        r = 1;
+      if (g > 1)
+        g = 1;
+      if (b > 1)
+        b = 1;
+      float min = Math.min(r, Math.min(g, b));
+      float max = Math.max(r, Math.max(g, b));
+
+      //  Calculate the Hue
+
+      float h = 0;
+
+      if (max == min)
+        h = 0;
+      else if (max == r)
+        h = ((60 * (g - b) / (max - min)) + 360) % 360;
+      else if (max == g)
+        h = (60 * (b - r) / (max - min)) + 120;
+      else if (max == b)
+        h = (60 * (r - g) / (max - min)) + 240;
+
+      //  Calculate the Luminance
+
+      float l = (max + min) / 2;
+
+      //  Calculate the Saturation
+
+      float s = 0;
+
+      if (max == min)
+        s = 0;
+      else if (l <= .5f)
+        s = (max - min) / (max + min);
+      else
+        s = (max - min) / (2 - max - min);
+      
+      //if (s == 1 && l != 0.5f) {
+      //    s = Float.NaN;
+      // }
+      
+      ret[0] = h / 360;
+      ret[1] = s;
+      ret[2] = l;
+    }
+
+
 }
