@@ -25,11 +25,11 @@
 
 package org.jmol.util;
 
-import java.util.BitSet;
-import java.util.Enumeration;
-import java.util.Hashtable;
-import java.util.List;
 import java.util.ArrayList;
+import java.util.BitSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.AxisAngle4f;
 import javax.vecmath.Matrix3f;
@@ -59,13 +59,42 @@ public class Escape {
     return r + g + b;
   }
 
+  
+  /**
+   * must be its own, because of the possibility of being null
+   * @param xyz
+   * @return  {x y z}
+   */
+  public static String escape(Tuple3f xyz) {
+    if (xyz == null)
+      return "null";
+    return "{" + xyz.x + " " + xyz.y + " " + xyz.z + "}";
+  }
+
+  @SuppressWarnings("unchecked")
   public static String escape(Object x) {
     if (x instanceof String)
-      return escape("" + x);
-    if (x instanceof String[])
-      return escape((String[]) x, true);
+      return escape((String) x);
+    if (x instanceof List<?>)
+      return escape((ArrayList<ScriptVariable>) x);
     if (x instanceof BitSet) 
       return escape((BitSet) x, true);
+    if (x instanceof Matrix3f) 
+      return TextFormat.simpleReplace(((Matrix3f) x).toString(), "\t", ",\t");
+    if (x instanceof Matrix4f) 
+      return TextFormat.simpleReplace(((Matrix4f) x).toString(), "\t", ",\t");
+    if (x instanceof Tuple3f)
+      return escape((Tuple3f) x);
+    if (x instanceof Point4f) {
+      Point4f xyzw = (Point4f) x;
+      return "{" + xyzw.x + " " + xyzw.y + " " + xyzw.z + " " + xyzw.w + "}";
+    }
+    if (x instanceof AxisAngle4f) {
+      AxisAngle4f a = (AxisAngle4f) x;
+      return "{" + a.x + " " + a.y + " " + a.z + " " + (float) (a.angle * 180d/Math.PI) + "}";
+    }    
+    if (x instanceof String[])
+      return escape((String[]) x, true);
     if (x instanceof int[] 
           || x instanceof int[][]
           || x instanceof float[]
@@ -74,77 +103,16 @@ public class Escape {
       return toJSON(null, x);
     if (x instanceof Point3f[])
       return escapeArray(x);
-    if (x instanceof Hashtable)
-      return escape((Hashtable) x);
+    if (x instanceof Map)
+      return escape((Map<String, Object>) x);
     return (x == null ? "null" : x.toString());
   }
 
-  public static String escape(Point4f xyzw) {
-    return "{" + xyzw.x + " " + xyzw.y + " " + xyzw.z + " " + xyzw.w + "}";
-  }
-
-  public static String escape(AxisAngle4f a) {
-    return "{" + a.x + " " + a.y + " " + a.z + " " + (float) (a.angle * 180d/Math.PI) + "}";
-  }
-
-  public static String escape(Tuple3f xyz) {
-    if (xyz == null)
-      return "null";
-    return "{" + xyz.x + " " + xyz.y + " " + xyz.z + "}";
-  }
-
-  public static String escape(float[] f, boolean asArray) {
-    if (asArray)
-      return toJSON(null, f);
-    StringBuffer sb = new StringBuffer();
-    for (int i = 0; i < f.length; i++) {
-      if (i > 0)
-        sb.append('\n');
-      sb.append(f[i]);
-    }
-    return sb.toString();
-  }
-
-  public static String escape(float[][] f, boolean addSemi) {
-    StringBuffer sb = new StringBuffer();
-    String eol = (addSemi ? ";\n" : "\n");
-    for (int i = 0; i < f.length; i++)
-      if (f[i] != null) {
-        if (i > 0)
-          sb.append(eol);
-        for (int j = 0; j < f[i].length; j++)
-          sb.append(f[i][j]).append('\t');
-      }
-    return sb.toString();
-  }
-
-  public static String escape(float[][][] f, boolean addSemi) {
-    StringBuffer sb = new StringBuffer();
-    String eol = (addSemi ? ";\n" : "\n");
-    if (f[0] == null || f[0][0] == null)
-      return "0 0 0" + eol;
-    sb.append(f.length).append(" ")
-      .append(f[0].length).append(" ")
-      .append(f[0][0].length);
-    for (int i = 0; i < f.length; i++)
-      if (f[i] != null) {
-        sb.append(eol);
-        for (int j = 0; j < f[i].length; j++)
-          if (f[i][j] != null) {
-            sb.append(eol);
-            for (int k = 0; k < f[i][j].length; k++)
-              sb.append(f[i][j][k]).append('\t');
-          }
-      }
-    return sb.toString();
-  }
+//static String ESCAPE_SET = " ,./;:_+-~=><?'!@#$%^&*";
+//static int nEscape = ESCAPE_SET.length();
 
   private final static String escapable = "\\\\\tt\rr\nn\"\""; 
 
-  public static String escapeChopped(String str) {
-    return chop(escape(str)); 
-  }
-  
   public static String escape(String str) {
     if (str == null)
       return "\"\"";
@@ -176,8 +144,93 @@ public class Escape {
     return "\"" + str + "\"";
   }
 
-  static String ESCAPE_SET = " ,./;:_+-~=><?'!@#$%^&*";
-  static int nEscape = ESCAPE_SET.length();
+  private static String unicode(char c) {
+    String s = "0000" + Integer.toHexString(c);
+    return "\\u" + s.substring(s.length() - 4);
+  }
+
+  public static String escape(ArrayList<ScriptVariable> list) {
+    if (list == null)
+      return escape("");
+    StringBuilder s = new StringBuilder();
+    s.append("[");
+    for (int i = 0; i < list.size(); i++) {
+      if (i > 0)
+        s.append(", ");
+      s.append(escapeNice(list.get(i).asString()));
+    }
+    s.append("]");
+    return s.toString();
+  }
+
+  public static String escape(Map<String, Object> ht) {
+    StringBuilder sb = new StringBuilder();
+    sb.append("{ ");
+    String sep = "";
+    for (Map.Entry<String, Object> entry : ht.entrySet()) {
+      String key = entry.getKey();
+      sb.append(sep).append(escape(key)).append(':');
+      Object val = entry.getValue();
+      if (!(val instanceof ScriptVariable))
+        val = ScriptVariable.getVariable(val);
+      sb.append(((ScriptVariable)val).escape());
+      sep = ","; 
+    }
+    sb.append(" }");
+    return sb.toString();
+  }
+  
+  /**
+   * 
+   * @param f
+   * @param asArray -- FALSE allows bypassing of escape(Object f); TRUE: unnecssary
+   * @return tabular string
+   */
+  public static String escape(float[] f, boolean asArray) {
+    if (asArray)
+      return toJSON(null, f); // or just use escape(f)
+    StringBuilder sb = new StringBuilder();
+    for (int i = 0; i < f.length; i++) {
+      if (i > 0)
+        sb.append('\n');
+      sb.append(f[i]);
+    }
+    return sb.toString();
+  }
+
+  public static String escape(float[][] f, boolean addSemi) {
+    StringBuilder sb = new StringBuilder();
+    String eol = (addSemi ? ";\n" : "\n");
+    for (int i = 0; i < f.length; i++)
+      if (f[i] != null) {
+        if (i > 0)
+          sb.append(eol);
+        for (int j = 0; j < f[i].length; j++)
+          sb.append(f[i][j]).append('\t');
+      }
+    return sb.toString();
+  }
+
+  public static String escape(float[][][] f, boolean addSemi) {
+    StringBuilder sb = new StringBuilder();
+    String eol = (addSemi ? ";\n" : "\n");
+    if (f[0] == null || f[0][0] == null)
+      return "0 0 0" + eol;
+    sb.append(f.length).append(" ")
+      .append(f[0].length).append(" ")
+      .append(f[0][0].length);
+    for (int i = 0; i < f.length; i++)
+      if (f[i] != null) {
+        sb.append(eol);
+        for (int j = 0; j < f[i].length; j++)
+          if (f[i][j] != null) {
+            sb.append(eol);
+            for (int k = 0; k < f[i][j].length; k++)
+              sb.append(f[i][j][k]).append('\t');
+          }
+      }
+    return sb.toString();
+  }
 
   /**
    * 
@@ -189,7 +242,7 @@ public class Escape {
   public static String escape(String[] list, boolean nicely) {
     if (list == null)
       return escape("");
-    StringBuffer s = new StringBuffer();
+    StringBuilder s = new StringBuilder();
     s.append("[");
     for (int i = 0; i < list.length; i++) {
       if (i > 0)
@@ -206,7 +259,7 @@ public class Escape {
       return escape("");
     if (x instanceof Float)
       return "" + x;
-    StringBuffer s = new StringBuffer();
+    StringBuilder s = new StringBuilder();
     s.append("[");
     if (x instanceof double[]) {
       double[] dlist = (double[]) x;
@@ -231,7 +284,7 @@ public class Escape {
       }
     } else if (x instanceof Point3f[]) {
       Point3f[] plist = (Point3f[]) x;
-      s = new StringBuffer("[");
+      s = new StringBuilder("[");
       for (int i = 0; i < plist.length; i++) {
         if (i > 0)
           s.append(", ");
@@ -251,27 +304,6 @@ public class Escape {
     return (Float.isNaN(f) ? escape(s) : s);
   }
 
-  private static String unicode(char c) {
-    String s = "0000" + Integer.toHexString(c);
-    return "\\u" + s.substring(s.length() - 4);
-  }
-
-  private static String chop(String s) {
-    int len = s.length();
-    if (len < 512)
-      return s;
-    StringBuffer sb = new StringBuffer();
-    String sep = "\"\\\n    + \"";
-    int pt = 0;
-    for (int i = 72; i < len; pt = i, i += 72) {
-      while (s.charAt(i - 1) == '\\')
-        i++;
-      sb.append((pt == 0 ? "" : sep)).append(s.substring(pt, i));
-    }
-    sb.append(sep).append(s.substring(pt, len));
-    return sb.toString();
-  }
-
   public static Object unescapePointOrBitsetOrMatrixOrArray(String s) {
     if (s.charAt(0) == '{')
       return unescapePoint(s);
@@ -418,7 +450,7 @@ public class Escape {
     char chClose = (isAtoms ? ')' : ']');
     if (bs == null)
       return chOpen + "{}" + chClose;
-    StringBuffer s = new StringBuffer(chOpen + "{");
+    StringBuilder s = new StringBuilder(chOpen + "{");
     int imax = bs.length();
     int iLast = -1;
     int iFirst = -2;
@@ -444,36 +476,7 @@ public class Escape {
     return s.toString();
   }
 
-  public static String escape(BitSet bs) {
-    return escape(bs, true);
-  }
-
-  public static String escape(Matrix3f m3) {
-    return m3.toString();
-  }
-
-  public static String escape(Matrix4f m4) {
-    return (m4.toString());
-  }
-
-
-  public static String escape(Hashtable ht) {
-    StringBuffer sb = new StringBuffer();
-    sb.append("{ ");
-    String sep = "";
-    Enumeration e = ((Hashtable) ht).keys();
-    while (e.hasMoreElements()) {
-      String key = (String) e.nextElement();
-      sb.append(sep).append(escape(key)).append(':');
-      Object val = ht.get(key);
-      sb.append(((ScriptVariable)val).escape());
-      sep = ","; 
-    }
-    sb.append(" }");
-    return sb.toString();
-  }
-  
-  private static String packageJSON(String infoType, StringBuffer sb) {
+  private static String packageJSON(String infoType, StringBuilder sb) {
     return packageJSON(infoType, sb.toString());
   }
 
@@ -491,11 +494,12 @@ public class Escape {
     return "\"" + s + "\"";
   }
 
+  @SuppressWarnings("unchecked")
   public static String toJSON(String infoType, Object info) {
 
     //Logger.debug(infoType+" -- "+info);
 
-    StringBuffer sb = new StringBuffer();
+    StringBuilder sb = new StringBuilder();
     String sep = "";
     if (info == null)
       return packageJSON(infoType, (String) null);
@@ -533,6 +537,17 @@ public class Escape {
       sb.append("]");
       return packageJSON(infoType, sb);
     }
+    if (info instanceof Point3f[]) {
+      sb.append("[");
+      int imax = ((Point3f[]) info).length;
+      for (int i = 0; i < imax; i++) {
+        sb.append(sep);
+        addJsonTuple(sb, ((Point3f[]) info)[i]);
+        sep = ",";
+      }
+      sb.append("]");
+      return packageJSON(infoType, sb);
+    }
     if (info instanceof int[][]) {
       sb.append("[");
       int imax = ((int[][]) info).length;
@@ -565,9 +580,9 @@ public class Escape {
     }
     if (info instanceof List) {
       sb.append("[ ");
-      int imax = ((List) info).size();
+      int imax = ((List<?>) info).size();
       for (int i = 0; i < imax; i++) {
-        sb.append(sep).append(toJSON(null, ((List) info).get(i)));
+        sb.append(sep).append(toJSON(null, ((List<?>) info).get(i)));
         sep = ",";
       }
       sb.append(" ]");
@@ -600,10 +615,7 @@ public class Escape {
       return packageJSON(infoType, sb);
     }
     if (info instanceof Tuple3f) {
-      sb.append("[")
-        .append(((Tuple3f) info).x).append(",")
-        .append(((Tuple3f) info).y).append(",")
-        .append(((Tuple3f) info).z).append("]");
+      addJsonTuple(sb, (Tuple3f) info);
       return packageJSON(infoType, sb);
     }
     if (info instanceof AxisAngle4f) {
@@ -622,13 +634,13 @@ public class Escape {
         .append(((Point4f) info).w).append("]");
       return packageJSON(infoType, sb);
     }
-    if (info instanceof Hashtable) {
+    if (info instanceof Map) {
       sb.append("{ ");
-      Enumeration e = ((Hashtable) info).keys();
-      while (e.hasMoreElements()) {
-        String key = (String) e.nextElement();
+      Iterator<String> e = ((Map<String, ?>) info).keySet().iterator();
+      while (e.hasNext()) {
+        String key = e.next();
         sb.append(sep)
-            .append(packageJSON(key, toJSON(null, ((Hashtable) info).get(key))));
+            .append(packageJSON(key, toJSON(null, ((Map<?, ?>) info).get(key))));
         sep = ",";
       }
       sb.append(" }");
@@ -637,12 +649,19 @@ public class Escape {
     return packageJSON(infoType, fixString(info.toString()));
   }
 
+  private static void addJsonTuple(StringBuilder sb, Tuple3f pt) {
+    sb.append("[")
+    .append(pt.x).append(",")
+    .append(pt.y).append(",")
+    .append(pt.z).append("]");
+  }
+
   public static String toReadable(Object info) {
     return toReadable(null, info);
   }
-  
+
   public static String toReadable(String name, Object info) {
-    StringBuffer sb =new StringBuffer();
+    StringBuilder sb =new StringBuilder();
     String sep = "";
     if (info == null)
       return "null";
@@ -678,6 +697,16 @@ public class Escape {
       sb.append("]");
       return packageReadable(name, "float[" + imax + "]", sb);
     }
+    if (info instanceof Point3f[]) {
+      sb.append("[");
+      int imax = ((Point3f[]) info).length;
+      for (int i = 0; i < imax; i++) {
+        sb.append(sep).append(escape(((Point3f[])info)[i]));
+        sep = ",";
+      }
+      sb.append("]");
+      return packageReadable(name, "point3f[" + imax + "]", sb);
+    }
     if (info instanceof int[][]) {
       sb.append("[");
       int imax = ((int[][]) info).length;
@@ -698,35 +727,26 @@ public class Escape {
       sb.append("]");
       return packageReadable(name, "float[][]", sb);
     }
-    if (info instanceof List) {
-      int imax = ((List) info).size();
+    if (info instanceof List<?>) {
+      int imax = ((List<?>) info).size();
       for (int i = 0; i < imax; i++) {
-        sb.append(toReadable(name + "[" + (i + 1) + "]", ((List) info).get(i)));
+        sb.append(toReadable(name + "[" + (i + 1) + "]", ((List<?>) info).get(i)));
       }
-      return packageReadable(name, "Vector[" + imax + "]", sb);
+      return packageReadable(name, "List[" + imax + "]", sb);
     }
-    if (info instanceof Matrix3f) {
-      sb.append(escape((Matrix3f) info));
+    if (info instanceof Matrix3f
+        || info instanceof Tuple3f
+        || info instanceof Point4f
+        || info instanceof AxisAngle4f) {
+      sb.append(escape(info));
       return packageReadable(name, null, sb);
     }
-    if (info instanceof Tuple3f) {
-      sb.append(escape((Tuple3f) info));
-      return packageReadable(name, null, sb);
-    }
-    if (info instanceof Point4f) {
-      sb.append(escape((Point4f) info));
-      return packageReadable(name, null, sb);
-    }
-    if (info instanceof AxisAngle4f) {
-      sb.append(escape((AxisAngle4f) info));
-      return packageReadable(name, null, sb);
-    }
-    if (info instanceof Hashtable) {
-      Enumeration e = ((Hashtable) info).keys();
-      while (e.hasMoreElements()) {
-        String key = (String) e.nextElement();
+    if (info instanceof Map<?, ?>) {
+      Iterator<?> e = ((Map<?, ?>) info).keySet().iterator();
+      while (e.hasNext()) {
+        String key = (String) e.next();
         sb.append(toReadable((name == null ? "" : name + ".") + key,
-            ((Hashtable) info).get(key)));
+            ((Map<?, ?>) info).get(key)));
       }
       return sb.toString();
     }
@@ -734,7 +754,7 @@ public class Escape {
   }
 
   private static String packageReadable(String infoName, String infoType,
-                                        StringBuffer sb) {
+                                        StringBuilder sb) {
     return packageReadable(infoName, infoType, sb.toString());
   }
   
@@ -768,20 +788,9 @@ public class Escape {
     return XmlUtil.wrapCdata(toReadable(null, value));
   }
 
-  public static List unescapePointVector(String[] pts) {
-    List data = new ArrayList();
-    for (int i = 0; i < pts.length; i++) {
-      Object pt = Escape.unescapePoint(pts[i]);
-      if (!(pt instanceof Point3f))
-        return null;
-      data.add(pt);
-    }
-    return data;
-  }
-
   public static String unescapeUnicode(String s) {
     int ichMax = s.length();
-    StringBuffer sb = new StringBuffer(ichMax);
+    StringBuilder sb = new StringBuilder(ichMax);
     int ich = 0;
     while (ich < ichMax) {
       char ch = s.charAt(ich++);
@@ -824,7 +833,7 @@ public class Escape {
     //TODO -- should recognize '..' as well as "..." ?
     if (data == null || !data.startsWith("[") || !data.endsWith("]"))
       return null;
-    List v = new ArrayList();
+    List<String> v = new ArrayList<String>();
     int[] next = new int[1];
     next[0] = 1;
     while (next[0] < data.length()) {
@@ -836,31 +845,9 @@ public class Escape {
         next[0]++;
     }    
     String[] array = new String[v.size()];
-    for (int i = 0; i < array.length; i++)
-      array[i] = (String) v.get(i);
+    for (int i = 0; i < array.length; i++) {
+      array[i] = v.get(i);
+    }
     return array;
   }
-
-  public static BitSet unEscapeBitSetArray(String[] list, boolean allowNull) {
-    BitSet bs = new BitSet();
-    for (int i = 0; i < list.length; i++) {
-      BitSet bs1 = unescapeBitset(list[i]);
-      if (bs1 != null)
-        bs.or(bs1);
-      else if (allowNull)
-        return null;
-    }
-    return bs;
-  }
-
-  public static BitSet[] unEscapeBitSetArray(String[] list) {
-    BitSet[] b = new BitSet[list.length];
-    for (int i = 0; i < list.length; i++) {
-      b[i] = unescapeBitset(list[i]);
-      if (b[i] == null)
-        return null;
-    }
-    return b;
-  }
-
 }
