@@ -32,10 +32,12 @@
 
 package org.jmol.util;
 
-import java.awt.*;
-import java.awt.image.*;
-import java.io.*;
+import java.io.BufferedOutputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
 
+import org.jmol.api.ApiPlatform;
 
 /*
  * JpegEncoder - The JPEG main program which performs a jpeg compression of
@@ -45,10 +47,20 @@ import java.io.*;
  * 
  */
 
-public class JpegEncoder extends Frame
-{
-  // implemented in Jmol 12.1.36; here to ensure compatibility only -- see ScriptCompiler
+public class JpegEncoder {
+  // A system to allow the full Jmol state -- regardless of length -- 
+  // to be encoded in a set of APP1 (FFE1) tags.
+  // But we have to be careful about line ends for backward compatibility. 
+  // This solution is not 100% effective, because some data lines may in principle be 
+  // Very large and may not contain new lines for more than 65500 characters, 
+  // But that would be very unusual. Perhaps a huge data set loaded from a 
+  // string. Introduced in Jmol 12.1.36. 
+  
   public static final String CONTINUE_STRING = " #Jmol...\0"; 
+  // this string will GENERALLY appear at the end of lines and escape the 
+  private static final int CONTINUE_MAX = 65500;  // some room to spare here. 
+  private static final int CONTINUE_MAX_BUFFER = CONTINUE_MAX + 10; // never break up last 10 bytes
+  
   //Thread runner;
   private BufferedOutputStream outStream;
   //Image image;
@@ -58,15 +70,12 @@ public class JpegEncoder extends Frame
   private int Quality;
   //int code;
 
-  public JpegEncoder(Image image, int quality, OutputStream out, String comment)
-  {
-    MediaTracker tracker = new MediaTracker(this);
-    tracker.addImage(image, 0);
+  public JpegEncoder(ApiPlatform apiPlatform, Object image, int quality, OutputStream out, String comment) {
     try {
-      tracker.waitForID(0);
-    }
-    catch (InterruptedException e) {
-      // Got to do something?
+      if (!apiPlatform.waitForDisplay(null, image)) 
+        return;
+    } catch (InterruptedException e) {
+      // ignore
     }
     /*
      * Quality of the image.
@@ -79,16 +88,16 @@ public class JpegEncoder extends Frame
      * Getting picture information
      * It takes the Width, Height and RGB scans of the image. 
      */
-    JpegObj = new JpegInfo(image, comment);
+    JpegObj = new JpegInfo(apiPlatform, image, comment);
     
     outStream = new BufferedOutputStream(out);
     dct = new DCT(Quality);
     Huf=new Huffman(JpegObj.imageWidth,JpegObj.imageHeight);
   }
   
-  public static byte[] getBytes(Image image, int quality, String comment) {
+  public static byte[] getBytes(ApiPlatform apiPlatform, Object image, int quality, String comment) {
     ByteArrayOutputStream os = new ByteArrayOutputStream();
-    write(image, quality, os, comment);
+    write(apiPlatform, image, quality, os, comment);
     try {
       os.flush();
       os.close();
@@ -98,8 +107,8 @@ public class JpegEncoder extends Frame
     return os.toByteArray();
   }
 
-  public static void write(Image image, int quality, OutputStream os, String comment) {
-    (new JpegEncoder(image, quality, os, comment)).Compress();
+  public static void write(ApiPlatform apiPlatform, Object image, int quality, OutputStream os, String comment) {
+    (new JpegEncoder(apiPlatform, image, quality, os, comment)).Compress();
   }
 
   public void setQuality(int quality) {
@@ -111,6 +120,8 @@ public class JpegEncoder extends Frame
   }
 */  
   public void Compress() {
+    if (JpegObj == null)
+      return;
     String longState = WriteHeaders(outStream, JpegObj, dct);
     WriteCompressedData(outStream, JpegObj, dct, Huf);
     WriteEOI(outStream);
@@ -166,7 +177,7 @@ public class JpegEncoder extends Frame
         for (comp = 0; comp < JpegObj.NumberOfComponents; comp++) {
           //Width = JpegObj.BlockWidth[comp];
           //Height = JpegObj.BlockHeight[comp];
-          inputArray = (float[][]) JpegObj.Components[comp];
+          inputArray = JpegObj.Components[comp];
           int VsampF = JpegObj.VsampFactor[comp];
           int HsampF = JpegObj.HsampFactor[comp];
           int QNumber = JpegObj.QtableNumber[comp];
@@ -219,24 +230,24 @@ public class JpegEncoder extends Frame
   }
   
   static private String WriteHeaders(BufferedOutputStream out, JpegInfo JpegObj, DCT dct) {
-    int i, j, index, offset, length;
+    int i, j, index, offset;
     int tempArray[];
 
     // the SOI marker
     byte[] SOI = {(byte) 0xFF, (byte) 0xD8};
     WriteMarker(SOI, out);
     
-    // The order of the following headers is quiet inconsequential.
+    // The order of the following headers is quite inconsequential.
     // the JFIF header
     byte JFIF[] = new byte[18];
     JFIF[0] = (byte) 0xff;
     JFIF[1] = (byte) 0xe0;
-    JFIF[2] = (byte) 0x00;
-    JFIF[3] = (byte) 0x10;
-    JFIF[4] = (byte) 0x4a;
-    JFIF[5] = (byte) 0x46;
-    JFIF[6] = (byte) 0x49;
-    JFIF[7] = (byte) 0x46;
+    JFIF[2] = 0;
+    JFIF[3] = 16;
+    JFIF[4] = (byte) 0x4a; //'J'
+    JFIF[5] = (byte) 0x46; //'F'
+    JFIF[6] = (byte) 0x49; //'I'
+    JFIF[7] = (byte) 0x46; //'F'
     JFIF[8] = (byte) 0x00;
     JFIF[9] = (byte) 0x01;
     JFIF[10] = (byte) 0x00;
@@ -247,35 +258,21 @@ public class JpegEncoder extends Frame
     JFIF[15] = (byte) 0x01;
     JFIF[16] = (byte) 0x00;
     JFIF[17] = (byte) 0x00;
-    WriteArray(JFIF, out);
+    writeArray(JFIF, out);
     
     // Comment Header
-    String comment = JpegObj.Comment;
-    if (JpegObj.Comment.length() >= 65000) {
-      JpegObj.Comment = "JPEG Encoder Copyright 1998, James R. Weeks and BioElectroMech.\n\n";
-    } else {
-      JpegObj.Comment = "JPEG Encoder Copyright 1998, James R. Weeks and BioElectroMech.\n\n" + comment;
-      comment = null;
-    }
-    // bug fix -- hansonr@stolaf.edu -- 4/2010 -- Jmol 12.0.RC7 -- 
-    // length was two short here.
-    length = JpegObj.Comment.length() + 2;
-    byte COM[] = new byte[length + 2];
-    COM[0] = (byte) 0xFF;
-    COM[1] = (byte) 0xFE;
-    COM[2] = (byte) ((length >> 8) & 0xFF);
-    COM[3] = (byte) (length & 0xFF);
-    java.lang.System.arraycopy(JpegObj.Comment.getBytes(), 0, 
-          COM, 4, JpegObj.Comment.length());
-    WriteArray(COM, out);
-
+    String comment = null;    
+    if (JpegObj.Comment.length() > 0) 
+      writeString(JpegObj.Comment, (byte) 0xE1, out); // App data 1
+    writeString("JPEG Encoder Copyright 1998, James R. Weeks and BioElectroMech.\n\n", (byte) 0xFE, out);
+    
     // The DQT header
     // 0 is the luminance index and 1 is the chrominance index
     byte DQT[] = new byte[134];
     DQT[0] = (byte) 0xFF;
     DQT[1] = (byte) 0xDB;
-    DQT[2] = (byte) 0x00;
-    DQT[3] = (byte) 0x84;
+    DQT[2] = 0;
+    DQT[3] = (byte) 132;
     offset = 4;
     for (i = 0; i < 2; i++) {
       DQT[offset++] = (byte) ((0 << 4) + i);
@@ -284,14 +281,14 @@ public class JpegEncoder extends Frame
         DQT[offset++] = (byte) tempArray[Huffman.jpegNaturalOrder[j]];
       }
     }
-    WriteArray(DQT, out);
+    writeArray(DQT, out);
     
     // Start of Frame Header
     byte SOF[] = new byte[19];
     SOF[0] = (byte) 0xFF;
     SOF[1] = (byte) 0xC0;
-    SOF[2] = (byte) 0x00;
-    SOF[3] = (byte) 17;
+    SOF[2] = 0;
+    SOF[3] = 17;
     SOF[4] = (byte) JpegObj.Precision;
     SOF[5] = (byte) ((JpegObj.imageHeight >> 8) & 0xFF);
     SOF[6] = (byte) ((JpegObj.imageHeight) & 0xFF);
@@ -304,7 +301,7 @@ public class JpegEncoder extends Frame
       SOF[index++] = (byte) ((JpegObj.HsampFactor[i] << 4) + JpegObj.VsampFactor[i]);
       SOF[index++] = (byte) JpegObj.QtableNumber[i];
     }
-    WriteArray(SOF, out);
+    writeArray(SOF, out);
     
     WriteDHTHeader(Huffman.bitsDCluminance, Huffman.valDCluminance, out);
     WriteDHTHeader(Huffman.bitsACluminance, Huffman.valACluminance, out);
@@ -315,8 +312,8 @@ public class JpegEncoder extends Frame
     byte SOS[] = new byte[14];
     SOS[0] = (byte) 0xFF;
     SOS[1] = (byte) 0xDA;
-    SOS[2] = (byte) 0x00;
-    SOS[3] = (byte) 12;
+    SOS[2] = 0;
+    SOS[3] = 12;
     SOS[4] = (byte) JpegObj.NumberOfComponents;
     index = 5;
     for (i = 0; i < SOS[4]; i++) {
@@ -327,10 +324,43 @@ public class JpegEncoder extends Frame
     SOS[index++] = (byte) JpegObj.Ss;
     SOS[index++] = (byte) JpegObj.Se;
     SOS[index++] = (byte) ((JpegObj.Ah << 4) + JpegObj.Al);
-    WriteArray(SOS, out);
+    writeArray(SOS, out);
     return comment;
   }
 
+  private static void writeString(String s, byte id, BufferedOutputStream out) {
+    int len = s.length();
+    int i0 = 0;
+    String suffix = CONTINUE_STRING;
+    while (i0 < len) {
+      int nBytes = len - i0;
+      if (nBytes > CONTINUE_MAX_BUFFER) {
+        nBytes = CONTINUE_MAX;
+        // but break only at line breaks
+        int pt = s.lastIndexOf('\n', i0 + nBytes);
+        if (pt > i0 + 1)
+          nBytes = pt - i0;
+      }
+      if (i0 + nBytes == len)
+        suffix = "";
+      writeTag(nBytes + suffix.length(), id, out);
+      writeArray(s.substring(i0, i0 + nBytes).getBytes(), out);
+      if (suffix.length() > 0)
+        writeArray(suffix.getBytes(), out);
+      i0 += nBytes;
+    }
+  }
+
+  private static void writeTag(int length, byte id, BufferedOutputStream out) {
+    length += 2;
+    byte COM[] = new byte[4];
+    COM[0] = (byte) 0xFF;
+    COM[1] = id;
+    COM[2] = (byte) ((length >> 8) & 0xFF);
+    COM[3] = (byte) (length & 0xFF);
+    writeArray(COM, out);
+  }
+
   static void WriteDHTHeader(int[] bits, int[] val, BufferedOutputStream out) {
     byte DHT1[], DHT2[], DHT3[], DHT4[];
     int bytes, temp, oldindex, intermediateindex;
@@ -360,7 +390,7 @@ public class JpegEncoder extends Frame
     oldindex = index;
     DHT4[2] = (byte) (((index - 2) >> 8) & 0xFF);
     DHT4[3] = (byte) ((index - 2) & 0xFF);
-    WriteArray(DHT4, out);
+    writeArray(DHT4, out);
   }
   
   static void WriteMarker(byte[] data, BufferedOutputStream out) {
@@ -371,11 +401,9 @@ public class JpegEncoder extends Frame
     }
   }
   
-  static void WriteArray(byte[] data, BufferedOutputStream out) {
-    int length;
+  static void writeArray(byte[] data, BufferedOutputStream out) {
     try {
-      length = ((data[2] & 0xFF) << 8) + (data[3] & 0xFF) + 2;
-      out.write(data, 0, length);
+      out.write(data);
     } catch (IOException e) {
       Logger.error("IO Error", e);
     }
@@ -1181,7 +1209,7 @@ class Huffman
 class JpegInfo
 {
   String Comment;
-  private Image imageobj;
+  private Object imageobj;
   int imageHeight;
   int imageWidth;
   int BlockWidth[];
@@ -1204,10 +1232,9 @@ class JpegInfo
   int Al = 0;
   private int compWidth[], compHeight[];
   private int MaxHsampFactor;
-  private int MaxVsampFactor;
+  private int MaxVsampFactor;  
   
-  
-  public JpegInfo(Image image, String comment)
+  public JpegInfo(ApiPlatform apiPlatform, Object image, String comment)
   {
     Components = new float[NumberOfComponents][][];
     compWidth = new int[NumberOfComponents];
@@ -1215,10 +1242,10 @@ class JpegInfo
     BlockWidth = new int[NumberOfComponents];
     BlockHeight = new int[NumberOfComponents];
     imageobj = image;
-    imageWidth = image.getWidth(null);
-    imageHeight = image.getHeight(null);
+    imageWidth = apiPlatform.getImageWidth(image);
+    imageHeight = apiPlatform.getImageHeight(image);
     Comment = comment;
-    getYCCArray();
+    getYCCArray(apiPlatform);
   }
 
   /*
@@ -1226,7 +1253,7 @@ class JpegInfo
    * input image.
    */
   
-  private void getYCCArray()
+  private void getYCCArray(ApiPlatform apiPlatform)
   {
     int values[] = new int[imageWidth * imageHeight];
     int r, g, b, y, x;
@@ -1235,9 +1262,6 @@ class JpegInfo
     // those before going for more.  The time expense may be prohibitive.
     // However, for a situation where memory overhead is a concern, this may be
     // the only choice.
-    PixelGrabber grabber = 
-      new PixelGrabber(imageobj.getSource(), 0, 0, imageWidth, imageHeight, 
-          values, 0, imageWidth);
     MaxHsampFactor = 1;
     MaxVsampFactor = 1;
     for (y = 0; y < NumberOfComponents; y++) {
@@ -1260,18 +1284,7 @@ class JpegInfo
       }
       BlockHeight[y] = (int) Math.ceil(compHeight[y]/8.0);
     }
-    try
-    {
-      if(grabber.grabPixels() != true)
-      {
-        try
-        {
-          throw new AWTException("Grabber returned false: " + grabber.status());
-        }
-        catch (Exception e) {}
-      }
-    }
-    catch (InterruptedException e) {}
+    apiPlatform.grabPixels(imageobj, imageWidth, imageHeight, values);
     float Y[][] = new float[compHeight[0]][compWidth[0]];
     float Cr1[][] = new float[compHeight[0]][compWidth[0]];
     float Cb1[][] = new float[compHeight[0]][compWidth[0]];
