@@ -23,6 +23,7 @@
  */
 package org.jmol.util;
 
+
 import java.util.ArrayList;
 import java.util.List;
 
@@ -39,8 +40,6 @@ import org.jmol.script.Token;
 final public class Measure {
 
   public final static float radiansPerDegree = (float) (2 * Math.PI / 360);
-
-  
   
   public static float computeAngle(Tuple3f pointA, Tuple3f pointB, Tuple3f pointC, Vector3f vectorBA, Vector3f vectorBC, boolean asDegrees) {
     vectorBA.sub(pointA, pointB);
@@ -52,7 +51,7 @@ final public class Measure {
   public static float computeAngle(Tuple3f pointA, Tuple3f pointB, Tuple3f pointC, boolean asDegrees) {
     Vector3f vectorBA = new Vector3f();
     Vector3f vectorBC = new Vector3f();        
-    return Measure.computeAngle(pointA, pointB, pointC, vectorBA, vectorBC, asDegrees);
+    return computeAngle(pointA, pointB, pointC, vectorBA, vectorBC, asDegrees);
   }
 
   public static float computeTorsion(Tuple3f p1, Tuple3f p2, Tuple3f p3, Tuple3f p4, boolean asDegrees) {
@@ -173,7 +172,7 @@ final public class Measure {
           +"measure " + Escape.escape(a) + Escape.escape(pt_a_prime) + Escape.escape(pt_b_prime) + Escape.escape(b));
     */
     if (tokType == Token.draw)
-      return "draw ID " + id + " VECTOR " + Escape.escape(pt_a_prime)
+      return "draw ID \"" + id + "\" VECTOR " + Escape.escape(pt_a_prime)
           + " " + Escape.escape(n) + " color "
           + (theta < 0 ? "{255.0 200.0 0.0}" : "{255.0 0.0 128.0}");
     if (tokType == Token.measure)
@@ -196,16 +195,16 @@ final public class Measure {
     }
   }
 
-  public static Point4f getPlaneThroughPoints(Point3f pointA,
+  public static void getPlaneThroughPoints(Point3f pointA,
                                               Point3f pointB,
                                               Point3f pointC, Vector3f vNorm,
-                                              Vector3f vAB, Vector3f vAC) {
+                                              Vector3f vAB, Vector3f vAC, Point4f plane) {
     float w = getNormalThroughPoints(pointA, pointB, pointC, vNorm, vAB, vAC);
-    return new Point4f(vNorm.x, vNorm.y, vNorm.z, w);
+    plane.set(vNorm.x, vNorm.y, vNorm.z, w);
   }
   
-  public static Point4f getPlaneThroughPoint(Point3f pt, Vector3f normal) {
-    return new Point4f(normal.x, normal.y, normal.z, -normal.dot(new Vector3f(pt)));
+  public static void getPlaneThroughPoint(Point3f pt, Vector3f normal, Point4f plane) {
+    plane.set(normal.x, normal.y, normal.z, -normal.dot(new Vector3f(pt)));
   }
   
   public static float distanceToPlane(Point4f plane, Point3f pt) {
@@ -261,16 +260,16 @@ final public class Measure {
     // ax + by + cz + d = 0
     // so if a point is in the plane, then N dot X = -d
     vAB.set(pointA);
-    float d = -vAB.dot(vNorm);
-    return d;
+    return -vAB.dot(vNorm);
   }
 
-  public static void moveToPlane(Point4f plane, Point3f a) {
-    float dist = Measure.distanceToPlane(plane, a);
-    Vector3f vAC = new Vector3f(plane.x, plane.y, plane.z);
-    vAC.normalize();
-    vAC.scale(-dist);
-    a.add(vAC);
+  public static void getPlaneProjection(Point3f pt, Point4f plane, Point3f ptProj, Vector3f vNorm) {
+    float dist = distanceToPlane(plane, pt);
+    vNorm.set(plane.x, plane.y, plane.z);
+    vNorm.normalize();
+    vNorm.scale(-dist);
+    ptProj.set(pt);
+    ptProj.add(vNorm);
   }
 
   /**
@@ -296,18 +295,19 @@ final public class Measure {
     return !isReversed;
   }
 
-  public static void calcXYNormalToLine(Point3f pointA, Point3f pointB,
+  public static void getNormalToLine(Point3f pointA, Point3f pointB,
                                    Vector3f vNormNorm) {
     // vector in xy plane perpendicular to a line between two points RMH
-    Vector3f axis = new Vector3f(pointA);
-    axis.sub(pointB);
-    vNormNorm.cross(axis, JmolConstants.axisY);
+    vNormNorm.sub(pointA, pointB);
+    vNormNorm.cross(vNormNorm, JmolConstants.axisY);
     vNormNorm.normalize();
     if (Float.isNaN(vNormNorm.x))
       vNormNorm.set(1, 0, 0);
   }
   
-  public static void projectOntoAxis (Point3f point, Point3f axisA, Vector3f axisUnitVector, Vector3f vectorProjection) {
+  public static void projectOntoAxis(Point3f point, Point3f axisA,
+                                     Vector3f axisUnitVector,
+                                     Vector3f vectorProjection) {
     vectorProjection.sub(point, axisA);
     float projectedLength = vectorProjection.dot(axisUnitVector);
     point.set(axisUnitVector);
@@ -372,7 +372,7 @@ final public class Measure {
      * 
      */
 
-    Measure.calcAveragePointN(points, nPoints, axisA);
+    calcAveragePointN(points, nPoints, axisA);
 
     int nTries = 0;
     while (nTries++ < nTriesMax
@@ -437,19 +437,20 @@ final public class Measure {
     averagePoint.scale(1f / nPoints);
   }
 
-  public static Point3f[] getCenterAndPoints(List vPts) {
+  public static Point3f[] getCenterAndPoints(List<Point3f> vPts) {
     int n = vPts.size();
     Point3f[] pts = new Point3f[n + 1];
     pts[0] = new Point3f();
     if (n > 0) {
-      for (int i = 0; i < n; i++)
-        pts[0].add(pts[i + 1] = (Point3f) vPts.get(i));
+      for (int i = 0; i < n; i++) {
+        pts[0].add(pts[i + 1] = vPts.get(i));
+      }
       pts[0].scale(1f / n);
     }
     return pts;
   }
 
-  public static float getTransformMatrix4(List ptsA, List ptsB, Matrix4f m, Point3f centerA) {
+  public static float getTransformMatrix4(List<Point3f> ptsA, List<Point3f> ptsB, Matrix4f m, Point3f centerA) {
     Point3f[] cptsA = getCenterAndPoints(ptsA);
     Point3f[] cptsB = getCenterAndPoints(ptsB);
     float[] retStddev = new float[2];
@@ -574,10 +575,10 @@ final public class Measure {
     return (float) Math.sqrt((sum2 - sum * sum / n) / (n - 1));
   }
 
-  public static List transformPoints(List vPts, Matrix4f m4, Point3f center) {
-    List v = new ArrayList();
+  public static List<Point3f> transformPoints(List<Point3f> vPts, Matrix4f m4, Point3f center) {
+    List<Point3f> v = new ArrayList<Point3f>();
     for (int i = 0; i < vPts.size(); i++) {
-      Point3f pt = new Point3f((Point3f)vPts.get(i));
+      Point3f pt = new Point3f(vPts.get(i));
       pt.sub(center);
       m4.transform(pt, pt);
       pt.add(center);
@@ -586,4 +587,157 @@ final public class Measure {
     return v;
   }
 
+  public static boolean isInTetrahedron(Point3f pt, Point3f ptA, Point3f ptB,
+                                        Point3f ptC, Point3f ptD,
+                                        Point4f plane, Vector3f vTemp,
+                                        Vector3f vTemp2, Vector3f vTemp3, boolean fullyEnclosed) {
+    getPlaneThroughPoints(ptC, ptD, ptA, vTemp, vTemp2, vTemp3, plane);
+    boolean b = (distanceToPlane(plane, pt) >= 0);
+    getPlaneThroughPoints(ptA, ptD, ptB, vTemp, vTemp2, vTemp3, plane);
+    if (b != (distanceToPlane(plane, pt) >= 0))
+      return false;
+    getPlaneThroughPoints(ptB, ptD, ptC, vTemp, vTemp2, vTemp3, plane);
+    if (b != (distanceToPlane(plane, pt) >= 0))
+      return false;
+    getPlaneThroughPoints(ptA, ptB, ptC, vTemp, vTemp2, vTemp3, plane);
+    float d = distanceToPlane(plane, pt);
+    if (fullyEnclosed)
+      return (b == (d >= 0));
+    float d1 = distanceToPlane(plane, ptD);
+    return d1 * d <= 0 || Math.abs(d1) > Math.abs(d);
+  }
+
+
+  /**
+   * 
+   * @param plane1
+   * @param plane2
+   * @return       [ point, vector ] or []
+   */
+  public static List<Object> getIntersection(Point4f plane1, Point4f plane2) {
+    float a1 = plane1.x;
+    float b1 = plane1.y;
+    float c1 = plane1.z;
+    float d1 = plane1.w;
+    float a2 = plane2.x;
+    float b2 = plane2.y;
+    float c2 = plane2.z;
+    float d2 = plane2.w;
+    Vector3f norm1 = new Vector3f(a1, b1, c1);
+    Vector3f norm2 = new Vector3f(a2, b2, c2);
+    Vector3f nxn = new Vector3f();
+    nxn.cross(norm1, norm2);
+    float ax = Math.abs(nxn.x);
+    float ay = Math.abs(nxn.y);
+    float az = Math.abs(nxn.z);
+    float x, y, z, diff;
+    int type = (ax > ay ? (ax > az ? 1 : 3) : ay > az ? 2 : 3);
+    switch(type) {
+    case 1:
+      x = 0;
+      diff = (b1 * c2 - b2 * c1);
+      if (Math.abs(diff) < 0.01) return null;
+      y = (c1 * d2 - c2 * d1) / diff;
+      z = (b2 * d1 - d2 * b1) / diff;
+      break;
+    case 2:
+      diff = (a1 * c2 - a2 * c1);
+      if (Math.abs(diff) < 0.01) return null;
+      x = (c1 * d2 - c2 * d1) / diff;
+      y = 0;
+      z = (a2 * d1 - d2 * a1) / diff;
+      break;
+    case 3:
+    default:
+      diff = (a1 * b2 - a2 * b1);
+      if (Math.abs(diff) < 0.01) return null;
+      x = (b1 * d2 - b2 * d1) / diff;
+      y = (a2 * d1 - d2 * a1) / diff;
+      z = 0;
+    }
+    List<Object>list = new ArrayList<Object>();
+    list.add(new Point3f(x, y, z));
+    list.add(nxn);
+    return list;
+  }
+
+  /**
+   * 
+   * @param pt1  point on line
+   * @param v    unit vector of line
+   * @param plane 
+   * @param ptRet  point of intersection of line with plane
+   * @param tempNorm 
+   * @param vTemp 
+   * @return       ptRte
+   */
+  public static Point3f getIntersection(Point3f pt1, Vector3f v,
+                                               Point4f plane, Point3f ptRet, Vector3f tempNorm, Vector3f vTemp) {
+    getPlaneProjection(pt1, plane, ptRet, tempNorm);
+    tempNorm.set(plane.x, plane.y, plane.z);
+    tempNorm.normalize();
+    if (v == null)
+      v = new Vector3f(tempNorm);
+    float l_dot_n = v.dot(tempNorm);
+    if (Math.abs(l_dot_n) < 0.01) return null;
+    vTemp.set(ptRet);
+    vTemp.sub(pt1);
+    ptRet.scaleAdd(vTemp.dot(tempNorm) / l_dot_n, v, pt1);
+    return ptRet;
+  }
+/*
+  public static Point3f getTriangleIntersection(Point3f a1, Point3f a2,
+                                               Point3f a3, Point4f plane,
+                                               Point3f b1,
+                                               Point3f b2, Point3f b3,
+                                               Vector3f vNorm, Vector3f vTemp, 
+                                               Point3f ptRet, Point3f ptTemp, Vector3f vTemp2, Point4f pTemp, Vector3f vTemp3) {
+    
+    if (getTriangleIntersection(b1, b2, a1, a2, a3, vTemp, plane, vNorm, vTemp2, vTemp3, ptRet, ptTemp))
+      return ptRet;
+    if (getTriangleIntersection(b2, b3, a1, a2, a3, vTemp, plane, vNorm, vTemp2, vTemp3, ptRet, ptTemp))
+      return ptRet;
+    if (getTriangleIntersection(b3, b1, a1, a2, a3, vTemp, plane, vNorm, vTemp2, vTemp3, ptRet, ptTemp))
+      return ptRet;
+    return null;
+  }
+*/
+/*  
+  public static boolean getTriangleIntersection(Point3f b1, Point3f b2,
+                                                Point3f a1, Point3f a2,
+                                                Point3f a3, Vector3f vTemp,
+                                                Point4f plane, Vector3f vNorm,
+                                                Vector3f vTemp2, Vector3f vTemp3,
+                                                Point3f ptRet,
+                                                Point3f ptTemp) {
+    if (distanceToPlane(plane, b1) * distanceToPlane(plane, b2) >= 0)
+      return false;
+    vTemp.sub(b2, b1);
+    vTemp.normalize();
+    if (getIntersection(b1, vTemp, plane, ptRet, vNorm, vTemp2) != null) {
+      if (isInTriangle(ptRet, a1, a2, a3, vTemp, vTemp2, vTemp3))
+        return true;
+    }
+    return false;
+  }
+  private static boolean isInTriangle(Point3f p, Point3f a, Point3f b,
+                                      Point3f c, Vector3f v0, Vector3f v1,
+                                      Vector3f v2) {
+    // from http://www.blackpawn.com/texts/pointinpoly/default.html
+    // Compute barycentric coordinates
+    v0.sub(c, a);
+    v1.sub(b, a);
+    v2.sub(p, a);
+    float dot00 = v0.dot(v0);
+    float dot01 = v0.dot(v1);
+    float dot02 = v0.dot(v2);
+    float dot11 = v1.dot(v1);
+    float dot12 = v1.dot(v2);
+    float invDenom = 1 / (dot00 * dot11 - dot01 * dot01);
+    float u = (dot11 * dot02 - dot01 * dot12) * invDenom;
+    float v = (dot00 * dot12 - dot01 * dot02) * invDenom;
+    return (u > 0 && v > 0 && u + v < 1);
+  }
+*/
+
 }
