@@ -42,7 +42,7 @@ public class Parser {
    * thus, a data set can be incrementally added to in this way.
    * 
    *  @param str     the string to parse
-   *  @param bs      the atom positions to assign or null
+   *  @param bs      the atom positions to assign
    *  @param data    the (sparce) array to fill
    * @return  number of floats
    */
@@ -101,9 +101,16 @@ public class Parser {
     return max + 1;
   }
 
-  public static float[][] parseFloatArray2d(String str) {
+  private static String fixDataString(String str) {
     str = str.replace(';', str.indexOf('\n') < 0 ? '\n' : ' ');
     str = TextFormat.trim(str, "\n \t");
+    str = TextFormat.simpleReplace(str, "\n ", "\n");
+    str = TextFormat.simpleReplace(str, "\n\n", "\n");
+    return str;    
+  }
+  
+  public static float[][] parseFloatArray2d(String str) {
+    str = fixDataString(str);
     int[] lines = markLines(str, '\n');
     int nLines = lines.length;
     float[][] data = new float[nLines][];
@@ -115,8 +122,7 @@ public class Parser {
   }
 
   public static float[][][] parseFloatArray3d(String str) {
-    str = str.replace(';', str.indexOf('\n') < 0 ? '\n' : ' ');
-    str = TextFormat.trim(str, "\n \t");
+    str = fixDataString(str);
     int[] lines = markLines(str, '\n');
     int nLines = lines.length;
     String[] tokens = getTokens(str.substring(0, lines[0]));
