@@ -25,6 +25,7 @@
 
 package org.jmol.util;
 
+
 import java.text.DecimalFormat;
 import java.util.List;
 
@@ -90,6 +91,16 @@ public class TextFormat {
     return format(formatDecimal(value, precision), width, 0, alignLeft, zeroPad);
   }
 
+  /**
+   * 
+   * @param value
+   * @param width
+   * @param precision
+   * @param alignLeft
+   * @param zeroPad
+   * @param allowOverflow IGNORED
+   * @return formatted string
+   */
   public static String format(double value, int width, int precision,
                               boolean alignLeft, boolean zeroPad, boolean allowOverflow) {
     return format(formatDecimal((float)value, -1 - precision), width, 0, alignLeft, zeroPad);
@@ -453,6 +464,8 @@ public class TextFormat {
    */
   public static String replaceAllCharacters(String str, String strFrom,
                                             char chTo) {
+    if (str == null)
+      return null;
     for (int i = strFrom.length(); --i >= 0;)
       str = str.replace(strFrom.charAt(i), chTo);
     return str;
@@ -590,21 +603,22 @@ public class TextFormat {
     return true;
   }
 
-  public static String join(String[] s, char c, int i0) {
+  public static String join(Object[] s, char c, int i0) {
     if (s.length < i0)
       return null;
-    StringBuffer sb = new StringBuffer(s[i0++]);
+    StringBuffer sb = new StringBuffer();
+    sb.append(s[i0++]);
     for (int i = i0; i < s.length; i++)
       sb.append(c).append(s[i]);
     return sb.toString();
   }
 
-  public static String replaceQuotedStrings(String s, List list,
-                                            List newList) {
+  public static String replaceQuotedStrings(String s, List<String> list,
+                                            List<String> newList) {
     int n = list.size();
     for (int i = 0; i < n; i++) {
-      String name = (String) list.get(i);
-      String newName = (String) newList.get(i);
+      String name = list.get(i);
+      String newName = newList.get(i);
       if (!newName.equals(name))
         s = simpleReplace(s, "\"" + name + "\"", "\"" + newName
             + "\"");
@@ -612,12 +626,12 @@ public class TextFormat {
     return s;
   }
 
-  public static String replaceStrings(String s, List list,
-                                            List newList) {
+  public static String replaceStrings(String s, List<String> list,
+                                      List<String> newList) {
     int n = list.size();
     for (int i = 0; i < n; i++) {
-      String name = (String) list.get(i);
-      String newName = (String) newList.get(i);
+      String name = list.get(i);
+      String newName = newList.get(i);
       if (!newName.equals(name))
         s = simpleReplace(s, name, newName);
     }
