@@ -1,5 +1,6 @@
 package org.jmol.util;
 
+
 import java.util.BitSet;
 import java.util.List;
 
@@ -305,11 +306,11 @@ public class TriangleData {
    * @param v 
    * @param flags
    *          0 -- polygon int[]  1 -- edges only 2 -- triangles only 3 -- both
-   * @return List of Point3f[3] triangles and Point3f[2] edge lines
+   * @return Vector of Point3f[3] triangles and Point3f[2] edge lines
    */
 
 
-  public static List intersectPlane(Point4f plane, List v, int flags) {
+  public static List<Object> intersectPlane(Point4f plane, List<Object> v, int flags) {
     if (plane == null) {
       v.add(fullCubePolygon);
       return v;
