@@ -776,6 +776,17 @@ public class ActionManager {
         }
         return;
       }
+      if (isBound(action, ACTION_popupMenu)) {
+        char type = 'j';
+        if (viewer.getModelkitMode()) {
+          Map<String, Object> t = viewer.checkObjectClicked(x, y, Binding.getMouseAction(1,
+              Binding.LEFT));
+          type = (t != null && "bond".equals(t.get("type")) ? 'b' : viewer
+              .findNearestAtomIndex(x, y) >= 0 ? 'a' : 'm');
+        }
+        viewer.popupMenu(x, y, type);
+        return;
+      }
       if (dragSelectedMode) {
         haveSelection = true;
         if (isSelectAndDrag) {
@@ -789,17 +800,6 @@ public class ActionManager {
               Integer.MIN_VALUE, Integer.MIN_VALUE, null, false, false);
         return;
       }
-      if (isBound(action, ACTION_popupMenu)) {
-        char type = 'j';
-        if (viewer.getModelkitMode()) {
-          Map<String, Object> t = viewer.checkObjectClicked(x, y, Binding.getMouseAction(1,
-              Binding.LEFT));
-          type = (t != null && "bond".equals(t.get("type")) ? 'b' : viewer
-              .findNearestAtomIndex(x, y) >= 0 ? 'a' : 'm');
-        }
-        viewer.popupMenu(x, y, type);
-        return;
-      }
       if (viewer.useArcBall())
         viewer.rotateArcBall(x, y, 0);
       checkMotionRotateZoom(action, x, 0, 0, true);
@@ -950,19 +950,20 @@ public class ActionManager {
   }
 
   private void calcRectRubberBand() {
+    int factor = (viewer.isAntialiased() ? 2 : 1);
     if (current.x < pressed.x) {
-      rectRubber.x = current.x;
-      rectRubber.width = pressed.x - current.x;
+      rectRubber.x = current.x * factor;
+      rectRubber.width = (pressed.x - current.x) * factor;
     } else {
-      rectRubber.x = pressed.x;
-      rectRubber.width = current.x - pressed.x;
+      rectRubber.x = pressed.x * factor;
+      rectRubber.width = (current.x - pressed.x) * factor;
     }
     if (current.y < pressed.y) {
-      rectRubber.y = current.y;
-      rectRubber.height = pressed.y - current.y;
+      rectRubber.y = current.y * factor;
+      rectRubber.height = (pressed.y - current.y) * factor;
     } else {
-      rectRubber.y = pressed.y;
-      rectRubber.height = current.y - pressed.y;
+      rectRubber.y = pressed.y * factor;
+      rectRubber.height = (current.y - pressed.y) * factor;
     }
   }
 
