@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-03-24 04:45:53 +0100 (jeu., 24 mars 2011) $
- * $Revision: 15334 $
+ * $Date: 2011-09-28 23:55:43 +0200 (mer., 28 sept. 2011) $
+ * $Revision: 16175 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -23,11 +23,13 @@
  */
 package org.jmol.viewer;
 
+
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Enumeration;
 import java.util.Hashtable;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.Matrix3f;
 
@@ -56,10 +58,11 @@ public class PropertyManager {
     "fileName"        , "", "",
     "fileHeader"      , "", "",
     "fileContents"    , "<pathname>", "",
-    "fileContents"    , "", "",  
+    "fileContents"    , "", "",
     "animationInfo"   , "", "",
-    "modelInfo"       , atomExpression, "(*)",
-    "X -vibrationInfo", "", "",  //not implemented -- see modelInfo
+    "modelInfo"       , atomExpression, "{*}",
+    //"X -vibrationInfo", "", "",  //not implemented -- see modelInfo
+    "ligandInfo"      , atomExpression, "{*}",
     "shapeInfo"       , "", "",
     "measurementInfo" , "", "",
     
@@ -79,7 +82,7 @@ public class PropertyManager {
     "jmolStatus"      , "statusNameList", "",
     "jmolViewer"      , "", "",
     "messageQueue"    , "", "",
-    "auxiliaryInfo"   , atomExpression, "*",
+    "auxiliaryInfo"   , atomExpression, "{*}",
     
     "boundBoxInfo"    , "", "",  
     "dataInfo"        , "<data type>", "types",
@@ -88,9 +91,11 @@ public class PropertyManager {
     "menu"            , "<type>", "current",
     "minimizationInfo", "", "",
     "pointGroupInfo"  , atomExpression, "(visible)",
-    "fileInfo"         , "<type>", "",
-    "errorMessage", "", "",
-    "mouseInfo", "", "",
+    "fileInfo"        , "<type>", "",
+    "errorMessage"    , "", "",
+    "mouseInfo"       , "", "",
+    "isosurfaceInfo"  , "", "",
+    "consoleText"     , "", "",
   };
 
   private final static int PROP_APPLET_INFO = 0;
@@ -102,7 +107,8 @@ public class PropertyManager {
   private final static int PROP_ANIMATION_INFO = 5;
   private final static int PROP_MODEL_INFO = 6;
   //private final static int PROP_VIBRATION_INFO = 7; //not implemented -- see auxiliaryInfo
-  private final static int PROP_SHAPE_INFO = 8;
+  private final static int PROP_LIGAND_INFO = 7;
+   private final static int PROP_SHAPE_INFO = 8;
   private final static int PROP_MEASUREMENT_INFO = 9;
   
   private final static int PROP_CENTER_INFO = 10;
@@ -133,7 +139,9 @@ public class PropertyManager {
   private final static int PROP_FILE_INFO = 32;
   private final static int PROP_ERROR_MESSAGE = 33;
   private final static int PROP_MOUSE_INFO = 34;
-  private final static int PROP_COUNT = 35;
+  private final static int PROP_ISOSURFACE_INFO = 35;
+  private final static int PROP_CONSOLE_TEXT = 36;
+  private final static int PROP_COUNT = 37;
 
   //// static methods used by Eval and Viewer ////
   
@@ -197,6 +205,7 @@ public class PropertyManager {
     return extractProperty(getProperty(viewer, null, propertyName, propertyValue), args, 1);
   }
 
+  @SuppressWarnings("unchecked")
   public static Object extractProperty(Object property, ScriptVariable[] args, int ptr) {
     if (ptr >= args.length)
       return property;
@@ -205,8 +214,8 @@ public class PropertyManager {
     switch (arg.tok) {
     case Token.integer:
       pt = ScriptVariable.iValue(arg) - 1;  //one-based, as for array selectors
-      if (property instanceof List) {
-        List v = (List) property;
+      if (property instanceof List<?>) {
+        List<Object> v = (List<Object>) property;
         if (pt < 0)
           pt += v.size();
         if (pt >= 0 && pt < v.size())
@@ -246,7 +255,7 @@ public class PropertyManager {
         if (pt < 0)
           pt += ilist.length;
         if (pt >= 0 && pt < ilist.length)
-          return new Integer(ilist[pt]);
+          return Integer.valueOf(ilist[pt]);
         return "";
       }
       if (property instanceof float[][]) {
@@ -275,21 +284,21 @@ public class PropertyManager {
       }
       break;
     case Token.string:
-      String key = ScriptVariable.sValue(arg);
-      if (property instanceof Hashtable) {
-        Hashtable h = (Hashtable) property;
+      String key = arg.asString();
+      if (property instanceof Map<?,?>) {
+        Map<String, Object> h = (Map<String, Object>) property;
         if (key.equalsIgnoreCase("keys")) {
-          List keys = new ArrayList();
-          Enumeration e = h.keys();
-          while (e.hasMoreElements())
-            keys.add(e.nextElement()); 
+          List<Object> keys = new ArrayList<Object>();
+          Iterator<String> e = h.keySet().iterator();
+          while (e.hasNext())
+            keys.add(e.next()); 
           return extractProperty(keys, args, ptr);
         }
         if (!h.containsKey(key)) {
-          Enumeration e = h.keys();
+          Iterator<String> e = h.keySet().iterator();
           String newKey = "";
-          while (e.hasMoreElements())
-            if ((newKey = ((String) e.nextElement())).equalsIgnoreCase(key)) {
+          while (e.hasNext())
+            if ((newKey = e.next()).equalsIgnoreCase(key)) {
               key = newKey;
               break;
             }
@@ -298,6 +307,18 @@ public class PropertyManager {
           return extractProperty(h.get(key), args, ptr);
         return "";
       }
+      if (property instanceof List<?>) {
+        // drill down into vectors for this key
+        List<Object> v = (List<Object>) property;
+        List<Object> v2 = new ArrayList<Object>();
+        ptr--;
+        for (pt = 0; pt < v.size(); pt++) {
+          Object o = v.get(pt);
+          if (o instanceof Map<?,?>)
+            v2.add(extractProperty(o, args, ptr));
+        }
+        return v2;
+      }
       break;
     }
     return property;
@@ -318,7 +339,7 @@ public class PropertyManager {
   }
   
   private final static String[] readableTypes = {
-    "", "stateinfo", "extractmodel", "filecontents", "fileheader", "image", "menu"};
+    "", "stateinfo", "extractmodel", "filecontents", "fileheader", "image", "menu", "minimizationInfo"};
   
   private static boolean isReadableAsString(String infoType) {
     for (int i = readableTypes.length; --i >= 0; )
@@ -327,7 +348,8 @@ public class PropertyManager {
     return false;
   }
   
-  private static Object getPropertyAsObject(Viewer viewer, String infoType, Object paramInfo, String returnType) {
+  private static Object getPropertyAsObject(Viewer viewer, String infoType,
+                                            Object paramInfo, String returnType) {
     //Logger.debug("getPropertyAsObject(\"" + infoType+"\", \"" + paramInfo + "\")");
     if (infoType.equals("tokenList")) {
       return Token.getTokensLike((String) paramInfo);
@@ -356,8 +378,18 @@ public class PropertyManager {
       return viewer.getRotationCenter();
     case PROP_CHAIN_INFO:
       return viewer.getAllChainInfo(myParam);
+    case PROP_CONSOLE_TEXT:
+      return viewer.getProperty("DATA_API", "consoleText", null);
+    case PROP_DATA_INFO:
+      return viewer.getData(myParam.toString());
+    case PROP_ERROR_MESSAGE:
+      return viewer.getErrorMessageUntranslated();
+    case PROP_EVALUATE:
+      return ScriptEvaluator.evaluateExpression(viewer, myParam.toString());
     case PROP_EXTRACT_MODEL:
-      return viewer.getModelExtract(myParam, true, false, false);
+      return viewer.getModelExtract(myParam, true, "MOL");
+    case PROP_FILE_INFO:
+      return getFileInfo(viewer.getFileData(), myParam.toString());
     case PROP_FILENAME:
       return viewer.getFullPathName();
     case PROP_FILEHEADER:
@@ -367,44 +399,6 @@ public class PropertyManager {
       if (iHaveParameter)
         return viewer.getFileAsString(myParam.toString());
       return viewer.getCurrentFileAsString();
-    case PROP_JMOL_STATUS:
-      return viewer.getStatusChanged(myParam.toString()); 
-    case PROP_JMOL_VIEWER:
-      return viewer;
-    case PROP_MEASUREMENT_INFO:
-      return viewer.getMeasurementInfo();
-    case PROP_MENU:
-      return viewer.getMenu(myParam.toString());
-    case PROP_MESSAGE_QUEUE:
-      return viewer.getMessageQueue();
-    case PROP_MINIMIZATION_INFO:
-      return viewer.getMinimizationInfo();
-    case PROP_MODEL_INFO:
-      return viewer.getModelInfo(myParam);
-    case PROP_MOLECULE_INFO:
-      return viewer.getMoleculeInfo(myParam);
-    case PROP_ORIENTATION_INFO:
-      return viewer.getOrientationInfo();
-    case PROP_POLYMER_INFO:
-      return viewer.getAllPolymerInfo(myParam);
-    case PROP_SHAPE_INFO:
-      return viewer.getShapeInfo();
-    case PROP_STATE_INFO:
-      return viewer.getStateInfo(myParam.toString());
-    case PROP_POINTGROUP_INFO:
-      return viewer.getPointGroupInfo(myParam);
-    case PROP_FILE_INFO:
-      return getFileInfo(viewer.getFileData(), myParam.toString());
-    case PROP_ERROR_MESSAGE:
-      return viewer.getErrorMessageUntranslated();
-    case PROP_TRANSFORM_INFO:
-      return viewer.getMatrixRotate();
-    case PROP_DATA_INFO:
-      return viewer.getData(myParam.toString());
-    case PROP_MOUSE_INFO:
-      return viewer.getMouseInfo();
-    case PROP_EVALUATE:
-      return ScriptEvaluator.evaluateExpression(viewer, myParam.toString());
     case PROP_IMAGE:
       String params = myParam.toString();
       int height = -1, width = -1;
@@ -421,6 +415,40 @@ public class PropertyManager {
         height = width;        
       return viewer.getImageAs(returnType == null ? "JPEG" : "JPG64", -1, width, height,
           null, null);
+    case PROP_ISOSURFACE_INFO:
+      return viewer.getShapeProperty(JmolConstants.SHAPE_ISOSURFACE, "getInfo");
+    case PROP_JMOL_STATUS:
+      return viewer.getStatusChanged(myParam.toString());
+    case PROP_JMOL_VIEWER:
+      return viewer;
+    case PROP_LIGAND_INFO:
+      return viewer.getLigandInfo(myParam);
+    case PROP_MEASUREMENT_INFO:
+      return viewer.getMeasurementInfo();
+    case PROP_MENU:
+      return viewer.getMenu(myParam.toString());
+    case PROP_MESSAGE_QUEUE:
+      return viewer.getMessageQueue();
+    case PROP_MINIMIZATION_INFO:
+      return viewer.getMinimizationInfo();
+    case PROP_MODEL_INFO:
+      return viewer.getModelInfo(myParam);
+    case PROP_MOLECULE_INFO:
+      return viewer.getMoleculeInfo(myParam);
+    case PROP_MOUSE_INFO:
+      return viewer.getMouseInfo();
+    case PROP_ORIENTATION_INFO:
+      return viewer.getOrientationInfo();
+    case PROP_POINTGROUP_INFO:
+      return viewer.getPointGroupInfo(myParam);
+    case PROP_POLYMER_INFO:
+      return viewer.getAllPolymerInfo(myParam);
+    case PROP_SHAPE_INFO:
+      return viewer.getShapeInfo();
+    case PROP_STATE_INFO:
+      return viewer.getStateInfo(myParam.toString(), 0, 0);
+    case PROP_TRANSFORM_INFO:
+      return viewer.getMatrixRotate();
     }
     String[] data = new String[PROP_COUNT];
     for (int i = 0; i < PROP_COUNT; i++) {
@@ -441,17 +469,18 @@ public class PropertyManager {
     return info.toString();
   }
   
+  @SuppressWarnings("unchecked")
   static Object getFileInfo(Object objHeader, String type) {
-    Hashtable ht = new Hashtable();
+    Map ht = new Hashtable();
     if (objHeader == null)
       return ht;
     boolean haveType = (type != null && type.length() > 0);
-    if (objHeader instanceof Hashtable) {
-      return (haveType ? ((Hashtable) objHeader).get(type) : objHeader);
+    if (objHeader instanceof Map) {
+      return (haveType ? ((Map) objHeader).get(type) : objHeader);
     }
     String[] lines = TextFormat.split((String)objHeader, '\n');
     String keyLast = "";
-    StringBuffer sb = new StringBuffer();
+    StringBuilder sb = new StringBuilder();
     if (haveType)
       type = type.toUpperCase();
     String key = "";
@@ -468,7 +497,7 @@ public class PropertyManager {
           return sb.toString();
         if (sb != null && !haveType) {
           ht.put(keyLast, sb.toString());
-          sb = new StringBuffer();
+          sb = new StringBuilder();
         }
         keyLast = key;
       }
