@@ -555,7 +555,7 @@ public class StateManager {
         + ";debug;debugscript;defaultlatttice;defaults;diffusepercent;exportdrivers"
         + ";_filecaching;_filecache;fontcaching;fontscaling;language"
         + ";legacyautobonding"
-        + ";loglevel;logfile;loggestures;logcommands;measureStyleChime"
+        + ";loglevel;logfile;loggestures;logcommands;measurestylechime"
         + ";loadformat;loadligandformat;smilesurlformat;nihresolverformat;edsurlformat;edsurlcutoff;multiprocessor;navigationmode;"
         + ";perspectivedepth;phongexponent;perspectivemodel;preservestate;refreshing;repaintwaitms;rotationradius"
         + ";showaxes;showaxis1;showaxis2;showaxis3;showboundbox;showfrank;showunitcell"
@@ -731,6 +731,9 @@ public class StateManager {
       setParameterValue("spinFps", TransformManager.DEFAULT_SPIN_FPS);
       setParameterValue("stereoDegrees", EnumStereoMode.DEFAULT_STEREO_DEGREES); 
       setParameterValue("stateversion", 0); // only set by a saved state being recalled
+      setParameterValue("syncScript", viewer.getStatusManager().syncingScripts);
+      setParameterValue("syncMouse", viewer.getStatusManager().syncingMouse);
+      setParameterValue("syncStereo", viewer.getStatusManager().stereoSync);
       setParameterValue("windowCentered", true); // maintained by TransformManager
       setParameterValue("zoomEnabled", true);    // maintained by TransformManager
       setParameterValue("zShade", false);        // maintained by TransformManager
