@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Date: 2012-04-24 00:35:37 +0200 (mar., 24 avr. 2012) $
+ * $Revision: 17042 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -28,18 +28,19 @@ import org.jmol.script.ScriptEvaluator;
 import org.jmol.util.Logger;
 import org.jmol.util.TextFormat;
 
-import java.util.Enumeration;
+import java.util.ArrayList;
+import java.util.BitSet;
+import java.util.Collections;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
-
-import javax.swing.SwingUtilities;
+import java.util.Map;
 
 import org.jmol.api.Interface;
 import org.jmol.api.JmolAppConsoleInterface;
 import org.jmol.api.JmolCallbackListener;
 import org.jmol.api.JmolDialogInterface;
 import org.jmol.api.JmolStatusListener;
+import org.jmol.constant.EnumCallback;
 
 /**
  * 
@@ -179,8 +180,8 @@ class StatusManager {
    * 
    */
   
-  private Hashtable messageQueue = new Hashtable();
-  Hashtable getMessageQueue() {
+  private Map<String, List<List<Object>>> messageQueue = new Hashtable<String, List<List<Object>>>();
+  Map<String, List<List<Object>>> getMessageQueue() {
     return messageQueue;
   }
   
@@ -198,22 +199,22 @@ class StatusManager {
       int intInfo, Object statusInfo, boolean isReplace) {
     if (!recordStatus(statusName))
       return;
-    List msgRecord = new ArrayList();
-    msgRecord.add(new Integer(++statusPtr));
+    List<Object> msgRecord = new ArrayList<Object>();
+    msgRecord.add(Integer.valueOf(++statusPtr));
     msgRecord.add(statusName);
-    msgRecord.add(new Integer(intInfo));
+    msgRecord.add(Integer.valueOf(intInfo));
     msgRecord.add(statusInfo);
-    List statusRecordSet = (isReplace ? null : (List)messageQueue.get(statusName));
+    List<List<Object>> statusRecordSet = (isReplace ? null : messageQueue.get(statusName));
     if (statusRecordSet == null)
-      messageQueue.put(statusName, statusRecordSet = new ArrayList());
-    if (statusRecordSet.size() == MAXIMUM_QUEUE_LENGTH)
+      messageQueue.put(statusName, statusRecordSet = new ArrayList<List<Object>>());
+    else if (statusRecordSet.size() == MAXIMUM_QUEUE_LENGTH)
       statusRecordSet.remove(0);    
     statusRecordSet.add(msgRecord);
   }
   
-  synchronized List getStatusChanged(String newStatusList) {
+  synchronized List<List<List<Object>>> getStatusChanged(String newStatusList) {
     /*
-     * returns a List of statusRecordSets, one per status type,
+     * returns a Vector of statusRecordSets, one per status type,
      * where each statusRecordSet is itself a vector of vectors:
      * [int statusPtr,String statusName,int intInfo, String statusInfo]
      * 
@@ -241,14 +242,10 @@ class StatusManager {
           Logger.debug("StatusManager messageQueue = " + statusList);
       }
     }
-    List msgList = new ArrayList();
-    if (getList) {
-      Enumeration e = messageQueue.keys();
-      while (e.hasMoreElements())
-        msgList.add(messageQueue.remove(e.nextElement()));
-    } else {
-      messageQueue.clear();
-    }
+    List<List<List<Object>>> msgList = (getList ? Collections.list(Collections
+        .enumeration(messageQueue.values()))
+        : new ArrayList<List<List<Object>>>());
+    messageQueue.clear();
     statusPtr = 0;
     return msgList;
   }
@@ -262,11 +259,11 @@ class StatusManager {
     this.jmolCallbackListener = jmolCallbackListener;
   }
   
-  private String[] jmolScriptCallbacks = new String[JmolConstants.CALLBACK_COUNT];
-  
-  private String jmolScriptCallback(int iCallback) {
+  Map<EnumCallback, String> jmolScriptCallbacks = new Hashtable<EnumCallback, String>();
+
+  private String jmolScriptCallback(EnumCallback callback) {
     //System.out.println("callback " + iCallback + " + " + JmolConstants.getCallbackName(iCallback));
-    String s = jmolScriptCallbacks[iCallback];
+    String s = jmolScriptCallbacks.get(callback);
     if (s != null)
       viewer.evalStringQuiet(s, true, false);
     return s;
@@ -275,63 +272,80 @@ class StatusManager {
   synchronized void setCallbackFunction(String callbackType,
                                         String callbackFunction) {
     // menu and language setting also use this route
-    int iCallback = JmolConstants.getCallbackId(callbackType);
-    if (iCallback >= 0) {
+    EnumCallback callback = EnumCallback.getCallback(callbackType);
+    if (callback != null) {
       int pt = (callbackFunction == null ? 0
           : callbackFunction.length() > 7
               && callbackFunction.toLowerCase().indexOf("script:") == 0 ? 7
               : callbackFunction.length() > 11
                   && callbackFunction.toLowerCase().indexOf("jmolscript:") == 0 ? 11
                   : 0);
-      jmolScriptCallbacks[iCallback] = (pt == 0 ? null : callbackFunction
-          .substring(pt).trim());
+      if (pt == 0)
+        jmolScriptCallbacks.remove(callback);
+      else
+        jmolScriptCallbacks.put(callback, callbackFunction.substring(pt).trim());
     }
     if (jmolCallbackListener != null)
       jmolCallbackListener.setCallbackFunction(callbackType, callbackFunction);
   }
   
-  private boolean notifyEnabled(int type) {
+  private boolean notifyEnabled(EnumCallback type) {
     return jmolCallbackListener != null && jmolCallbackListener.notifyEnabled(type);
   }
 
+  synchronized void setStatusAppletReady(String htmlName, boolean isReady) {
+    String sJmol = (isReady ? jmolScriptCallback(EnumCallback.APPLETREADY) : null);
+    if (notifyEnabled(EnumCallback.APPLETREADY))
+      jmolCallbackListener.notifyCallback(EnumCallback.APPLETREADY,
+          new Object[] { sJmol, htmlName, Boolean.valueOf(isReady), null });
+  }
+
+  synchronized void setStatusAtomMoved(BitSet bsMoved) {
+    String sJmol = jmolScriptCallback(EnumCallback.ATOMMOVED);
+    setStatusChanged("atomMoved", -1, bsMoved, false);
+    if (notifyEnabled(EnumCallback.ATOMMOVED))
+      jmolCallbackListener.notifyCallback(EnumCallback.ATOMMOVED,
+          new Object[] { sJmol, bsMoved });
+  }
+
   synchronized void setStatusAtomPicked(int atomIndex, String strInfo) {
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_PICK);
+    String sJmol = jmolScriptCallback(EnumCallback.PICK);
     Logger.info("setStatusAtomPicked(" + atomIndex + "," + strInfo + ")");
     setStatusChanged("atomPicked", atomIndex, strInfo, false);
-    if (notifyEnabled(JmolConstants.CALLBACK_PICK))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_PICK,
-          new Object[] { sJmol, strInfo, new Integer(atomIndex) });
+    if (notifyEnabled(EnumCallback.PICK))
+      jmolCallbackListener.notifyCallback(EnumCallback.PICK,
+          new Object[] { sJmol, strInfo, Integer.valueOf(atomIndex) });
   }
 
   synchronized int setStatusClicked(int x, int y, int action, int clickCount, int mode) {
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_CLICK);
-    if (!notifyEnabled(JmolConstants.CALLBACK_CLICK))
+    String sJmol = jmolScriptCallback(EnumCallback.CLICK);
+    if (!notifyEnabled(EnumCallback.CLICK))
       return action;
     // allows modification of action
     int[] m = new int[] { action, mode };
-    jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_CLICK,
-        new Object[] { sJmol, new Integer(x), new Integer(y), new Integer(action), new Integer(clickCount), m });
+    jmolCallbackListener.notifyCallback(EnumCallback.CLICK,
+        new Object[] { sJmol, Integer.valueOf(x), Integer.valueOf(y), Integer.valueOf(action), Integer.valueOf(clickCount), m });
     return m[0];
   }
 
   synchronized void setStatusResized(int width, int height){
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_RESIZE);
-    if (notifyEnabled(JmolConstants.CALLBACK_RESIZE))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_RESIZE,
-          new Object[] { sJmol, new Integer(width), new Integer(height) }); 
+    String sJmol = jmolScriptCallback(EnumCallback.RESIZE);
+    if (notifyEnabled(EnumCallback.RESIZE))
+      jmolCallbackListener.notifyCallback(EnumCallback.RESIZE,
+          new Object[] { sJmol, Integer.valueOf(width), Integer.valueOf(height) }); 
   }
 
   synchronized void setStatusAtomHovered(int iatom, String strInfo) {
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_HOVER);
-    if (notifyEnabled(JmolConstants.CALLBACK_HOVER))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_HOVER, 
-          new Object[] {sJmol, strInfo, new Integer(iatom) });
+    String sJmol = jmolScriptCallback(EnumCallback.HOVER);
+    if (notifyEnabled(EnumCallback.HOVER))
+      jmolCallbackListener.notifyCallback(EnumCallback.HOVER, 
+          new Object[] {sJmol, strInfo, Integer.valueOf(iatom) });
   }
   
   synchronized void setFileLoadStatus(String fullPathName, String fileName,
                                         String modelName, String errorMsg,
                                         int ptLoad, boolean doCallback) {
-    if (fullPathName == null && fileName != null && fileName.equals("resetUndo")) {
+    if (fullPathName == null && "resetUndo".equals(fileName)) {
       JmolAppConsoleInterface appConsole = (JmolAppConsoleInterface) viewer.getProperty("DATA_API", "getAppConsole", null);
       if (appConsole != null)
         appConsole.zap();
@@ -340,12 +354,12 @@ class StatusManager {
     setStatusChanged("fileLoaded", ptLoad, fullPathName, false);
     if (errorMsg != null)
       setStatusChanged("fileLoadError", ptLoad, errorMsg, false);
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_LOADSTRUCT);
-    if (doCallback && notifyEnabled(JmolConstants.CALLBACK_LOADSTRUCT)) {
+    String sJmol = jmolScriptCallback(EnumCallback.LOADSTRUCT);
+    if (doCallback && notifyEnabled(EnumCallback.LOADSTRUCT)) {
       String name = (String) viewer.getParameter("_smilesString");
       if (name.length() != 0)
         fileName = name;
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_LOADSTRUCT,
+      jmolCallbackListener.notifyCallback(EnumCallback.LOADSTRUCT,
           new Object[] { sJmol, 
               fullPathName, fileName, modelName, errorMsg, Integer.valueOf(ptLoad)
           , viewer.getParameter("_modelNumber"), viewer.getModelNumberDotted(viewer.getModelCount() - 1) });
@@ -364,9 +378,9 @@ class StatusManager {
     setStatusChanged("frameChanged", frameNo, (f >= 0 ? viewer
         .getModelNumberDotted(f) : ""), false);
     String entryName = viewer.getMenuName(f);
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_ANIMFRAME);
-    if (notifyEnabled(JmolConstants.CALLBACK_ANIMFRAME)) {
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_ANIMFRAME,
+    String sJmol = jmolScriptCallback(EnumCallback.ANIMFRAME);
+    if (notifyEnabled(EnumCallback.ANIMFRAME)) {
+      jmolCallbackListener.notifyCallback(EnumCallback.ANIMFRAME,
           new Object[] { sJmol,
               new int[] { frameNo, fileNo, modelNo, firstNo, lastNo }, entryName });
     }
@@ -382,10 +396,10 @@ class StatusManager {
     if (strEcho == null)
       return;
     setStatusChanged("scriptEcho", 0, strEcho, false);
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_ECHO);
-    if (notifyEnabled(JmolConstants.CALLBACK_ECHO))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_ECHO,
-          new Object[] { sJmol, strEcho, new Integer(isScriptQueued ? 1 : 0) });
+    String sJmol = jmolScriptCallback(EnumCallback.ECHO);
+    if (notifyEnabled(EnumCallback.ECHO))
+      jmolCallbackListener.notifyCallback(EnumCallback.ECHO,
+          new Object[] { sJmol, strEcho, Integer.valueOf(isScriptQueued ? 1 : 0) });
   }
 
   synchronized void setStatusMeasuring(String status, int intInfo, String strMeasure, float value) {
@@ -393,30 +407,30 @@ class StatusManager {
     String sJmol = null;
     if(status.equals("measureCompleted")) { 
       Logger.info("measurement["+intInfo+"] = "+strMeasure);
-      sJmol = jmolScriptCallback(JmolConstants.CALLBACK_MEASURE);
+      sJmol = jmolScriptCallback(EnumCallback.MEASURE);
     } else if (status.equals("measurePicked")) {
         setStatusChanged("measurePicked", intInfo, strMeasure, false);
         Logger.info("measurePicked " + intInfo + " " + strMeasure);
     }
-    if (notifyEnabled(JmolConstants.CALLBACK_MEASURE))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_MEASURE, 
+    if (notifyEnabled(EnumCallback.MEASURE))
+      jmolCallbackListener.notifyCallback(EnumCallback.MEASURE, 
           new Object[] { sJmol, strMeasure,  Integer.valueOf(intInfo), status , Float.valueOf(value)});
   }
   
   synchronized void notifyError(String errType, String errMsg,
                                 String errMsgUntranslated) {
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_ERROR);
-    if (notifyEnabled(JmolConstants.CALLBACK_ERROR))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_ERROR,
+    String sJmol = jmolScriptCallback(EnumCallback.ERROR);
+    if (notifyEnabled(EnumCallback.ERROR))
+      jmolCallbackListener.notifyCallback(EnumCallback.ERROR,
           new Object[] { sJmol, errType, errMsg, viewer.getShapeErrorState(),
               errMsgUntranslated });
   }
   
   synchronized void notifyMinimizationStatus(String minStatus, Integer minSteps, 
                                              Float minEnergy, Float minEnergyDiff) {
-    String sJmol = jmolScriptCallback(JmolConstants.CALLBACK_MINIMIZATION);
-    if (notifyEnabled(JmolConstants.CALLBACK_MINIMIZATION))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_MINIMIZATION,
+    String sJmol = jmolScriptCallback(EnumCallback.MINIMIZATION);
+    if (notifyEnabled(EnumCallback.MINIMIZATION))
+      jmolCallbackListener.notifyCallback(EnumCallback.MINIMIZATION,
           new Object[] { sJmol, minStatus, minSteps, minEnergy, minEnergyDiff });
   }
   
@@ -432,7 +446,7 @@ class StatusManager {
     } else if (strStatus == null) {
       return;
     }
-    String sJmol = (msWalltime == 0 ? jmolScriptCallback(JmolConstants.CALLBACK_SCRIPT)
+    String sJmol = (msWalltime == 0 ? jmolScriptCallback(EnumCallback.SCRIPT)
         : null);
     boolean isScriptCompletion = (strStatus == ScriptEvaluator.SCRIPT_COMPLETED);
 
@@ -450,18 +464,18 @@ class StatusManager {
     if (isScriptCompletion && viewer.getMessageStyleChime()
         && viewer.getDebugScript()) {
       data = new Object[] { null, "script <exiting>", statusMessage,
-          new Integer(-1), strErrorMessageUntranslated };
-      if (notifyEnabled(JmolConstants.CALLBACK_SCRIPT))
+          Integer.valueOf(-1), strErrorMessageUntranslated };
+      if (notifyEnabled(EnumCallback.SCRIPT))
         jmolCallbackListener
-            .notifyCallback(JmolConstants.CALLBACK_SCRIPT, data);
+            .notifyCallback(EnumCallback.SCRIPT, data);
       processScript(data);
       strStatus = "Jmol script completed.";
     }
     data = new Object[] { sJmol, strStatus, statusMessage,
-        new Integer(isScriptCompletion ? -1 : msWalltime),
+        Integer.valueOf(isScriptCompletion ? -1 : msWalltime),
         strErrorMessageUntranslated };
-    if (notifyEnabled(JmolConstants.CALLBACK_SCRIPT))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_SCRIPT, data);
+    if (notifyEnabled(EnumCallback.SCRIPT))
+      jmolCallbackListener.notifyCallback(EnumCallback.SCRIPT, data);
     processScript(data);
   }
 
@@ -506,9 +520,9 @@ class StatusManager {
   synchronized void setSync(String mouseCommand) {
     if (syncingMouse) {
       if (mouseCommand != null)
-        syncSend(mouseCommand, "*");
+        syncSend(mouseCommand, "*", 0);
     } else if (!syncingScripts)
-      syncSend("!" + viewer.getMoveToText(minSyncRepeatMs / 1000f), "*");
+      syncSend("!" + viewer.getMoveToText(minSyncRepeatMs / 1000f), "*", 0);
   }
 
   boolean drivingSync = false;
@@ -532,7 +546,7 @@ class StatusManager {
     //  5 stereo
     //System.out.println(viewer.getHtmlName() +" setting mode=" + syncMode);
     if (stereoSync && syncMode != SYNC_ENABLE) {
-      syncSend(Viewer.SYNC_NO_GRAPHICS_MESSAGE, "*");
+      syncSend(Viewer.SYNC_NO_GRAPHICS_MESSAGE, "*", 0);
       stereoSync = false;
     }
     switch (syncMode) {
@@ -568,11 +582,11 @@ class StatusManager {
     }
   }
 
-  void syncSend(String script, String appletName) {
+  void syncSend(String script, String appletName, int port) {
     // no jmolscript option for syncSend
-    if (notifyEnabled(JmolConstants.CALLBACK_SYNC))
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_SYNC,
-          new Object[] { null, script, appletName });
+    if (port != 0 || notifyEnabled(EnumCallback.SYNC))
+      jmolCallbackListener.notifyCallback(EnumCallback.SYNC,
+          new Object[] { null, script, appletName, Integer.valueOf(port) });
   }
   
   int getSyncMode() {
@@ -589,7 +603,7 @@ class StatusManager {
       viewer.appConsole.sendConsoleMessage(null);
     }
     if (jmolStatusListener != null)
-      jmolCallbackListener.notifyCallback(JmolConstants.CALLBACK_MESSAGE, null);
+      jmolCallbackListener.notifyCallback(EnumCallback.MESSAGE, null);
   }
 
   float[][] functionXY(String functionName, int nX, int nY) {
@@ -622,7 +636,7 @@ class StatusManager {
       jmolStatusListener.createImage(fileNameOrError, type, text_or_bytes, quality));
   }
 
-  Hashtable getRegistryInfo() {
+  Map<String, Object> getRegistryInfo() {
     /* 
 
      //note that the following JavaScript retrieves the registry:
@@ -641,62 +655,26 @@ class StatusManager {
     return (jmolStatusListener == null ? null : jmolStatusListener.getRegistryInfo());
   }
 
-  String inputFileName;
-  String outputFileName;
-  String dialogType;
-  
-  final protected static String[] imageChoices = { "JPEG", "PNG", "GIF", "PPM" };
-  final protected static String[] imageExtensions = { "jpg", "png", "gif", "ppm" };
-
-  static JmolDialogInterface newDialog(boolean forceNewTranslation) {
-    JmolDialogInterface sd = (JmolDialogInterface) Interface
-        .getOptionInterface("export.dialog.Dialog");
-    sd.setupUI(forceNewTranslation);
-    return sd;
-  }
-
-  int qualityJPG = -1;
-  int qualityPNG = -1;
-  String imageType;
-  int imageQuality;
+  private int qualityJPG = -1;
+  private int qualityPNG = -1;
+  private String imageType;
 
   String dialogAsk(String type, String fileName) {
-    inputFileName = fileName;
-    dialogType = type;
-    //System.out.println("Jvm12 thread: " + Thread.currentThread().getName());
-    try {
-      SwingUtilities.invokeAndWait(new Runnable() {
-        public void run() {
-          if (dialogType.equals("load")) {
-            outputFileName = newDialog(false).getOpenFileNameFromDialog(
-                viewer.getAppletContext(), viewer, inputFileName, null, null, false);
-            return;
-          }
-          JmolDialogInterface sd = newDialog(false);
-          if (dialogType.equals("save")) {
-            outputFileName = sd.getSaveFileNameFromDialog(viewer,
-                inputFileName, null);
-            return;
-          }
-          if (dialogType.startsWith("saveImage")) {
-            outputFileName = sd.getImageFileNameFromDialog(viewer,
-                inputFileName, imageType, imageChoices, imageExtensions,
-                qualityJPG, qualityPNG);
-            qualityJPG = sd.getQuality("JPG");
-            qualityPNG = sd.getQuality("PNG");
-            String sType = sd.getType();
-            if (sType != null)
-              imageType = sType;
-            int iQuality = sd.getQuality(sType);
-            if (iQuality >= 0)
-              imageQuality = iQuality;
-            return;
-          }
-          outputFileName = null;
-        }
-      });
-    } catch (Exception e) {
-      Logger.error(e.getMessage());
+    boolean isImage = (type.startsWith("saveImage"));
+    JmolDialogInterface sd = (JmolDialogInterface) Interface
+    .getOptionInterface("export.dialog.Dialog");
+    if (sd == null)
+      return null;
+    sd.setupUI(false);
+    if (isImage) 
+      sd.setImageInfo(qualityJPG, qualityPNG, imageType);
+    String outputFileName = sd.getFileNameFromDialog(viewer, type, fileName);    
+    if (isImage && outputFileName != null) {
+      qualityJPG = sd.getQuality("JPG");
+      qualityPNG = sd.getQuality("PNG");
+      String sType = sd.getType();
+      if (sType != null)
+        imageType = sType;
     }
     return outputFileName;
   }
