@@ -32,6 +32,7 @@ import javax.vecmath.Matrix4f;
 import javax.vecmath.Matrix3f;
 import javax.vecmath.AxisAngle4f;
 
+import org.jmol.constant.EnumStereoMode;
 import org.jmol.g3d.Text3D;
 import org.jmol.script.Token;
 import org.jmol.util.Escape;
@@ -43,6 +44,7 @@ import java.util.BitSet;
 import java.util.Date;
 import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 
 abstract class TransformManager {
 
@@ -158,6 +160,7 @@ abstract class TransformManager {
     navigating = false;
     slabPlane = null;
     depthPlane = null;
+    zSlabPoint = null;
     resetNavigationPoint(true);
   }
 
@@ -186,10 +189,9 @@ abstract class TransformManager {
     
     
     StateManager.appendCmd(commands, getMoveToText(0, false));
-    if (stereoMode != JmolConstants.STEREO_NONE)
+    if (stereoMode != EnumStereoMode.NONE)
       StateManager.appendCmd(commands, "stereo "
-          + (stereoColors == null ? 
-              JmolConstants.getStereoModeName(stereoMode)
+          + (stereoColors == null ? stereoMode.getName()
               : Escape.escapeColor(stereoColors[0])
                   + " " + Escape.escapeColor(stereoColors[1]))
           + " " + stereoDegrees);
@@ -198,8 +200,16 @@ abstract class TransformManager {
     commands.append("  slab ").append(slabPercentSetting).append(";depth ")
         .append(depthPercentSetting).append(
             slabEnabled && mode != MODE_NAVIGATION ? ";slab on" : "").append(";\n");
+    commands.append("  set slabRange ").append(slabRange).append(";\n");
     if (zShadeEnabled)
       commands.append("  set zShade;\n");
+    try {
+    if (zSlabPoint != null)
+      commands.append("  set zSlab ").append(Escape.escape(zSlabPoint)).append(
+      ";\n");
+    } catch (Exception e) {
+      // don't care
+    }
     if (slabPlane != null)
       commands.append("  slab plane ").append(Escape.escape(slabPlane)).append(
           ";\n");
@@ -219,6 +229,10 @@ abstract class TransformManager {
     return commands.toString();
   }
 
+  /**
+   * @param isAll  
+   * @return spin state
+   */
   String getSpinState(boolean isAll) {
     String s = "  set spinX " + (int) spinX + "; set spinY " + (int) spinY
         + "; set spinZ " + (int) spinZ + "; set spinFps " + (int) spinFps + ";";
@@ -291,12 +305,6 @@ abstract class TransformManager {
   final static int MAXIMUM_ZOOM_PERCENTAGE = 200000;
   final static int MAXIMUM_ZOOM_PERSPECTIVE_DEPTH = 10000;
 
-  private boolean rotateMolecule;
-
-  void setRotateMolecule(boolean TF) {
-    rotateMolecule = TF;
-  }
-
   private void setFixedRotationCenter(Point3f center) {
     if (center == null)
       return;
@@ -387,32 +395,31 @@ abstract class TransformManager {
     matrixRotate.rotZ(angleRadians);
   }
 
-  private void applyRotation(Matrix3f mNew, boolean isInternal, BitSet bsAtoms) {
+  private void applyRotation(Matrix3f mNew, boolean isInternal, BitSet bsAtoms, Vector3f translation) {
     if (bsAtoms == null) {
       matrixRotate.mul(mNew, matrixRotate);
       return;
     }
-    viewer.rotateAtoms(mNew, matrixRotate, rotateMolecule,
+    viewer.moveAtoms(mNew, matrixRotate, translation,
         internalRotationCenter, isInternal, bsAtoms);
-    if (internalTranslation != null) {
-      viewer.setAtomCoordRelative(internalTranslation, bsAtoms);
-      internalRotationCenter.add(internalTranslation);
+    if (translation != null) {
+      internalRotationCenter.add(translation);
     }
   }
 
   synchronized void rotateXRadians(float angleRadians, BitSet bsAtoms) {
     matrixTemp3.rotX(angleRadians);
-    applyRotation(matrixTemp3, false, bsAtoms);
+    applyRotation(matrixTemp3, false, bsAtoms, null);
   }
 
   synchronized void rotateYRadians(float angleRadians, BitSet bsAtoms) {
     matrixTemp3.rotY(angleRadians);
-    applyRotation(matrixTemp3, false, bsAtoms);
+    applyRotation(matrixTemp3, false, bsAtoms, null);
   }
 
   synchronized void rotateZRadians(float angleRadians) {
     matrixTemp3.rotZ(angleRadians);
-    applyRotation(matrixTemp3, false, null);
+    applyRotation(matrixTemp3, false, null, null);
   }
 
   protected void rotateAxisAngle(Vector3f rotAxis, float radians) {
@@ -423,7 +430,7 @@ abstract class TransformManager {
   synchronized void rotateAxisAngle(AxisAngle4f axisAngle, BitSet bsAtoms) {
     //matrixTemp3.setIdentity();
     matrixTemp3.set(axisAngle);
-    applyRotation(matrixTemp3, false, bsAtoms);
+    applyRotation(matrixTemp3, false, bsAtoms, null);
   }
 
   /*
@@ -482,7 +489,7 @@ abstract class TransformManager {
                                  float degreesPerSecond, float endDegrees,
                                  boolean isClockwise, boolean isSpin,
                                  BitSet bsAtoms, boolean isGesture,
-                                 Vector3f translation, List finalPoints) {
+                                 Vector3f translation, List<Point3f> finalPoints) {
 
     // *THE* Viewer INTERNAL frame rotation entry point
 
@@ -547,7 +554,7 @@ abstract class TransformManager {
     // NOW apply that rotation  
 
     matrixTemp3.set(axisangleT);
-    applyRotation(matrixTemp3, true, bsAtoms);
+    applyRotation(matrixTemp3, true, bsAtoms, internalTranslation);
     if (bsAtoms == null)
       getNewFixedRotationCenter();
   }
@@ -664,11 +671,11 @@ abstract class TransformManager {
   }
 
   float getTranslationXPercent() {
-    return (fixedTranslation.x - width / 2) * 100 / width;
+    return (fixedTranslation.x - width / 2f) * 100 / width;
   }
 
   float getTranslationYPercent() {
-    return (fixedTranslation.y - height / 2) * 100 / height;
+    return (fixedTranslation.y - height / 2f) * 100 / height;
   }
 
   float getTranslationZPercent() {
@@ -703,8 +710,8 @@ abstract class TransformManager {
     }
   }
 
-  Hashtable getOrientationInfo() {
-    Hashtable info = new Hashtable();
+  Map<String, Object> getOrientationInfo() {
+    Map<String, Object> info = new Hashtable<String, Object>();
     info.put("moveTo", getMoveToText(1, false));
     info.put("center", "center " + getCenterText());
     info.put("centerPt", fixedRotationCenter);
@@ -865,12 +872,25 @@ abstract class TransformManager {
 
   int slabPercentSetting;
   int depthPercentSetting;
-
+  int zSlabPercentSetting = 50; // new default for 12.3.6 and 12.2.6
+  int zDepthPercentSetting = 0;
+  Point3f zSlabPoint;
+  
+  void setZslabPoint(Point3f pt) {
+    zSlabPoint = (pt == null ? null : new Point3f(pt));
+  }
+  
   int slabValue;
   int depthValue;
   int zSlabValue;
   int zDepthValue;
-  
+
+  float slabRange = 0f;
+
+  public void setSlabRange(float value) {
+    slabRange = value;    
+  }
+
   void setSlabEnabled(boolean slabEnabled) {
     this.slabEnabled = slabEnabled;
     viewer.getGlobalSettings().setParameterValue("slabEnabled", slabEnabled);
@@ -886,6 +906,9 @@ abstract class TransformManager {
     viewer.getGlobalSettings().setParameterValue("zoomEnabled", zoomEnabled);
   }
 
+  Point4f slabPlane = null;
+  Point4f depthPlane = null;
+
   void slabReset() {
     slabToPercent(100);
     depthToPercent(0);
@@ -902,8 +925,15 @@ abstract class TransformManager {
   void slabByPercentagePoints(int percentage) {
     slabPlane = null;
     slabPercentSetting += percentage;
+    slabDepthChanged();
     if (depthPercentSetting >= slabPercentSetting)
       depthPercentSetting = slabPercentSetting - 1;
+//    System.out.println("transformManager slab/depthpercentSetting: " + slabPercentSetting + " " + depthPercentSetting);
+  }
+
+  private void slabDepthChanged() {
+    viewer.getGlobalSettings().setParameterValue("slab", slabPercentSetting);
+    viewer.getGlobalSettings().setParameterValue("depth", depthPercentSetting);
   }
 
   void depthByPercentagePoints(int percentage) {
@@ -911,6 +941,8 @@ abstract class TransformManager {
     depthPercentSetting += percentage;
     if (slabPercentSetting <= depthPercentSetting)
       slabPercentSetting = depthPercentSetting + 1;
+    slabDepthChanged();
+//    System.out.println("transformManager slab/depthpercentSetting: " + slabPercentSetting + " " + depthPercentSetting);
   }
 
   void slabDepthByPercentagePoints(int percentage) {
@@ -918,22 +950,37 @@ abstract class TransformManager {
     depthPlane = null;
     slabPercentSetting += percentage;
     depthPercentSetting += percentage;
+    slabDepthChanged();
+//    System.out.println("transformManager slab/depthpercentSetting: " + slabPercentSetting + " " + depthPercentSetting);
   }
 
   void slabToPercent(int percentSlab) {
+    viewer.setFloatProperty("slabRange", 0);
     slabPercentSetting = percentSlab;
     slabPlane = null;
     if (depthPercentSetting >= slabPercentSetting)
       depthPercentSetting = slabPercentSetting - 1;
+    slabDepthChanged();
   }
 
-  Point4f slabPlane = null;
-  Point4f depthPlane = null;
-
   void depthToPercent(int percentDepth) {
+    viewer.getGlobalSettings().setParameterValue("depth", percentDepth);
     depthPercentSetting = percentDepth;
     if (slabPercentSetting <= depthPercentSetting)
       slabPercentSetting = depthPercentSetting + 1;
+    slabDepthChanged();
+  }
+
+  void zSlabToPercent(int percentSlab) {
+    zSlabPercentSetting = percentSlab;
+    if (zDepthPercentSetting > zSlabPercentSetting)
+      zDepthPercentSetting = percentSlab;
+  }
+
+  void zDepthToPercent(int percentDepth) {
+    zDepthPercentSetting = percentDepth;
+    if (zDepthPercentSetting > zSlabPercentSetting)
+      zSlabPercentSetting = percentDepth;
   }
 
   void slabInternal(Point4f plane, boolean isDepth) {
@@ -1145,7 +1192,7 @@ abstract class TransformManager {
   protected float cameraDepthSetting = 3f;
   protected float visualRange; // set in stateManager to 5f;
   protected float cameraDistance = 1000f; // prevent divide by zero on startup
-
+  
   /**
    * This method returns data needed by the VRML, X3D, and IDTF/U3D exporters
    * It also should serve as a valuable resource for anyone adapting Jmol and 
@@ -1423,8 +1470,27 @@ abstract class TransformManager {
    */
 
   protected void calcSlabAndDepthValues() {
-    zSlabValue = slabValue = zValueFromPercent(slabPercentSetting);
-    zDepthValue = depthValue = zValueFromPercent(depthPercentSetting);
+    if (slabRange < 1)
+      slabValue = zValueFromPercent(slabPercentSetting);
+    else
+      slabValue = (int) (modelCenterOffset * slabRange / (2 * modelRadius) * (zoomPercentSetting / 100));
+    depthValue = zValueFromPercent(depthPercentSetting);
+    if (zSlabPercentSetting == zDepthPercentSetting) {
+      zSlabValue = slabValue;
+      zDepthValue = depthValue;
+    } else {
+      zSlabValue = zValueFromPercent(zSlabPercentSetting);
+      zDepthValue = zValueFromPercent(zDepthPercentSetting);
+    }
+    if (zSlabPoint != null) {
+      try {
+      transformPoint(zSlabPoint, pointT2);
+      zSlabValue = (int) pointT2.z;
+      } catch (Exception e) {
+        // don't care
+      }
+    }
+      
     viewer.getGlobalSettings().setParameterValue("_slabPlane",
         Escape.escape(getSlabDepthPlane(false)));
     viewer.getGlobalSettings().setParameterValue("_depthPlane",
@@ -1729,7 +1795,7 @@ abstract class TransformManager {
     //setSpinOn(false);// trouble here with Viewer.checkHalt
   }
 
-  class MotionThread extends Thread implements Runnable {
+  class MotionThread extends Thread {
     private final Vector3f aaStepCenter = new Vector3f();
     private final Vector3f aaStepNavCenter = new Vector3f();
     private final AxisAngle4f aaStep = new AxisAngle4f();
@@ -1782,6 +1848,7 @@ abstract class TransformManager {
         run();
     }
 
+    @Override
     public void run() {
       if (totalSteps > 0)
         viewer.setInMotion(true);
@@ -2039,7 +2106,8 @@ abstract class TransformManager {
       truncate2(sb, tY);
       sb.append(";");
     }
-    if (modelRadius != rotationRadiusDefault) {
+    if (modelRadius != rotationRadiusDefault || modelRadius == 10) {
+      // after ZAP;load APPEND   we need modelRadius, which is 10
       sb.append(" set rotationRadius");
       truncate2(sb, modelRadius);
       sb.append(";");
@@ -2057,17 +2125,25 @@ abstract class TransformManager {
 
   private String getRotateZyzText(boolean iAddComment) {
     StringBuffer sb = new StringBuffer();
-    float m22 = matrixRotate.m22;
+    Matrix3f m = (Matrix3f) viewer.getModelSetAuxiliaryInfo("defaultOrientationMatrix");
+    if (m == null) {
+      m = matrixRotate;
+    } else {
+      m = new Matrix3f(m);
+      m.invert();
+      m.mul(matrixRotate, m);
+    }
+    float m22 = m.m22;
     float rY = (float) (Math.acos(m22) * degreesPerRadian);
     float rZ1, rZ2;
     if (m22 > .999f || m22 < -.999f) {
-      rZ1 = (float) (Math.atan2(matrixRotate.m10, matrixRotate.m11)
+      rZ1 = (float) (Math.atan2(m.m10, m.m11)
           * degreesPerRadian);
       rZ2 = 0;
     } else {
-      rZ1 = (float) (Math.atan2(matrixRotate.m21, -matrixRotate.m20)
+      rZ1 = (float) (Math.atan2(m.m21, -m.m20)
           * degreesPerRadian);
-      rZ2 = (float) (Math.atan2(matrixRotate.m12, matrixRotate.m02)
+      rZ2 = (float) (Math.atan2(m.m12, m.m02)
           * degreesPerRadian);
     }
     if (rZ1 != 0 && rY != 0 && rZ2 != 0 && iAddComment)
@@ -2133,6 +2209,9 @@ abstract class TransformManager {
       navZ = z;
   }
 
+  /**
+   * @param value  
+   */
   protected void setNavFps(int value) {
     // see TransformManager11
   }
@@ -2161,25 +2240,26 @@ abstract class TransformManager {
     setSpinOn(spinOn, Float.MAX_VALUE, null, null, false);
   }
 
-  private void setSpinOn(boolean spinOn, float endDegrees, List endPositions, BitSet bsAtoms, boolean isGesture) {
+  private void setSpinOn(boolean spinOn, float endDegrees,
+                         List<Point3f> endPositions, BitSet bsAtoms,
+                         boolean isGesture) {
     if (navOn && spinOn)
       setNavOn(false);
     this.spinOn = spinOn;
     viewer.getGlobalSettings().setParameterValue("_spinning", spinOn);
     if (spinOn) {
       if (spinThread == null) {
-        spinThread = new SpinThread(endDegrees, endPositions, bsAtoms, false, isGesture);
+        spinThread = new SpinThread(endDegrees, endPositions, bsAtoms, false,
+            isGesture);
         if (bsAtoms == null)
           spinThread.start();
-        else 
+        else
           spinThread.run();
       }
-    } else {
-      if (spinThread != null) {
-        spinThread.isReset = true;
-        spinThread.interrupt();
-        spinThread = null;
-      }
+    } else if (spinThread != null) {
+      spinThread.isReset = true;
+      spinThread.interrupt();
+      spinThread = null;
     }
   }
 
@@ -2208,16 +2288,16 @@ abstract class TransformManager {
     }
   }
 
-  private class SpinThread extends Thread implements Runnable {
+  private class SpinThread extends Thread {
     float endDegrees;
-    List endPositions;
+    List<Point3f> endPositions;
     float nDegrees;
     BitSet bsAtoms;
     boolean isNav;
     boolean isGesture;
     boolean isReset;
     
-    SpinThread(float endDegrees, List endPositions, BitSet bsAtoms, boolean isNav, boolean isGesture) {
+    SpinThread(float endDegrees, List<Point3f> endPositions, BitSet bsAtoms, boolean isNav, boolean isGesture) {
       setName("SpinThread" + new Date());
       this.endDegrees = Math.abs(endDegrees);
       this.endPositions = endPositions;
@@ -2226,6 +2306,7 @@ abstract class TransformManager {
       this.isGesture = isGesture;
     }
 
+    @Override
     public void run() {
       float myFps = (isNav ? navFps : spinFps);
       viewer.getGlobalSettings().setParameterValue(isNav ? "_navigating" : "_spinning", true);
@@ -2340,6 +2421,9 @@ abstract class TransformManager {
     vibrationScale = scale;
   }
 
+  /**
+   * @param navigatingSurface  
+   */
   protected void setNavigationOffsetRelative(boolean navigatingSurface) {
    // only in Transformmanager11
   }
@@ -2410,12 +2494,13 @@ abstract class TransformManager {
   }
 
   /*private -- removed for fixing warning*/
-  class VibrationThread extends Thread implements Runnable {
+  class VibrationThread extends Thread {
 
     VibrationThread() {
       this.setName("VibrationThread");
     }
 
+    @Override
     public void run() {
       long startTime = System.currentTimeMillis();
       long lastRepaintTime = startTime;
@@ -2442,15 +2527,15 @@ abstract class TransformManager {
   // stereo support
   ////////////////////////////////////////////////////////////////
 
-  int stereoMode;
+  EnumStereoMode stereoMode = EnumStereoMode.NONE;
   int[] stereoColors;
 
   void setStereoMode(int[] twoColors) {
-    stereoMode = JmolConstants.STEREO_CUSTOM;
+    stereoMode = EnumStereoMode.CUSTOM;
     stereoColors = twoColors;
   }
 
-  void setStereoMode(int stereoMode) {
+  void setStereoMode(EnumStereoMode stereoMode) {
     stereoColors = null;
     this.stereoMode = stereoMode;
   }
@@ -2689,6 +2774,10 @@ abstract class TransformManager {
     return Float.NaN;
   }
 
+  /**
+   * @param XorY 
+   * @return percent
+   */
   float getNavigationOffsetPercent(char XorY) {
     return 0;
   }
@@ -2697,6 +2786,10 @@ abstract class TransformManager {
     viewer.getGlobalSettings().setParameterValue("navigationSlab", offset);
   }
 
+  /**
+   * @param addComments  
+   * @return navigation text if in navigation mode
+   */
   String getNavigationText(boolean addComments) {
     return "";
   }
@@ -2714,7 +2807,12 @@ abstract class TransformManager {
     frameOffsets = offsets;
   }
 
+  /**
+   * @param timeSeconds  
+   * @param name 
+   */
   void navigateSurface(float timeSeconds, String name) {
   }
 
+
 }
