@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-09-05 03:26:52 +0200 (dim., 05 sept. 2010) $
- * $Revision: 14238 $
+ * $Date: 2012-01-11 12:59:35 +0100 (mer., 11 janv. 2012) $
+ * $Revision: 16660 $
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
  *
@@ -23,16 +23,14 @@
  */
 package org.jmol.viewer;
 
+import org.jmol.api.Event;
+import org.jmol.g3d.Graphics3D;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
 
 import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
 
-import java.awt.event.KeyEvent;
-import java.awt.event.InputEvent;
-
-import org.jmol.g3d.Graphics3D;
 
 class TransformManager11 extends TransformManager {
 
@@ -49,6 +47,7 @@ class TransformManager11 extends TransformManager {
     setNavFps(10);
   }
 
+  @Override
   protected void setNavFps(int navFps) {
     this.navFps = navFps;
   }
@@ -58,6 +57,7 @@ class TransformManager11 extends TransformManager {
     setNavFps(10);
   }
 
+  @Override
   void zoomByFactor(float factor, int x, int y) {
     if (!zoomEnabled || factor <= 0 || mode != MODE_NAVIGATION) {
       super.zoomByFactor(factor, x, y);
@@ -88,6 +88,7 @@ class TransformManager11 extends TransformManager {
 */    
   }
 
+  @Override
   protected void calcCameraFactors() {
     // (m) model coordinates
     // (s) screen coordinates = (m) * screenPixelsPerAngstrom
@@ -108,7 +109,7 @@ class TransformManager11 extends TransformManager {
     // conversion factor Angstroms --> pixels
     // so that "full window" is visualRange
     scalePixelsPerAngstrom = (scale3D && !perspectiveDepth
-        && mode != MODE_NAVIGATION ? 72 / scale3DAngstromsPerInch
+        && mode != MODE_NAVIGATION ? 72 / scale3DAngstromsPerInch * (antialias ? 2 : 1)
         : screenPixelCount / visualRange); // (s/m)
 
     // model radius in pixels
@@ -166,12 +167,14 @@ class TransformManager11 extends TransformManager {
     
   }
 
+  @Override
   protected float getPerspectiveFactor(float z) {
     // System.out.println (z + " getPerspectiveFactor " + referencePlaneOffset +
     // " " + (z <= 0 ? referencePlaneOffset : referencePlaneOffset / z));
     return (z <= 0 ? referencePlaneOffset : referencePlaneOffset / z);
   }
 
+  @Override
   protected void adjustTemporaryScreenPoint() {
 
     // fixedRotation point is at the origin initially
@@ -255,6 +258,7 @@ class TransformManager11 extends TransformManager {
 
   private int navMode = NAV_MODE_RESET;
 
+  @Override
   void setScreenParameters(int screenWidth, int screenHeight,
                            boolean useZoomLarge, boolean antialias,
                            boolean resetSlab, boolean resetZoom) {
@@ -278,6 +282,7 @@ class TransformManager11 extends TransformManager {
    * All the magic happens here.
    * 
    */
+  @Override
   protected void calcNavigationPoint() {
     // called by finalize
     calcNavigationDepthPercent();
@@ -355,7 +360,7 @@ class TransformManager11 extends TransformManager {
     transformPoint(navigationCenter, navigationOffset);
     navigationOffset.z = referencePlaneOffset;
     navMode = NAV_MODE_NONE;
-    calcNavigationSlabAndDepth();
+    calcNavSlabAndDepthValues();
   }
 
   private float getNavPtHeight() {
@@ -363,19 +368,14 @@ class TransformManager11 extends TransformManager {
     return height / (navigateSurface ? 1f : 2f);
   }
 
-  protected void calcNavigationSlabAndDepth() {
-    slabValue = 0;
-    depthValue = Integer.MAX_VALUE;
-    if (!slabEnabled)
-      return;
-    zSlabValue = slabValue = (mode == MODE_NAVIGATION ? -100 : 0)
-        + (int) (referencePlaneOffset - navigationSlabOffset);
-    zDepthValue = depthValue = zValueFromPercent(depthPercentSetting);
-
-    viewer.getGlobalSettings().setParameterValue("navigationDepth",
-        getNavigationDepthPercent());
-    viewer.getGlobalSettings().setParameterValue("navigationSlab",
-        getNavigationSlabOffsetPercent());
+  protected void calcNavSlabAndDepthValues() {
+    calcSlabAndDepthValues();
+    if (slabEnabled) {
+      slabValue = (mode == MODE_NAVIGATION ? -100 : 0)
+          + (int) (referencePlaneOffset - navigationSlabOffset);
+      if (zSlabPercentSetting == zDepthPercentSetting)
+        zSlabValue = slabValue;
+    }
 
     if (Logger.debugging)
       Logger.debug("\n" + "\nperspectiveScale: " + referencePlaneOffset
@@ -425,6 +425,7 @@ class TransformManager11 extends TransformManager {
     mode = MODE_NAVIGATION;
   }
 
+  @Override
   boolean canNavigate() {
     return true;
   }
@@ -432,6 +433,7 @@ class TransformManager11 extends TransformManager {
   private int nHits;
   private int multiplier = 1;
 
+  @Override
   protected void resetNavigationPoint(boolean doResetSlab) {
 
     // no release from navigation mode if too far zoomed in!
@@ -448,12 +450,15 @@ class TransformManager11 extends TransformManager {
     } else if (doResetSlab) {
       slabPercentSetting = 100;
     }
-    if (doResetSlab)
+    viewer.setFloatProperty("slabRange", 0);
+    if (doResetSlab) {
       slabEnabled = (mode == MODE_NAVIGATION);
+    }
     zoomFactor = Float.MAX_VALUE;
     zoomPercentSetting = zoomPercent;
   }
 
+  @Override
   protected void setNavigationOffsetRelative(boolean navigatingSurface) {
     if (navigatingSurface) {
       navigateSurface(Integer.MAX_VALUE);
@@ -472,6 +477,7 @@ class TransformManager11 extends TransformManager {
     navigate(0, pt);
   }
 
+  @Override
   synchronized void navigate(int keyCode, int modifiers) {
     // 0 0 here means "key released"
     String key = null;
@@ -490,22 +496,22 @@ class TransformManager11 extends TransformManager {
     if (nHits % 10 == 0)
       multiplier *= (multiplier == 4 ? 1 : 2);
     boolean navigateSurface = viewer.getNavigateSurface();
-    boolean isShiftKey = ((modifiers & InputEvent.SHIFT_MASK) > 0);
-    boolean isAltKey = ((modifiers & InputEvent.ALT_MASK) > 0);
-    boolean isCtrlKey = ((modifiers & InputEvent.CTRL_MASK) > 0);
+    boolean isShiftKey = ((modifiers & Event.SHIFT_MASK) > 0);
+    boolean isAltKey = ((modifiers & Event.ALT_MASK) > 0);
+    boolean isCtrlKey = ((modifiers & Event.CTRL_MASK) > 0);
     float speed = viewer.getNavigationSpeed() * (isCtrlKey ? 10 : 1);
     // race condition viewer.cancelRendering();
     switch (keyCode) {
-    case KeyEvent.VK_PERIOD:
+    case Event.VK_PERIOD:
       navX = navY = navZ = 0;
       homePosition(true);
       return;
-    case KeyEvent.VK_SPACE:
+    case Event.VK_SPACE:
       if (!navOn)
         return;
       navX = navY = navZ = 0;
       return;
-    case KeyEvent.VK_UP:
+    case Event.VK_UP:
       if (navOn) {
         if (isAltKey) {
           navY += multiplier;
@@ -536,7 +542,7 @@ class TransformManager11 extends TransformManager {
           * (viewer.getNavigationPeriodic() ? 1 : multiplier);
       navMode = NAV_MODE_NEWZ;
       break;
-    case KeyEvent.VK_DOWN:
+    case Event.VK_DOWN:
       if (navOn) {
         if (isAltKey) {
           navY -= multiplier;
@@ -567,7 +573,7 @@ class TransformManager11 extends TransformManager {
           * (viewer.getNavigationPeriodic() ? 1 : multiplier);
       navMode = NAV_MODE_NEWZ;
       break;
-    case KeyEvent.VK_LEFT:
+    case Event.VK_LEFT:
       if (navOn) {
         navX -= multiplier;
         value = navX;
@@ -586,7 +592,7 @@ class TransformManager11 extends TransformManager {
           null);
       navMode = NAV_MODE_NEWXYZ;
       break;
-    case KeyEvent.VK_RIGHT:
+    case Event.VK_RIGHT:
       if (navOn) {
         navX += multiplier;
         value = navX;
@@ -620,10 +626,11 @@ class TransformManager11 extends TransformManager {
     if (viewer.isRepaintPending())
       return;
     viewer.setShapeProperty(JmolConstants.SHAPE_ISOSURFACE, "navigate",
-        new Integer(dz == Integer.MAX_VALUE ? 2 * multiplier : dz));
+        Integer.valueOf(dz == Integer.MAX_VALUE ? 2 * multiplier : dz));
     viewer.requestRepaintAndWait();
   }
 
+  @Override
   void navigate(float seconds, Point3f pt) {
     if (seconds > 0) {
       navigateTo(seconds, null, Float.NaN, pt, Float.NaN, Float.NaN, Float.NaN);
@@ -636,6 +643,7 @@ class TransformManager11 extends TransformManager {
     navigating = false;
   }
 
+  @Override
   void navigate(float seconds, Vector3f rotAxis, float degrees) {
     if (degrees == 0)
       return;
@@ -651,6 +659,7 @@ class TransformManager11 extends TransformManager {
     navigating = false;
   }
 
+  @Override
   void setNavigationDepthPercent(float timeSec, float percent) {
     if (timeSec > 0) {
       navigateTo(timeSec, null, Float.NaN, null, percent, Float.NaN, Float.NaN);
@@ -659,6 +668,7 @@ class TransformManager11 extends TransformManager {
     setNavigationDepthPercent(percent);
   }
 
+  @Override
   void navTranslate(float seconds, Point3f pt) {
     Point3f pt1 = new Point3f();
     transformPoint(pt, pt1);
@@ -669,6 +679,7 @@ class TransformManager11 extends TransformManager {
     navTranslatePercent(-1, pt1.x, pt1.y);
   }
 
+  @Override
   void navTranslatePercent(float seconds, float x, float y) {
     // if either is Float.NaN, then the other is RELATIVE to current
     transformPoint(navigationCenter, navigationOffset);
@@ -789,15 +800,25 @@ class TransformManager11 extends TransformManager {
     viewer.setInMotion(false);
   }
 
+  @Override
   void navigate(float seconds, Point3f[][] pathGuide) {
     navigate(seconds, pathGuide, null, null, 0, Integer.MAX_VALUE);
   }
 
+  @Override
   void navigate(float seconds, Point3f[] path, float[] theta, int indexStart,
                 int indexEnd) {
     navigate(seconds, null, path, theta, indexStart, indexEnd);
   }
 
+  /**
+   * @param seconds 
+   * @param pathGuide 
+   * @param path 
+   * @param theta  
+   * @param indexStart 
+   * @param indexEnd 
+   */
   private void navigate(float seconds, Point3f[][] pathGuide, Point3f[] path,
                         float[] theta, int indexStart, int indexEnd) {
     if (seconds <= 0) // PER station
@@ -862,6 +883,7 @@ class TransformManager11 extends TransformManager {
     }
   }
 
+  @Override
   void navigateSurface(float timeSeconds, String name) {
   }
 
@@ -910,14 +932,17 @@ class TransformManager11 extends TransformManager {
     matrixRotate.transform(ptVectorWing, pt2s);
   }
 
+  @Override
   Point3f getNavigationCenter() {
     return navigationCenter;
   }
 
+  @Override
   float getNavigationDepthPercent() {
     return navigationDepth;
   }
 
+  @Override
   void setNavigationSlabOffsetPercent(float percent) {
     viewer.getGlobalSettings().setParameterValue("navigationSlab", percent);
     calcCameraFactors(); // current
@@ -929,6 +954,7 @@ class TransformManager11 extends TransformManager {
     return 50 * navigationSlabOffset / modelRadiusPixels;
   }
 
+  @Override
   Point3f getNavigationOffset() {
     transformPoint(navigationCenter, navigationOffset);
     return navigationOffset;
@@ -953,6 +979,7 @@ class TransformManager11 extends TransformManager {
             / modelRadiusPixels));
   }
 
+  @Override
   float getNavigationOffsetPercent(char XorY) {
     getNavigationOffset();
     if (width == 0 || height == 0)
@@ -961,6 +988,7 @@ class TransformManager11 extends TransformManager {
         : (navigationOffset.y - getNavPtHeight()) * 100f / height);
   }
 
+  @Override
   protected String getNavigationText(boolean addComments) {
     getNavigationOffset();
     return (addComments ? " /* navigation center, translation, depth */ " : " ")
@@ -971,6 +999,7 @@ class TransformManager11 extends TransformManager {
         + getNavigationOffsetPercent('Y') + " " + getNavigationDepthPercent();
   }
 
+  @Override
   protected String getNavigationState() {
     if (mode != MODE_NAVIGATION)
       return "";
