@@ -24,6 +24,7 @@
 package org.openscience.jmol.app;
 
 import java.awt.Dimension;
+import java.awt.GraphicsEnvironment;
 import java.awt.Point;
 import java.io.File;
 import java.util.Scanner;
@@ -119,7 +120,11 @@ public class JmolApp {
 
   public void parseCommandLine(String[] args) {
 
-    Options options = getOptions();
+    if (GraphicsEnvironment.isHeadless()) {
+      System.out.println("Use Jmol 12.3 for headless operation");
+      System.exit(0);
+    }
+      Options options = getOptions();
 
     CommandLine line = null;
     try {
