@@ -1,7 +1,7 @@
 /* $RCSfile$
- * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Author$
+ * $Date$
+ * $Revision$
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -28,14 +28,21 @@ import java.awt.Color;
 import java.awt.Container;
 import java.awt.Component;
 import java.awt.Dimension;
-import java.awt.event.*;
+import java.awt.Window;
+import java.awt.event.KeyEvent;
+import java.awt.event.ActionEvent;
 import java.net.URL;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+
+import javax.swing.AbstractButton;
 import javax.swing.JButton;
+import javax.swing.JDialog;
 import javax.swing.JPanel;
 import javax.swing.JFrame;
 import javax.swing.JSplitPane;
-import javax.swing.JTextPane;
-//import javax.swing.SwingUtilities;
+import javax.swing.JTextPane; //import javax.swing.SwingUtilities;
 import javax.swing.text.Position;
 import javax.swing.text.DefaultStyledDocument;
 import javax.swing.text.BadLocationException;
@@ -44,34 +51,47 @@ import javax.swing.text.SimpleAttributeSet;
 import javax.swing.text.StyleConstants;
 import javax.swing.JScrollPane;
 
-import java.util.List;
-
 import org.jmol.api.JmolAppConsoleInterface;
 import org.jmol.api.JmolViewer;
 import org.jmol.console.JmolConsole;
 import org.jmol.i18n.GT;
 import org.jmol.util.CommandHistory;
 import org.jmol.util.Logger;
+import org.jmol.util.Parser;
 import org.jmol.util.TextFormat;
 import org.jmol.viewer.JmolConstants;
-import org.jmol.viewer.Viewer;
 
-public final class AppConsole extends JmolConsole implements JmolAppConsoleInterface, EnterListener{
-  
-  protected ConsoleTextPane console;
-  private JButton varButton, haltButton, closeButton, clearButton, 
-                  questButton, helpButton, undoButton, redoButton;
-  
-  private JButton checkButton;
-  protected JButton stepButton;
-  private JButton topButton;
+public class AppConsole extends JmolConsole implements JmolAppConsoleInterface,
+    EnterListener {
 
-  /*
-   * methods sendConsoleEcho, sendConsoleMessage(strStatus), notifyScriptStart(), notifyScriptTermination()
-   * are public in case developers want to use appConsole separate from the Jmol application.
+  public static final String ALL_BUTTONS = "Editor Variables Clear History State UndoRedo Close Help";
+
+  // note:  "Check" "Top" "Step" not included in 12.1
+
+  /**
+   * general entry point
    * 
+   * @param viewer
+   * @param externalContainer   a JFrame or JPanel or JDialog
+   * @param enabledButtons
    */
-  
+  public AppConsole(JmolViewer viewer,
+      Container externalContainer, String enabledButtons) {
+    this.viewer = viewer;
+    Window w = getWindow((Container) viewer.getDisplay());
+    viewerFrame = (w instanceof JFrame ? (JFrame) w : null);
+    if (externalContainer == null) {
+      jcd = new JDialog(viewerFrame, null, false);
+      jcd.setSize(645, 400);
+      jcd.setLocationRelativeTo(viewerFrame);
+      this.externalContainer = jcd;
+    } else {
+      this.externalContainer = externalContainer;
+      viewer.setConsole(this);
+    }
+    addWindowListener();
+    layoutWindow(enabledButtons);
+  }
 
   public AppConsole() {
     // required for Class.forName  
@@ -79,99 +99,46 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     // appConsole = ((JmolApplicationConsoleInterface) Interface
     //       .getApplicationInterface("jmolpanel.AppConsole")).getAppConsole(viewer, display);
   }
-  
-  public JmolAppConsoleInterface getAppConsole(Viewer viewer, Component display) {
-    return new AppConsole(viewer, display instanceof DisplayPanel ? 
-        ((DisplayPanel)display).getFrame() 
-        : display instanceof JFrame ? (JFrame) display : null);
+
+  /**
+   * don't delete! used by Viewer after it gets the class by name
+   * 
+   * @param viewer
+   * @return         AppConsole or AppletConsole
+   */
+  public JmolAppConsoleInterface getAppConsole(JmolViewer viewer) {
+    // used after reflection gets the class
+    return new AppConsole(viewer, null, null);
   }
 
-  private AppConsole(JmolViewer viewer, JFrame frame) {
-    super(viewer, frame);
-    layoutWindow(getContentPane());
-    setSize(645, 400);
-    setLocationRelativeTo(frame);
-  }
 
-  JButton setButton(String s) {
-    JButton b = new JButton(s);
-    b.addActionListener(this);
-    buttonPanel.add(b);
-    return b;
-  }
-  
-  JPanel buttonPanel = new JPanel();
-  
-  void layoutWindow(Container container) {
-    console = new ConsoleTextPane(this);    
-    console.setPrompt();
-    console.setDragEnabled(true);
-    JScrollPane consolePane = new JScrollPane(console);
-        
-    
-    
-    editButton = setButton(GT._("Editor"));
-    checkButton = setButton(GT._("Check"));
-    topButton = setButton(GT._("Top"));
-    stepButton = setButton(GT._("Step"));
+  JDialog jcd;
+
+  protected ConsoleTextPane console;
+  protected JButton stepButton;
+  protected Map<String, AbstractButton> buttons = new HashMap<String, AbstractButton>();
+
+  private JButton varButton, haltButton, closeButton, clearButton;
+  private JButton helpButton, undoButton, redoButton, checkButton, topButton;
+  private JPanel buttonPanel = new JPanel();
+
+  /*
+   * methods sendConsoleEcho and sendConsoleMessage(strStatus)
+   * are public in case developers want to use appConsole separate from the Jmol application.
+   * 
+   */
 
 
-    //questButton = setButton("?");
-    //questButton.addActionListener(this);
-    //buttonPanel.add(questButton);
-
-    varButton = setButton(GT._("Variables"));
-    clearButton = setButton(GT._("Clear"));
-    haltButton = setButton(GT._("Halt"));
-
-    historyButton = setButton(GT._("History"));
-    stateButton = setButton(GT._("State"));
-
-    helpButton = setButton(GT._("Help"));
-    closeButton = setButton(GT._("Close"));
-    undoButton = setButton(GT._("Undo"));
-    redoButton = setButton(GT._("Redo"));
-
-    undoButton.setEnabled(false);
-    redoButton.setEnabled(false);
-
-
-    
-//    container.setLayout(new BorderLayout());
-  //  container.add(consolePane, BorderLayout.CENTER);
-    JPanel buttonPanelWrapper = new JPanel();
-    buttonPanelWrapper.setLayout(new BorderLayout());
-    buttonPanelWrapper.add(buttonPanel, BorderLayout.CENTER);
-
-    JSplitPane spane = new JSplitPane(
-        JSplitPane.VERTICAL_SPLIT,
-        consolePane, buttonPanelWrapper);
-    consolePane.setMinimumSize(new Dimension(300,300));
-    consolePane.setPreferredSize(new Dimension(5000,5000));
-    buttonPanelWrapper.setMinimumSize(new Dimension(60,60));
-    buttonPanelWrapper.setMaximumSize(new Dimension(1000,60));
-    buttonPanelWrapper.setPreferredSize(new Dimension(60,60));
-    spane.setDividerSize(0);
-    spane.setResizeWeight(0.95);
-    container.add(spane);
-//    container.setLayout(new BorderLayout());
-  //  container.add(consolePane,BorderLayout.CENTER);
-    //container.add(buttonPanelWrapper,BorderLayout.SOUTH);
-
-  }
-
+  @Override
   public void sendConsoleEcho(String strEcho) {
-    if (strEcho != null) {      
+    if (strEcho == null)  // new language
+      updateLabels();
+    else
       console.outputEcho(strEcho);
-    }
     setError(false);
   }
 
-  boolean isError = false;
-  private void setError(boolean TF) {
-    isError = TF;
-  }
-  
+  @Override
   public void sendConsoleMessage(String strStatus) {
     if (strStatus == null) {
       console.clearContent(null);
@@ -185,45 +152,167 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     }
   }
 
+  @Override
+  protected JButton setButton(String label) {
+    JButton b = super.setButton(label);
+    buttonPanel.add(b);
+    return b;
+  }
+
+  @Override
+  protected void setupLabels() {
+    labels.put("Check", GT._("Check"));
+    labels.put("Clear", GT._("Clear"));
+    labels.put("Close", GT._("Close"));
+    labels.put("Halt", GT._("Halt"));
+    labels.put("Help", GT._("Help"));
+    labels.put("Editor", GT._("Editor"));
+    labels.put("History", GT._("History"));
+    labels.put("State", GT._("State"));
+    labels.put("Step", GT._("Step"));
+    labels.put("Top", GT._("Top"));
+    labels.put("Undo", GT._("Undo"));
+    labels.put("Redo", GT._("Redo"));
+    labels.put("Variables", GT._("Variables"));
+  }
+
+  private void layoutWindow(String enabledButtons) {
+    setTitle();
+    console = new ConsoleTextPane(this);
+    console.setPrompt();
+    console.setDragEnabled(true);
+    if (enabledButtons == null)
+      enabledButtons = ALL_BUTTONS;
+    JScrollPane consolePane = new JScrollPane(console);
+    String[] tokens = Parser.getTokens(enabledButtons);
+    for (int i = 0; i < tokens.length; i++)
+      enableButton(tokens[i]);
+    setEnabled(undoButton, false);
+    setEnabled(redoButton, false);
+
+    //    container.setLayout(new BorderLayout());
+    //  container.add(consolePane, BorderLayout.CENTER);
+    JPanel buttonPanelWrapper = new JPanel();
+    buttonPanelWrapper.setLayout(new BorderLayout());
+    buttonPanelWrapper.add(buttonPanel, BorderLayout.CENTER);
+
+    JSplitPane spane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, consolePane,
+        buttonPanelWrapper);
+    consolePane.setMinimumSize(new Dimension(300, 300));
+    consolePane.setPreferredSize(new Dimension(5000, 5000));
+    buttonPanelWrapper.setMinimumSize(new Dimension(60, 60));
+    buttonPanelWrapper.setMaximumSize(new Dimension(1000, 60));
+    buttonPanelWrapper.setPreferredSize(new Dimension(60, 60));
+    spane.setDividerSize(0);
+    spane.setResizeWeight(0.95);
+    getPane().add(spane);
+    //    container.setLayout(new BorderLayout());
+    //  container.add(consolePane,BorderLayout.CENTER);
+    //container.add(buttonPanelWrapper,BorderLayout.SOUTH);
+
+  }
+
+  private void enableButton(String name) {
+    switch((
+        "Check     " +
+    		"Clear     " +
+    		"Close     " +
+    		"Editor    " +
+    		"Halt      " +
+    		"Help      " +
+    		"History   " +
+    		"State     " +
+    		"Step      " +
+    		"Top       " +
+    		"UndoRedo  " +
+    		"Variables ").indexOf(name)) {
+    case 0:
+       checkButton = setButton("Check");
+      break;
+    case 10:
+       clearButton = setButton("Clear");
+      break;
+    case 20:
+       closeButton = setButton("Close");
+      break;
+    case 30:
+       editButton = setButton("Editor");
+      break;
+    case 40:
+       haltButton = setButton("Halt");
+     break;
+    case 50:
+       helpButton = setButton("Help");
+      break;
+    case 60:
+       historyButton = setButton("History");
+      break;
+    case 70:
+       stateButton = setButton("State");
+      break;
+    case 80:
+       stepButton = setButton("Step");
+      break;
+    case 90:
+       topButton = setButton("Top");
+      break;
+    case 100:
+       undoButton = setButton("Undo");
+       redoButton = setButton("Redo");
+      break;
+    case 110:
+       varButton = setButton("Variables");
+      break;
+    }
+  }
+  
+  boolean isError = false;
+
+  private void setError(boolean TF) {
+    isError = TF;
+  }
+
   public void enterPressed() {
     executeCommandAsThread(null);
   }
-  
+
   class ExecuteCommandThread extends Thread {
 
     String strCommand;
-    ExecuteCommandThread (String command) {
+
+    ExecuteCommandThread(String command) {
       strCommand = command;
       this.setName("appConsoleExecuteCommandThread");
     }
-    
+
+    @Override
     public void run() {
-      
+
       try {
-        
+
         while (console.checking) {
-            try {
-              Thread.sleep(100); //wait for command checker
-            } catch (Exception e) {
-              break; //-- interrupt? 
-            }
+          try {
+            Thread.sleep(100); //wait for command checker
+          } catch (Exception e) {
+            break; //-- interrupt? 
+          }
         }
 
         executeCommand(strCommand);
       } catch (Exception ie) {
-        Logger.error("execution command interrupted!",ie);
+        Logger.error("execution command interrupted!", ie);
       }
     }
   }
-   
+
   ExecuteCommandThread execThread;
-  
+
+  @Override
   protected void execute(String strCommand) {
-    //System.out.println("appConsole executing " + strCommand);
     executeCommandAsThread(strCommand);
   }
-  
-  void executeCommandAsThread(String strCommand){ 
+
+  void executeCommandAsThread(String strCommand) {
     if (strCommand == null)
       strCommand = console.getCommandString().trim();
     if (strCommand.equalsIgnoreCase("undo")) {
@@ -240,8 +329,9 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       System.exit(0);
     } else if (strCommand.length() == 0) {
       strCommand = "!resume";
+      undoSetEnabled();
     }
-      
+
     if (strCommand.length() > 0) {
       execThread = new ExecuteCommandThread(strCommand);
       execThread.start();
@@ -252,6 +342,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     }
   }
 
+  
   private static int MAXUNDO = 10;
   private String[] undoStack = new String[MAXUNDO + 1];
   private int undoPointer = 0;
@@ -261,7 +352,6 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     //undoClear();
   }
 
-/*  
   private void undoClear() {
     if (undoButton == null)
       return;
@@ -271,24 +361,25 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     undoButton.setEnabled(false);
     redoButton.setEnabled(false);
   }
-*/
   
-  void undoSetEnabled() {
+  private boolean undoSetEnabled() {
     if (undoButton == null)
-      return;
+      return false;
+    boolean undoAllowed = (viewer.getBooleanProperty("undo") && viewer
+        .getBooleanProperty("preserveState"));
     undoButton
-        .setEnabled(undoPointer > 0 && undoStack[undoPointer - 1] != null);
-    redoButton.setEnabled(undoPointer < MAXUNDO
+        .setEnabled(undoAllowed && undoPointer > 0 && undoStack[undoPointer - 1] != null);
+    redoButton.setEnabled(undoAllowed && undoPointer < MAXUNDO
         && undoStack[undoPointer + 1] != null);
+    return undoAllowed;
   }
 
-  void undoRedo(boolean isRedo) {
+  private void undoRedo(boolean isRedo) {
     if (undoButton == null)
       return;
     // pointer is always left at the undo slot when a command is given
     // redo at CURRENT pointer position
-    if (!viewer.getBooleanProperty("undo")
-        || !viewer.getBooleanProperty("preserveState"))
+    if (!undoSetEnabled())
       return;
     //dumpUndo("undoRedo1");
     int ptr = undoPointer + (isRedo ? 1 : -1);
@@ -309,6 +400,8 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     //dumpUndo("undoRedo DONE");
   }
 
+  private boolean dontsave;
+  
   private void undoSave(boolean incrementPtr) {
     if (undoButton == null)
       return;
@@ -319,7 +412,8 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     for (int i = undoPointer + 1; i <= MAXUNDO; i++)
       undoStack[i] = null;
     Logger.startTimer();
-    undoStack[undoPointer] = (String) viewer.getProperty("readable",
+    try {
+      undoStack[undoPointer] = (String) viewer.getProperty("readable",
         "stateInfo", null);
     //shift stack if full
     if (incrementPtr && undoPointer == MAXUNDO) {
@@ -328,9 +422,12 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       undoStack[MAXUNDO] = null;
     } else if (incrementPtr)
       undoPointer++;
-    if (Logger.checkTimer(null) > 1000) {
-      //viewer.setBooleanProperty("undo", false);
-      //undoClear();
+    } catch (Error e) {
+      dontsave = true;
+    }
+    if (dontsave || Logger.checkTimer(null) > 2000) {
+      viewer.setBooleanProperty("undo", false);
+      undoClear();
       Logger.info("command processing slow; undo disabled");
     } else {
       undoSetEnabled();
@@ -348,6 +445,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     return;
   }
 */
+  @SuppressWarnings("unchecked")
   void executeCommand(String strCommand) {
     boolean doWait;
     console.appendNewline();
@@ -356,7 +454,8 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       console.grabFocus();
       return;
     }
-    if (strCommand.charAt(0) != '!' && viewer.getBooleanProperty("executionPaused"))
+    if (strCommand.charAt(0) != '!'
+        && viewer.getBooleanProperty("executionPaused"))
       strCommand = "!" + strCommand;
     if (strCommand.charAt(0) != '!' && !isError) {
       undoSave(true);
@@ -368,21 +467,24 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     doWait = (strCommand.indexOf("WAITTEST ") == 0);
     if (doWait) { //for testing, mainly
       // demonstrates using the statusManager system; probably hangs application.
-      List info = (List) viewer
+      Object o = viewer
           .scriptWaitStatus(strCommand.substring(5),
               "+fileLoaded,+scriptStarted,+scriptStatus,+scriptEcho,+scriptTerminated");
-      /*
-       * info = [ statusRecortSet0, statusRecortSet1, statusRecortSet2, ...]
-       * statusRecordSet = [ statusRecord0, statusRecord1, statusRecord2, ...]
-       * statusRecord = [int msgPtr, String statusName, int intInfo, String msg]    
-       */
-      for (int i = 0; i < info.size(); i++) {
-        List statusRecordSet = (List) info.get(i);
-        for (int j = 0; j < statusRecordSet.size(); j++) {
-          List statusRecord = (List) statusRecordSet.get(j);
-          Logger.info("msg#=" + statusRecord.get(0) + " " + statusRecord.get(1)
-              + " intInfo=" + statusRecord.get(2) + " stringInfo="
-              + statusRecord.get(3));
+      if (o instanceof List) {
+        List<List<List<Object>>> info = (List<List<List<Object>>>) o;
+        /*
+         * info = [ statusRecortSet0, statusRecortSet1, statusRecortSet2, ...]
+         * statusRecordSet = [ statusRecord0, statusRecord1, statusRecord2, ...]
+         * statusRecord = [int msgPtr, String statusName, int intInfo, String msg]    
+         */
+        for (int i = 0; i < info.size(); i++) {
+          List<List<Object>> statusRecordSet = info.get(i);
+          for (int j = 0; j < statusRecordSet.size(); j++) {
+            List<Object> statusRecord = statusRecordSet.get(j);
+            Logger.info("msg#=" + statusRecord.get(0) + " "
+                + statusRecord.get(1) + " intInfo=" + statusRecord.get(2)
+                + " stringInfo=" + statusRecord.get(3));
+          }
         }
       }
       console.appendNewline();
@@ -400,17 +502,21 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       if (strErrorMessage.length() > 0) {
         console.outputError(strErrorMessage);
       } else {
-        viewer.script(strCommand + (strCommand.indexOf("\1##") >= 0 ? "" : JmolConstants.SCRIPT_EDITOR_IGNORE));
+        viewer.script(strCommand
+            + (strCommand.indexOf("\1##") >= 0 ? ""
+                : JmolConstants.SCRIPT_EDITOR_IGNORE));
       }
     }
     if (strCommand.indexOf("\1##") < 0)
       console.grabFocus();
   }
 
+  @Override
   protected void clearContent(String text) {
     console.clearContent(text);
   }
 
+  @Override
   public void actionPerformed(ActionEvent e) {
     console.grabFocus(); // always grab the focus (e.g., after clear)
     Object source = e.getSource();
@@ -430,9 +536,6 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       return;
     }
 
-    
-    
-    
     if (source == closeButton) {
       setVisible(false);
       return;
@@ -441,10 +544,6 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       viewer.haltScriptExecution();
       return;
     }
-    if (source == questButton) {
-      execute("!?");
-      return;
-    }
     if (source == varButton) {
       execute("!show variables");
       return;
@@ -462,26 +561,28 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       return;
     }
     if (source == helpButton) {
-        URL url = this.getClass().getClassLoader()
-            .getResource("org/openscience/jmol/Data/guide/ch04.html");
-        HelpDialog hd = new HelpDialog(null, url);
-        hd.setVisible(true);
+      URL url = this.getClass().getClassLoader().getResource(
+          "org/openscience/jmol/Data/guide/ch04.html");
+      if (url == null)
+        viewer.script("help");
+      else
+        (new HelpDialog(null, url)).setVisible(true);
     }
     super.actionPerformed(e);
   }
-  
+
   class ConsoleTextPane extends JTextPane {
 
     private ConsoleDocument consoleDoc;
     private EnterListener enterListener;
-    
+
     boolean checking = false;
-    
+
     ConsoleTextPane(AppConsole appConsole) {
       super(new ConsoleDocument());
-      consoleDoc = (ConsoleDocument)getDocument();
+      consoleDoc = (ConsoleDocument) getDocument();
       consoleDoc.setConsoleTextPane(this);
-      this.enterListener = (EnterListener) appConsole;
+      this.enterListener = appConsole;
     }
 
     public String getCommandString() {
@@ -517,19 +618,19 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       if (enterListener != null)
         enterListener.enterPressed();
     }
-    
+
     public void clearContent(String text) {
       consoleDoc.clearContent();
       if (text != null)
-        consoleDoc.outputEcho(text);  
+        consoleDoc.outputEcho(text);
       setPrompt();
     }
-    
-     /*
-     * (non-Javadoc)
-     * 
-     * @see java.awt.Component#processKeyEvent(java.awt.event.KeyEvent)
-     */
+
+    /*
+    * (non-Javadoc)
+    * 
+    * @see java.awt.Component#processKeyEvent(java.awt.event.KeyEvent)
+    */
 
     /**
      * Custom key event processing for command 0 implementation.
@@ -540,6 +641,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
      * 
      * @see java.awt.Component#processKeyEvent(java.awt.event.KeyEvent)
      */
+    @Override
     protected void processKeyEvent(KeyEvent ke) {
       // Id Control key is down, captures events does command
       // history recall and inhibits caret vertical shift.
@@ -604,7 +706,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
           checkCommand();
       }
     }
-    
+
     /**
      * Recall command history.
      * 
@@ -613,7 +715,6 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
      */
     void recallCommand(boolean up) {
       String cmd = viewer.getSetHistory(up ? -1 : 1);
-      //System.out.println(cmd);
       if (cmd == null) {
         return;
       }
@@ -628,29 +729,28 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       } catch (BadLocationException e) {
         e.printStackTrace();
       }
-    }  
-     
+    }
+
     synchronized void checkCommand() {
       String strCommand = consoleDoc.getCommandString();
       if (strCommand.length() == 0 || strCommand.charAt(0) == '!'
-          || viewer.isScriptExecuting() || viewer.getBooleanProperty("executionPaused"))
+          || viewer.isScriptExecuting()
+          || viewer.getBooleanProperty("executionPaused"))
         return;
       checking = true;
       consoleDoc
-          .colorCommand(viewer.scriptCheck(strCommand) instanceof String ? 
-             consoleDoc.attError : consoleDoc.attUserInput);
+          .colorCommand(viewer.scriptCheck(strCommand) instanceof String ? consoleDoc.attError
+              : consoleDoc.attUserInput);
       checking = false;
     }
   }
 
+  @Override
   protected String completeCommand(String thisCmd) {
     return super.completeCommand(thisCmd);
   }
 
-  public Object getMyMenuBar() {
-    return null;
-  }
-
+  @Override
   public String getText() {
     return console.getText();
   }
@@ -677,7 +777,6 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       attUserInput = new SimpleAttributeSet();
       StyleConstants.setForeground(attUserInput, Color.black);
 
-      SimpleAttributeSet attEcho;
       attEcho = new SimpleAttributeSet();
       StyleConstants.setForeground(attEcho, Color.blue);
       StyleConstants.setBold(attEcho, true);
@@ -692,12 +791,13 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     }
 
     private Position positionBeforePrompt; // starts at 0, so first time isn't tracked (at least on Mac OS X)
-    private Position positionAfterPrompt;  // immediately after $, so this will track
-    private int offsetAfterPrompt;         // only still needed for the insertString override and replaceCommand
+    private Position positionAfterPrompt; // immediately after $, so this will track
+    private int offsetAfterPrompt; // only still needed for the insertString override and replaceCommand
 
     boolean isAtEnd() {
       return consoleTextPane.getCaretPosition() == getLength();
     }
+
     /** 
      * Removes all content of the script window, and add a new prompt.
      */
@@ -708,7 +808,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
         Logger.error("Could not clear script window content", exception);
       }
     }
-    
+
     void setPrompt() {
       try {
         super.insertString(getLength(), "$ ", attPrompt);
@@ -749,12 +849,12 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
         int pt = consoleTextPane.getCaretPosition();
         Position caretPosition = createPosition(pt);
         pt = positionBeforePrompt.getOffset();
-        super.insertString(pt, str+"\n", attribute);
+        super.insertString(pt, str + "\n", attribute);
         //setOffsetPositions();
         offsetAfterPrompt += str.length() + 1;
         positionBeforePrompt = createPosition(offsetAfterPrompt - 2);
         positionAfterPrompt = createPosition(offsetAfterPrompt - 1);
-        
+
         pt = caretPosition.getOffset();
         consoleTextPane.setCaretPosition(pt);
       } catch (Exception e) {
@@ -769,7 +869,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
 
     void outputErrorForeground(String strError) {
       try {
-        super.insertString(getLength(), strError+"\n", attError);
+        super.insertString(getLength(), strError + "\n", attError);
         consoleTextPane.setCaretPosition(getLength());
       } catch (BadLocationException e) {
         e.printStackTrace();
@@ -796,15 +896,16 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
 
     // override the insertString to make sure everything typed ends up at the end
     // or in the 'command line' using the proper font, and the newline is processed.
+    @Override
     public void insertString(int offs, String str, AttributeSet a)
-      throws BadLocationException {
+        throws BadLocationException {
       int ichNewline = str.indexOf('\n');
       if (ichNewline != 0) {
         if (offs < offsetAfterPrompt) {
           offs = getLength();
         }
         super.insertString(offs, str, a == attError ? a : attUserInput);
-        consoleTextPane.setCaretPosition(offs+str.length());
+        consoleTextPane.setCaretPosition(offs + str.length());
       }
       if (ichNewline >= 0) {
         consoleTextPane.enterPressed();
@@ -815,7 +916,7 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       String strCommand = "";
       try {
         int cmdStart = positionAfterPrompt.getOffset();
-        strCommand =  getText(cmdStart, getLength() - cmdStart);
+        strCommand = getText(cmdStart, getLength() - cmdStart);
         while (strCommand.length() > 0 && strCommand.charAt(0) == ' ')
           strCommand = strCommand.substring(1);
       } catch (BadLocationException e) {
@@ -824,8 +925,8 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
       return strCommand;
     }
 
-    public void remove(int offs, int len)
-      throws BadLocationException {
+    @Override
+    public void remove(int offs, int len) throws BadLocationException {
       if (offs < offsetAfterPrompt) {
         len -= offsetAfterPrompt - offs;
         if (len <= 0)
@@ -833,11 +934,12 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
         offs = offsetAfterPrompt;
       }
       super.remove(offs, len);
-//      consoleTextPane.setCaretPosition(offs);
+      //      consoleTextPane.setCaretPosition(offs);
     }
 
+    @Override
     public void replace(int offs, int length, String str, AttributeSet attrs)
-      throws BadLocationException {
+        throws BadLocationException {
       if (offs < offsetAfterPrompt) {
         if (offs + length < offsetAfterPrompt) {
           offs = getLength();
@@ -848,18 +950,19 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
         }
       }
       super.replace(offs, length, str, attrs);
-//      consoleTextPane.setCaretPosition(offs + str.length());
+      //      consoleTextPane.setCaretPosition(offs + str.length());
     }
 
-     /**
-     * Replaces current command on script.
-     * 
-     * @param newCommand new command value
-     * @param isError    true to set error color  ends with #??
-     * 
-     * @throws BadLocationException
-     */
-    void replaceCommand(String newCommand, boolean isError) throws BadLocationException {
+    /**
+    * Replaces current command on script.
+    * 
+    * @param newCommand new command value
+    * @param isError    true to set error color  ends with #??
+    * 
+    * @throws BadLocationException
+    */
+    void replaceCommand(String newCommand, boolean isError)
+        throws BadLocationException {
       if (positionAfterPrompt == positionBeforePrompt)
         return;
       replace(offsetAfterPrompt, getLength() - offsetAfterPrompt, newCommand,
@@ -869,13 +972,12 @@ public final class AppConsole extends JmolConsole implements JmolAppConsoleInter
     void colorCommand(SimpleAttributeSet att) {
       if (positionAfterPrompt == positionBeforePrompt)
         return;
-      setCharacterAttributes(offsetAfterPrompt, getLength() - offsetAfterPrompt, att, true);
+      setCharacterAttributes(offsetAfterPrompt,
+          getLength() - offsetAfterPrompt, att, true);
     }
   }
-
 }
 
 interface EnterListener {
   public void enterPressed();
 }
-
