@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-07-23 00:55:50 +0200 (ven., 23 juil. 2010) $
- * $Revision: 13656 $
+ * $Date: 2011-09-08 17:07:15 +0200 (jeu., 08 sept. 2011) $
+ * $Revision: 16044 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -41,8 +41,9 @@ import javax.vecmath.Point3f;
 import java.awt.*;
 import java.awt.event.*;
 
+import java.util.Iterator;
+import java.util.Map;
 import java.util.Properties;
-import java.util.Hashtable;
 import java.util.Enumeration;
 
 import org.jmol.i18n.GT;
@@ -428,7 +429,7 @@ ActionListener, ChangeListener, Runnable {
     try {
       currentIndex = index;
       int atomSetIndex = indexes[index];
-      viewer.evalStringQuiet("frame " + viewer.getModelNumber(atomSetIndex));
+      viewer.evalStringQuiet("frame " + viewer.getModelNumberDotted(atomSetIndex));
       infoLabel.setText(viewer.getModelName(atomSetIndex));
       showProperties(viewer.getModelProperties(atomSetIndex));
       showAuxiliaryInfo(viewer.getModelAuxiliaryInfo(atomSetIndex));
@@ -444,7 +445,7 @@ ActionListener, ChangeListener, Runnable {
   protected void setIndexes(DefaultMutableTreeNode node) {
     int atomSetCount = node.getLeafCount();
     indexes = new int[atomSetCount];
-    Enumeration e = node.depthFirstEnumeration();
+    Enumeration<?> e = node.depthFirstEnumeration();
     int idx=0;
     while (e.hasMoreElements()) {
       node = (DefaultMutableTreeNode) e.nextElement();
@@ -576,26 +577,28 @@ ActionListener, ChangeListener, Runnable {
   public void stateChanged(ChangeEvent e) {
     Object src = e.getSource();
     int value = ((JSlider) src).getValue();
+    String cmd = null;
     if (src == selectSlider) {
       showAtomSetIndex(value, false);
     } else if (src == fpsSlider) {
       if (value == 0)
         fpsSlider.setValue(1); // make sure I never set it to 0...
       else
-        viewer.evalStringQuiet("animation fps " + value);
+        cmd = "animation fps " + value;
     } else if (src == radiusSlider) {
       if (value == 0)
         radiusSlider.setValue(1); // make sure I never set it to 0..
       else
-        viewer.evalStringQuiet("vector " + value);
+        cmd = "vector " + value;
     } else if (src == scaleSlider) {
-      viewer.evalStringQuiet("vector scale " + (value * SCALE_PRECISION));
+      cmd = "vector scale " + (value * SCALE_PRECISION);
     } else if (src == amplitudeSlider) {
-      viewer
-          .evalStringQuiet("vibration scale " + (value * AMPLITUDE_PRECISION));
+      cmd = "vibration scale " + (value * AMPLITUDE_PRECISION);
     } else if (src == periodSlider) {
-      viewer.evalStringQuiet("vibration " + (value * PERIOD_PRECISION));
+      cmd = "vibration " + (value * PERIOD_PRECISION);
     }
+    if (cmd != null)
+      viewer.evalStringQuiet(cmd);
   }
   
   /**
@@ -607,7 +610,7 @@ ActionListener, ChangeListener, Runnable {
     boolean needLF = false;
     propertiesTextArea.setText("");
     if (properties != null) {
-      Enumeration e = properties.propertyNames();
+      Enumeration<?> e = properties.propertyNames();
       while (e.hasMoreElements()) {
         String propertyName = (String)e.nextElement();
         if (propertyName.startsWith("."))
@@ -624,13 +627,13 @@ ActionListener, ChangeListener, Runnable {
    * AtomSetChooser window
    * @param auxiliaryInfo Hashtable to be shown.
    */
-  protected void showAuxiliaryInfo(Hashtable auxiliaryInfo) {
+  protected void showAuxiliaryInfo(Map<String, Object> auxiliaryInfo) {
     String separator = " ";
     //propertiesTextArea.setText(""); AFTER properties
     if (auxiliaryInfo != null) {
-      Enumeration e = auxiliaryInfo.keys();
-      while (e.hasMoreElements()) {
-        String keyName = (String) e.nextElement();
+      Iterator<String> e = auxiliaryInfo.keySet().iterator();
+      while (e.hasNext()) {
+        String keyName = e.next();
         if (keyName.startsWith("."))
           continue; // skip the 'hidden' ones
         //won't show objects properly, of course
@@ -732,6 +735,7 @@ ActionListener, ChangeListener, Runnable {
       return atomSetIndex;
     }
     
+    @Override
     public String toString() {
       return atomSetName;
     }
