@@ -1,7 +1,7 @@
 /* $RCSfile$
- * $Author: hansonr $
- * $Date: 2011-01-23 07:24:52 +0100 (dim., 23 janv. 2011) $
- * $Revision: 15021 $
+ * $Author: nicove $
+ * $Date: 2010-08-05 07:33:23 +0200 (jeu., 05 août 2010) $
+ * $Revision: 13833 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -129,6 +129,7 @@ public class PovrayDialog extends JDialog {
       }
     };
     InputVerifier updateInputVerifier = new InputVerifier() {
+      @Override
       public boolean verify(JComponent component) {
         updateScreen();
         return true;
@@ -270,7 +271,7 @@ public class PovrayDialog extends JDialog {
     imageSizeWidth.setToolTipText(text);
     imageSizeXYBox.add(imageSizeWidth);
     imageSizeTextWidth = new JFormattedTextField();
-    imageSizeTextWidth.setValue(new Integer(outputWidth));
+    imageSizeTextWidth.setValue(Integer.valueOf(outputWidth));
     imageSizeTextWidth.addPropertyChangeListener("value",
       new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent e) {
@@ -287,7 +288,7 @@ public class PovrayDialog extends JDialog {
     imageSizeHeight.setToolTipText(text);
     imageSizeXYBox.add(imageSizeHeight);
     imageSizeTextHeight = new JFormattedTextField();
-    imageSizeTextHeight.setValue(new Integer(outputHeight));
+    imageSizeTextHeight.setValue(Integer.valueOf(outputHeight));
     imageSizeTextHeight.addPropertyChangeListener("value",
       new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent e) {
@@ -670,15 +671,15 @@ public class PovrayDialog extends JDialog {
   	        break;
   	      case 1: // Jmol
   	        height = (int)(((double) width) * outputHeight / outputWidth);
-  	        imageSizeTextHeight.setValue(new Integer(height));
+  	        imageSizeTextHeight.setValue(Integer.valueOf(height));
   	        break;
   	      case 2: // 4/3
   	        height = (int)(((double) width) * 3 / 4);
-  	        imageSizeTextHeight.setValue(new Integer(height));
+  	        imageSizeTextHeight.setValue(Integer.valueOf(height));
   	        break;
   	      case 3: // 16/9
   	        height = (int)(((double) width) * 9 / 16);
-  	        imageSizeTextHeight.setValue(new Integer(height));
+  	        imageSizeTextHeight.setValue(Integer.valueOf(height));
   	        break;
   	      }
   	    }
@@ -1035,6 +1036,7 @@ public class PovrayDialog extends JDialog {
      * Closes the dialog when window closing event occurs.
      * @param e Event
      */
+    @Override
     public void windowClosing(WindowEvent e) {
       cancelPressed();
       setVisible(false);
