@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-12-22 14:32:44 +0100 (jeu., 22 déc. 2011) $
- * $Revision: 16625 $
+ * $Date: 2012-03-19 22:33:37 +0100 (lun., 19 mars 2012) $
+ * $Revision: 16913 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -27,6 +27,7 @@ import org.jmol.api.*;
 import org.jmol.console.KeyJMenuItem;
 import org.jmol.i18n.GT;
 import org.jmol.util.Logger;
+import org.jmol.viewer.JmolConstants;
 import org.openscience.jmol.app.jmolpanel.GuiMap;
 import org.openscience.jmol.app.jmolpanel.JmolPanel;
 
@@ -109,11 +110,11 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     "showBoundingBox",                "false",
     "axesOrientationRasmol",          "false",
     "openFilePreview",                "true",
-    "percentVdwAtom",                 "23",
     "autoBond",                       "true",
-    "marBond",                        "150",
-    "minBondDistance",                "0.40",
-    "bondTolerance",                  "0.45",
+    "percentVdwAtom",                 "" + JmolConstants.DEFAULT_PERCENT_VDW_ATOM,
+    "marBond",                        "" + JmolConstants.DEFAULT_BOND_MILLIANGSTROM_RADIUS,
+    "minBondDistance",                "" + JmolConstants.DEFAULT_MIN_BOND_DISTANCE,
+    "bondTolerance",                  "" + JmolConstants.DEFAULT_BOND_TOLERANCE,
   };
 
   final static String[] rasmolOverrides = {
