@@ -180,6 +180,7 @@ public class SimpleMarchingCubesOld {
     }
     int cellIndex0 = cubeCountY * cubeCountZ - 1;
     int cellIndex = cellIndex0;
+    float[][][] voxelData = (mode == MODE_CUBE ? volumeData.getVoxelData() : null);
     for (int x = x0; x != x1; x += xStep, ptX += ptStep, pt = ptX, cellIndex = cellIndex0) {
       if (mode == MODE_GETXYZ) {
         float[] plane = xyPlanes[0];
@@ -219,7 +220,7 @@ public class SimpleMarchingCubesOld {
               break;
             default:
             case MODE_CUBE:
-              vertexValues[i] = volumeData.voxelData[x + offset.x][y + offset.y][z
+              vertexValues[i] = voxelData[x + offset.x][y + offset.y][z
                   + offset.z];
               isInside = isInside(vertexValues[i], cutoff, isCutoffAbsolute);
               if (isInside)
@@ -252,7 +253,9 @@ public class SimpleMarchingCubesOld {
 
   BitSet bsValues = new BitSet();
 
-  private float getValue(int i, int x, int y, int z, int pt, float[] tempValues) {
+  private float getValue(@SuppressWarnings("unused") int i,
+                         int x, int y, int z,
+                         int pt, float[] tempValues) {
     //if (bsValues.get(pt))
       //return tempValues[pt % yzCount];
     bsValues.set(pt);
@@ -267,7 +270,8 @@ public class SimpleMarchingCubesOld {
   private final int[] nullNeighbor = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1 };
 
-  private int[] propagateNeighborPointIndexes(int x, int y, int z, int pt,
+  private int[] propagateNeighborPointIndexes(int x, int y, int z,
+                                              @SuppressWarnings("unused") int pt,
                                               int[][] isoPointIndexes,
                                               int cellIndex) {
     /*
@@ -388,7 +392,10 @@ public class SimpleMarchingCubesOld {
     256, 512, 1024, 2048 };
 
   private boolean processOneCubical(int insideMask, int[] voxelPointIndexes,
-                                    int x, int y, int z, int pt) {
+                                    @SuppressWarnings("unused") int x,
+                                    @SuppressWarnings("unused") int y,
+                                    @SuppressWarnings("unused") int z,
+                                    @SuppressWarnings("unused") int pt) {
     
     // the key to the algorithm is that we have a catalog that
     // maps the inside-vertex mask to an edge mask. 
